* SUBROUTINE EIQSXX                ALL SYSTEMS                00/12/01
C PORTABILITY : ALL SYSTEMS
C 00/12/01 RU : ORIGINAL VERSION
*
* PURPOSE :
*  INPUT OF HESSIAN MATRIX FOR SPARSE QP.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  II  M  NUMBER OF CONSTRAINTS.
*  II  NEQ  NUMBER OF EQUALITY CONSTRAINTS.
*  II  NAC  NUMBER OF ACTIVE CONSTRAINTS.
*  II  CONDG  LOGARITHM OF THE CONDITION NUMBER OF THE HESSIAN MATRIX.
*  II  CONDB  LOGARITHM OF THE CONDITION NUMBER OF THE JACOBIAN MATRIX.
*  II  CONDZ  LOGARITHM OF THE CONDITION NUMBER OF THE REDUCED HESSIAN
*         MATRIX.
*  II  CONDA  LOGARITHM OF THE CONDITION NUMBER OF THE ACTIVE JACOBIAN
*         MATRIX.
*  RI  GLMIN  MINIMUM NONZERO EIGENVALUE OF THE HESSIAN MATRIX.
*  RI  BLMIN  MINIMUM NONZERO EIGENVALUE OF THE JACOBIAN MATRIX.
*  RI  ZLMIN  MINIMUM NONZERO EIGENVALUE OF THE REDUCED HESSIAN MATRIX.
*  RI  ALMIN  MINIMUM NONZERO EIGENVALUE OF THE ACTIVE JACOBIAN MATRIX.
*  II  GRANK  RANK OF THE HESSIAN MATRIX.
*  II  ZRANK  RANK OF THE REDUCED HESSIAN MATRIX.
*  II  INDG  DISTRIBUTION FOR SINGULAR VALUES OF THE HESSIAN MATRIX.
*         INDG=0-LOGARITHMIC DISTRIBUTION. INDG=1-UNIFORM DISTRIBUTION.
*         INDG=2-EQUALLY SPACED VALUES.
*  II  INDB  DISTRIBUTION FOR SINGULAR VALUES OF THE JACOBIAN MATRIX.
*         INDB=0-LOGARITHMIC DISTRIBUTION. INDB=1-UNIFORM DISTRIBUTION.
*         INDB=2-EQUALLY SPACED VALUES.
*  II  IRAND  SEED FOR THE GENERATION OF RANDOM NUMBERS.
*  II  NDEG  LEVEL OF DEGENERACY OF THE CONSTRAINTS.
*  RI  SPARG  SPARSITY OF THE HESSIAN MATRIX (PERCENTAGE).
*  RI  SPARB  SPARSITY OF THE JACOBIAN MATRIX (PERCENTAGE).
*  II  INDEQ  BANDWIDTH OF BOTH THE HESSIAN AND THE JACOBIAN MATRICES
*         IF INDEQ IS NONZERO. IF INDEQU IS ZERO, NO BAND STRUCTURE IS
*         ASSUMED.
*  II  IMAX  MAXIMUM NUMBER OF NONZERO ELEMENTS IN THE SPARSE ROW.
*  II  LMAX  MAXIMUM NUMBER OF NONZERO ELEMENTS IN THE PROBLEM MATRICES.
*  IO  MG  NUMBER OF NONZERO ELEMENTS OF THE HESSIAN MATRIX.
*  RO  VG(LMAX)  NUMERICAL VALUES OF NONZERO ELEMENTS OF THE HESSIAN
*         MATRIX.
*  IO  JG(LMAX)  COLUMN INDICES OF NONZERO ELEMENTS OF THE HESSIAN
*         MATRIX.
*  IO  KG(N+1)  ROW POINTERS OF THE HESSIAN MATRIX.
*  RO  Q(N)  GRADIENT VECTOR.
*  IO  MB  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  RO  VB(LMAX)  NUMERICAL VALUES OF NONZERO ELEMENTS OF THE JACOBIAN
*         MATRIX.
*  IO  JB(LMAX)  COLUMN INDICES OF NONZERO ELEMENTS OF THE JACOBIAN
*         MATRIX.
*  IO  KB(N+1)  ROW POINTERS OF THE JACOBIAN MATRIX.
*  RO  D(M)  RIGHT HAND SIDES OF CONSTRAINTS.
*  RO  X(N)  THE SOLUTION VECTOR.
*  RO  FMIN  MINIMUM VALUE OF THE OBJECTIVE FUNCTION.
*  RO  LAMBDA(M) VECTOR OF THE LAGRANGE MULTIPLIERS.
*  RA  G(N,IMAX) AUXILIARY VECTOR.
*  IA  IG(N,IMAX) AUXILIARY VECTOR.
*  RA  B(M,IMAX) AUXILIARY VECTOR.
*  IA  IB(M,IMAX) AUXILIARY VECTOR.
*  RA  TEMPI(N+M) AUXILIARY VECTOR.
*  RA  TEMPJ(N+M) AUXILIARY VECTOR.
*  IA  INDI(N+M) AUXILIARY VECTOR.
*  IA  INDJ(N+M) AUXILIARY VECTOR.
*
* METHOD :
* V.RUGGIERO, L.ZANI : ON A CLASS OF ITERATIVE METHODS FOR LARGE-SCALE
* CONVEX QUADRATIC PROGRAMS. REND. DEL CIRC. MATEM. DI PALERMO,
* SERIE II, SUPPL. 58 (1999), PP. 213-227.
* G.W.STEWART : THE EFFICIENT GENERATION OF RANDOM ORTHOGONAL MATRICES
* WITH AN APPLICATION TO CONDITION ESTIMATORS. SIAM J. NUMER. ANAL. 17,
* NO. 3 (1980), PP. 403-409.
*
      SUBROUTINE EIQSXX(N,M,NEQ,NAC,CONDG,CONDB,CONDZ,CONDA,GLMIN,BLMIN,
     & ZLMIN,ALMIN,GRANK,ZRANK,INDG,INDB,IRAND,NDEG,SPARG,SPARB,INDEQ,
     & IMAX,LMAX,MG,VG,JG,KG,Q,MB,VB,JB,KB,D,X,FMIN,LAMBDA,G,IG,B,IB,
     & TEMPI,TEMPJ,INDI,INDJ)
      INTEGER N,M,NEQ,NAC,CONDG,CONDB,CONDZ,CONDA,GRANK,ZRANK,INDG,INDB,
     & IRAND,NDEG,INDEQ,IMAX,LMAX,MG,JG(LMAX),KG(N+1),MB,JB(LMAX),
     & KB(M+1),IG(N,IMAX),IB(M,IMAX),INDI(N+M),INDJ(N+M)
      REAL*8 GLMIN,BLMIN,ZLMIN,ALMIN,SPARG,SPARB,VG(LMAX),Q(N),VB(LMAX),
     & D(M),X(N),FMIN,LAMBDA(M),G(N,IMAX),B(M,IMAX),TEMPI(N+M),
     & TEMPJ(N+M)
      INTEGER ISEME,NULLD1,NULLD2,NNULLG,NNULLB,IEQG,ICONTG,ICONDG,
     & ICONTB,ICONDB,NDTOT,IEQUI,I,J,K,L,II,IR,JC,MN
      REAL*8 TOL,UCOS,RANDF,VC
      COMMON /SEME/ISEME
      ISEME=IRAND
      N=MAX(N,0)
      M=MAX(M,0)
      NEQ=MIN(N,MAX(NEQ,0))
      NAC=MIN(M,MAX(NAC,NEQ))
      ZRANK=MIN(N-NAC,ZRANK)
C
C--------- Executable statements ---------------------------------------
C
       tol=1.0d-15
       if(indeq.eq.0)indeq=n
       nulld1=n-nac-zrank               ! zero diag. elements of D1
       nulld2=n-grank-nulld1            ! zero diag. elements of D2
       nnullb=M*n*(1.0d0-sparb/1.0d2) ! nonzero diag. elements of B
       nnullg=n*n*(1.0d0-sparg/1.0d2) ! nonzero diag. elements of G
C
C----- Generating G ----------------------------------------------------
C
       ieqg=nnullg/n  +1
       do i=1,n
        do j=1,imax
        g(i,j)=0.0d0
        ig(i,j) =0
        enddo
       enddo
       icontg=grank
       icondg=grank
       do i=1,n
        kg(i)=1
        ig(i,1)=i
       enddo
       call sing(g,nac,nac-nulld2,condg,glmin,indg)
       if(n-nac.ne.0)then
          call sing(g(nac+1,1),n-nac,zrank,condz,zlmin,indg)
       endif
       k=0
1007   if(icondg.lt.n)then
        do ir=1,n/2
          if(abs(g(ir,1)).lt.tol)then
            jc=randf()*(n-ir)+1+ir
            goto 345
          endif
        enddo
        do jc=n/2+1,n
          if(abs(g(jc,1)).lt.tol)then
            ir=randf()*(jc-1)+1
            goto 345
          endif
        enddo
       endif
       if(icontg.gt.nnullg)goto 2001
       ii=0
2344   ir=randf()*(n-n/10)+1
       jc=randf()*(min(n-ir,indeq))+1+ir
       ii=ii+1
         if ((kg(ir).gt.ieqg.or.kg(jc).gt.ieqg)
     1          .and.ii.lt.1000)goto 2344
         if(ii.ge.1000)then
            write(*,*)'sparg is too big'
            stop
          endif
        if(ir.eq.jc)goto 2344
345    k=k+1
       if (k.gt.lmax) then
         write(*,*)' error: lmax too small'
              stop
       endif
       jg(k)=ir
       jb(k)=jc
       vc=randf()*2.0d0-1.0d0
       vg(k)=vc
       call tleft(g,ig,kg,n,vc,icontg,icondg,tol,n,imax,ir,jc,
     1  tempi,tempj,indi,indj)
       call tright(g,ig,kg,n,vc,icontg,icondg,tol,n,imax,ir,jc,
     1  tempi,tempj,indi,indj)
        do j=2,kg(ir)
         ii=ig(ir,j)
         do l=1,kg(ii)
            if(ig(ii,l).eq.ir)then
              g(ii,l)=g(ir,j)
              goto 333
            endif
         enddo
333       continue
        enddo
        do j=2,kg(jc)
         ii=ig(jc,j)
         do l=1,kg(ii)
            if(ig(ii,l).eq.jc)then
              g(ii,l)=g(jc,j)
              goto 3334
            endif
         enddo
3334    continue
        enddo
        goto 1007
2001    do i=1,n
         x(i)=kg(i)
        enddo
       sparg=(1.0d0-icontg/real(n*n))*100
C
C----- Generating B ----------------------------------------------------
C
       do i=1,m
        kb(i)=0
        do j=1,imax
          b(i,j)=0.0d0
          ib(i,j)=0
        enddo
       enddo
       mn=min(m,n)
       icontb=mn
       icondb=mn
       do i=1,mn
        kb(i)=1
        ib(i,1)=i
       enddo
       call sing(b,nac,nac,conda,almin,indb)
        if (m.gt.nac) then
       call sing(b(nac+1,1),m-nac,mn-nac,condb,blmin,indb)
        endif
       ndtot=m*n*(1.0d0-sparb/100.0d0)
          iequi=ndtot/m +1
       do i=1,k
       call tleft(b,ib,kb,m,vg(i),icontb,icondb,tol,
     +            m,imax,jg(i),jb(i),tempi,tempj,indi,indj)
       enddo
       do jc=n+1,m
         ii=0
1211      ir=randf()*(jc-nac-1) + nac+1
         ii=ii+1
         if (kb(ir).gt.iequi.and. ii.lt.1000) goto 1211
          if(ii.eq.1000)then
             write(*,*)'sparb id too big'
             stop
         endif
       ucos=randf()*2.0d0-1.0d0
       call tright(b,ib,kb,n,ucos,icontb,icondb,tol,m,imax,ir,jc,
     1  tempi,tempj,indi,indj)
       enddo
       if(icontb.ge.ndtot)goto 100
999    continue
         ii=0
1212      ir=randf()*(m-m/10)+1
          ii=ii+1
          if (kb(ir).gt.iequi.and. ii.lt.1000) goto 1212
           if(ii.eq.1000)then
              write(*,*)'sparb is too big'
              stop
           endif
        if (ir.lt.nac) then
         jc=randf()*(min(indeq, nac-ir))+1+ir
        elseif (ir.eq.nac)then
            goto 999
        else
         jc=randf()*(min(m-ir,indeq))+1+ir
        endif
       ucos=randf()*2.0d0-1.0d0
       call tright(b,ib,kb,n,ucos,icontb,icondb,tol,m,imax,ir,jc,
     1  tempi,tempj,indi,indj)
       if(icontb.ge.ndtot)goto 100
       goto 999
100    continue
       sparb=(1.0d0-icontb/real(m*n))*100
       do i=1,m
        d(i)=kb(i)
       enddo
C
C----- Generating the solution x and the non-zero multiplier lambda: ---
C
       do i=1,n
        x(i)=randf()*2.0d0-1.0d0
       enddo
       do i=1,nac
        lambda(i)=10**(-randf()*ndeg)
       enddo
C
C------Generating the vector d -----------------------------------------
C
       do i=1,m
        d(i)=0.0d0
        do j=1,kb(i)
         d(i)=d(i)+b(i,j)*x(ib(i,j))
        enddo
       enddo
       do i=nac+1,m
        d(i)=d(i)-10*randf()
       enddo
C
C----- Generating the vector q -----------------------------------------
C
        do i=1,n
        q(i)=0.0d0
        do j=1,kg(i)
          q(i)=q(i)-g(i,j)*x(ig(i,j))
        enddo
        enddo
        fmin=0.0d0
        do i=1,n
        fmin=fmin-q(i)*x(i)
        enddo
        fmin=fmin*0.5d0
        do i=1,nac
        do j=1,kb(i)
        q(ib(i,j))=q(ib(i,j))+lambda(i)*b(i,j)
        enddo
        enddo
C
C----- Compute the minimum value of the function -----------------------
C
        do i=1,n
        fmin=fmin+q(i)*x(i)
        enddo
C
C---------- Postprocessing ---------------------------------------------
C
        k=0
        do i=1,n
        ii=kg(i)
        kg(i)=k+1
        do j=1,ii
        if (ig(i,j).ge.i) then
        k=k+1
        jg(k)=ig(i,j)
        vg(k)=g(i,j)
        endif
        enddo
        enddo
        mg=k
        kg(n+1)=k+1
        do i=1,n
        k=kg(i)
        l=kg(i+1)-k
        call uxvsr4(l,jg(k),vg(k))
        enddo
        k=0
        do i=1,m
        ii=kb(i)
        kb(i)=k+1
        do j=1,ii
        k=k+1
        jb(k)=ib(i,j)
        vb(k)=b(i,j)
        enddo
        enddo
        mb=k
        kb(m+1)=k+1
        do i=1,m
        k=kb(i)
        l=kb(i+1)-k
        call uxvsr4(l,jb(k),vb(k))
        enddo
        return
        end
*====================== Subroutine sing ================================
* If the matrix is square, it computes its eigenvalues, otherwise its
* singular values
*
       subroutine sing(a,n,nrango,conda,almin,ind)
       integer n,nrango,conda,ind
       real*8  a(*),almin
       integer i,iseme
       real*8 t,randf,ra
          common /seme/iseme
       do i=nrango+1,n
         a(i)=0.0d0
       enddo
       a(1)=10**conda
       a(nrango)=1.0d0
C
C--------  Logarithmic distribution of the singular values -------------
C
       if (ind.eq.0) then
        do i=2,nrango-1
          a(i)=10**(real(nrango-i)/real(nrango-1)*conda)
        enddo
C
C--------  Uniform distribution of the singular values -----------------
C
       elseif (ind.eq.1)then
        do i=2,nrango-1
           t=randf()
          a(i)=t*(a(1)-a(nrango))+1
        enddo
       else
C
C--------  Equally spaced singular values (between the smallest and the
C          largest one) ------------------------------------------------
C
          ra=(a(1)-a(nrango))/(nrango-1)
        do i=2,nrango-1
          a(i)=a(i-1)-ra
        enddo
       endif
        do i=1,nrango
          a(i)=a(i)*almin
        enddo
       return
       end
*============ Subroutine tleft =========================================
* It makes a Givens' transformation on matrix A:
*     A* Gt(ir,jc)
* using random numbers in [-1,1] as non-zero elements of G
*
       subroutine tleft(a,ia,kr,n,c,icont,icontd,tol,lda,imax,ir,jc,
     +  tempi,tempj,indi,indj)
       integer n,icont,icontd,lda,imax,ir,jc,ia(lda,imax),
     +  kr(*),indi(n),indj(n)
       real*8 a(lda,imax),c,tol,tempi(n),tempj(n)
       integer iseme,i,j,k,icol
       real*8 c2,s2,s,coeff
       common /seme/iseme
C
C     n = number of rows of A
C
       c2=c**2
       s2=1.0d0-c2
       s=sqrt(s2)
       do k=1,n
        tempi(k)=0.0d0
        tempj(k)=0.0d0
        indi(k)=0
        indj(k)=0
       do i=1,kr(k)
        if(ia(k,i).eq.ir)then
           tempi(k)=a(k,i)
           indi(k)=i
        endif
        if(ia(k,i).eq.jc)then
           tempj(k)=a(k,i)
           indj(k)=i
        endif
       enddo
       enddo
       do k=1,n
       if(k.ne.ir)then
        coeff=c*tempi(k)-s*tempj(k)
             if(dabs(coeff).gt.tol)then
              if(indi(k).ne.0)then
              icol=indi(k)
              else
              kr(k)=kr(k)+1
              if (kr(k).ge.imax) then
              write(*,*)'imax is too small'
              stop
              endif
              icol=kr(k)
              icont=icont+1
              ia(k,icol)=ir
              endif
             a(k,icol)=coeff
             else
              if(indi(k).ne.0)then
              icont=icont-1
              j=indi(k)
                do i=j,kr(k)-1
                 a(k,i)=a(k,i+1)
                 ia(k,i)=ia(k,i+1)
                enddo
              a(k,kr(k))=0.0d0
              ia(k,kr(k))=0
           if(indj(k).ne.0.and.indj(k).gt.indi(k))indj(k)=indj(k)-1
              indi(k)=0
              kr(k)=kr(k)-1
              endif
             endif
        endif
        if(k.ne.jc)then
        coeff=s*tempi(k)+c*tempj(k)
             if(abs(coeff).gt.tol)then
              if(indj(k).ne.0)then
              icol=indj(k)
              else
              kr(k )=kr(k )+1
              if (kr(k ).ge.imax) then
              write(*,*)'imax is too small'
              stop
              endif
              icol=kr(k )
              icont=icont+1
              ia(k ,icol)=jc
              endif
             a(k ,icol)=coeff
             else
              if(indj(k).ne.0)then
              icont=icont-1
              j=indj(k)
             do i=j,kr(k )-1
                 a(k ,i)=a(k ,i+1)
                 ia(k ,i)=ia(k ,i+1)
                enddo
              a(k ,kr(k ))=0.0d0
              ia(k ,kr(k ))=0
          if(indi(k).ne.0.and.indi(k).gt.indj(k))indi(k)=indi(k)-1
              indj(k)=0
              kr(k )=kr(k )-1
              endif
             endif
        endif
        enddo
        if (ir.le.n) then
          coeff=c*tempi(ir)-s*tempj(ir)
          if(abs(coeff).gt.tol)then
          if (abs(a(ir,1)).le.tol) then
          icont=icont+1
          icontd=icontd+1
          endif
          a(ir,1)=coeff
          else
          if (abs(a(ir,1)).gt.tol) then
          icont=icont-1
          icontd=icontd-1
          endif
          a(ir,1)=0.0d0
          endif
         endif
         if (jc.le.n) then
          coeff=s*tempi(jc)+c*tempj(jc)
          if(abs(coeff).gt.tol)then
          if (abs(a(jc,1)).le.tol) then
          icont=icont+1
          icontd=icontd+1
          endif
          a(jc,1)=coeff
          else
          if (abs(a(jc,1)).gt.tol) then
          icont=icont-1
          icontd=icontd-1
          endif
          a(jc,1)=0.0d0
          endif
         endif
          return
          end
*============ Subroutine tright ========================================
* It makes a Givens' transformation on matrix A:
*     G(ir,jc) * A
* using random numbers in [-1,1] as non-zero elements of G
*
       subroutine tright(a,ia,kr,n,c,icont,icontd,tol,lda,imax,ir,jc,
     +  tempi,tempj,indi,indj)
       integer n,icont,icontd,lda,imax,ir,jc,ia(lda,imax),
     +  kr(*),indi(n),indj(n)
       real*8 a(lda,imax),c,tol,tempi(n),tempj(n)
       integer iseme,i,j,k,icol
       real*8 c2,s2,s,coeff
       common /seme/iseme
C
C    n=number of columns of A
C
       c2=c**2
       s2=1.0d0-c2
       s=sqrt(s2)
       do i=1,n
        tempi(i)=0.0d0
        tempj(i)=0.0d0
        indi(i)=0
        indj(i)=0
       enddo
       do i=1,kr(ir)
       tempi(ia(ir,i))=a(ir,i)
       indi(ia(ir,i))=i
       enddo
       do i=1,kr(jc)
       tempj(ia(jc,i))=a(jc,i)
       indj(ia(jc,i))=i
       enddo
       do k=1,n
        if(k.ne.ir)then
        coeff=c*tempi(k)-s*tempj(k)
             if(abs(coeff).gt.tol)then
              if(indi(k).ne.0)then
              icol=indi(k)
              else
              kr(ir)=kr(ir)+1
              if (kr(ir).ge.imax) then
              write(*,*)'imax is too small'
              stop
              endif
              icol=kr(ir)
              icont=icont+1
              ia(ir,icol)=k
              endif
             a(ir,icol)=coeff
             else
              if(indi(k).ne.0)then
              icont=icont-1
              j=indi(k)
                do i=j,kr(ir)-1
                 a(ir,i)=a(ir,i+1)
                 ia(ir,i)=ia(ir,i+1)
                enddo
              a(ir,kr(ir))=0.0d0
              ia(ir,kr(ir))=0
              do i=1,n
              if(indi(i).gt.j)indi(i)=indi(i)-1
              enddo
              indi(k)=0
              kr(ir)=kr(ir)-1
              endif
             endif
        endif
        if(k.ne.jc)then
        coeff=s*tempi(k)+c*tempj(k)
             if(abs(coeff).gt.tol)then
              if(indj(k).ne.0)then
              icol=indj(k)
              else
              kr(jc)=kr(jc)+1
              if (kr(jc).ge.imax) then
              write(*,*)'imax is too small'
              stop
              endif
              icol=kr(jc)
              icont=icont+1
              ia(jc,icol)=k
              endif
             a(jc,icol)=coeff
             else
              if(indj(k).ne.0)then
              icont=icont-1
              j=indj(k)
                do i=j,kr(jc)-1
                 a(jc,i)=a(jc,i+1)
                 ia(jc,i)=ia(jc,i+1)
                enddo
              a(jc,kr(jc))=0.0d0
              ia(jc,kr(ir))=0
              do i=1,n
              if(indj(i).gt.j)indj(i)=indj(i)-1
              enddo
              indj(k)=0
              kr(jc)=kr(jc)-1
              endif
             endif
        endif
        enddo
        if (ir.le.n) then
          coeff=c*tempi(ir)-s*tempj(ir)
          if(abs(coeff).gt.tol)then
          if (abs(a(ir,1)).le.tol) then
          icont=icont+1
          icontd=icontd+1
          endif
          a(ir,1)=coeff
          else
          if (abs(a(ir,1)).gt.tol) then
          icont=icont-1
          icontd=icontd-1
          endif
          a(ir,1)=0.0d0
          endif
         endif
         if (jc.le.n) then
          coeff=s*tempi(jc)+c*tempj(jc)
          if(abs(coeff).gt.tol)then
          if (abs(a(jc,1)).le.tol) then
          icont=icont+1
          icontd=icontd+1
          endif
          a(jc,1)=coeff
          else
          if (abs(a(jc,1)).gt.tol) then
          icont=icont-1
          icontd=icontd-1
          endif
          a(jc,1)=0.0d0
          endif
         endif
          return
          end
*========= FUNCTION RANF ===============================================
          double precision function randf()
          real*4 unran3
          integer iseme
          common /seme/iseme
  10       randf=unran3(iseme)
          if (randf.eq.0.0d0) goto 10
          return
          end
