* SUBROUTINE SFFGUX                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  EVALUATE BOTH THE VALUE AND THE GRADIENT OF THE OBJECTIVE FUNCTION.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  RI  X(N)  VECTOR OF VARIABLES.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RO  G(N)  GRADIENT OF THE OBJECTIVE FUNCTION.
*
      SUBROUTINE SFFGUX ( N, X, F, G )
      INTEGER            N
      DOUBLE PRECISION   F
      DOUBLE PRECISION   X( N ), G( N )
C
C  Based on the subroutines ufn.f and ugr.f by Nick Gould, which are
C  in turn based on the subroutine SBMIN by Conn, Gould and Toint.
C
C  Nick Gould, for CGT productions.
C  October 1991.
C
      INTEGER            LIWK  , LWK   , LFUVAL, LLOGIC, LCHARA
C
C  ---------------------------------------------------------------------
C
C  Parameters whose value might be changed by the user:
C
C  The following parameters define the sizes of problem
C  dependent arrays. These may be changed by the user to
C  suit a particular problem or system configuration.
C
C  The TOOLS will issue error messages if any of these sizes
C  is too small, telling which parameter to increase.
C
C  ---------------------------------------------------------------------
C
C  Integer workspace
C
      PARAMETER       ( LIWK   =  3000000 )
CMED  PARAMETER       ( LIWK   =  300000  )
CTOY  PARAMETER       ( LIWK   =  10000   )
COLD  PARAMETER       ( LIWK   =  1000000 )
      INTEGER                IWK( LIWK    )
C
C  Real/double precision workspace
C
      PARAMETER       ( LWK    =  3000000 )
CMED  PARAMETER       ( LWK    =  300000  )
CTOY  PARAMETER       ( LWK    =  10000   )
COLD  PARAMETER       ( LWK    =  500000  )
      DOUBLE PRECISION       WK ( LWK     )
C
C  Logical workspace
C
      PARAMETER       ( LLOGIC =  350000  )
CMED  PARAMETER       ( LLOGIC =  15000   )
CTOY  PARAMETER       ( LLOGIC =  1500    )
COLD  PARAMETER       ( LLOGIC =  100000  )
      LOGICAL              LOGI ( LLOGIC  )
C
C  Character workspace
C
      PARAMETER       ( LCHARA =  300000  )
CMED  PARAMETER       ( LCHARA =  30000   )
CTOY  PARAMETER       ( LCHARA =  3000    )
COLD  PARAMETER       ( LCHARA =  100000  )
      CHARACTER * 10       CHA  ( LCHARA  )
C
C  Workspace to store the problem's function and derivatives values
C
      PARAMETER       ( LFUVAL =  960000  )
CMED  PARAMETER       ( LFUVAL =  50000   )
CTOY  PARAMETER       ( LFUVAL =  5000    )
COLD  PARAMETER       ( LFUVAL =  200000  )
      DOUBLE PRECISION  FUVALS  ( LFUVAL  )
C
C  ---------------------------------------------------------------------
C
C  End of parameters which might be changed by the user.
C
C  ---------------------------------------------------------------------
C
C  integer variables from the GLOBAL common block.
C
      INTEGER            NG    , NELNUM, NGEL  , NVARS , NNZA  , NGPVLU
      INTEGER            NEPVLU, NG1   , NEL1  , ISTADG, ISTGP , ISTADA
      INTEGER            ISTAEV, ISTEP , ITYPEG, KNDOFC, ITYPEE
      INTEGER            IELING, IELVAR, ICNA  , ISTADH, INTVAR, IVAR
      INTEGER            ICALCF, IWRK  , A     , B
      INTEGER            U     , GPVALU, EPVALU
      INTEGER            ESCALE, GSCALE, VSCALE, GVALS , XT    , DGRAD
      INTEGER            Q     , WRK   , INTREP, GXEQX , GNAMES, VNAMES
      INTEGER            LO    , CH    , LIWORK, LWORK , NGNG  , FT
      INTEGER            LA, LB, NOBJGR, LU, LELVAR
      INTEGER            LSTAEV, LSTADH, LNTVAR, LCALCF
      INTEGER            LELING, LINTRE, LFT, LGXEQX, LSTADG, LGVALS
      INTEGER            LICNA , LSTADA, LKNDOF
      INTEGER            LGSCAL, LESCAL, LVSCAL, LCALCG
C
C  integer variables from the LOCAL common block.
C
      INTEGER            LFXI  , LGXI  , LHXI  , LGGFX , LDX   , LGRJAC
      INTEGER            LQGRAD, LBREAK, LP    , LXCP  , LX0   , LGX0
      INTEGER            LDELTX, LBND  , LWKSTR, LSPTRS, LSELTS, LINDEX
      INTEGER            LSWKSP, LSTAGV, LSTAJC, LIUSED, LFREEC
      INTEGER            LNNONZ, LNONZ2, LSYMMD, LSYMMH
      INTEGER            LSLGRP, LSVGRP, LGCOLJ, LVALJR, LSEND
      INTEGER            LNPTRS, LNELTS, LNNDEX, LNWKSP, LNSTGV
      INTEGER            LNSTJC, LNIUSE, LNFREC, LNNNON, LNNNO2, LNSYMD
      INTEGER            LNSYMH, LNLGRP, LNVGRP, LNGCLJ, LNVLJR, LNQGRD
      INTEGER            LNBRAK, LNP   , LNBND , LNFXI , LNGXI , LNGUVL
      INTEGER            LNHXI , LNHUVL, LNGGFX, LNDX  , LNGRJC, LIWK2
      INTEGER            LWK2  , MAXSIN, NINVAR, MAXSEL
      INTEGER            NTYPE , NSETS , LSTYPE, LSSWTR, LSSIWT, LSIWTR
      INTEGER            LSWTRA, LNTYPE, LNSWTR, LNSIWT, LNIWTR
      INTEGER            LNWTRA, LSISET, LSSVSE, LNISET, LNSVSE
      LOGICAL            ALTRIV, FIRSTG
      COMMON / GLOBAL /  IWK   , WK    , FUVALS, LOGI,
     *                   NG    , NELNUM, NGEL  , NVARS , NNZA  , NGPVLU,
     *                   NEPVLU, NG1   , NEL1  , ISTADG, ISTGP , ISTADA,
     *                   ISTAEV, ISTEP , ITYPEG, KNDOFC, ITYPEE,
     *                   IELING, IELVAR, ICNA  , ISTADH, INTVAR, IVAR  ,
     *                   ICALCF, IWRK  , A     , B     ,
     *                   U     , GPVALU, EPVALU,
     *                   ESCALE, GSCALE, VSCALE, GVALS , XT    , DGRAD ,
     *                   Q     , WRK   , INTREP, GXEQX , GNAMES, VNAMES,
     *                   LO    , CH    , LIWORK, LWORK , NGNG  , FT    ,
     *                   ALTRIV, FIRSTG,
     *                   LA, LB, NOBJGR, LU, LELVAR,
     *                   LSTAEV, LSTADH, LNTVAR, LCALCF,
     *                   LELING, LINTRE, LFT, LGXEQX, LSTADG, LGVALS,
     *                   LICNA , LSTADA, LKNDOF,
     *                   LGSCAL, LESCAL, LVSCAL, LCALCG
      COMMON / CHARA /   CHA
      COMMON / LOCAL /   LFXI  , LGXI  , LHXI  , LGGFX , LDX   , LGRJAC,
     *                   LQGRAD, LBREAK, LP    , LXCP  , LX0   , LGX0  ,
     *                   LDELTX, LBND  , LWKSTR, LSPTRS, LSELTS, LINDEX,
     *                   LSWKSP, LSTAGV, LSTAJC, LIUSED, LFREEC,
     *                   LNNONZ, LNONZ2, LSYMMD, LSYMMH,
     *                   LSLGRP, LSVGRP, LGCOLJ, LVALJR, LSEND ,
     *                   LNPTRS, LNELTS, LNNDEX, LNWKSP, LNSTGV,
     *                   LNSTJC, LNIUSE, LNFREC, LNNNON, LNNNO2, LNSYMD,
     *                   LNSYMH, LNLGRP, LNVGRP, LNGCLJ, LNVLJR, LNQGRD,
     *                   LNBRAK, LNP   , LNBND , LNFXI , LNGXI , LNGUVL,
     *                   LNHXI , LNHUVL, LNGGFX, LNDX  , LNGRJC, LIWK2 ,
     *                   LWK2  , MAXSIN, NINVAR, MAXSEL, NTYPE ,
     *                   NSETS , LSTYPE, LSSWTR, LSSIWT, LSIWTR,
     *                   LSWTRA, LNTYPE, LNSWTR, LNSIWT, LNIWTR,
     *                   LNWTRA, LSISET, LSSVSE, LNISET, LNSVSE
      INTEGER            NUMVAR, NUMCON
      COMMON / DIMS /    NUMVAR, NUMCON
      SAVE             / GLOBAL /, / LOCAL /, / CHARA /, / DIMS /
C
C  local variables.
C
      INTEGER            I , J , IEL, K, IG, II, IG1, L, LL, ICON
      INTEGER            NIN   , NVAREL, NELOW , NELUP , ISTRGV, IENDGV
      LOGICAL            NONTRV
      EXTERNAL           DSETVL, DSETVI, RANGES
      DOUBLE PRECISION   FTT   , ONE   , ZERO  , GI    , SCALEE
      PARAMETER        ( ZERO  = 0.0D+0, ONE  = 1.0D+0 )
C
C  There are non-trivial group functions.
C
      DO 10 I = 1, MAX( NELNUM, NG )
        IWK( ICALCF + I ) = I
   10 CONTINUE
C
C  Evaluate the element function values.
C
      CALL ELFUNS( FUVALS, X, WK( EPVALU + 1 ), NELNUM,
     *             IWK( ITYPEE + 1 ), IWK( ISTAEV + 1 ),
     *             IWK( IELVAR + 1 ), IWK( INTVAR + 1 ),
     *             IWK( ISTADH + 1 ), IWK( ISTEP  + 1 ),
     *             IWK( ICALCF + 1 ), 1 )
C
C  Compute the group argument values ft.
C
      IF ( NUMCON .GT. 0 ) THEN
         DO 100 IG = 1, NG
            FTT     = ZERO
C
C  Consider only those groups in the objective function.
C
            IF ( IWK( KNDOFC + IG ) .EQ. 0 ) THEN
               FTT     = - WK( B + IG )
C
C  Include the contribution from the linear element
C  only if the variable belongs to the first N variables.
C
               DO 30 I = IWK( ISTADA + IG ), IWK( ISTADA + IG + 1 ) - 1
                  J = IWK( ICNA + I )
                  IF ( J .LE. N )
     *               FTT  = FTT + WK( A + I ) * X( J )
   30          CONTINUE
C
C  Include the contributions from the nonlinear elements.
C
               DO 60 I = IWK( ISTADG + IG ), IWK( ISTADG + IG + 1 ) - 1
                  FTT  = FTT +
     *                   WK( ESCALE + I ) * FUVALS( IWK( IELING + I ) )
   60          CONTINUE
C
C  Record the derivatives of trivial groups.
C
               IF ( LOGI( GXEQX + IG ) ) WK( GVALS + NG + IG ) = ONE
            END IF
            WK( FT + IG ) = FTT
  100    CONTINUE
      ELSE
C
C  There are no constraints, so we need not check IWK( KNDOFC + IG ).
C
         DO 300 IG = 1, NG
            FTT     = - WK( B + IG )
C
C  Include the contribution from the linear element
C  only if the variable belongs to the first N variables.
C
            DO 330 I = IWK( ISTADA + IG ), IWK( ISTADA + IG + 1 ) - 1
               J = IWK( ICNA + I )
               IF ( J .LE. N )
     *            FTT  = FTT + WK( A + I ) * X( J )
  330       CONTINUE
C
C  Include the contributions from the nonlinear elements.
C
            DO 360 I = IWK( ISTADG + IG ), IWK( ISTADG + IG + 1 ) - 1
               FTT  = FTT +
     *                WK( ESCALE + I ) * FUVALS( IWK( IELING + I ) )
  360       CONTINUE
C
C  Record the derivatives of trivial groups.
C
            IF ( LOGI( GXEQX + IG ) ) WK( GVALS + NG + IG ) = ONE
            WK( FT + IG ) = FTT
  300    CONTINUE
      END IF
C
C  Compute the group function values.
C
C  All group functions are trivial.
C
      IF ( ALTRIV ) THEN
         CALL DCOPY( NG, WK( FT + 1 ), 1, WK( GVALS + 1 ), 1 )
         CALL DSETVL( NG, WK( GVALS + NG + 1 ), 1, ONE )
      ELSE
C
C  Evaluate the group function values.
C
         CALL GROUPS( WK ( GVALS  + 1 ), NG, WK( FT + 1 ),
     *                WK ( GPVALU + 1 ), NG,
     *                IWK( ITYPEG + 1 ), IWK( ISTGP + 1 ),
     *                IWK( ICALCF + 1 ), .FALSE. )
      END IF
C
C  Compute the objective function value.
C
      F         = ZERO
      IF ( NUMCON .GT. 0 ) THEN
         DO 110 IG = 1, NG
            IF ( IWK( KNDOFC + IG ) .EQ. 0 ) THEN
               IF ( LOGI( GXEQX + IG ) ) THEN
                  F = F + WK( GSCALE + IG ) * WK( FT + IG )
               ELSE
                  F = F + WK( GSCALE + IG ) * WK( GVALS + IG )
               END IF
            END IF
  110    CONTINUE
      ELSE
C
C  There are no constraints, so we need not check IWK( KNDOFC + IG ).
C
         DO 310 IG = 1, NG
            IF ( LOGI( GXEQX + IG ) ) THEN
               F = F + WK( GSCALE + IG ) * WK( FT + IG )
            ELSE
               F = F + WK( GSCALE + IG ) * WK( GVALS + IG )
            END IF
  310    CONTINUE
      END IF
      IF ( .TRUE. ) THEN
C
C  Evaluate the element function derivatives.
C
         CALL ELFUNS( FUVALS, X, WK( EPVALU + 1 ), NELNUM,
     *                IWK( ITYPEE + 1 ), IWK( ISTAEV + 1 ),
     *                IWK( IELVAR + 1 ), IWK( INTVAR + 1 ),
     *                IWK( ISTADH + 1 ), IWK( ISTEP  + 1 ),
     *                IWK( ICALCF + 1 ), 2 )
C
C  Evaluate the group derivative values.
C
         IF ( .NOT. ALTRIV ) CALL GROUPS( WK ( GVALS + 1 ), NG,
     *         WK( FT + 1 ), WK ( GPVALU + 1 ), NG,
     *         IWK( ITYPEG + 1 ), IWK( ISTGP + 1 ),
     *         IWK( ICALCF + 1 ), .TRUE. )
         IF ( NUMCON .GT. 0 ) THEN
C
C  Compute the gradient values. Initialize the gradient as zero.
C
            DO 120 J = 1, N
               G( J ) = ZERO
  120       CONTINUE
C
C  Consider the IG-th group.
C
            DO 290 IG = 1, NG
               ICON   = IWK( KNDOFC + IG )
C
C  Consider only those groups in the objective function.
C
               IF ( ICON .GT. 0 ) GO TO 290
               IG1    = IG + 1
               ISTRGV = IWK( LSTAGV + IG  )
               IENDGV = IWK( LSTAGV + IG1 ) - 1
               NELOW  = IWK( ISTADG + IG  )
               NELUP  = IWK( ISTADG + IG1 ) - 1
               NONTRV = .NOT. LOGI( GXEQX + IG )
C
C  Compute the first derivative of the group.
C
               GI = WK( GSCALE + IG )
               IF ( NONTRV ) GI  = GI  * WK( GVALS + NG + IG )
C
C  The group has nonlinear elements.
C
               IF ( NELOW .LE. NELUP ) THEN
                  CALL DSETVI( IENDGV - ISTRGV + 1, WK( WRK + 1 ),
     *                         IWK( LSVGRP + ISTRGV ), ZERO )
C
C  Loop over the group's nonlinear elements.
C
                  DO 150 II  = NELOW, NELUP
                     IEL    = IWK( IELING + II      )
                     K      = IWK( INTVAR + IEL     )
                     L      = IWK( ISTAEV + IEL     )
                     NVAREL = IWK( ISTAEV + IEL + 1 ) - L
                     SCALEE = WK( ESCALE + II )
                     IF ( LOGI( INTREP + IEL ) ) THEN
C
C  The IEL-th element has an internal representation.
C
                        NIN = IWK( INTVAR + IEL + 1 ) - K
                        CALL RANGES( IEL, .TRUE., FUVALS( K ),
     *                               WK( WRK + N + 1 ), NVAREL, NIN )
                        DO 130 I         = 1, NVAREL
                           J             = IWK( IELVAR + L )
                           WK( WRK + J ) = WK( WRK + J ) +
     *                                        SCALEE * WK( WRK + N + I )
                           L             = L + 1
  130                   CONTINUE
                     ELSE
C
C  The IEL-th element has no internal representation.
C
                        DO 140 I         = 1, NVAREL
                           J             = IWK( IELVAR + L )
                           WK( WRK + J ) = WK( WRK + J ) +
     *                                        SCALEE * FUVALS( K )
                           K             = K + 1
                           L             = L + 1
  140                   CONTINUE
                     END IF
  150             CONTINUE
C
C  Include the contribution from the linear element.
C
                  DO 160 K         = IWK( ISTADA + IG  ),
     *                               IWK( ISTADA + IG1 ) - 1
                     J             = IWK( ICNA + K )
                     WK( WRK + J ) = WK( WRK + J ) + WK( A + K )
  160             CONTINUE
C
C  Allocate a gradient.
C
                  DO 190 I = ISTRGV, IENDGV
                     LL    = IWK( LSVGRP + I )
C
C  Include the contributions from only the first N variables.
C
                     IF ( LL .LE. N )
     *                  G( LL ) = G( LL ) +  GI * WK( WRK + LL )
  190             CONTINUE
C
C  The group has only linear elements.
C
               ELSE
C
C  Allocate a gradient.
C
                  DO 210 K = IWK( ISTADA + IG ), IWK( ISTADA + IG1 ) - 1
                     LL    = IWK( ICNA + K )
C
C  Include the contributions from linear elements for only the first
C  N variables.
C
                     IF ( LL .LE. N )
     *                  G( LL ) = G( LL ) +  GI * WK( A + K )
  210             CONTINUE
               END IF
  290       CONTINUE
         ELSE
C
C  Compute the gradient value.
C  When EFFGU is called by MINOS, the passed number of variables, N,
C  may be less than the true number of variables, NUMVAR.
C  (MINOS requires the gradient wrt the nonlinear variables only.)
C  Therefore use NUMVAR in the call to ELGRD.
C
            CALL DELGRD( NUMVAR, NG, FIRSTG, IWK( ICNA + 1 ), LICNA,
     *                   IWK( ISTADA + 1 ), LSTADA, IWK( IELING + 1 ),
     *                   LELING, IWK( ISTADG + 1 ), LSTADG,
     *                   IWK( ISTAEV + 1 ), LSTAEV, IWK( IELVAR + 1 ),
     *                   LELVAR, IWK( INTVAR + 1 ), LNTVAR,
     *                   IWK( LSVGRP + 1 ),
     *                   LNVGRP, IWK( LSTAJC + 1 ), LNSTJC,
     *                   IWK( LSTAGV + 1 ), LNSTGV, WK( A + 1 ), LA,
     *                   WK( GVALS + NG + 1 ), LGVALS,
     *                   FUVALS, LNGUVL, FUVALS( LGGFX + 1 ),
     *                   WK( GSCALE + 1 ), LGSCAL,
     *                   WK( ESCALE + 1 ), LESCAL, FUVALS( LGRJAC + 1 ),
     *                   LNGRJC, WK( WRK + 1 ), WK( WRK + NUMVAR + 1 ),
     *                   MAXSEL, LOGI( GXEQX + 1 ), LGXEQX,
     *                   LOGI( INTREP + 1 ), LINTRE, RANGES )
C
C  Store the gradient value.
C
            DO 400 I = 1, N
               G( I ) = FUVALS( LGGFX + I )
  400       CONTINUE
         END IF
      END IF
      RETURN
C
C  end of EFFGU.
C
      END
