* SUBROUTINE SINSXX             ALL SYSTEMS                 99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  SET UP THE INPUT DATA FOR CONSTRAINED OPTIMIZATION.
*  GENERAL SPARSE VERSION.
*
* PARAMETERS :
*  IO  N  NUMBER OF VARIABLES.
*  IO  NC  NUMBER OF CONSTRAINTS.
*  IO  MC  NUMBER OF NONZERO ELEMENTS IN THE CONSTRAINT GROUP JACOBIAN
*         MATRIX.
*  IO  M  NUMBER OF NONZERO ELEMENTS IN THE LAGRANGE HESSIAN MATRIX.
*  RO  X(NMAX)  VECTOR OF VARIABLES.
*  IO  IX(NMAX)  VECTOR CONTAINING TYPES OF BOUNDS. IX(I)=0-FREE
*         VARIABLE. IX(I)=1-LOWER BOX CONSTRAINT. IX(I)=2-UPPER BOX
*         CONSTRAINT. IX(I)=3 TWO-SIDED BOX CONSTRAINT. IX(I)=5-FIXED
*         VARIABLE.
*  RO  BL(NMAX)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RO  BU(NMAX)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  IO  IH(NMAX+1)  ROW POINTERS IN THE ARRAY JH. IH(I) IS THE POSITION
*         OF THE I-TH DIAGONAL ELEMENT OF THE LAGRANGE HESSIAN MATRIX
*         IN THE COLUMN LIST JH.
*  IO  JH(MMAX)  COLUMN INDICES OF NONZERO ELEMENTS IN THE ROW-ORDERED
*         UPPER-HALF PART OF THE LAGRANGE HESSIAN MATRIX
*  IA  KH(MMAX)  AUXILIARY ARRAY.
*  RI  C(NCMAX)  VECTOR CONTAINING VALUES OF CONSTRAINT FUNCTIONS.
*  II  IC(NCMAX)  VECTOR CONTAINING TYPES OF CONSTRAINTS. IC(I)=0-FREE
*         CONSTRAINT (THIS CONSTRAINT IS NOT CONSIDERED). IC(I)=1-LOWER
*         INEQUALITY CONSTRAINT. IX(I)=2-UPPER INEQUALITY CONSTRAINT.
*         IX(I)=3 TWO-SIDED INEQUALITY CONSTRAINT. IX(I)=5-EQUALITY
*         CONSTRAINT.
*  RI  CL(NCMAX)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT
*         FUNCTIONS.
*  RI  CU(NCMAX)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT
*         FUNCTIONS.
*  IO  ICG(NCMAX+1)  ROW POINTERS IN THE ARRAY JCG. ICG(I) IS THE
*         POSITION OF THE FIRST NONZERO ELEMENT OF THE I-TH CONSTRAINT
*         GROUP GRADIENT IN THE COLUMN LIST JCG.
*  IO  JCG(MCMAX)  COLUMN INDICES OF NONZERO ELEMENTS IN THE ROW-ORDERED
*         PACKED CONSTRAINT GROUP JACOBIAN MATRIX.
*  II  NMAX  DIMENSION OF ARRAYS X, IX, BL, BU AND IH (DECREASED BY 1).
*  II  NCMAX DIMENSION OF ARRAYS C, IC, CL, CU AND ICG (DECREASED BY 1).
*  II  MCMAX DIMENSION OF THE ARRAY JCG.
*  II  MMAX  DIMENSION OF ARRAYS JH, KH.
*
      SUBROUTINE SINSXX ( N, NC, MC, M, X, IX, BL, BU, IH, JH, KH,
     *                    C, IC, CL, CU, ICG, JCG, NMAX, NCMAX,
     *                    MCMAX, MMAX ,ITERM )
      INTEGER             INPUT, IOUT, ITERM, N, NC, MC, M, MLIN, NMAX
      INTEGER             NCMAX, MCMAX, MMAX
      DOUBLE PRECISION    X(  NMAX ), BL(  NMAX ), BU(  NMAX )
      DOUBLE PRECISION    C( NCMAX ), CL( NCMAX ), CU( NCMAX )
      INTEGER             IX( NMAX ), IC( NCMAX )
      INTEGER             IH(  NMAX  + 1 ), JH(  MMAX  ), KH(  MMAX )
      INTEGER             ICG( NCMAX + 1 ), JCG( MCMAX )
C
C  Set up the input data for the constrained optimization tools.
C
C  Nick Gould, for CGT productions,
C  30th October, 1991.
C  Ingrid Bongartz added option to reorder variables, so that
C  nonlinear variables come first.  Within the nonlinear variables,
C  the smaller set of either the nonlinear objective or nonlinear
C  Jacobian variables appears first.
C  5th November, 1992.
C
C  Workspace arrays.
C
      INTEGER            LIWK  , LWK   , LFUVAL, LLOGIC, LCHARA
C
C  ---------------------------------------------------------------------
C
C  Parameters whose value might be changed by the user:
C
C  The following parameters define the sizes of problem
C  dependent arrays. These may be changed by the user to
C  suit a particular problem or system configuration.
C
C  The TOOLS will issue error messages if any of these sizes
C  is too small, telling which parameter to increase.
C
C  ---------------------------------------------------------------------
C
C  Integer workspace
C
      PARAMETER       ( LIWK   =  3000000 )
CMED  PARAMETER       ( LIWK   =  300000  )
CTOY  PARAMETER       ( LIWK   =  10000   )
COLD  PARAMETER       ( LIWK   =  1000000 )
      INTEGER                IWK( LIWK    )
C
C  Real/double precision workspace
C
      PARAMETER       ( LWK    =  3000000 )
CMED  PARAMETER       ( LWK    =  300000  )
CTOY  PARAMETER       ( LWK    =  10000   )
COLD  PARAMETER       ( LWK    =  500000  )
      DOUBLE PRECISION       WK ( LWK     )
C
C  Logical workspace
C
      PARAMETER       ( LLOGIC =  350000  )
CMED  PARAMETER       ( LLOGIC =  15000   )
CTOY  PARAMETER       ( LLOGIC =  1500    )
COLD  PARAMETER       ( LLOGIC =  100000  )
      LOGICAL              LOGI ( LLOGIC  )
C
C  Character workspace
C
      PARAMETER       ( LCHARA =  300000  )
CMED  PARAMETER       ( LCHARA =  30000   )
CTOY  PARAMETER       ( LCHARA =  3000    )
COLD  PARAMETER       ( LCHARA =  100000  )
      CHARACTER * 10       CHA  ( LCHARA  )
C
C  Workspace to store the problem's function and derivatives values
C
      PARAMETER       ( LFUVAL =  960000  )
CMED  PARAMETER       ( LFUVAL =  50000   )
CTOY  PARAMETER       ( LFUVAL =  5000    )
COLD  PARAMETER       ( LFUVAL =  200000  )
      DOUBLE PRECISION  FUVALS  ( LFUVAL  )
C
C  ---------------------------------------------------------------------
C
C  End of parameters which might be changed by the user.
C
C  ---------------------------------------------------------------------
C
C  Integer variables from the GLOBAL common block.
C
      INTEGER            NG    , NELNUM, NGEL  , NVARS , NNZA  , NGPVLU
      INTEGER            NEPVLU, NG1   , NEL1  , ISTADG, ISTGP , ISTADA
      INTEGER            ISTAEV, ISTEP , ITYPEG, KNDOFC, ITYPEE
      INTEGER            IELING, IELVAR, ICNA  , ISTADH, INTVAR, IVAR
      INTEGER            ICALCF, IWRK  , A     , B
      INTEGER            U     , GPVALU, EPVALU
      INTEGER            ESCALE, GSCALE, VSCALE, GVALS , XT    , DGRAD
      INTEGER            Q     , WRK   , INTREP, GXEQX , GNAMES, VNAMES
      INTEGER            LO    , CH    , LIWORK, LWORK , NGNG  , FT
      INTEGER            LA, LB, NOBJGR, LU, LELVAR
      INTEGER            LSTAEV, LSTADH, LNTVAR, LCALCF
      INTEGER            LELING, LINTRE, LFT, LGXEQX, LSTADG, LGVALS
      INTEGER            LICNA , LSTADA, LKNDOF
      INTEGER            LGSCAL, LESCAL, LVSCAL, LCALCG
C
C  Integer variables from the LOCAL common block.
C
      INTEGER            LFXI  , LGXI  , LHXI  , LGGFX , LDX   , LGRJAC
      INTEGER            LQGRAD, LBREAK, LP    , LXCP  , LX0   , LGX0
      INTEGER            LDELTX, LBND  , LWKSTR, LSPTRS, LSELTS, LINDEX
      INTEGER            LSWKSP, LSTAGV, LSTAJC, LIUSED, LFREEC
      INTEGER            LNNONZ, LNONZ2, LSYMMD, LSYMMH
      INTEGER            LSLGRP, LSVGRP, LGCOLJ, LVALJR, LSEND
      INTEGER            LNPTRS, LNELTS, LNNDEX, LNWKSP, LNSTGV
      INTEGER            LNSTJC, LNIUSE, LNFREC, LNNNON, LNNNO2, LNSYMD
      INTEGER            LNSYMH, LNLGRP, LNVGRP, LNGCLJ, LNVLJR, LNQGRD
      INTEGER            LNBRAK, LNP   , LNBND , LNFXI , LNGXI , LNGUVL
      INTEGER            LNHXI , LNHUVL, LNGGFX, LNDX  , LNGRJC, LIWK2
      INTEGER            LWK2  , MAXSIN, NINVAR, MAXSEL, LNIWTR
      INTEGER            NTYPE , NSETS , LSTYPE, LSSWTR, LSSIWT
      INTEGER            LSIWTR, LSWTRA, LNTYPE, LNSWTR, LNSIWT
      INTEGER            LNWTRA, LSISET, LSSVSE, LNISET, LNSVSE
      LOGICAL            ALTRIV, FIRSTG
      COMMON / GLOBAL /  IWK   , WK    , FUVALS, LOGI  ,
     *                   NG    , NELNUM, NGEL  , NVARS , NNZA  , NGPVLU,
     *                   NEPVLU, NG1   , NEL1  , ISTADG, ISTGP , ISTADA,
     *                   ISTAEV, ISTEP , ITYPEG, KNDOFC, ITYPEE,
     *                   IELING, IELVAR, ICNA  , ISTADH, INTVAR, IVAR  ,
     *                   ICALCF, IWRK  , A     , B     ,
     *                   U     , GPVALU, EPVALU,
     *                   ESCALE, GSCALE, VSCALE, GVALS , XT    , DGRAD ,
     *                   Q     , WRK   , INTREP, GXEQX , GNAMES, VNAMES,
     *                   LO    , CH    , LIWORK, LWORK , NGNG  , FT    ,
     *                   ALTRIV, FIRSTG,
     *                   LA, LB, NOBJGR, LU, LELVAR,
     *                   LSTAEV, LSTADH, LNTVAR, LCALCF,
     *                   LELING, LINTRE, LFT, LGXEQX, LSTADG, LGVALS,
     *                   LICNA , LSTADA, LKNDOF,
     *                   LGSCAL, LESCAL, LVSCAL, LCALCG
      COMMON / CHARA /   CHA
      COMMON / LOCAL /   LFXI  , LGXI  , LHXI  , LGGFX , LDX   , LGRJAC,
     *                   LQGRAD, LBREAK, LP    , LXCP  , LX0   , LGX0  ,
     *                   LDELTX, LBND  , LWKSTR, LSPTRS, LSELTS, LINDEX,
     *                   LSWKSP, LSTAGV, LSTAJC, LIUSED, LFREEC,
     *                   LNNONZ, LNONZ2, LSYMMD, LSYMMH,
     *                   LSLGRP, LSVGRP, LGCOLJ, LVALJR, LSEND ,
     *                   LNPTRS, LNELTS, LNNDEX, LNWKSP, LNSTGV,
     *                   LNSTJC, LNIUSE, LNFREC, LNNNON, LNNNO2, LNSYMD,
     *                   LNSYMH, LNLGRP, LNVGRP, LNGCLJ, LNVLJR, LNQGRD,
     *                   LNBRAK, LNP   , LNBND , LNFXI , LNGXI , LNGUVL,
     *                   LNHXI , LNHUVL, LNGGFX, LNDX  , LNGRJC, LIWK2 ,
     *                   LWK2  , MAXSIN, NINVAR, MAXSEL, NTYPE ,
     *                   NSETS , LSTYPE, LSSWTR, LSSIWT, LSIWTR,
     *                   LSWTRA, LNTYPE, LNSWTR, LNSIWT, LNIWTR,
     *                   LNWTRA, LSISET, LSSVSE, LNISET, LNSVSE
      INTEGER            NNOV  , NNJV
      COMMON / NNVARS /  NNOV  , NNJV
      INTEGER            IOUT2
      COMMON / OUTPUT /  IOUT2
      INTEGER            NUMVAR, NUMCON, NUMAPR
      COMMON / DIMS /    NUMVAR, NUMCON, NUMAPR
      SAVE             / GLOBAL /, / LOCAL /, / CHARA /, / NNVARS /,
     *                 / OUTPUT /, / DIMS /
C
C  Local variables.
C
      INTEGER            IALGOR, IPRINT, INFORM, I , IG, J , JG, MEND
      INTEGER            MEQ   , NSLACK, NELTYP, NGRTYP, NELOW , NELUP
      INTEGER            K , LL, ITEMP , ISTRGV, IENDGV, ICON  , IG1
      LOGICAL            FDGRAD, DEBUG , NONTRV
      INTEGER            LH2 , LIH   , LIWKH , LNXTRW, LINXTR
      CHARACTER * 8      PNAME
      DOUBLE PRECISION   ATEMP , ZERO , ONE
      PARAMETER        ( ZERO = 0.0D+0 , ONE  = 1.0D+0 )
      DOUBLE PRECISION   OBFBND( 2 ), BIG
      PARAMETER        ( BIG  = 1.0D+20 )
      EXTERNAL           SETTYP, RANGES
      INPUT   = 99
      IOUT    = 6
      IOUT2   = IOUT
      DEBUG   = .FALSE.
      DEBUG   = DEBUG .AND. IOUT .GT. 0
      IPRINT  = 0
      IF ( DEBUG ) IPRINT = 3
C
C  Input the problem dimensions.
C
      OPEN( INPUT, FILE='OUTSDIF.DAT')
      READ( INPUT, 1001 ) N , NG, NELNUM, NGEL , NVARS , NNZA , NGPVLU,
     *                    NEPVLU, NELTYP, NGRTYP
      IF ( N .LE. 0 ) THEN
         CLOSE( INPUT )
         IF ( IOUT .GT. 0 ) WRITE( IOUT, 2020 )
         STOP
      END IF
      IF ( NG .LE. 0 ) THEN
         CLOSE( INPUT )
         IF ( IOUT .GT. 0 ) WRITE( IOUT, 2030 )
         STOP
      END IF
      IF ( N .GT. NMAX ) THEN
         CLOSE( INPUT )
         IF ( IOUT .GT. 0 ) THEN
            WRITE( IOUT, 2000 ) 'X   ', 'NMAX  ', N - NMAX
            WRITE( IOUT, 2000 ) 'BL  ', 'NMAX  ', N - NMAX
            WRITE( IOUT, 2000 ) 'BU  ', 'NMAX  ', N - NMAX
         END IF
         STOP
      END IF
C
C  Input the problem type.
C
      READ( INPUT, 1000 ) IALGOR, PNAME
C
C  Set useful integer values.
C
      NG1    = NG     + 1
      NGNG   = NG     + NG
      NEL1   = NELNUM + 1
C
C  Partition the integer workspace.
C
      ISTADG = 0
      ISTGP  = ISTADG + NG1
      ISTADA = ISTGP  + NG1
      ISTAEV = ISTADA + NG1
      ISTEP  = ISTAEV + NEL1
      ITYPEG = ISTEP  + NEL1
      KNDOFC = ITYPEG + NG
      ITYPEE = KNDOFC + NG
      IELING = ITYPEE + NELNUM
      IELVAR = IELING + NGEL
      ICNA   = IELVAR + NVARS
      ISTADH = ICNA   + NNZA
      INTVAR = ISTADH + NEL1
      IVAR   = INTVAR + NEL1
      ICALCF = IVAR   + N
      IWRK   = ICALCF + MAX( NELNUM, NG )
      LIWORK = LIWK   - IWRK
C
C  Ensure there is sufficient room.
C
      IF ( LIWORK .LT. 0 ) THEN
         CLOSE( INPUT )
         IF ( IOUT .GT. 0 ) WRITE( IOUT, 2000 )
     *       'IWK   ', 'LIWK  ', - LIWORK
         STOP
      END IF
C
C  Partition the real workspace.
C
      A      = 0
      B      = A      + NNZA
      U      = B      + NG
      GPVALU = U      + NG
      EPVALU = GPVALU + NGPVLU
      ESCALE = EPVALU + NEPVLU
      GSCALE = ESCALE + NGEL
      VSCALE = GSCALE + NG
      GVALS  = VSCALE + N
      XT     = GVALS  + 3 * NG
      DGRAD  = XT     + N
      Q      = DGRAD  + N
      FT     = Q      + N
      WRK    = FT     + NG
      LWORK  = LWK    - WRK
C
C  Ensure there is sufficient room.
C
      IF ( LWORK .LT. 0 ) THEN
         CLOSE( INPUT )
         IF ( IOUT .GT. 0 ) WRITE( IOUT, 2000 )
     *       'WK   ', 'LWK   ', - LWORK
         STOP
      END IF
C
C  Partition the logical workspace.
C
      INTREP = 0
      GXEQX  = INTREP + NELNUM
      LO     = GXEQX  + NGNG
C
C  Ensure there is sufficient room.
C
      IF ( LLOGIC .LT. LO ) THEN
         CLOSE( INPUT )
         IF ( IOUT .GT. 0 ) WRITE( IOUT, 2000 )
     *       'LOGI  ', 'LLOGIC', LO - LLOGIC
         STOP
      END IF
C
C  Partition the character workspace.
C
      GNAMES = 0
      VNAMES = GNAMES + NG
      CH     = VNAMES + N
C
C  Ensure there is sufficient room.
C
      IF ( LCHARA .LT. CH + 1 ) THEN
         CLOSE( INPUT )
         IF ( IOUT .GT. 0 ) WRITE( IOUT, 2000 )
     *       'CHA   ', 'LCHARA', CH + 1 - LCHARA
         STOP
      END IF
C
C  Record the lengths of arrays.
C
      LSTADG = MAX( 1, NG1 )
      LSTADA = MAX( 1, NG1 )
      LSTAEV = MAX( 1, NEL1 )
      LKNDOF = MAX( 1, NG )
      LELING = MAX( 1, NGEL )
      LELVAR = MAX( 1, NVARS )
      LICNA  = MAX( 1, NNZA )
      LSTADH = MAX( 1, NEL1 )
      LNTVAR = MAX( 1, NEL1 )
      LCALCF = MAX( 1, NELNUM, NG )
      LCALCG = MAX( 1, NG )
      LA     = MAX( 1, NNZA )
      LB     = MAX( 1, NG )
      LU     = MAX( 1, NG )
      LESCAL = MAX( 1, NGEL )
      LGSCAL = MAX( 1, NG )
      LVSCAL = MAX( 1, N )
      LFT    = MAX( 1, NG )
      LGVALS = MAX( 1, NG )
      LINTRE = MAX( 1, NELNUM )
      LGXEQX = MAX( 1, NGNG )
C
C  Print out problem data. input the number of variables, groups,
C  elements and the identity of the objective function group.
C
      IF ( IALGOR .EQ. 2 ) THEN
         READ( INPUT, 1002 ) NSLACK, NOBJGR
      ELSE
         NSLACK = 0
      END IF
      IF ( DEBUG ) WRITE( IOUT, 1100 ) PNAME, N, NG, NELNUM
      CHA( CH + 1 ) = PNAME // '  '
C
C  Input the starting addresses of the elements in each group,
C  of the parameters used for each group and
C  of the nonzeros of the linear element in each group.
C
      READ( INPUT, 1010 ) ( IWK( ISTADG + I ), I = 1, NG1 )
      IF ( DEBUG ) WRITE( IOUT, 1110 ) 'ISTADG',
     *   ( IWK( ISTADG + I ), I = 1, NG1 )
      READ( INPUT, 1010 ) ( IWK( ISTGP + I ), I = 1, NG1 )
      IF ( DEBUG ) WRITE( IOUT, 1110 ) 'ISTGP ',
     *   ( IWK( ISTGP + I ), I = 1, NG1 )
      READ( INPUT, 1010 ) ( IWK( ISTADA + I ), I = 1, NG1 )
      IF ( DEBUG ) WRITE( IOUT, 1110 ) 'ISTADA',
     *   ( IWK( ISTADA + I ), I = 1, NG1 )
C
C  Input the starting addresses of the variables and parameters
C  in each element.
C
      READ( INPUT, 1010 ) ( IWK( ISTAEV + I ), I = 1, NEL1 )
      IF ( DEBUG ) WRITE( IOUT, 1110 ) 'ISTAEV',
     *   ( IWK( ISTAEV + I ), I = 1, NEL1 )
      READ( INPUT, 1010 ) ( IWK( ISTEP + I ), I = 1, NEL1 )
      IF ( DEBUG ) WRITE( IOUT, 1110 ) 'ISTEP ',
     *   ( IWK( ISTEP + I ), I = 1, NEL1 )
C
C  Input the group type of each group
C
      READ( INPUT, 1010 ) ( IWK( ITYPEG + I ), I = 1, NG )
      IF ( DEBUG ) WRITE( IOUT, 1110 ) 'ITYPEG',
     *   ( IWK( ITYPEG + I ), I = 1, NG )
      IF ( IALGOR .GE. 2 ) THEN
         READ( INPUT, 1010 ) ( IWK( KNDOFC +  I ), I = 1, NG )
         IF ( DEBUG ) WRITE( IOUT, 1110 ) 'KNDOFC',
     *      ( IWK( KNDOFC + I ), I = 1, NG )
      END IF
C
C  Input the element type of each element
C
      READ( INPUT, 1010 ) ( IWK( ITYPEE + I ), I = 1, NELNUM )
      CALL SETTYP( NELNUM, IWK( ITYPEE + 1 ) )
      IF ( DEBUG ) WRITE( IOUT, 1110 ) 'ITYPEE',
     *   ( IWK( ITYPEE + I ), I = 1, NELNUM )
C
C  Input the number of internal variables for each element.
C
      READ( INPUT, 1010 ) ( IWK( INTVAR + I ), I = 1, NELNUM )
      IF ( DEBUG ) WRITE( IOUT, 1110 ) 'INTVAR',
     *   ( IWK( INTVAR + I ), I = 1, NELNUM )
C
C  Input the identity of each individual element.
C
      READ( INPUT, 1010 ) ( IWK( IELING + I ), I = 1, NGEL )
      IF ( DEBUG ) WRITE( IOUT, 1110 ) 'IELING',
     *   ( IWK( IELING + I ), I = 1, NGEL )
C
C  Input the variables in each group's elements.
C
      NVARS = IWK( ISTAEV + NEL1 ) - 1
      READ( INPUT, 1010 ) ( IWK( IELVAR + I ), I = 1, NVARS )
      IF ( DEBUG ) WRITE( IOUT, 1110 ) 'IELVAR',
     *   ( IWK( IELVAR + I ), I = 1, NVARS )
C
C  Input the column addresses of the nonzeros in each linear element.
C
      READ( INPUT, 1010 ) ( IWK( ICNA + I ), I = 1, NNZA )
      IF ( DEBUG ) WRITE( IOUT, 1110 ) 'ICNA  ',
     *   ( IWK( ICNA + I ), I = 1, NNZA )
C
C  Input the values of the nonzeros in each linear element, the
C  constant term in each group, the lower and upper bounds on
C  the variables and the starting point for the minimization.
C
      READ( INPUT, 1020 ) ( WK( A + I ), I = 1, NNZA )
      IF ( DEBUG ) WRITE( IOUT, 1120 ) 'A     ',
     *   ( WK( A + I ), I = 1, NNZA )
      READ( INPUT, 1020 ) ( WK( B + I ), I = 1, NG )
      IF ( DEBUG ) WRITE( IOUT, 1120 ) 'B     ',
     *   ( WK( B + I ), I = 1, NG )
      IF ( IALGOR .LE. 2 ) THEN
         READ( INPUT, 1020 ) ( BL( I ), I = 1, N )
         IF ( DEBUG ) WRITE( IOUT, 1120 ) 'BL    ',
     *      ( BL( I ), I = 1, N )
         READ( INPUT, 1020 ) ( BU( I ), I = 1, N )
         IF ( DEBUG ) WRITE( IOUT, 1120 ) 'BU    ',
     *      ( BU( I ), I = 1, N )
      ELSE
C
C  Use GVALS and FT as temporary storage for the constraint bounds.
C
         READ( INPUT, 1020 ) ( BL( I ), I = 1, N ),
     *                       ( WK( GVALS + I ), I = 1, NG )
         IF ( DEBUG ) WRITE( IOUT, 1120 ) 'BL    ',
     *      ( BL( I ), I = 1, N ), ( WK( GVALS + I ), I = 1, NG )
         READ( INPUT, 1020 ) ( BU( I ), I = 1, N ),
     *                       ( WK( FT + I ), I = 1, NG )
         IF ( DEBUG ) WRITE( IOUT, 1120 ) 'BU    ',
     *      ( BU( I ), I = 1, N ), ( WK( FT + I ), I = 1, NG )
      END IF
C
C   Input the starting point for the minimization.
C
      READ( INPUT, 1020 ) ( X( I ), I = 1, N )
      IF ( DEBUG ) WRITE( IOUT, 1120 ) 'X     ',
     *   ( X( I ), I = 1, N )
      IF ( IALGOR .GE. 2 ) THEN
         READ( INPUT, 1020 )( WK( U + I ), I = 1, NG )
         IF ( DEBUG ) WRITE( IOUT, 1120 ) 'U     ',
     *      ( WK( U + I ), I = 1, NG )
      END IF
      DO 5 I = 1, N
         IX( I ) = 3
         IF ( BU( I ) .LE. BL( I ) ) THEN
            IX( I ) = 5
         ELSE IF ( BL( I ) .LE. -BIG ) THEN
            IX( I ) = 2
            IF ( BU( I ) .GE.  BIG ) IX( I ) = 0
         ELSE IF ( BU( I ) .GE.  BIG ) THEN
            IX( I ) = 1
            IF ( BL( I ) .LE. -BIG ) IX( I ) = 0
         END IF
    5 CONTINUE
C
C  Input the parameters in each group.
C
      READ( INPUT, 1020 ) ( WK( GPVALU + I ), I = 1, NGPVLU )
      IF ( DEBUG ) WRITE( IOUT, 1120 ) 'GPVALU',
     *   ( WK( GPVALU + I ), I = 1, NGPVLU )
C
C  Input the parameters in each individual element.
C
      READ( INPUT, 1020 ) ( WK( EPVALU + I ), I = 1, NEPVLU )
      IF ( DEBUG ) WRITE( IOUT, 1120 ) 'EPVALU',
     *   ( WK( EPVALU + I ), I = 1, NEPVLU )
C
C  Input the scale factors for the nonlinear elements.
C
      READ( INPUT, 1020 ) ( WK( ESCALE + I ), I = 1, NGEL )
      IF ( DEBUG ) WRITE( IOUT, 1120 ) 'ESCALE',
     *   ( WK( ESCALE + I ), I = 1, NGEL )
C
C  Input the scale factors for the groups.
C
      READ( INPUT, 1020 ) ( WK( GSCALE + I ), I = 1, NG )
      IF ( DEBUG ) WRITE( IOUT, 1120 ) 'GSCALE',
     *   ( WK( GSCALE + I ), I = 1, NG )
C
C  Input the scale factors for the variables.
C
      READ( INPUT, 1020 ) ( WK( VSCALE + I ), I = 1, N )
      IF ( DEBUG ) WRITE( IOUT, 1120 ) 'VSCALE',
     *   ( WK( VSCALE + I ), I = 1, N )
C
C  Input the lower and upper bounds on the objective function.
C
      READ( INPUT, 1080 ) OBFBND( 1 ), OBFBND( 2 )
      IF ( DEBUG ) WRITE( IOUT, 1180 ) 'OBFBND',
     *    OBFBND( 1 ), OBFBND( 2 )
C
C  Input a logical array which says whether an element has internal
C  varaiables.
C
      READ( INPUT, 1030 ) ( LOGI( INTREP + I ), I = 1, NELNUM )
      IF ( DEBUG ) WRITE( IOUT, 1130 ) 'INTREP',
     *   ( LOGI( INTREP + I ), I = 1, NELNUM )
C
C  Input a logical array which says whether a group is trivial.
C
      READ( INPUT, 1030 ) ( LOGI( GXEQX + I ), I = 1, NG )
      IF ( DEBUG ) WRITE( IOUT, 1130 ) 'GXEQX ',
     *   ( LOGI( GXEQX + I ), I = 1, NG )
C
C  Input the names given to the groups and to the variables.
C
      READ( INPUT, 1040 ) ( CHA( GNAMES + I ), I = 1, NG )
      IF ( DEBUG ) WRITE( IOUT, 1140 ) 'GNAMES',
     *   ( CHA( GNAMES + I ), I = 1, NG )
      READ( INPUT, 1040 ) ( CHA( VNAMES + I ), I = 1, N )
      IF ( DEBUG ) WRITE( IOUT, 1140 ) 'VNAMES',
     *   ( CHA( VNAMES + I ), I = 1, N )
C
C  Consider which groups are constraints. Of these, decide which are
C  equations, which are linear, allocate starting values for the
C  Lagrange multipliers and set lower and upper bounds on any
C  inequality constraints. Reset KNDOFC to point to the list of
C  constraint groups.
C
      NC = 0
      DO 10 I = 1, NG
         IF ( IWK( KNDOFC + I ) .EQ. 1 ) THEN
           IWK( KNDOFC + I ) = - 1
         ELSE
            NC = NC + 1
            IF ( NC .LE. NCMAX ) THEN
               C( NC ) = ZERO
               IF ( LOGI( GXEQX + I ) .AND. IWK( ISTADG + I )
     *                       .GE. IWK( ISTADG + I + 1 ) ) C( NC ) = ONE
               IF ( IWK( KNDOFC + I ) .EQ. 2 ) THEN
                  IC    ( NC ) = 5
                  CL    ( NC ) = ZERO
                  CU    ( NC ) = ZERO
               ELSE
                  IC    ( NC ) = 3
                  CL    ( NC ) = WK( GVALS + I )
                  CU    ( NC ) = WK( FT    + I )
               END IF
            END IF
            IWK( KNDOFC + I ) = NC
         END IF
   10 CONTINUE
      IF ( NC .EQ. 0 .AND. IOUT .GT. 0 ) WRITE( IOUT, 2010 )
      IF ( NC .GT. NCMAX ) THEN
         CLOSE( INPUT )
         IF ( IOUT .GT. 0 ) THEN
            WRITE( IOUT, 2000 ) 'CL    ', 'NCMAX  ', NC - NCMAX
            WRITE( IOUT, 2000 ) 'CU    ', 'NCMAX  ', NC - NCMAX
            WRITE( IOUT, 2000 ) 'IC    ', 'NCMAX  ', NC - NCMAX
         END IF
         STOP
      END IF
C
C  Partition the workspace arrays FUVALS, IWK and WK. Initialize
C  certain portions of IWK.
C
      NUMVAR = N
      NUMCON = NC
      FDGRAD = .FALSE.
      CALL DINITW( N, NG, NELNUM, IWK(IELING+1), LELING, IWK(ISTADG+1),
     *             LSTADG, IWK(IELVAR+1), LELVAR, IWK(ISTAEV+1), LSTAEV,
     *             IWK(INTVAR+1), LNTVAR, IWK(ISTADH+1), LSTADH,
     *             IWK(ICNA+1), LICNA, IWK(ISTADA+1), LSTADA,
     *             LOGI(GXEQX+1), LGXEQX, LOGI(INTREP+1), LINTRE,
     *             LFUVAL, ALTRIV, .TRUE., FDGRAD, LFXI,LGXI,LHXI,LGGFX,
     *             LDX   , LGRJAC, LQGRAD, LBREAK, LP,     LXCP  ,
     *             LX0   , LGX0  , LDELTX, LBND,   LWKSTR, LSPTRS,
     *             LSELTS, LINDEX, LSWKSP, LSTAGV, LSTAJC, LIUSED,
     *             LFREEC, LNNONZ, LNONZ2, LSYMMD, LSYMMH, LSLGRP,
     *             LSVGRP, LGCOLJ, LVALJR, LSEND , LNPTRS, LNELTS,
     *             LNNDEX, LNWKSP, LNSTGV, LNSTJC, LNIUSE, LNFREC,
     *             LNNNON, LNNNO2, LNSYMD, LNSYMH, LNLGRP, LNVGRP,
     *             LNGCLJ, LNVLJR, LNQGRD, LNBRAK, LNP,    LNBND ,
     *             LNFXI,  LNGXI,  LNGUVL, LNHXI,  LNHUVL, LNGGFX,
     *             LNDX  , LNGRJC, LIWK2 , LWK2  , MAXSIN, NINVAR,
     *             NTYPE , NSETS , MAXSEL, LSTYPE, LSSWTR, LSSIWT,
     *             LSIWTR, LSWTRA, LNTYPE, LNSWTR, LNSIWT, LNIWTR,
     *             LNWTRA, LSISET, LSSVSE, LNISET, LNSVSE, RANGES,
     *           IWK(IWRK+1),LIWORK,WK(WRK+1),LWORK,IPRINT,IOUT,INFORM )
      IF ( INFORM .NE. 0 ) STOP
C
C  Shift the starting addresses for the real workspace relative to WRK.
C
      LQGRAD = LQGRAD + WRK
      LBREAK = LBREAK + WRK
      LP     = LP     + WRK
      LXCP   = LXCP   + WRK
      LX0    = LX0    + WRK
      LGX0   = LGX0   + WRK
      LDELTX = LDELTX + WRK
      LBND   = LBND   + WRK
      LSWTRA = LSWTRA + WRK
      LWKSTR = LWKSTR + WRK
C
C  Shift the starting addresses for the integer workspace relative
C  to IWRK.
C
      LSPTRS = LSPTRS + IWRK
      LSELTS = LSELTS + IWRK
      LINDEX = LINDEX + IWRK
      LSWKSP = LSWKSP + IWRK
      LSTAGV = LSTAGV + IWRK
      LSTAJC = LSTAJC + IWRK
      LIUSED = LIUSED + IWRK
      LFREEC = LFREEC + IWRK
      LNNONZ = LNNONZ + IWRK
      LNONZ2 = LNONZ2 + IWRK
      LSYMMD = LSYMMD + IWRK
      LSYMMH = LSYMMH + IWRK
      LSLGRP = LSLGRP + IWRK
      LSVGRP = LSVGRP + IWRK
      LGCOLJ = LGCOLJ + IWRK
      LVALJR = LVALJR + IWRK
      LSTYPE = LSTYPE + IWRK
      LSSWTR = LSSWTR + IWRK
      LSSIWT = LSSIWT + IWRK
      LSIWTR = LSIWTR + IWRK
      LSISET = LSISET + IWRK
      LSSVSE = LSSVSE + IWRK
      LSEND  = LSEND  + IWRK
      IF ( NUMCON .EQ. 0 ) GO TO 770
C
C  Record which group is associated with each constraint.
C
      IF ( NC .GT. LIWK2 ) THEN
         WRITE( IOUT, 2040 )
         STOP
      END IF
      MEQ  = 0
      MLIN = 0
      DO 100 IG = 1, NG
         I      = IWK( KNDOFC + IG )
         IF ( I .GT. 0 ) THEN
            IWK( LSEND + I ) = IG
            IF ( IC( I ) .EQ. 5) THEN
               MEQ  = MEQ  + 1
            ELSE IF ( CL( I ) .LE. -BIG ) THEN
               IC( I ) = 2
               IF ( CU( I ) .GE.  BIG ) IC( I ) = 0
            ELSE IF ( CU( I ) .GE.  BIG ) THEN
               IC( I ) = 1
               IF ( CL( I ) .LE. -BIG ) IC( I ) = 0
            END IF
            IF ( C( I ) .EQ. ONE ) MLIN = MLIN + 1
         END IF
  100 CONTINUE
         IF ( MLIN .EQ. 0 .OR. MLIN .EQ. NC ) GO TO 130
C
C  Reorder the constraints so that the linear constraints occur before the
C  nonlinear ones.
C
         MEND = NC
C
C  Run forward through the constraints until a nonlinear constraint
C  is encountered.
C
         DO 120 I = 1, NC
            IF ( I .GT. MEND ) GO TO 130
            IG = IWK( LSEND + I )
            IF ( C( I ) .EQ. ZERO ) THEN
C
C  Constraint I is nonlinear. Now, run backwards through the
C  constraints until a linear one is encountered.
C
               DO 110 J = MEND, I, - 1
                  JG    = IWK( LSEND + J )
                  IF ( C( J ) .EQ. ONE ) THEN
                     MEND = J - 1
C
C  Interchange the data for constraints I and J.
C
                     IWK( LSEND  +  I ) = JG
                     IWK( LSEND  +  J ) = IG
                     IWK( KNDOFC + IG ) = J
                     IWK( KNDOFC + JG ) = I
                     ATEMP              = C     ( I )
                     C     ( I )        = C     ( J )
                     C     ( J )        = ATEMP
                     ITEMP              = IC    ( I )
                     IC    ( I )        = IC    ( J )
                     IC    ( J )        = ITEMP
                     ATEMP              = CL    ( I )
                     CL    ( I )        = CL    ( J )
                     CL    ( J )        = ATEMP
                     ATEMP              = CU    ( I )
                     CU    ( I )        = CU    ( J )
                     CU    ( J )        = ATEMP
                     GO TO 120
                  END IF
  110          CONTINUE
               GO TO 130
            END IF
  120    CONTINUE
  130    CONTINUE
C
C  Set sparse objective structure.
C
      IF ( NUMCON .EQ. 0 ) GO TO 770
C
C  Set sparse constraint structure.
C
      MC = 0
      ICG( 1 ) = 1
      DO 760 ICON = 1, NUMCON
C
C  Find group index IG of constraint ICON.
C
      IG = 0
      DO 710 I = 1, NG
         IF ( IWK( KNDOFC + I ) .EQ. ICON ) THEN
           IG = I
           GOTO 720
         END IF
  710 CONTINUE
  720 IF ( IG .EQ. 0 ) THEN
         WRITE( IOUT, 3100 )
         STOP
      END IF
C
C  Determine nonlinear elements in group IG.
C  Record their indices in IWK( ICALCF ).
C
      NELOW  = IWK( ISTADG + IG )
      NELUP  = IWK( ISTADG + IG + 1 ) - 1
C
C  Consider only group IG.
C
         IG1    = IG + 1
         ISTRGV = IWK( LSTAGV + IG  )
         IENDGV = IWK( LSTAGV + IG1 ) - 1
         NONTRV = .NOT. LOGI( GXEQX + IG )
C
C  The group has nonlinear elements.
C
         IF ( NELOW .LE. NELUP ) THEN
C
C  Allocate a gradient.
C
            DO 740 I = ISTRGV, IENDGV
               LL    = IWK( LSVGRP + I )
C
C  Include contributions from the first N variables only.
C
               IF ( LL .LE. N ) THEN
                  MC           = MC + 1
                  JCG( MC ) = LL
               END IF
  740       CONTINUE
C
C  The group has only linear elements.
C
         ELSE
C
C  Allocate a gradient.
C
            DO 750 K = IWK( ISTADA + IG ), IWK( ISTADA + IG1 ) - 1
               LL    = IWK( ICNA + K )
C
C  Include contributions from the first N variables only.
C
               IF ( LL .LE. N ) THEN
                  MC           = MC + 1
                  JCG( MC ) = LL
               END IF
  750       CONTINUE
         END IF
         ICG( ICON + 1 ) = MC + 1
         CALL UXVSRT(ICG(ICON+1)-ICG(ICON),JCG(ICG(ICON)))
  760 CONTINUE
C
C  Set sparse objective structure.
C
C
C  there are non-trivial group functions.
C
  770 FIRSTG = .FALSE.
C
C  Define the real work space needed for ASMBL.
C  Ensure that there is sufficient space.
C
      IF ( NUMCON .GT. 0 ) THEN
         IF ( LWK2 .LT. N + 3 * MAXSEL + NG ) THEN
            IF ( IOUT .GT. 0 ) WRITE( IOUT, 2000 )
            STOP
         END IF
      ELSE
         IF ( LWK2 .LT. N + 3 * MAXSEL ) THEN
            IF ( IOUT .GT. 0 ) WRITE( IOUT, 2000 )
            STOP
         END IF
      END IF
C
C  Define the integer work space needed for ASMBL.
C  Ensure that there is sufficient space.
C
      LIWKH  = LIWK2 - N
      LH2    = MIN( MMAX, ( LIWKH - 3 * N ) / 2 )
      LINXTR = LH2 + N
      IF ( LH2 .LE. 0 ) THEN
         IF ( IOUT .GT. 0 ) WRITE( IOUT, 2010 )
         STOP
      END IF
C
C  Set starting addresses for partitions of the integer workspace.
C
      LIH    = MMAX
      LNXTRW = 0
      DO 690 I           = 1, N
         IWK( IVAR + I ) = I
  690 CONTINUE
C
C  Assemble the Hessian.
C
      IF ( NUMCON .GT. 0 ) THEN
         CALL UASMBL( N , NG, MAXSEL, N, LH2 , LIH , M ,
     *             N, IWK( IVAR + 1), IWK( ISTADH + 1 ), LSTADH,
     *             IWK( ICNA + 1 )  , LICNA ,
     *             IWK( ISTADA + 1 ), LSTADA, IWK( INTVAR + 1 ), LNTVAR,
     *             IWK( IELVAR + 1 ), LELVAR, IWK( IELING + 1 ), LELING,
     *             IWK( ISTADG + 1 ), LSTADG, IWK( ISTAEV + 1 ), LSTAEV,
     *             IWK( LSTAGV + 1 ), LNSTGV, IWK( LSVGRP + 1 ), LNVGRP,
     *             KH , JH , IWK( LSEND + LNXTRW + 1 ) , LINXTR,
     *             IWK( LIWKH + 1 ) , LIWK2 - LIWKH  ,
     *             WK( A + 1 ) , LA, FUVALS, LNGUVL, FUVALS, LNHUVL,
     *             WK( GVALS + NG + 1 ), WK( GVALS + 2 * NG + 1 ),
     *             WK( LWKSTR + 1 ), WK( ESCALE + 1 ), LESCAL,
     *             IH, WK ( LWKSTR + NG + 1 ), LWK2 - NG ,
     *             LOGI( GXEQX + 1 ), LGXEQX, LOGI( INTREP + 1 ),
     *             LINTRE, RANGES, 1, IOUT  , .FALSE., I, INFORM )
      ELSE
         CALL UASMBL( N , NG, MAXSEL, N, LH2 , LIH , M ,
     *             N, IWK( IVAR + 1), IWK( ISTADH + 1 ), LSTADH,
     *             IWK( ICNA + 1 )  , LICNA ,
     *             IWK( ISTADA + 1 ), LSTADA, IWK( INTVAR + 1 ), LNTVAR,
     *             IWK( IELVAR + 1 ), LELVAR, IWK( IELING + 1 ), LELING,
     *             IWK( ISTADG + 1 ), LSTADG, IWK( ISTAEV + 1 ), LSTAEV,
     *             IWK( LSTAGV + 1 ), LNSTGV, IWK( LSVGRP + 1 ), LNVGRP,
     *             KH , JH , IWK( LSEND + LNXTRW + 1 ) , LINXTR,
     *             IWK( LIWKH + 1 ) , LIWK2 - LIWKH  ,
     *             WK( A + 1 ) , LA, FUVALS, LNGUVL, FUVALS, LNHUVL,
     *             WK( GVALS + NG + 1 ), WK( GVALS + 2 * NG + 1 ),
     *             WK( GSCALE + 1 ), WK( ESCALE + 1 ), LESCAL,
     *             IH, WK ( LWKSTR + 1 ), LWK2 - NG ,
     *             LOGI( GXEQX + 1 ), LGXEQX, LOGI( INTREP + 1 ),
     *             LINTRE, RANGES, 1, IOUT  , .FALSE., I, INFORM )
      END IF
C
C  Check that there is sufficient integer workspace.
C
      IF ( INFORM .GT. 0 ) THEN
C         IF ( IOUT .GT. 0 ) WRITE( IOUT, 2010 )
C         STOP
          CALL UOERR1('SINSXX',75)
          ITERM= -75
      END IF
      RETURN
C
C  Non-executable statements.
C
 1000 FORMAT( I2, A8 )
 1001 FORMAT( 10I6 )
 1002 FORMAT( 2I6 )
 1010 FORMAT( ( 12I6 ) )
 1020 FORMAT( ( 1P, 4D16.8 ) )
 1030 FORMAT( ( 72L1 ) )
 1040 FORMAT( ( 8A10 ) )
 1080 FORMAT( 1P, 2D16.8 )
 1100 FORMAT( A8, 3I6 )
 1110 FORMAT( 1X, A6, /, ( 1X, 12I6 ) )
 1120 FORMAT( 1X, A6, /, ( 1X, 1P, 4D16.8 ) )
 1130 FORMAT( 1X, A6, /, ( 1X, 72L1 ) )
 1140 FORMAT( 1X, A6, /, ( 1X, 8A10 ) )
 1180 FORMAT( 1X, A6, /, 1P, 2D16.6 )
 2000 FORMAT( /, ' ** SUBROUTINE EINS: array length ', A6,
     *        ' too small.', /, ' -- Miminimization abandoned.',
     *        /, ' -- Increase the parameter ', A6, ' by at least ', I8,
     *           ' and restart.'  )
 2010 FORMAT( /, ' ** SUBROUTINE EINS: ** Warning. The problem has',
     *           ' no general constraints. ', /,
     *           ' Other tools may be preferable' )
 2020 FORMAT( /, ' ** SUBROUTINE EINS: the problem uses no variables.'
     *         , ' Execution terminating ' )
 2030 FORMAT( /, ' ** SUBROUTINE EINS: the problem is vacuous.',
     *           ' Execution terminating ' )
 2040 FORMAT( ' ** SUBROUTINE EINS: Increase the size of IWK ' )
 3100 FORMAT( ' ** SUBROUTINE EINS: invalid constraint index ICON ' )
C
C  End of EINS.
C
      END
