* SUBROUTINE UA0HS2              ALL SYSTEMS                  97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL COMPUTATION OF THE HESSIAN MATRIX OF THE MODEL FUNCTION
* USING ITS VALUES.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF FUNCTIONS.
*  II  NAL  NUMBER OF LINEAR FUNCTIONS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  AH(NF*NA*(NF+1)/2) SIMULTANEOUSLY COMPUTED DENSE HESSIAN
*         MATRICES.
*  RA  GO(NF)  AUXILIARY VECTOR.
*  RA  AGO(NF*NA)  AUXILIARY ARRAY.
*  RI  AF(NA)  VECTOR OF FUNCTION VALUES.
*  RA  AFO(NA)  AUXILIARY VECTOR.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*
* COMMON DATA :
*  II  KBF  TYPE OF BOUNDS. KBF=0-BOUNDS ARE NOT USED. KBF=1-ONE SIDED
*         BOUNDS. KBF=1-TWO SIDED BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*
      SUBROUTINE UA0HS2(NF,NA,NAL,X,IX,XN,AH,GO,AGO,AF,AFO,ETA1)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NA,NAL,IX(NF)
      REAL*8 X(NF),XN(NF),AH(NF*NA*(NF+1)/2),GO(NF),AGO(NF*NA),AF(NA),
     & AFO(NA),ETA1
      REAL*8 XTEMPI,XTEMPJ,ETA
      INTEGER I,J,IJ,KA,K,L,IVAR,JVAR,IND
      SAVE XTEMPI,XTEMPJ,ETA
      SAVE IVAR,JVAR
      IND(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
      IF (TSS(NS).NE.'UAHX') CALL UYPRO1('UAHX',-1)
      GO TO (1,3,8) ISP(NS)
    1 CONTINUE
      ETA=ETA1**(ONE/THREE)
      CALL UXVCOP(NA,AF,AFO)
      IVAR=0
    2 CONTINUE
      IVAR=IVAR+1
      IF (IVAR.GT.NF) GO TO 4
      IF (KBF.GT.0) THEN
      IF (IX(IVAR).LE.-5) GO TO 2
      ENDIF
      IF (NORMF.GT.0) THEN
      GO(IVAR)=XN(IVAR)
      ELSE
      GO(IVAR)=ONE
      ENDIF
      GO(IVAR)=ETA*MAX(ABS(X(IVAR)),GO(IVAR))*SIGN(ONE,X(IVAR))
      XTEMPI=X(IVAR)
      X(IVAR)=X(IVAR)+GO(IVAR)
      GO(IVAR)=X(IVAR)-XTEMPI
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
      K=NF*NAL
      DO 33 KA=NAL+1,NA
      AGO(K+IVAR)=AF(KA)
      K=K+NF
   33 CONTINUE
      X(IVAR)=XTEMPI
      GO TO 2
    4 IVAR=0
    5 CONTINUE
      IVAR=IVAR+1
      IF (IVAR.GT.NF) GO TO 10
      IF (KBF.GT.0) THEN
      IF (IX(IVAR).LE.-5) GO TO 5
      ENDIF
      XTEMPI=X(IVAR)
      X(IVAR)=XTEMPI+GO(IVAR)
      JVAR=IVAR-1
    7 CONTINUE
      JVAR=JVAR+1
      IF (KBF.GT.0) THEN
      IF (IX(JVAR).LE.-5) GO TO 7
      ENDIF
      IF (JVAR.GT.NF) GO TO 9
      XTEMPJ=X(JVAR)
      X(JVAR)=X(JVAR)+GO(JVAR)
      CALL UYSET0(1,3)
      RETURN
    8 CONTINUE
      IJ=IND(IVAR,JVAR)
      K=NF*NAL
      L=NF*NAL*(NF+1)/2
      DO 88 KA=NAL+1,NA
      AH(L+IJ)=((AFO(KA)-AGO(K+IVAR))+(AF(KA)-AGO(K+JVAR)))/
     & (GO(IVAR)*GO(JVAR))
      K=K+NF
      L=L+NF*(NF+1)/2
   88 CONTINUE
      X(JVAR)=XTEMPJ
      GO TO 7
    9 X(IVAR)=XTEMPI
      GO TO 5
   10 CALL UXVCOP(NA,AFO,AF)
      CALL UYEPI1(-1)
      RETURN
      END
