* SUBROUTINE UA2MX1             ALL SYSTEMS                 97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE OBJECTIVE FUNCTION
* WHICH IS DEFINED AS A SUM OF SQUARES.
*
* PARAMETERS:
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  NAL  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  KA  INDEX OF THE APPROXIMATED FUNCTION.
*  II  KAP  OPTIMUM INDEX OF THE APPROXIMATED FUNCTION.
*  RI  GA(NF)  GRADIENT OF THE APPROXIMATED FUNCTION.
*  RO  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  HA(NF*(NF+1)/2)  HESSIAN MATRIX OF THE APPROXIMATED FUNCTION.
*  RO  H(NF*(NF+1)/2)  HESSIAN MATRIX OF THE OBJECTIVE FUNCTION.
*  RI  FA  VALUE OF THE SELECTED FUNCTION.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*
* COMMON DATA :
*  II  KD  DEGREE OF REQUIRED DERVATIVES.
*  IO  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  IU  NAV  NUMBER OF APPROXIMATED FUNCTION VALUES COMPUTED.
*  IU  NAG  NUMBER OF APPROXIMATED FUNCTION GRADIENTS COMPUTED.
*  IU  NAH  NUMBER OF APPROXIMATED FUNCTION HESSIAN MATRICES COMPUTED.
*  IU  NFV  NUMBER OF OBJECTIVE FUNCTION VALUES COMPUTED.
*  IU  NFG  NUMBER OF OBJECTIVE FUNCTION GRADIENTS COMPUTED.
*  IU  NFH  NUMBER OF OBJECTIVE FUNCTION HESSIAN MATRICES COMPUTED.
*  II  IEXT  TYPE OF EXTREMUM. IEXT<0-MINIMAX. IEXT=0-CHEBYSHEV
*         APPROXIMATION, IEXT>0-MAXIMIN.
*  IU  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*
      SUBROUTINE UA2MX1(NF,NA,NAL,KA,KAP,GA,G,HA,H,FA,F)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NA,NAL,KA,KAP
      REAL*8 GA(NF),G(NF),HA(NF*(NF+1)/2),H(NF*(NF+1)/2),FA,F
      REAL*8 FTEMP,FVAL
      INTEGER K,L
      SAVE FVAL
      SAVE L
      IF (TSS(NS).NE.'UAXX') CALL UYPRO1('UAXX',-1)
      GO TO (1,4,9,3) ISP(NS)
    1 CONTINUE
      IF (KD.LE.LD) GO TO 11
      KA=1
    2 IF (KA.GT.NA) GO TO 7
      IF (KD.LT.0) GO TO 5
      IF (LD.GE.0) GO TO 5
      CALL UYSET0(1,2)
      RETURN
    4 CONTINUE
      IF (IEXT.EQ.0.AND.FA.GE.ZERO.OR.IEXT.LT.0) THEN
      FTEMP= FA
      K= 1
      ELSE
      FTEMP=-FA
      K=-1
      ENDIF
      IF (KA.EQ.1) THEN
      FVAL=FTEMP
      KAP=KA
      L=K
      ELSE IF (FVAL.LT.FTEMP) THEN
      FVAL=FTEMP
      KAP=KA
      L=K
      ENDIF
    5 KA=KA+1
      GO TO 2
    7 IF (LD.GE.0) GO TO 8
      F=FVAL
    8 IF (KD.LT.1) GO TO 10
      IF (LD.GE.1) GO TO 20
      KA=KAP
      CALL UYSET0(2,3)
      RETURN
    9 CONTINUE
      IF (L.GE.0) THEN
      CALL UXVCOP(NF,GA,G)
      ELSE
      CALL UXVNEG(NF,GA,G)
      ENDIF
   20 IF (KD.LT.2) GO TO 10
      IF (LD.GE.2) GO TO 10
      KA=KAP
      IF (L.GE.0) THEN
      CALL UXVCOP(NF,G,GA)
      ELSE
      CALL UXVNEG(NF,G,GA)
      ENDIF
      CALL UYSET0(3,4)
      RETURN
    3 CONTINUE
      IDECF=0
      IF (L.GE.0) THEN
      CALL UXVCOP(NF*(NF+1)/2,HA,H)
      ELSE
      CALL UXVNEG(NF*(NF+1)/2,HA,H)
      ENDIF
   10 NFV=NAV/(NA-NAL)
      NFG=NAG/(NA-NAL)
      NFH=NAH/(NA-NAL)
      LD=KD
   11 CALL UYEPI1(-1)
      RETURN
      END
