* SUBROUTINE UD1TL1                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* TEST ON DESCENT DIRECTION AND PREPARATION OF LINE SEARCH.
*
* PARAMETERS :
*  II  NDIM DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  G(NDIM) GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  S(NDIM) DIRECTION VECTOR.
*  RI  EPS0  TOLERANCE FOR UNIFORM DESCENT DIRECTION.
*  RI  ALF1  LOWER BOUND FOR STEPSIZE PARAMETER.
*  RI  ALF2  UPPER BOUND FOR STEPSIZE PARAMETER.
*  RO  R  STEPSIZE PARAMETER.
*  RO  P  DIRECTIONAL DERIVATE.
*  RI  GNORM  EUCLIDEAN NORM OF THE GRADIENT.
*  RI  SNORM  EUCLIDEAN NORM OF THE DIRECTION VECTOR.
*  RO  RMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RO  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  XMAX  MAXIMUM STEPSIZE.
*  RI  XDEL  TRUST REGION RADIUS.
*  II  MES  METHOD SELECTION.
*  II  INITD  CHOICE OF THE RESTART CRITERION. INITD<0-CHANGE OF THE
*         SIGN IF P<0. INITD>0-NO CHANGE IF P<0. ABS(INITD)=0-NO
*         RESTART CRITERION. ABS(INITD)=1-UNIFORM DESCENT CRITERION.
*  II  INITH  CHOICE OF THE INITIAL APPROXIMATION OF THE HESSIAN
*         MATRIX. INITH=1-UNIT INITIAL APPROXIMATION.
*         INITH=2-RANDOM INITIAL APPROXIMATION.
*
* COMMON DATA :
*  IU  ITERD  TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  IU  IREST  RESTART SPECIFICATION. IF IREST>0 THEN RESTART IS
*         PERFORMED.
*  IU  ITERM  TERMINATION INDICATOR. TERMINATION IF ITERM .NE. 0.
*  IU  NRED  ITERATION COUNT IN LINE SEARCH.
*  TO  TDXX  TEXT INFORMATION ON THE DIRECTION VECTOR OBTAINED.
*
* SUBPROGRAMS USED :
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UYRES1  DEFINITION OF THE RESTART VARIABLES.
*  S   UOD1D8  INTERMEDIATE PRINT.
*
      SUBROUTINE UD1TL1(NDIM,N,G,S,EPS0,ALF1,ALF2,R,P,GNORM,SNORM,RMIN,
     & RMAX,XMAX,XDEL,MES,INITD,INITH)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NDEM
      COMMON /UMCOMD/ NDEM
      INTEGER NDIM,N,MES,INITD,INITH
      REAL*8 G(NDIM),S(NDIM),EPS0,ALF1,ALF2,R,P,GNORM,SNORM,RMIN,RMAX,
     & XMAX,XDEL
      REAL*8 SNOR
      REAL*8 CON1
      PARAMETER (CON1=5.0D-1)
      SAVE SNOR
      NDEM=NDIM
      IF(ITERD.LT.0) THEN
        ITERM=ITERD
        RETURN
      ENDIF
      IF (MES.LT.0) THEN
      IF (NIT.LE.LIT+1) THEN
      XDEL=XMAX
      ELSE
      IF (R.LT.CON1) THEN
      XDEL=R*SNOR
      ELSE IF (XDEL.LE.SNOR) THEN
      XDEL=TWO*XDEL
      ENDIF
      ENDIF
      IF (SNORM.GT.XDEL) THEN
      CALL UXVSCL(N,XDEL/SNORM,S,S)
      SNORM=XDEL
      ENDIF
      SNOR=SNORM
      ENDIF
      IF (INITD.LT.0.AND.P.GT.ZERO) THEN
C
C     CHANGE OF THE SIGN
C
        CALL UXVNEG(N,S,S)
        P=-P
      ENDIF
C
C     TEST ON DESCENT DIRECTION
C
      IF(SNORM.LE.ZERO) THEN
        CALL UYRES1('ZERO DIR')
      ELSEIF(ABS(INITD).EQ.0.OR.P+EPS0*GNORM*SNORM.LE.ZERO) THEN
        IREST=0
      ELSEIF(IREST.NE.0.OR.INITH.LT.0) THEN
C
C     STEEPEST DESCENT DIRECTION
C
        IREST=0
        CALL UXVNEG(N,G,S)
        P=-GNORM*GNORM
        SNORM=GNORM
        ITERD=0
        TDXX='ST. DES.'
      ELSEIF(ABS(INITD).EQ.1) THEN
C
C     UNIFORM DESCENT CRITERION
C
        CALL UYRES1('UNIFORM ')
      ENDIF
      IF (IREST.EQ.0) THEN
C
C     PREPARATION OF LINE SEARCH
C
        NRED = 0
        RMIN=ALF1*GNORM/SNORM
        RMAX=MIN(ALF2*GNORM/SNORM,XMAX/SNORM)
        RMIN=MIN(RMIN,RMAX/TEN)
      ENDIF
      CALL UOD1D8(N,G,S,GNORM,SNORM,P)
      RETURN
      END
