* SUBROUTINE UEVEX1                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF THE INITIAL VALUE PROBLEM FOR A SET OF THE FIRST ORDER
* ORDINARY DIFFERENTIAL EQUATIONS USING THE FIFTH ORDER DORMAN AND
* PRINCE INTEGRATION FORMULA.
*
* PARAMETERS :
*  II  NA  NUMBER OF MESH POINTS.
*  II  NAL  NUMBER OF MESH POINTS.
*  IU  MAL
*  II  NE  DECLARED NUMBER OF DIFFERENTIAL EQUATIONS.
*  II  MEL  ACTUAL NUMBER OF DIFFERENTIAL EQUATIONS.
*  IU  KA  INDEX OF THE MESH POINT.
*  RI  AT(NA)  MESH POINTS.
*  RO  AY(NA*NE)  FIELD OF SOLUTION VECTORS IN THE MESH POINTS.
*  RU  YA(MEL)  SOLUTION VECTOR.
*  RA  YAO(MEL)  OLD SOLUTION VECTOR.
*  RI  EF(MEL)  RIGHT HAND SIDE VECTOR.
*  RA  ES(12*MEL)  AUXILIARY VECTOR.
*  RI  ETA0  MACHINE PRECISION.
*  RI  TET1  LOVER PARAMETER FOR STEPSIZE SELECTION.
*  RI  TET2  UPPER PARAMETER FOR STEPSIZE SELECTION.
*  RU  TA  VALUE OF THE INTEGRATION PARAMETER.
*  RU  TAO  OLD VALUE OF THE INTEGRATION PARAMETER.
*  RI  TAMAX  MAXIMUM VALUE OF THE INTEGRATION PARAMETER.
*  RU  HE  ACTUAL STEPSIZE.
*  RU  HEMAX  MAXIMUM STEPSIZE.
*  RI  FMAX  UPPER BOUND FOR SOLUTION NORM.
*  RO  ERR  INTEGRATION ERROR.
*  RO  ERRO  OLD INTEGRATION ERROR.
*  II  MED  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  II  MED3  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  II  MED4  CHOICE OF STEPSIZE SEQUENCE.
*  II  MED5  DEGREE OF INTERPOLATION FORMULA.
*  II  MED6  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  IO  NACC  NUMBER OF ACCEPTED INTEGRATION STEPS.
*  IO  NREJ  NUMBER OF REJECTED INTEGRATION STEPS.
*  IU  KAP  AUXILIARY VARIABLE.
*  IO  IER  ERROR INDICATOR. IER=0-SOLUTION OBTAINED. IER=-1-SYSTEM
*         IS UNSTABLE. IER=-2-MAXIMUM NUMBER OF STEPS EXCEEDED.
*         IER=-3-TOO SMALL STEP REQUIRED. IER=-4-SYSTEM IS PROBABLY
*         STIFF. IER=-5-JACOBIAN MATRIX IS SINGULAR. IER=-1-LACK
*         OF SPACE FOR OUTPUT DATA.
*
* COMMON DATA :
*  IO  NSTP  ACTUAL  NUMBER OF INTEGRATION STEPS.
*  II  MSTP  MAXIMUM NUMBER OF INTEGRATION STEPS.
*  RI  TOLR  RELATIVE PRECISION.
*  RI  TOLA  ABSOLUTE PRECISION.
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UEVEXB  HERMITE INTERPOLATION.
*  S   UNVEX1  INTERPOLATION IN MESH POINTS.
*  S   UXVCOP  COPYING OF A VECTOR.
*
* METHOD :
* E.HAIRER, S.P.NORSETT, G.WANNER: SOLVING ORDINARY DIFFERENTIAL
* EQUATIONS I. SPRINGER SERIES IN COMPUTATIONAL MATHEMATICS.
* SPRINGER VERLAG, BERLIN 1980.
*
      SUBROUTINE UEVEX1(NA,NAL,MAL,NE,MEL,KA,AT,AY,YA,YAO,EF,ES,IES,
     & ATOV,REJECT,ETA0,TET1,TET2,TA,TAO,TAMAX,HE,HEMAX,FMAX,ERR,ERRO,
     & MED,MED3,MED4,MED5,MED6,NACC,NREJ,KAP,IER)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NA,NAL,MAL,NE,MEL,KA,MED,MED3,MED4,MED5,MED6,NACC,NREJ,
     & KAP,IER,IES(2*MED3+1)
      REAL*8 AT(*),AY(*),YA(MEL),YAO(MEL),EF(MEL),
     & ES(*),ETA0,TET1,TET2,
     & TA,TAO,TAMAX,HE,HEMAX,FMAX,ERR,ERRO
      LOGICAL ATOV,REJECT
      REAL*8 UXVMAX,ERRX,PROD,HOPTDE,H1,XOLDD,HHH,DBLENJ,FACTOR,FACNJ,
     & ERRINT
      INTEGER NE1,NE2,NE3,NE4,NE5,NE6,NE7,NE8,NE9,NE10,NE11,NE12,NE13,
     & NE14,NE15,NE16,NE17,NE18,I,J,K,L,LFS,NJADD,MU,KMI,KMIT,KBEG,KK,
     & KCE,KRN,LBEG,LEND,KOPT
      LOGICAL LAST
      SAVE HOPTDE
      SAVE NE1,NE2,NE3,NE4,NE5,NE6,NE7,NE8,NE9,NE10,NE11,NE12,NE13,
     & NE14,NE15,NE16,NE17,NE18,K,KMIT,LFS,KCE
      SAVE LAST
      IF (TSS(NS).NE.'UEDX') CALL UYPRO1('UEDX',-1)
      GO TO (1,2,3,4,5,6,7) ISP(NS)
    1 CONTINUE
      LFS=2*MED3*MED3+MED3
      NE1=0
      NE2=MEL
      NE3=2*MEL
      NE4=3*MEL
      NE5=4*MEL
      NE6=NE5+MED3*MEL
      NE7=NE6+MED3
      NE8=NE7+MED3
      NE9=NE8+MED3
      NE10=NE9+(2*MED3*MED3+MED3)*NE
      NE11=NE10+MED3*NE
      NE12=NE11+2*MED3
      NE13=NE12+NE
      NE14=NE13+NE
      NE15=NE14+NE
      NE16=NE15+NE
      NE17=NE16+2*MED3*NE
      NE18=NE17+NE
C
      MED3=MAX(MED3,3)
      IF(MED.LE.1) MED4=MAX(MED4,1)
      IF(MED.GE.2) MED4=MAX(MED4,4)
      MED4=MIN(MED4,5)
C
C     DEFINE THE STEP SIZE SEQUENCE
C
      IF (MED4.EQ.1) THEN
        DO 81 I=1,MED3
  81    IES(I)=2*I
      END IF
      IF (MED4.EQ.2) THEN
        IES(1)=2
        DO 82 I=2,MED3
  82    IES(I)=4*I-4
      END IF
      IF (MED4.EQ.3) THEN
        IES(1)=2
        IES(2)=4
        IES(3)=6
        DO 83 I=4,MED3
   83   IES(I)=2*IES(I-2)
      END IF
      IF (MED4.EQ.4) THEN
        DO 84 I=1,MED3
   84   IES(I)=4*I-2
      END IF
      IF (MED4.EQ.5) THEN
        DO 85 I=1,MED3
   85   IES(I)=4*I
      END IF
C
C     DEFINE THE A(I) FOR ORDER SELECTION
C
      ES(NE8+1)=1.D0+IES(1)
      DO 86 I=2,MED3
   86 ES(NE8+I)=ES(NE8+I-1)+IES(I)
C
C     INITIAL SCALING
C
      DO 87 I=1,MEL
      ES(NE4+I)=TOLA + TOLR*ABS(YA(I))
   87 CONTINUE
C
C     INITIAL PREPARATIONS
C
      K=MAX(3,MIN(MED3-1,INT(-LOG10(TOLR)*0.6D0+1.5D0)))
      HE=MAX(ABS(HE),1.D-4)
      HE=MIN(HE,HEMAX,(TAMAX-TA)/2.D0)
      TAO=TA
      CALL UXVCOP(MEL,YA,YAO)
C
C     OUTPUT
C
      IF (MED.GE.1) THEN
        IF (MED.GE.2) THEN
          IES(MED3+1)=0
          DO 88 I=1,MED3
          NJADD=4*I-2
          IF (IES(I).GT.NJADD) NJADD=NJADD+1
   88     IES(MED3+I+1)=IES(MED3+I)+NJADD
          DO 90 MU=1,MED3*2
          ERRX=SQRT(MU/(MU+4.D0))*0.5D0
          PROD=1.D0/(MU+4.D0)**2
          DO 89 J=1,MU
   89     PROD=PROD*ERRX/J
   90     ES(NE11+MU)=PROD
          MAL=0
        END IF
        CALL UNVEX1(NA,NAL,NE,KA,AT,AY,YAO,KMIT,TA,TAO,HE,IER,-MED)
      END IF
C
      ERR=0.D0
      ERRO=1.D10
      HOPTDE=HEMAX
      ES(NE7+1)=0.D0
      REJECT=.FALSE.
      LAST=.FALSE.
  10  ATOV=.FALSE.
C
C     IS XEND REACHED IN THE NEXT STEP?
C
      H1=TAMAX-TA
      IF (H1.LE.ETA0) GO TO 110
      HE=MIN(HE,H1,HEMAX,HOPTDE)
      IF (HE.GE.H1-ETA0) LAST=.TRUE.
      IF (NSTP.NE.0.AND.MED.EQ.2) GO TO 22
      CALL UXVCOP(MEL,YAO,YA)
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE3+1))
C
C     THE FIRST AND LAST STEP
C
  22  IF (NSTP.NE.0.AND..NOT.LAST) GO TO 30
      MAL=0
      NSTP=NSTP+1
      KAP=1
  20  IF (KAP.GT.K) GO TO 55
      KCE=KAP
      CALL UYSET0(2,3)
      RETURN
    3 CONTINUE
      IF (ATOV) GO TO 10
      IF (KAP.GT.1.AND.ERR.LE.1.D0) GO TO 60
      KAP=KAP+1
      GO TO 20
C
C     BASIC INTEGRATION STEP
C
  30  CONTINUE
      IF (UXVMAX(NE,YA).GE.FMAX) GO TO 121
      MAL=0
      NSTP=NSTP+1
      IF (NSTP.GE.MSTP) GO TO 120
      KCE=K-1
      KAP=1
  23  IF(KAP.GT.KCE) GO TO 40
      CALL UYSET0(2,4)
      RETURN
    4 CONTINUE
      IF (ATOV) GO TO 10
      KAP=KAP+1
      GO TO 23
  40  CONTINUE
C
C     CONVERGENCE MONITOR
C
      IF (K.EQ.2.OR.REJECT) GO TO 50
      IF (ERR.LE.1.D0) GO TO 60
      IF (ERR.GT.((IES(K+1)*IES(K))/4.D0)**2) GO TO 100
  50  CONTINUE
      KAP=K
      CALL UYSET0(2,5)
      RETURN
    5 CONTINUE
      IF (ATOV) GO TO 10
      KCE=K
      IF (ERR.LE.1.D0) GO TO 60
C
C     HOPE FOR CONVERGENCE IN LINE K+1
C
  55  CONTINUE
      IF (ERR.GT.(IES(K+1)/2.D0)**2) GO TO 100
      KCE=K+1
      KAP=KCE
      CALL UYSET0(2,6)
      RETURN
    6 CONTINUE
      IF (ATOV) GO TO 10
      IF (ERR.GT.1.D0) GO TO 100
C
C     STEP IS ACCEPTED
C
  60  TAO=TA
      TA=TA+HE
C
C     OUTPUT
C
      IF (MED.NE.2) GO TO 71
      KMIT=2*KCE-MED5+1
      DO 69 I=1,NE
  69  ES(NE12+I)=YAO(I)
      XOLDD=TAO
      HHH=HE
      DO 76 I=1,NE
  76  ES(NE13+I)=HE*ES(NE3+I)
      DO 176 I=1,NE
 176  ES(NE14+I)=ES(NE5+MED3*(I-1)+1)
C
C     COMPUTE SOLUTION AT MID-POINT
C
      DO 473 J=2,KCE
      DBLENJ=IES(J)
      DO 473 L=J,2,-1
      FACTOR=(DBLENJ/IES(L-1))**2-1.D0
      DO 473 I=1,NE
      ES(NE10+MED3*(I-1)+L-1)=ES(NE10+MED3*(I-1)+L)+
     &  (ES(NE10+MED3*(I-1)+L)-ES(NE10+MED3*(I-1)+L-1))/FACTOR
 473  CONTINUE
      DO 474 I=1,NE
 474  ES(NE16+I)=ES(NE10+MED3*(I-1)+1)
C
C     COMPUTE FIRST DERIVATIVE AT RIGHT END
C
      DO 478 I=1,MEL
 478  YA(I)=ES(NE5+MED3*(I-1)+1)
      CALL UYSET0(1,7)
      RETURN
    7 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE2+1))
      DO 274 I=1,NE
 274  ES(NE15+I)=ES(NE2+I)*HE
C
C     THE LOOP
C
      DO 180 KMI=1,KMIT
C
C     COMPUTE KMI-TH DERIVATIVE AT MID-POINT
C
      KBEG=(KMI+1)/2
      DO 375 KK=KBEG,KCE
      FACNJ=(IES(KK)/2.D0)**(KMI-1)
      MAL=IES(MED3+KK+1)-2*KK+KMI
      DO 371 I=1,NE
 371  ES(NE10+MED3*(I-1)+KK)=ES(NE9+LFS*(I-1)+MAL)*FACNJ
 375  CONTINUE
      DO 373 J=KBEG+1,KCE
      DBLENJ=IES(J)
      DO 373 L=J,KBEG+1,-1
      FACTOR=(DBLENJ/IES(L-1))**2-1.D0
      DO 373 I=1,NE
      ES(NE10+MED3*(I-1)+L-1)=ES(NE10+MED3*(I-1)+L)+
     &  (ES(NE10+MED3*(I-1)+L)-ES(NE10+MED3*(I-1)+L-1))/FACTOR
 373  CONTINUE
      KRN=NE16+KMI*NE
      DO 374 I=1,NE
 374  ES(KRN+I)=ES(NE10+MED3*(I-1)+KBEG)*HE
      IF (KMI.EQ.KMIT) GOTO 180
C
C     COMPUTE DIFFERENCES
C
      DO 66 KK=(KMI+2)/2,KCE
      LBEG=IES(MED3+KK+1)
      LEND=IES(MED3+KK)+KMI+1
      IF (KMI.EQ.1.AND.MED4.EQ.4) LEND=LEND+2
      DO 64 L=LBEG,LEND,-2
      DO 64 I=1,NE
  64  ES(NE9+LFS*(I-1)+L)=
     &   ES(NE9+LFS*(I-1)+L)-ES(NE9+LFS*(I-1)+L-2)
      IF (KMI.EQ.1.AND.MED4.EQ.4) THEN
        L=LEND-2
        DO 65 I=1,NE
  65    ES(NE9+LFS*(I-1)+L)=
     &     ES(NE9+LFS*(I-1)+L)-ES(NE3+I)
      END IF
  66  CONTINUE
C
C     COMPUTE DIFFERENCES
C
      DO 166 KK=(KMI+2)/2,KCE
      LBEG=IES(MED3+KK+1)-1
      LEND=IES(MED3+KK)+KMI+2
      DO 164 L=LBEG,LEND,-2
      DO 164 I=1,NE
 164  ES(NE9+LFS*(I-1)+L)=
     &   ES(NE9+LFS*(I-1)+L)-ES(NE9+LFS*(I-1)+L-2)
 166  CONTINUE
 180  CONTINUE
      CALL UEVEXB(NE,ES(NE12+1),KMIT)
C
C     ESTIMATION OF INTERPOLATION ERROR
C
      IF (MED6.EQ.0.AND.KMIT.GE.1) THEN
        ERRINT=0.D0
        KRN=NE16+KMIT*NE
        DO 187 I=1,NE
 187    ERRINT=ERRINT
     &        +(ES(KRN+I)/ES(NE4+I))**2
        ERRINT=SQRT(ERRINT/NE)*ES(NE11+KMIT)
        HOPTDE=HE/MAX((ERRINT)**(1.D0/(KMIT+4)),0.01D0)
        IF (ERRINT.GT.10.D0) THEN
          HE=HOPTDE
          TA=TAO
          NREJ=NREJ+1
          REJECT=.TRUE.
          GOTO 10
        END IF
      END IF
      DO 189 I=1,MEL
 189  ES(NE3+I)=ES(NE2+I)
C
  71  DO 70 I=1,MEL
  70  YAO(I)=ES(NE5+MED3*(I-1)+1)
      NACC=NACC+1
C
C     OUTPUT
C
      IF (MED.GE.1) THEN
        IF (MED.EQ.1) CALL UNVEX1(NA,NAL,NE,KA,AT,AY,YAO,KMIT,TA,TAO,
     &      HE,IER,MED)
        IF (MED.GE.2) THEN
        IF (HE*(TA-AT(KA)).GE.ZERO) THEN
          CALL UNVEX1(NA,NAL,NE,KA,AT,AY,ES(NE12+1),KMIT,TA,TAO,HE,
     &      IER,MED)
        END IF
        END IF
        IF (IER.EQ.-11) GO TO 110
      END IF
C
C     COMPUTE OPTIMAL ORDER
C
      IF (KCE.EQ.2) THEN
        KOPT=3
        IF (REJECT) KOPT=2
        GO TO 80
      END IF
      IF (KCE.LE.K) THEN
        KOPT=KCE
        IF (ES(NE7+KCE-1).LT.ES(NE7+KCE)*TET1) KOPT=KCE-1
        IF (ES(NE7+KCE).LT.ES(NE7+KCE-1)*TET2) THEN
          KOPT=MIN(KCE+1,MED3-1)
        END IF
      ELSE
        KOPT=KCE-1
        IF (KCE.GT.3.AND.ES(NE8+KCE-2).LT.ES(NE7+KCE-1)*TET1) THEN
          KOPT=KCE-2
        END IF
        IF (ES(NE7+KCE).LT.ES(NE7+KOPT)*TET2) KOPT=MIN(KCE,MED3-1)
      END IF
C
C     AFTER A REJECTED STEP
C
  80  IF (REJECT) THEN
        K=MIN(KOPT,KCE)
        HE=MIN(HE,ES(NE6+K))
        REJECT=.FALSE.
        GO TO 10
      END IF
C
C     COMPUTE STEPSIZE FOR NEXT STEP
C
      IF (KOPT.LE.KCE) THEN
        HE=ES(NE6+KOPT)
      ELSE
        IF (KCE.LT.K.AND.ES(NE7+KCE).LT.ES(NE7+KCE-1)*TET2) THEN
          HE=ES(NE6+KCE)*ES(NE8+KOPT+1)/ES(NE8+KCE)
        ELSE
          HE=ES(NE6+KCE)*ES(NE8+KOPT)/ES(NE8+KCE)
        END IF
      END IF
      K=KOPT
      GO TO 10
C
C     STEP IS REJECTED
C
 100  CONTINUE
      K=MIN(K,KCE)
      IF (K.GT.2.AND.ES(NE7+K-1).LT.ES(NE7+K)*TET1) K=K-1
      NREJ=NREJ+1
      HE=ES(NE6+K)
      REJECT=.TRUE.
      GO TO 30
C
C     FAIL EXIT
C
 120  IER=-2
      TEDX='STEP LIM'
      GO TO 110
 121  IER=-1
      TEDX='UNSTABLE'
 110  CALL UYEPI1(-1)
      RETURN
      END
