* SUBROUTINE UF1FQ4                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE QUADRATIC OBJECTIVE
* FUNCTION - SPARSE CASE.
*
* PARAMETERS:
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF BLOCKS IN THE PARTITIONED STRUCTURE.
*  II  MA  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  MB  NUMBER OF NONZERO ELEMENTS THE PARTITIONED HESSIAN MATRIX.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  GF(NF)  GRADIENT OF THE QUADRATIC MODEL FUNCTION.
*  RO  G(NF)  GRADIENT OF THE QUADRATIC OBJECTIVE FUNCTION.
*  RI  HF(M)  HESSIAN ELEMENTS OF THE QUADRATIC MODEL FUNCTION.
*  II  IH(NA+1)  POINTERS OF THE DIAGONAL ELEMENTS OF HESSIAN MATRIX.
*  II  JH(MA)  INDICES OF THE NONZERO ELEMENTS OF HESSIAN MATRIX.
*  RI  FF  CONSTANT TERM OF THE QUADRATIC MODEL FUNCTION.
*  RO  F  VALUE OF THE QUADRATIC OBJECTIVE FUNCTION.
*  II  NAP  POINTER.
*
* COMMON DATA :
*  II  KD  DEGREE OF REQUIRED DERVATIVES.
*  II  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  II  IEXT  TYPE OF EXTREMUM. IEXT=0-MINIMUM. IEXT=1-MAXIMUM.
*
* SUBPROGRAMS USED :
*  S   UXBSMM  MATRIX VECTOR PRODUCT.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXVSUM  SUM OF TWO VECTORS.
*
      SUBROUTINE UF1FQ4(NF,NA,MA,MB,X,GF,G,HF,IH,JH,FF,F,NAP)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NA,MA,MB,IH(NA+1),JH(MA),NAP
      REAL*8 X(NF),GF(NF),G(NF),HF(MB),FF,F
      REAL*8 UXVDOT
      INTEGER NP
      IF (KD.LT.0) GO TO 1
      IF (LD.LT.0) THEN
      NP=NF-NAP
      CALL UXBSMM(NP,MB,NA,MA,HF,IH,JH,X,G)
      CALL UXVSUM(NF,GF,G,G)
      F=FF+HALF*(UXVDOT(NF,X,GF)+UXVDOT(NF,X,G))
      IF (IEXT.GT.0) F=-F
      ENDIF
      IF (KD.LT.1) GO TO 1
      IF (LD.LT.1) THEN
      IF (LD.GE.0) THEN
      CALL UXBSMM(NP,MB,NA,MA,HF,IH,JH,X,G)
      CALL UXVSUM(NF,GF,G,G)
      ENDIF
      IF (IEXT.GT.0) CALL UXVNEG(NF,G,G)
      ENDIF
    1 CONTINUE
      RETURN
      END
