* SUBROUTINE UF1HS3                ALL SYSTEMS                09/12/01
C PORTABILITY : ALL SYSTEMS
C 09/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL COMPUTATION OF THE HESSIAN MATRIX OF THE MODEL FUNCTION
* USING ITS GRADIENTS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RU  D(NF)  ELEMENTS OF THE DIAGONAL PRECONDITIONER.
*  RI  GF(NF)  GRADIENT OF THE MODEL FUNCTION.
*  RA  XS(NF)  AUXILIARY VECTOR.
*  RA  GS(NF)  AUXILIARY VECTOR.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RI  FF  VALUE OF THE MODEL FUNCTION.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*  II  MOT5  CHOICE OF DIFFERENCES. MOT5=1-VARIABLE DIFFERENCES.
*         MOT5=2-CONSTANT DIFFERENCES.
*
* COMMON DATA :
*  II  KBF  TYPE OF BOUNDS. KBF=0-BOUNDS ARE NOT USED. KBF=1-ONE SIDED
*         BOUNDS. KBF=1-TWO SIDED BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*
      SUBROUTINE UF1HS3(NF,X,IX,XN,D,GF,XS,GS,S,FF,ETA1,MOT5)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,IX(NF),MOT5
      REAL*8 X(NF),XN(NF),D(NF),GF(NF),XS(NF),GS(NF),S(NF),FF,ETA1
      REAL*8 XSTEP,XTEMP,FTEMP,ETA
      INTEGER I,J
      SAVE FTEMP,ETA
      IF (TSS(NS).NE.'UFHX') CALL UYPRO1('UFHX',-1)
      GO TO (1,4) ISP(NS)
    1 CONTINUE
      ETA=SQRT(ETA1)
      FTEMP=FF
      CALL UXVCOP(NF,GF,GS)
      CALL UXVCOP(NF,X,XS)
C
C     STEP SELECTION
C
      DO 3 I=1,NF
C      IF (KBF.GT.0) THEN
C      IF (IX(I).LE.-5) GO TO 3
C      ENDIF
      XTEMP=X(I)
      IF (MOT5.EQ.1) THEN
      IF (NORMF.GT.0) THEN
      XSTEP=XN(I)
      ELSE
      XSTEP=ONE
      ENDIF
      S(I)=SIGN(ETA*MAX(ABS(XTEMP),XSTEP),XTEMP)
      ELSE
      S(I)=SIGN(SQRT(ETA/DBLE(NF)),XTEMP)
      ENDIF
      X(I)=XTEMP+S(I)
      S(I)=X(I)-XTEMP
    3 CONTINUE
      CALL UYSET0(1,2)
      RETURN
    4 CONTINUE
C
C     NUMERICAL DIFFERENTIATION
C
      DO 5 I=1,NF
C      IF (KBF.GT.0) THEN
C      IF (IX(I).LE.-5) GO TO 5
C      ENDIF
      D(I)=(GF(I)-GS(I))/S(I)
      X(I)=XS(I)
    5 CONTINUE
C
C     COMPUTATION OF THE DIAGONAL MATRIX
C
      FF=FTEMP
      CALL UXVCOP(NF,GS,GF)
      CALL UYEPI1(-1)
      RETURN
      END
