* SUBROUTINE UKQGL1                 ALL SYSTEMS                98/08/01
C PORTABILITY : ALL SYSTEMS
C 98/08/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* DIAGONALLY PRECONDITIONED CONJUGATE GRADIENTS. ADDITIONAL PARAMETERS
* ARE TAKEN FROM THE TRUNCATED NEWTON ENVIRONMENT.
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE SPARSE MATRIX
*  II  M  NUMBER OF ROWS OF THE SPARSE MATRIX.
*  II  MCOLS DECLARED NUMBER OF VARIABLES.
*  II  MROWS DECLARED NUMBER OF ROWS.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NZA SIZE OF THE INDEX ARRAY RNR.
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  II  NUP  LENGTH OF THE ARRAYS LI,LJ,CM.
*  II  NEL  NUMBER OF TRIPLES IN ARRAYS (LI,LJ,CM).
*  II  MMAX SIZE OF THE VECTORS SH,H.
*  II  PTL(MROWS) POINTER VECTOR OF THE COLUMNS OF THE L-FACTOR.
*  II  LENL(MROWS) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  II  CNR(NZ) CONTAINS THE INDICES OF THE FACTORS.
*  II  PTR(MROWS) POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  II  LENR(MROWS) LENGTHS OF THE ROWS OF THE FACTOR U.
*  II  PH(MROWS+1) POINTER VECTOR OF SH,H.
*  II  SH(MMAX) INDEX VECTOR OF THE HESSIAN
*  RI  H(MMAX) NUMERICAL VALUES OF THE HESSIAN MATRIX.
*  II  RNR(NZA) COLUMNS OF THE ORIGINAL MATRIX A.
*  II  PRNR(MCOLS+1) INDEX VECTOR FOR THE ARRAY RNR.
*  II  ORDC(MROWS) COLUMN PERMUTATION.
*  II  ORDR(MROWS) ROW PERMUTATION.
*  II  ORDL(MROWS) ROW PERMUTATION OF THE ORIGINAL FACTOR L.
*  II  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  II  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RI  CM(NUP) UPDATE MULTIPLIERS.
*  RI  AU(NZ) NUMERICAL VALUES OF THE FACTORS.
*  RI  A(NZA) NUMERICAL VALUES OF THE MATRIX.
*  RA  WR01(MCOLS)  AUXILIARY ARRAY
*  II  BCOL(MCOLS) PERMUTATION ARRAY: FIRST M COMPONENTS FORM BASIS,
*              NEXT NS-M FORM SUPERBASIS AND REST CORRESPOND TO
*              THE NON-BASIC VARIABLES.
*  II  INBCOL(MCOLS) VECTOR WITH COMPONENTS :
*                INBCOL(K)=I WHEN EXISTS K=BCOL(I) AND I<=NS,
*                INBCOL(K)=0 OTHERWISE.
*  RI  LAM(MCOLS) VECTOR OF REDUCED COST/GRADIENT.
*  RO  PCG(LINP-M) DIRECTION VECTOR
*  RA  ECG(LINP-M) AUXILIARY VECTOR
*  RA  FCG(LINP-M) AUXILIARY VECTOR
*  RO  RCG(LINP-M) RESIDUUM VECTOR
*  RO  PV(LINP-M) PRECONDITIONING VECTOR
*  II  IPFL  TYPE OF PRECONDITIONING STRATEGY : =1 FOR NO
*            PRECONDITIONING, <>1 OTHERWISE.
*  RO  EPS6  TOLERANCE.
*  RO  EPS8  TOLERANCE.
*  RI  GNORM  EUCLIDEAN NORM OF THE GRADIENT
*  RI  T PARAMETER INFLUENCING ACCURACY OF CG ITERATIONS.
*  RO  TL  NORM OF THE CG-DIRECTION.
*  II  LINP NUMBER OF SUPERBASICS AND BASICS.
*  IU  NCGR  NUMBER OF CG-ITERATIONS COMPUTED.
*  IO  ITERD  EXIT INFORMATION : =0 FOR NORMAL EXIT , =1 WHEN NON
*             POSITIVE DEFINITENESS IS FACED , =2 WHEN MAXIMUM
*             NUMBER OF ITERATIONS MIT IS REACHED.
*
      SUBROUTINE UKQGL1(N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,MMAX,PTL,
     & LENL,CNR,PTR,LENR,PH,SH,H,RNR,PRNR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,
     & A,WR01,BCOL,INBCOL,LAM,PCG,ECG,FCG,RCG,PV,IPFL,EPS6,EPS8,GNORM,
     & T,TL,LINP,NCGR,ITERD)
      INTEGER N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,MMAX,PTL(MROWS),
     & LENL(MROWS),CNR(NZ),PTR(MROWS),LENR(MROWS),PH(MCOLS+1),SH(MMAX),
     & RNR(NZA),PRNR(MCOLS+1),ORDC(MROWS),ORDR(MROWS),ORDL(MROWS),
     & LI(NUP),LJ(NUP),BCOL(MCOLS),INBCOL(MCOLS),IPFL,LINP,
     & NCGR,ITERD
      REAL*8 H(MMAX),CM(NUP),AU(NAU),A(NZA),WR01(MCOLS),LAM(MCOLS),
     & PCG(MCOLS),ECG(MCOLS),FCG(MCOLS),RCG(MCOLS),PV(MCOLS),EPS6,EPS8,
     & GNORM,T,TL
      REAL*8 RNORM,ETA,EPS,EPS05
      REAL*8 ALFA,BETA,GAMA,DELTA,FI,PSI,UXVDOT
      INTEGER I,J,MITR
      RNORM=0.0D0
      EPS6=1.0D-6
      EPS=1.0D-16
      EPS8=1.0D-8
      MITR=4+LINP-M
C
C     INITIALIZE PRECONDITIONING VECTOR
C
      DO 50 I=1,LINP-M
      PV(I)=1.0D 0
50    CONTINUE
C
C     SET EPS05
C
      EPS05=EPS**0.5D 0
C
C     INITIALIZE P0,R0,E0,F0,DELTA0
C
      DO 100 I=1,LINP-M
      PCG(I)=0
100   CONTINUE
C
C     INTO R  ARE SHIFTED -LAM(M+1),...,-LAM(LINP).
C
      DO 150 I=1,LINP-M
      RCG(I)=-LAM(I+M)
150   CONTINUE
      I=0
      DO 200 J=1,LINP-M
      ECG(J)=RCG(J)/PV(J)
      FCG(J)=ECG(J)
200   CONTINUE
C
C     SET INITIAL ETA
C
      ETA=MAX(EPS05,MIN(T,GNORM))
C
C     PRECONDITIONING MATRIX
C
      CALL UKQGLS(M,MCOLS,PV,RCG,FCG,ECG,IPFL,LINP)
      DELTA=UXVDOT(LINP-M,RCG,ECG)
250   CONTINUE
C
C     SET Q,FI,GAMA. Q AND E USE THE SAME VECTOR.
C     Q=TRANS(Z)*H*Z*F. FI=TRANS(E)*R
C
      FI=UXVDOT(LINP-M,ECG,RCG)
      CALL UKQGLH(N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,MMAX,PTL,LENL,CNR,
     & PTR,LENR,RNR,PRNR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,A,ECG,WR01,BCOL,
     & INBCOL,FCG,PH,SH,H,LINP)
      GAMA=UXVDOT(LINP-M,ECG,FCG)
C
C     TEST ON POSITIVE DEFITENESS
C
      IF(GAMA.LE.EPS6*DELTA) THEN
C
C     ITERD=1
C
                             ITERD=1
                             NCGR=NCGR+I
          IF(I.EQ.0) THEN
                     DO 350 J=1,LINP-M
                     PCG(J)=RCG(J)
350                  CONTINUE
          END IF
          TL=RNORM
          RETURN
      END IF
C
C     ALFA -COEFFICIENT FOR THE INCREMENTATION OF DIRECTION VECTOR
C     AND DECREMENTATION OF THE RESIDUUM (GRADIENT) VECTOR :
C     ALFA=FI/GAMA
C
      ALFA=FI/GAMA
C
C     UPDATE P,R
C
      DO 370 J=1,LINP-M
      PCG(J)=PCG(J)+ALFA*FCG(J)
      RCG(J)=RCG(J)-ALFA*ECG(J)
370   CONTINUE
C
C     COMPUTE EUCLIDEAN NORM OF RESIDUUM (GRADIENT) R
C
      RNORM=DSQRT(UXVDOT(LINP-M,RCG,RCG))
C
C     TEST ON THE SIZE OF THE RNORM
C
      IF(RNORM/GNORM.LT.ETA.OR.RNORM.LE.EPS8) THEN
C
C     ITERD=0 - NORMAL EXIT
C
                             ITERD=0
                             NCGR=NCGR+I
                             TL=RNORM
                             RETURN
      END IF
C
C     NEW PRECONDITIONING MATRIX IS COMPUTED
C
      CALL UKQGLS(M,MCOLS,PV,RCG,FCG,ECG,IPFL,LINP)
C
C     UPDATE OF E
C
      DO 400 J=1,LINP-M
      ECG(J)=RCG(J)/PV(J)
400   CONTINUE
C
C     COMPUTE NEW BETA AND USING IT UPDATE F AND DELTA
C
      PSI=UXVDOT(LINP-M,ECG,RCG)
      BETA=UXVDOT(LINP-M,ECG,RCG)/FI
      DO 500 J=1,LINP-M
      FCG(J)=ECG(J)+BETA*FCG(J)
500   CONTINUE
      DELTA=PSI+BETA*BETA*DELTA
C
C     TEST ON THE END OF THE CYCLE
C
      IF(I.LT.MITR) THEN
        I=I+1
        GO TO 250
      END IF
C
C     ITERD=2 - MANY ITERATIONS
C
      ITERD=2
      NCGR=NCGR+I
      TL=RNORM
      RETURN
      END
