* SUBROUTINE UKQGLX                 ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* RMIN,RMAX,DMAX ARE SET UP.
*
* PARAMETERS :
*  II  MCOLS DECLARED NUMBER OF VARIABLES.
*  II  M  NUMBER OF ROWS OF THE MATRIX.
*  II  TYPE(MCOLS) TYPES OF THE VARIABLES:
*                                    =1 FOR NONBASIC LOWER BOUNDED
*                                    =2 FOR NONBASIC UPPER BOUNDED
*                                    =3 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS LOWER BOUND
*                                    =4 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS UPPER BOUND
*                                    =5 FOR NONBASIC FIXED VARIABLE
*                                    =6 FOR NONBASIC FREE VARIABLE
*                                    =-1 FOR BASIC LOWER BOUNDED
*                                    =-2 FOR BASIC UPPER BOUNDED
*                                    =-3 FOR BASIC LOWER AND UPPER
*                     BOUNDED VARIABLE
*                                    =-6 FOR BASIC FREE VARIABLE.
*  II  X(MCOLS) VECTOR OF VARIABLES.
*  RI  LB(MCOLS) LOWER BOUNDS FOR SOLUTION.
*  RI  UB(MCOLS) UPPER BOUNDS FOR SOLUTION.
*  RI  S(MCOLS) DIRECTION VECTOR.
*  II  BCOL(MCOLS) NUMBERS OF THE BASIC ,SUPERBASIC AND NONBASIC COLUMNS.
*  IO  NSB NUMBER OF SUPERBASICS AND BASICS SHIFTED AT THE BOUND,
*          WHEN STEP XMAX IS TAKEN.
*  IO  NBB NUMBER OF BASICS SHIFTED AT THE BOUND, WHEN STEP
*          OF THE SIZE XMAX IS TAKEN.
*  IA  WN01(MCOLS) AUXILIARY VECTOR
*  II  LINP NUMBER OF SUPERBASICS.
*  RI  ALF1  LOWER BOUND FOR STEPSIZE PARAMETER.
*  RI  ALF2  UPPER BOUND FOR STEPSIZE PARAMETER.
*  RI  EPS9 ZERO TOLERANCE.
*  RO  DMAX  MAXIMUM SIZE OF THE STEP THAT CAN BE ACCEPTED.
*  RI  XMAX  MAXIMUM STEPSIZE.
*  RO  RMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RO  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  GNORM  EUCLIDEAN NORM OF THE GRADIENT.
*  RI  XNORM  MAXIMUM NORM OF THE DIRECTION VECTOR.
*
      SUBROUTINE UKQGLX(MCOLS,M,TYPE,X,LB,UB,S,BCOL,NSB,NBB,WN01,ALF1,
     & ALF2,EPS9,RMIN,RMAX,DMAX,XMAX,GNORM,XNORM,LINP)
      INTEGER MCOLS,M,NSB,NBB,TYPE(MCOLS),WN01(MCOLS),BCOL(MCOLS),LINP
      REAL*8 X(MCOLS),LB(MCOLS),UB(MCOLS),S(MCOLS),ALF1,ALF2,EPS9,RMIN,
     & RMAX,DMAX,XMAX,GNORM,XNORM
      INTEGER TYPV,NMX
      REAL*8 TEMP,INFF,EPS13
      INTEGER I,J
      PARAMETER (INFF=1.0D 70,EPS13=1.0D-13)
C
C ALF1,ALF2 ARE SET UP FOR QP
C
      ALF1=1.0D-10
      ALF2=1.0D 10
C
C     INITIALIZE THE MAXIMUM STEP BY THE INFINITY
C
      DMAX=INFF
C
C     INITIALIZE NBB,NSB,PMAX
C
      NBB=0
      NSB=0
      NMX=0
      DO 100 I=1,M
      J=BCOL(I)
      TEMP=ABS(S(J))
      IF(TEMP.LE.EPS13) THEN
        GO TO 100
      ELSE
        TYPV=TYPE(J)
        IF(S(J).LT.0.AND.(TYPV.EQ.-1.OR.TYPV.EQ.-3)) THEN
          TEMP=(LB(J)-X(J))/S(J)
        ELSEIF(S(J).GT.0.AND.(TYPV.EQ.-2.OR.TYPV.EQ.-3)) THEN
          TEMP=(UB(J)-X(J))/S(J)
        ELSE
          GO TO 100
        END IF
        IF((DMAX-TEMP).GT.EPS9) THEN
          DMAX=TEMP
          NMX=1
          WN01(NMX)=I
        ELSEIF(ABS(DMAX-TEMP).LE.EPS9) THEN
          NMX=NMX+1
          WN01(NMX)=I
        END IF
      END IF
100   CONTINUE
C
C     BASIC VARIABLES AT THE BOUND ARE IN WN01(1,..,NBB)
C
      NBB=NMX
C
C     CYCLE OF SUPERBASICS
C
C
      DO 200 I=M+1,LINP
      J=BCOL(I)
      TEMP=ABS(S(J))
      IF(TEMP.LE.EPS13) THEN
        GO TO 200
      ELSE
        TYPV=TYPE(J)
        IF(S(J).LT.0.AND.(TYPV.NE.2.AND.TYPV.NE.6)) THEN
          TEMP=(LB(J)-X(J))/S(J)
        ELSEIF(S(J).GT.0.AND.(TYPV.NE.1.AND.TYPV.NE.6)) THEN
          TEMP=(UB(J)-X(J))/S(J)
        ELSE
          GO TO 200
        END IF
        IF((DMAX-TEMP).GT.EPS9) THEN
          DMAX=TEMP
          NMX=1
          NBB=0
          WN01(NMX)=I
        ELSEIF(ABS(DMAX-TEMP).LE.EPS9) THEN
          NMX=NMX+1
          WN01(NMX)=I
        END IF
      END IF
200   CONTINUE
C
C     SUPERBASICS AT THE BOUND ARE IN WN01(NBB+1,...,LINP)
C
      NSB=NMX
C
C     SET DMAX,RMIN,RMAX
C
      IF(.FALSE.) THEN
C GNORM.GT.EPS9
        DMAX=MIN(XMAX,DMAX)
        RMIN=ALF1*GNORM/XNORM
        RMAX=MIN(ALF2*GNORM/XNORM,DMAX)
      ELSE
        RMIN=MIN(1.0D-16,DMAX)
        RMAX=MIN(INFF,DMAX)
      END IF
      RETURN
      END
