* SUBROUTINE ULQDF1             ALL SYSTEMS                   06/12/01
C PORTABILITY : ALL SYSTEMS
C 06/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DUAL RANGE SPACE QUADRATIC PROGRAMMING METHOD FOR MINIMAX
* APPROXIMATION WITH LINEAR CONSTRAINTS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  IO  N  DIMENSION OF THE MANIFOLD DEFINED BY ACTIVE CONSTRAINTS.
*  II  NA  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  NC  NUMBER OF LINEAR CONSTRAINTS.
*  RU  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  II  IXA(NF)  VECTOR CONTAINING INFORMATION ON TRUST REGION ACTIVITY.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  AF(NA)  VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RO  AFD(NA)  VECTOR CONTAINING INCREMENTS OF THE APPROXIMATED
*         FUNCTIONS.
*  II  IA(2*NA)  VECTOR CONTAINING TYPES OF DEVIATIONS.
*  II  IAA(NF+1)  VECTOR CONTAINING INDICES OF ACTIVE FUNCTIONS.
*  RI  AG(NF*NA)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         APPROXIMATED FUNCTIONS.
*  RI  AR((NF+1)*(NF+2)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RO  AZ(NF+1)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RI  CF(NF)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RO  G(NF+1)  GRADIENT OF THE LAGRANGIAN FUNCTION.
*  RU  H(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OR INVERSION OF THE
*         HESSIAN MATRIX APPROXIMATION.
*  RI  S(NF+1)  AUXILIARY VECTOR.
*  RI  ETA0  MACHINE PRECISION.
*  RI  ETA2  TOLERANCE FOR POSITIVE DEFINITENESS OF THE HESSIAN MATRIX.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RI  EPS9  TOLERANCE FOR ACTIVITY OF CONSTRAINTS.
*  RU  XDEL  TRUST REGION BOUND.
*  RO  XNORM  VALUE OF LINEARIZED MINIMAX FUNCTION.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  II  MFP  TYPE OF FEASIBLE POINT. MFP=1-ARBITRARY FEASIBLE POINT.
*         MFP=2-OPTIMUM FEASIBLE POINT. MFP=3-REPEATED SOLUTION.
*  II  JOB  SPECIFICATION OF THE PROBLEM. JOB=0-QUADRATIC PROGRAMMING
*         PROBLEM. JOB=1-QUADRATIC PROGRAMMING SUBPROBLEM FOR DIRECTION
*         DETERMINATION.
*
* COMMON DATA :
*  II  LCS  VARIABLE WHICH SPECIFIES TRUST REGION BOUNDS.
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  IU  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*         IDECF=1-GILL-MURRAY DECOMPOSITION. IDECF=9-INVERSION.
*         IDECF=10-DIAGONAL MATRIX.
*  IU  NDECF  NUMBER OF DECOMPOSITIONS.
*  IO  ITERQ  TYPE OF FEASIBLE POINT. ITERQ=1-ARBITRARY FEASIBLE POINT.
*         ITERQ=2-OPTIMUM FEASIBLE POINT. ITERQ=-1 FEASIBLE POINT DOES
*         NOT EXISTS. ITERQ=-2 OPTIMUM FEASIBLE POINT DOES NOT EXISTS.
*  II  IEXT  TYPE OF EXTREMUM. IEXT<0-MINIMAX. IEXT=0-CHEBYSHEV
*         APPROXIMATION, IEXT>0-MAXIMIN.
*
* SUBPROGRAMS USED :
*  S   ULMINA  DETERMINATION OF THE NEW ACTIVE FUNCTION.
*  S   ULMINL  DETERMINATION OF THE NEW ACTIVE LINEAR CONSTRAINT.
*  S   ULMINS  DETERMINATION OF THE NEW ACTIVE SIMPLE BOUND.
*  S   ULMINT  DETERMINATION OF THE NEW ACTIVE TRUST REGION BOUND.
*  S   ULDLAG  DETERMINATION OF A VECTOR FOR THE DUAL QP METHOD.
*  S   ULADF1  CONSTRAINT ADDITION.
*  S   ULRMF0  CONSTRAINT DELETION.
*  S   ULSOB1  TRANSFORMATION OF THE LOCAL SOLUTION TO THE SOLUTION
*         OF THE ORIGINAL QP PROBLEM.
*  S   UXDPGF  GILL-MURRAY DECOMPOSITION OF A DENSE SYMMETRIC MATRIX.
*  S   UXDPGB  BACK SUBSTITUTION AFTER GILL-MURRAY DECOMPOSITION.
*  S   UXDPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXDSMM  MATRIX VECTOR PRODUCT.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVINA  ABSOLUTE VALUES OF ELEMENTS OF AN INTEGER VECTOR.
*  S   UXVINC  UPDATE OF AN INTEGER VECTOR.
*  S   UXVINV  CHANGE OF AN INTEGER VECTOR AFTER CONSTRAINT ADDITION.
*  S   UXVMUL  MULTIPLICATION OF A VECTOR BY A DIAGONAL MATRIX.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UOLQP1  PRINT OF ENTRY TO QUADRATIC PROGRAMMING.
*  S   UOLQP2  PRINT OF EXIT FROM QUADRATIC PROGRAMMING.
*  S   UOLQD5  PRINT OF INFORMATION DURING QUADRATIC PROGRAMMING.
*  S   UOERR1  ERROR MESSAGES.
*
      SUBROUTINE ULQDF1(NF,N,NA,NC,X,IX,IXA,XN,XL,XU,AF,AFD,IA,IAA,AG,
     & AR,AZ,CF,IC,CL,CU,CG,G,H,S,ETA0,ETA2,ETA9,EPS7,EPS9,XDEL,XNORM,
     & UMAX,GMAX,MFP,JOB)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,N,NA,NC,IX(NF),IXA(NF),IA(2*NA),IAA(NF+1),IC(NC),MFP,
     & JOB
      REAL*8 X(NF),XN(NF),XL(NF),XU(NF),AF(NA),AFD(NA),AG(NF*NA),
     & AR((NF+1)*(NF+2)/2),AZ(NF+1),CF(NC),CL(NC),CU(NC),CG(NF*NC),
     & G(NF+1),H(*),S(NF+1),ETA0,ETA2,ETA9,EPS7,EPS9,XDEL,XNORM,UMAX,
     & GMAX
      REAL*8 CON,BET,GAM,TEMP,STEP,STEP1,STEP2,E,T,DMAX,PAR,SNORM,Q,QO
      REAL*8 UXVDOT,UXVMAX
      INTEGER NAD,NAA,NAR,I,J,K,L,IOLD,JOLD,KOLD,INEW,JNEW,KNEW,INF,
     & MODE,IER,KREM,KC
      INTEGER KCYC,MCYC
      CALL UOLQP1('ULQDF1')
      T=ONE
      CON=ETA9
      IF (IEXT.EQ.0) THEN
      NAD=2*NA
      ELSE
      NAD=NA
      ENDIF
      IF (IDECF.LT.0) IDECF=1
      IF (IDECF.EQ.0) THEN
C
C     GILL-MURRAY DECOMPOSITION
C
      TEMP=ETA2
      CALL UXDPGF(NF,H,INF,TEMP,STEP)
      NDECF=NDECF+1
      IDECF=1
      ENDIF
      IF (IDECF.GE.2.AND.IDECF.LE.8) THEN
      ITERM=-1
      TDXX='BAD DEC.'
      CALL UOERR1('ULQDF1',1)
      GO TO 7
      ENDIF
C
C     INITIATION
C
      NRED=0
      MODE=0
      JOLD=0
      JNEW=0
      DMAX=ZERO
      KCYC=0
      MCYC=MAX(1000,100*(NA+NC))
      IF (MFP.EQ.3) GO TO 1
      N=NF
      NAA=0
      NAR=0
C      XNORM=-ETA9
      XNORM=-1.0D 20
      Q=TWO*XNORM
      CALL UXVINA(NAD,IA)
      IF (KBC.GT.0) CALL UXVINA(NC,IC)
      IF (KBF.GT.0) CALL UXVINA(NF,IX)
      IF (LCS.GT.0) CALL UXVINC(NF,IX,IXA)
C
C     DIRECTION DETERMINATION
C
    1 CALL UXVSET(NF,ZERO,S)
      KCYC = KCYC + 1
      DO 2 J=1,NAA
      L=IAA(J)
      IF (L.GT.NC) THEN
      L=L-NC
      IF (L.GT.NA) L=L-NA
      CALL UXVDIR(NF,AZ(J),AG((L-1)*NF+1),S,S)
      ELSE IF (L.GT.0) THEN
      CALL UXVDIR(NF,AZ(J),CG((L-1)*NF+1),S,S)
      ELSE
      L=-L
      S(L)=S(L)+AZ(J)
      ENDIF
    2 CONTINUE
      CALL UXVCOP(NF,S,G)
      IF (NAA.GT.0) THEN
      IF (IDECF.EQ.1) THEN
      CALL UXDPGB(NF,H,S,0)
      ELSE IF (IDECF.EQ.9) THEN
      CALL UXDSMM(NF,H,G,S)
      ELSE
      CALL UXVMUL(NF,H,S,S,-1)
      ENDIF
      ENDIF
C
C     INITIAL MINIMAX VARIABLE
C
      IF (NAA.EQ.1) THEN
      IF (INEW-NC.LE.0) THEN
      ELSE IF (INEW-NC.LE.NA) THEN
      TEMP=AF(INEW-NC)+UXVDOT(NF,AG((INEW-NC-1)*NF+1),S)
      ELSE
      TEMP=AF(INEW-NC-NA)+UXVDOT(NF,AG((INEW-NC-NA-1)*NF+1),S)
      ENDIF
      XNORM=-SIGN(1,KNEW)*TEMP
      ENDIF
C
C     CHECK OF FEASIBILITY
C
      INEW=0
      PAR=ZERO
      CALL ULMINA(NF,NA,NAD,NC,AF,AFD,IA,AG,S,EPS9,XNORM,PAR,INEW,KNEW)
      IF (NAA.GT.0) THEN
      CALL ULMINL(NF,NC,CF,IC,CL,CU,CG,S,EPS9,PAR,KBC,INEW,KNEW)
      CALL ULMINS(NF,IX,XN,X,XL,XU,S,EPS9,PAR,KBF,NORMF,INEW,KNEW)
      IF (LCS.GT.0) CALL ULMINT(NF,IX,XN,X,S,XDEL,EPS9,PAR,NORMF,INEW,
     & KNEW)
      ENDIF
      IF (INEW.EQ.0) THEN
C
C     SOLUTION ACHIEVED
C
      CALL UOLQD5(NF,N,NC,NA,S,IX,IC,IA,IAA,AR,AZ,PAR,DMAX,SNORM,XNORM,
     & JOLD,JNEW,MODE)
      IF (JOB.EQ.0) THEN
      CALL ULSOB1(NF,NC,X,X,CF,CG,S,KBC)
      ELSE
      CALL UXVNEG(NF,G,G)
      ENDIF
      ITERQ=2
      GO TO 7
      ELSE IF (KCYC.GE.MCYC) THEN
*
*     CYCLE OCCURRED
*
      IF (JOB.EQ.0) THEN
      CALL ULSOB1(NF,NC,X,X,CF,CG,S,KBC)
      ELSE
      CALL UXVNEG(NF,G,G)
      ENDIF
      ITERQ=4
      GO TO 7
      ELSE
*
*     IRREGULAR CASE
*
      QO=Q
      Q=HALF*UXVDOT(NF,G,S)+XNORM
      IF (Q.LE.QO) THEN
      IF (JOB.EQ.0) THEN
      CALL ULSOB1(NF,NC,X,X,CF,CG,S,KBC)
      ELSE
      CALL UXVNEG(NF,G,G)
      ENDIF
      ITERQ=3
      GO TO 7
      ENDIF
      SNORM=ZERO
      ENDIF
    4 IER=0
C
C     STEPSIZE DETERMINATION
C
      CALL UOLQD5(NF,N,NC,NA,S,IX,IC,IA,IAA,AR,AZ,PAR,DMAX,SNORM,XNORM,
     & JOLD,JNEW,MODE)
      CALL ULADF1(NF,N,NA,NC,NC,IA,IAA,AG,AR,CG,H,S,G,EPS7,GMAX,UMAX,
     & E,T,IDECF,INEW,KNEW,NADD,IER,1)
      CALL ULDLAG(NF,N,NA,NC,IA,IAA,S,KOLD)
      IF (KOLD.EQ.0) THEN
C
C     ZERO STEPSIZE
C
      STEP1=ZERO
      STEP=STEP1
      SNORM=SIGN(1,KNEW)
      XNORM=XNORM-PAR
      ELSE
C
C     PRIMAL STEPSIZE
C
      CALL UXDPRB(NAA,AR,S,1)
      BET=E-UXVDOT(NAA,S,G)
      GAM=BET/UXVDOT(NAA,S,S)
      UMAX=BET*GAM+UMAX
      IF (UMAX.LE.EPS7*GMAX) THEN
      STEP1=CON
      ELSE
      STEP1=-PAR/UMAX
      ENDIF
C
C     DUAL STEPSIZE
C
      CALL UXDPRB(NAA,AR,S,-1)
      CALL UXDPRB(NAA,AR,G,-1)
      CALL UXVDIR(NAA,GAM,S,G,G)
      IF (KNEW.LT.0) CALL UXVNEG(NAA,G,G)
      STEP=UXVMAX(NAA,G)
      IOLD=0
      STEP2=CON
      DO 5 J=1,NAA
      L=IAA(J)
      IF (L.GT.NC) THEN
      L=L-NC
      K=IA(L)
      ELSE IF (L.GT.0) THEN
      K=IC(L)
      ELSE
      L=-L
      K=IX(L)
      ENDIF
      IF (K.LE.-5) THEN
      ELSE IF ((K.EQ.-1.OR.K.EQ.-3.).AND.G(J).LE.ZERO) THEN
      ELSE IF ((K.EQ.-2.OR.K.EQ.-4.).AND.G(J).GE.ZERO) THEN
      ELSE IF (ABS(G(J)).LE.ETA0*STEP) THEN
      ELSE
      TEMP=AZ(J)/G(J)
      IF (STEP2.GT.TEMP) THEN
      IOLD=J
      STEP2=TEMP
      ENDIF
      ENDIF
    5 CONTINUE
C
C     FINAL STEPSIZE
C
      STEP=MIN(STEP1,STEP2)
      IF (STEP.GE.CON) THEN
C
C     FEASIBLE SOLUTION DOES NOT EXIST
C
      ITERQ=-1
      CALL UOERR1('ULQDF1',10-ITERQ)
      GO TO 7
      ENDIF
C
C     NEW LAGRANGE MULTIPLIERS
C
      DMAX=STEP
      CALL UXVDIR(NAA,-STEP,G,AZ,AZ)
      SNORM=SNORM+SIGN(1,KNEW)*STEP
      XNORM=XNORM+SIGN(1,KNEW)*STEP*GAM
      PAR=PAR-(STEP/STEP1)*PAR
      ENDIF
      IF (STEP.EQ.STEP1) THEN
      IF (N.LT.0) THEN
C
C     IMPOSSIBLE SITUATION
C
      ITERQ=-5
      CALL UOERR1('ULQDA1',10-ITERQ)
      GO TO 7
      ENDIF
C
C     CONSTRAINT ADDITION
C
      IF (IER.EQ.0) THEN
      N=N-1
      NAA=NAA+1
      NAR=NAR+NAA
      AZ(NAA)=SNORM
      ENDIF
      IF (INEW.GT.NC) THEN
      IA(INEW-NC)=-IA(INEW-NC)
      ELSE IF (INEW.GT.0) THEN
      KC=INEW
      CALL UXVINV(NC,IC,KC,KNEW)
      ELSE IF (ABS(KNEW).EQ.1) THEN
      I=-INEW
      CALL UXVINV(NF,IX,I,KNEW)
      ELSE
      I=-INEW
      IXA(I)=IX(I)
      IF (KNEW.GT.0) IX(I)=-3
      IF (KNEW.LT.0) IX(I)=-4
      ENDIF
      NRED=NRED+1
      NADD=NADD+1
      JNEW=INEW
      JOLD=0
      GO TO 1
      ELSE
C
C     CONSTRAINT DELETION
C
      DO 6 J=IOLD,NAA-1
      AZ(J)=AZ(J+1)
    6 CONTINUE
      CALL ULRMF0(NF,N,NA,NC,NC,IX,IXA,IA,IAA,AR,IC,G,LCS,IOLD,KREM,
     & NREM,IER)
      NAR=NAR-NAA
      NAA=NAA-1
      JOLD=IOLD
      JNEW=0
      CALL UXVINA(NAD,IA)
      IF (KBC.GT.0) CALL UXVINA(NC,IC)
      IF (KBF.GT.0.OR.LCS.GT.0) CALL UXVINA(NF,IX)
      DO 8 J=1,NAA
      L=IAA(J)
      IF (L.GT.NC) THEN
      L=L-NC
      IA(L)=-IA(L)
      ELSE IF (L.GT.0) THEN
      IC(L)=-IC(L)
      ELSE
      L=-L
      IX(L)=-IX(L)
      ENDIF
    8 CONTINUE
      GO TO 4
      ENDIF
    7 CALL UOLQP2
      RETURN
      END
