* SUBROUTINE ULSLAG               ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NEGATIVE PROJECTED GRADIENT IS DETERMINED USING LAGRANGE MULTIPLIERS.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF CURRENT LINEAR APPROXIMATED FUNCTIONS.
*  II  NB  NUMBER OF DECLARED CONSTRAINTS OR APPROXIMATED FUNCTIONS.
*  II  NC  NUMBER OF LINEARIZED CONSTRAINTS.
*  II  IAA(NF+1)  VECTOR CONTAINING INDICES OF ACTIVE FUNCTIONS.
*  RI  AG(NF*NA)  MATRIX WHOSE COLUMNS ARE GRADIENTS OF THE LINEAR
*         APPROXIMATED FUNCTIONS.
*  RO  AZ(NF+1)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RO  S(NF)  NEGATIVE PROJECTED GRADIENT OF THE QUADRATIC FUNCTION.
*  RI  EPS7  TOLERANCE FOR LINEAR AND QUADRATIC PROGRAMMING.
*  RO  GMAX  NORM OF THE TRANSFORMED GRADIENT.
*
* SUBPROGRAMS USED :
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXVMAX  L-INFINITY NORM OF A VECTOR.
*
      SUBROUTINE ULSLAG(NF,N,NA,NB,NC,IAA,AG,AZ,CG,S,EPS7,GMAX)
      INTEGER NF,N,NA,NB,NC,IAA(*)
      REAL*8 AG(NF*NA),AZ(*),CG(NF*NB),S(NF),EPS7,GMAX
      REAL*8 UXVMAX
      INTEGER NAA,J,L
      NAA=NF-N
      DO 3 J=1,NAA
      L=IAA(J)
      IF (L.GT.NC) THEN
      L=L-NC
      IF (L.GT.NA) L=L-NA
      CALL UXVDIR(NF,AZ(J),AG((L-1)*NF+1),S,S)
      ELSE IF (L.GT.0) THEN
      CALL UXVDIR(NF,AZ(J),CG((L-1)*NF+1),S,S)
      ELSE
      L=-L
      S(L)=S(L)+AZ(J)
      ENDIF
    3 CONTINUE
      GMAX=UXVMAX(NF,S)
      IF (GMAX.LE.EPS7) GMAX=0.0D 0
      RETURN
      END
