* SUBROUTINE UMQPB2             ALL SYSTEMS                   00/12/01
C PORTABILITY : ALL SYSTEMS
C 00/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE INITIAL FEASIBLE POINT AND THE LINEAR PROGRAMMING
* SUBROUTINE.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CURRENT LINEAR CONSTRAINTS.
*  II  MC  NUMBER OF ELEMENTS IN THE FIELD CG.
*  II  MMAX  MAXIMUM NUMBER OF NONZERO ELEMENTS OF THE MATRIX CR.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  II  IXA(NF)  VECTOR CONTAINING INFORMATION ON TRUST REGION ACTIVITY.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  XO(NF)  SAVED VECTOR OF VARIABLES.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  CF(NF)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCYIONS.
*  RO  CFD(NF)  VECTOR CONTAINING INCREMENTS OF THE CONSTRAINT
*         FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CG(MC)  SPARSE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC) COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  RI  CR(MMAX)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  II  ICR(NF+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CR.
*  II  JCR(MMAX) COLUMN INDICES OF ELEMENTS IN THE FIELD CR.
*  RO  CZ(NF)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  GO(NF)  SAVED GRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  H(M)  SPARSE HESSIAN MATRIX OF THE LAGRANGIAN FUNCTION.
*  II  IH(NF+1) POINTERS OF DIAGONAL ELEMENTS OF THE SPARSE HESSIAN
*         MATRIX.
*  II  JH(M)  COLUMN INDICES OF NONZERO ELEMENTS OF THE SPARSE HESSIAN
*         MATRIX.
*  RI  S(NF)  DIRECTION VECTOR.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RI  EPS9  TOLERANCE FOR ACTIVITY OF CONSTRAINTS.
*  RU  XDEL  TRUST REGION BOUND.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  II  MFP  TYPE OF FEASIBLE POINT. MFP=1-ARBITRARY FEASIBLE POINT.
*         MFP=2-OPTIMUM FEASIBLE POINT. MFP=3-REPEATED SOLUTION.
*  II  JOB  SPECIFICATION OF THE PROBLEM. JOB=0-QUADRATIC PROGRAMMING
*         PROBLEM. JOB=1-QUADRATIC PROGRAMMING SUBPROBLEM FOR DIRECTION
*         DETERMINATION.
*
* COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=M2-SCALING FACTORS ARE SUPPLIED BY USER.
*  IO  ITERQ  TYPE OF FEASIBLE POINT. ITERQ=1-ARBITRARY FEASIBLE POINT.
*         ITERQ=2-OPTIMUM FEASIBLE POINT. ITERQ=-1 FEASIBLE POINT DOES
*         NOT EXISTS. ITERQ=-2 OPTIMUM FEASIBLE POINT DOES NOT EXISTS.
*
* SUBPROGRAMS USED :
*  S   ULINIT  DETERMINATION OF INITIAL POINT SATISFYING SIMPLE BOUNDS.
*  S   UMMAXL  MAXIMUM STEPSIZE USING LINEAR CONSTRAINTS.
*  S   ULMAXS  MAXIMUM STEPSIZE USING SIMPLE BOUNDS.
*  S   ULMAXT  MAXIMUM STEPSIZE USING TRUST REGION BOUNDS.
*  S   ULNEWL  IDENTIFICATION OF ACTIVE LINEAR CONSTRAINTS.
*  S   ULNEWS  IDENTIFICATION OF ACTIVE SIMPLE BOUNDS.
*  S   ULNEWT  IDENTIFICATION OF ACTIVE TRUST REGION BOUNDS.
*  S   ULDIRL  NEW VALUES OF CONSTRAINT FUNCTIONS.
*  S   ULDIRS  NEW VALUES OF VARIABLES.
*  S   UMSETC  INITIAL VALUES OF CONSTRAINT FUNCTIONS.
*  S   UMSETG  DETERMINATION OF THE FIRST PHASE GRADIENT VECTOR.
*  S   UMGLAG  GRADIENT OF THE LAGRANGIAN FUNCTION IS DETERMINED.
*  S   UMSLAG  NEGATIVE PROJECTED GRADIENT IS DETERMINED.
*  S   ULTLAG  THE OPTIMUM LAGRANGE MULTIPLIER IS DETERMINED.
*  S   UMVLAG  AN AUXILIARY VECTOR IS DETERMINED.
*  S   UMADR0  CONSTRAINT ADDITION.
*  S   UMRMB0  CONSTRAINT DELETION.
*  S   UXSPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVINA  ABSOLUTE VALUES OF ELEMENTS OF AN INTEGER VECTOR.
*  S   UXVINC  UPDATE OF AN INTEGER VECTOR.
*  S   UXVIND  CHANGE OF THE INTEGER VECTOR FOR CONSTRAINT ADDITION.
*  S   UXVINT  CHANGE OF THE INTEGER VECTOR FOR TRUST REGION BOUND
*         ADDITION.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UOLLP1  PRINT OF ENTRY TO QUADRATIC PROGRAMMING.
*  S   UOLLP2  PRINT OF EXIT FROM QUADRATIC PROGRAMMING.
*  S   UOMLQ4  PRINT OF INFORMATION DURING QUADRATIC PROGRAMMING.
*  S   UOERR1  ERROR MESSAGES.
*
      SUBROUTINE UMQPB2(NF,N,NC,MC,MMAX,X,IX,IXA,XN,XO,XL,XU,CF,CFD,
     & IC,ICA,CL,CU,CG,ICG,JCG,CR,ICR,JCR,CZ,G,GO,H,IH,JH,S,SO,ETA9,
     & EPS7,EPS9,XDEL,UMAX,GMAX,MFP,JOB)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,N,NC,MC,MMAX,IX(NF),IXA(NF),IC(NC),ICA(NF),ICG(NC+1),
     & JCG(MC),ICR(NF+1),JCR(MMAX),IH(NF+1),JH(MMAX),MFP,JOB
      REAL*8 X(NF),XN(NF),XO(NF),XL(NF),XU(NF),CF(NC),CFD(NC),CL(NC),
     & CU(NC),CG(MC),CR(MMAX),CZ(NF),G(NF),GO(NF),H(MMAX),S(NF),SO(NF),
     & ETA9,EPS7,EPS9,XDEL,UMAX,GMAX
      REAL*8 POM,CON,DMAX,ALF,RHO,SIG,UXVDOT
      INTEGER IPOM,I,IOLD,JOLD,INEW,JNEW,MODE,IER,KREM,KC,KCGR
      CALL UOLQP1('UMQPB2')
      CON=ETA9
      NCGR=0
      KCGR=0
C
C     INITIATION
C
      CALL UXVCOP(NF,X,XO)
      IF (JOB.EQ.0) THEN
      CALL UXVCOP(NF,G,GO)
      ELSE
      CALL UXVCOP(NF,GO,G)
      ENDIF
      IPOM=0
      JOLD=0
      JNEW=0
      NRED=0
      KREM=0
      ITERQ=1
      DMAX=ZERO
      IF (MFP.EQ.3) GO TO 5
      N=NF
      ITERQ=0
      IF (LCS.GT.0) CALL UXVINC(NF,IX,IXA)
      CALL UXVINS(NF+1,1,ICR)
      IF (KBF.GT.0) THEN
C
C     SHIFT OF VARIABLES FOR SATISFYING SIMPLE BOUNDS
C
      CALL UXVINA(NF,IX)
      CALL ULINIT(NF,X,IX,XN,XO,XL,XU,XDEL,EPS9,INEW,ITERQ)
      IF (ITERQ.LT.0) THEN
      CALL UOERR1('UMQPB2',10-ITERQ)
      GO TO 6
      ENDIF
      DO 1 I=1,NF
      IF (IX(I).LT.0) THEN
      N=N-1
      ICA(NF-N)=-I
      ENDIF
    1 CONTINUE
      ICR(NF-N+1)=NF-N+1
      CALL UXVINP(NF-N,ICR)
      CALL UXVINP(NF-N,JCR)
      CALL UXVSET(NF-N,ONE,CR)
      ENDIF
      IF (NC.GT.0) THEN
C
C     ADDITION OF ACTIVE CONSTRAINTS AND INITIAL CHECK OF FEASIBILITY
C
      CALL UXVINA(NC,IC)
      IF (NF.GT.N) CALL UMSETC(NF,NC,MC,X,XN,XO,CF,IC,CG,ICG,JCG,S,
     & ISNC,NORMF)
      DO 2 KC=1,NC
      IF (IC(KC).NE.0) THEN
      INEW=0
      CALL ULNEWL(NC,KC,CF,IC,CL,CU,EPS9,INEW)
      CALL UMADR0(NF,N,NC,MC,MMAX,ICA,CG,ICG,JCG,CR,ICR,JCR,S,EPS7,
     & GMAX,UMAX,INEW,IER)
      CALL UXVIND(NC,IC,KC,IER)
      IF (ITERM.NE.0) GO TO 6
      IF (IC(KC).LT.-10) IPOM=1
      ENDIF
    2 CONTINUE
      ENDIF
    3 IF (IPOM.EQ.1) THEN
C
C     CHECK OF FEASIBILITY AND UPDATE OF THE FIRST PHASE OBJECTIVE
C     FUNCTION
C
      CALL UMSETG(NF,NC,MC,IC,CG,ICG,JCG,G,INEW)
      IF (INEW.EQ.0) IPOM=0
      ENDIF
      IF (IPOM.EQ.0.AND.ITERQ.EQ.0) THEN
C
C     FEASIBILITY ACHIEVED
C
      ITERQ=1
      IF (MFP.EQ.1) GO TO 6
      CALL UXVDIF(NF,X,XO,S)
      IF (NORMF.GT.0) CALL UXVMUL(NF,XN,S,S,-1)
      CALL UXSSMD(NF,IH(NF+1)-1,H,IH,JH,S,ONE,GO,G)
      ENDIF
C
C     LAGRANGE MULTIPLIERS DETERMINATION
C
    5 IF (NF.GT.N) THEN
      CALL UMVLAG(NF,N,NC,NC,NC,MC,MC,ICA,CG,ICG,JCG,CG,ICG,JCG,G,CZ)
      CALL UXSPRB(NF-N,MMAX,CR,ICR,JCR,CZ,0)
      CALL ULTLAG(NF,N,NC,NC,NC,IX,IC,ICA,CZ,IC,EPS7,UMAX,IOLD)
      ELSE
      IOLD=0
      UMAX=ZERO
      ENDIF
C
C     PROJECTED GRADIENT DETERMINATION
C
      IF (N.GT.0) THEN
      CALL UXVNEG(NF,G,S)
      CALL UMSLAG(NF,N,NC,NC,NC,MC,MC,ICA,CG,ICG,JCG,CZ,CG,ICG,JCG,S,
     & EPS7,GMAX)
      ELSE
      GMAX=ZERO
      ENDIF
      MODE=1-IPOM
      CALL UOMQP4(NF,N,NC,MMAX,X,IX,IC,ICA,CF,CR,ICR,JCR,CZ,DMAX,GMAX,
     & UMAX,JOLD,JNEW,MODE)
      INEW=0
      IF (GMAX.EQ.ZERO) THEN
C
C     OPTIMUM ON A LINEAR MANIFOLD OBTAINED
C
      IF (IOLD.EQ.0) THEN
      IF (IPOM.EQ.0) THEN
C
C     OPTIMUM SOLUTION ACHIEVED
C
      CALL UXVCOP(NF,GO,G)
      IF (JOB.GT.0) THEN
      CALL UXVDIF(NF,X,XO,S)
      CALL UXVCOP(NF,XO,X)
      CALL UMGLAG(NF,N,NC,NC,NC,MC,MC,ICA,CG,ICG,JCG,CZ,CG,ICG,JCG,G)
      ENDIF
      ITERQ= 2
      GO TO 6
      ELSE
      IPOM=0
      DO 22 KC=1,NC
      IF (IC(KC).LT.-10) THEN
      INEW=0
      CALL ULNEWL(NC,KC,CF,IC,CL,CU,EPS9,INEW)
      IF (IC(KC).LT.-10) IPOM=1
      ENDIF
   22 CONTINUE
      IF (IPOM.EQ.0) THEN
C
C     OPTIMUM SOLUTION ACHIEVED
C
      CALL UXVCOP(NF,GO,G)
      IF (JOB.GT.0) THEN
      CALL UXVDIF(NF,X,XO,S)
      CALL UXVCOP(NF,XO,X)
      CALL UMGLAG(NF,N,NC,NC,NC,MC,MC,ICA,CG,ICG,JCG,CZ,CG,ICG,JCG,G)
      ENDIF
      ITERQ= 2
      GO TO 6
      ELSE
C
C     FEASIBLE SOLUTION DOES NOT EXIST
C
      CALL UXVCOP(NF,GO,G)
      ITERQ=-1
      CALL UOERR1('UMQPB2',10-ITERQ)
      GO TO 6
      ENDIF
      ENDIF
      ELSE
C
C     CONSTRAINT DELETION
C
      KCGR=NCGR
      CALL UMRMB0(NF,N,NC,MMAX,IX,IXA,IC,ICA,CR,ICR,JCR,CZ,S,IOLD,KREM)
      IF (ITERM.NE.0) GO TO 6
      DMAX=ZERO
      JOLD=IOLD
      JNEW=0
      GO TO 5
      ENDIF
      ELSE
C
C     DIRECTION DETERMINATION
C
      IF (IPOM.EQ.0) THEN
      RHO=UXVDOT(NF,S,S)
      IF (NCGR.GT.KCGR) THEN
      CALL UXVDIR(NF,RHO/SIG,SO,S,S)
      ENDIF
      CALL UXSSMM(NF,IH(NF+1)-1,H,IH,JH,S,CZ)
      ALF=UXVDOT(NF,S,CZ)
      IF (ALF.EQ.ZERO) THEN
      ITERQ=-5
      CALL UOERR1('UMQPB2',10-ITERQ)
      GO TO 6
      ELSE
      ALF=RHO/ALF
      ENDIF
      ENDIF
C
C     STEPSIZE SELECTION
C
      POM=CON
      CALL UMMAXL(NF,NC,MC,CF,CFD,IC,CL,CU,CG,ICG,JCG,S,POM,KBC,KREM,
     & INEW)
      IF (NORMF.GT.0) CALL UXVMUL(NF,XN,S,S,1)
      IF (LCS.GT.0) CALL ULMAXT(NF,X,IX,IXA,XN,XO,S,POM,XDEL,NORMF,
     & KREM,INEW)
      CALL ULMAXS(NF,X,IX,XL,XU,S,POM,KBF,KREM,INEW)
      IF (IPOM.EQ.0) THEN
      IF (POM.GT.ALF) THEN
C
C     OPTIMUM STEPSIZE
C
      INEW=0
      POM=ALF
      ENDIF
      ELSE
      IF (INEW.EQ.0) THEN
C
C     FEASIBLE SOLUTION DOES NOT EXIST
C
      ITERQ=-3
      CALL UOERR1('ULQPB2',10-ITERQ)
      GO TO 6
      ENDIF
      ENDIF
C
C     STEP REALIZATION
C
      CALL ULDIRS(NF,X,IX,S,POM,KBF)
      CALL ULDIRL(NC,CF,CFD,IC,POM,KBC)
      IF (IPOM.EQ.0) THEN
      CALL UXVDIR(NF,POM,CZ,G,G)
      NCGR=NCGR+1
      IF (INEW.EQ.0) THEN
      CALL UXVCOP(NF,S,SO)
      SIG=RHO
      ELSE
      KCGR=NCGR
      ENDIF
      ENDIF
C
C     CONSTRAINT ADDITION
C
      IF (INEW.EQ.0) THEN
      ELSE IF (INEW.GT.0) THEN
      KC=INEW
      INEW=0
      CALL ULNEWL(NC,KC,CF,IC,CL,CU,EPS9,INEW)
      CALL UMADR0(NF,N,NC,MC,MMAX,ICA,CG,ICG,JCG,CR,ICR,JCR,S,EPS7,
     & GMAX,UMAX,INEW,IER)
      CALL UXVIND(NC,IC,KC,IER)
      ELSE IF (INEW+NF.GE.0) THEN
      I=-INEW
      INEW=0
      CALL ULNEWS(NF,X,IX,XN,XL,XU,EPS9,NORMF,I,INEW)
      CALL UMADR0(NF,N,NC,MC,MMAX,ICA,CG,ICG,JCG,CR,ICR,JCR,S,EPS7,
     & GMAX,UMAX,INEW,IER)
      CALL UXVIND(NF,IX,I,IER)
      ELSE
      I=-(INEW+NF)
      INEW=0
      CALL ULNEWT(NF,X,IX,IXA,XN,XO,XDEL,EPS9,NORMF,I,INEW)
      CALL UMADR0(NF,N,NC,MC,MMAX,ICA,CG,ICG,JCG,CR,ICR,JCR,S,EPS7,
     & GMAX,UMAX,INEW,IER)
      CALL UXVIND(NF,IX,I,IER)
C      CALL UXVINT(NF,IX,IXA,I,IER)
      ENDIF
      IF (ITERM.NE.0) GO TO 6
      DMAX=POM
      NRED=NRED+1
      JNEW=INEW
      JOLD=0
      GO TO 3
      ENDIF
    6 IF (ITERM.LT.0) ITERQ=ITERM
      CALL UOLQP2
      RETURN
      END
