* SUBROUTINE UNSTP1                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  STEPSIZE SELECTION USING POLYHEDRAL APPROXIMATION
*  FOR VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  MA  DECLARED NUMBER OF LINEAR APPROXIMATED FUNCTIONS
*  II  MAL  CURRENT NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  RU  X(N)  VECTOR OF VARIABLES.
*  RI  S(N)  DIRECTION VECTOR.
*  RI  AF(4*MA)  VECTOR OF BUNDLE FUNCTIONS VALUES.
*  RI  AG(N*MA)  MATRIX WHOSE COLUMNS ARE BUNDLE SUBGRADIENTS.
*  RI  AY(N*MA)  MATRIX WHOSE COLUMNS ARE VARIABLE VECTORS.
*  RO  T  VALUE OF THE STEPSIZE PARAMETER.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  DF  DIRECTIONAL DERIVATIVE.
*  RI  TMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  ETA5  DISTANCE MEASURE PARAMETER.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  IO  LQ  NEWTON DIRECTION INDICATOR.
*
      SUBROUTINE UNSTP1(N,MA,MAL,X,S,AF,AG,AY,T,F,DF,TMAX,ETA5,ETA9,LQ)
      INTEGER N,MA,MAL,LQ
      REAL*8 X(N),S(N),AF(MA*4),AG(N*MA),AY(N*MA)
      REAL*8 T,F,DF,TMAX,ETA5,ETA9
      REAL*8 ALF,ALFL,ALFR,BET,BETL,BETR,DX,Q,R,W
      INTEGER I,J,K,L,JN
      W=DF*T*(1.0D 0-T*0.5D 0)
C
C     INITIAL CHOICE OF POSSIBLY ACTIVE LINES
C
      K=0
      L=-1
      BETR=-ETA9
      DO 3 J=1,MAL-1
      JN=J*N-N
      R=0.0D 0
      BET=0.0D 0
      ALFL=AF(J)-F
      DO 2 I=1,N
      DX=X(I)-AY(JN+I)
      Q=AG(JN+I)
      R=R+DX*DX
      ALFL=ALFL+DX*Q
   2  BET=BET+S(I)*Q
      ALF=MAX(ABS(ALFL),ETA5*R)
      R=1.0D 0-BET/DF
      IF (R*R+(ALF+ALF)/DF.GT.1.0D-6) THEN
        K=K+1
        AF(MA+K)=ALF
        AF(MA+MA+K)=BET
        R=T*BET-ALF
        IF(R.GT.W) THEN
          W=R
          L=K
        ENDIF
      ENDIF
      BETR=MAX(BETR,BET-ALF)
  3   CONTINUE
      LQ=-1
      IF (BETR.LE.DF*0.5D 0) RETURN
      LQ=1
      IF (L.LE.0) RETURN
      BETR=AF(MA+MA+L)
      IF (BETR.LE.0.0D 0) THEN
        IF (T.LT.1.0D 0.OR.BETR.EQ.0.0D 0) RETURN
        LQ=2
      ENDIF
      ALFR=AF(MA+L)
C
C     ITERATION LOOP
C
  4   IF (LQ.GE.1) THEN
        R=1.0D 0-BETR/DF
        R=MIN(TMAX,1.95D 0,R-SIGN(SQRT(R*R+(ALFR+ALFR)/DF),BETR))
      ELSE
        IF (ABS(BETR-BETL)+ABS(ALFR-ALFL).LT.-1.0D-4*DF) RETURN
        R=(ALFR-ALFL)/(BETR-BETL)
      ENDIF
      IF (ABS(T-R).LT.1.0D-4) RETURN
      T=R
      AF(MA+L)=-1.0D 0
      W=T*BETR-ALFR
      L=-1
      DO 5 J=1,K
      ALF=AF(MA+J)
      IF (ALF.LT.0.0D 0) GO TO 5
      BET=AF(MA+MA+J)
      R=T*BET-ALF
      IF (R.GT.W) THEN
        W=R
        L=J
      ENDIF
  5   CONTINUE
      IF (L.LT.0) RETURN
      BET=AF(MA+MA+L)
      IF (BET.EQ.0.0D 0) RETURN
C
C     NEW INTERVAL SELECTION
C
      ALF=AF(MA+L)
      IF (BET.LT.0.0D 0) THEN
        IF (LQ.EQ.2) THEN
          ALFR=ALF
          BETR=BET
        ELSE
          ALFL=ALF
          BETL=BET
          LQ=0
        ENDIF
      ELSE
        IF (LQ.EQ.2) THEN
          ALFL=ALFR
          BETL=BETR
          LQ=0
        ENDIF
        ALFR=ALF
        BETR=BET
      ENDIF
      GO TO 4
      END
