* SUBROUTINE UNSTP3                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  STEPSIZE SELECTION USING POLYHEDRAL APPROXIMATION
*  FOR NULL STEP IN CONVEX VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  MA  DECLARED NUMBER OF LINEAR APPROXIMATED FUNCTIONS
*  II  MAL  CURRENT NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  RU  X(N)  VECTOR OF VARIABLES.
*  RI  AF(4*MA)  VECTOR OF BUNDLE FUNCTIONS VALUES.
*  RI  AG(N*MA)  MATRIX WHOSE COLUMNS ARE BUNDLE SUBGRADIENTS.
*  RI  AY(N*MA)  MATRIX WHOSE COLUMNS ARE VARIABLE VECTORS.
*  RI  S(N)  DIRECTION VECTOR.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RU  DF  DIRECTIONAL DERIVATIVE.
*  RO  T  VALUE OF THE STEPSIZE PARAMETER.
*  RO  TB  BUNDLE PARAMETER FOR MATRIX SCALING.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*
      SUBROUTINE UNSTP3(N,MA,MAL,X,AF,AG,AY,S,F,DF,T,TB,ETA9)
      INTEGER N,MA,MAL
      REAL*8 AF(MA*3),AG(N*MA),AY(N*MA),S(N),X(N),F,DF,T,TB,ETA9
      REAL*8 ALF,ALFL,ALFR,BET,BETL,BETR,R,TP,W
      INTEGER I,J,JN,K,L
      REAL*8 ZERO,ONE,HALF
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0,HALF=0.5D 0)
      W=DF*T
      TP=T
C
C     INITIAL CHOICE OF POSSIBLY ACTIVE LINES
C
      K=0
      L=-1
      JN=0
      TB=SQRT(ETA9)
      DO 3 J=1,MAL-1
      BET=ZERO
      ALFL=AF(J)-F
      DO 2 I=1,N
      R=AG(JN+I)
      ALFL=ALFL+(X(I)-AY(JN+I))*R
    2 BET=BET+S(I)*R
      ALF=ABS(ALFL)
      R=T*BET-ALF
      IF(R.GT.DF*T)THEN
        K=K+1
        AF(MA+K)=ALF
        AF(MA+MA+K)=BET
        IF(R.GT.W)THEN
          W=R
          L=K
        ENDIF
      ENDIF
      IF(BET.GT.ZERO)TB=MIN(TB,ALF/(BET-DF))
    3 JN=JN+N
      IF(L.LT.0)RETURN
      BETR=AF(MA+MA+L)
      IF(BETR.LE.ZERO)RETURN
      ALFR=AF(MA+L)
      ALF=ALFR
      BET=BETR
      ALFL=ZERO
      BETL=DF
C
C     ITERATION LOOP
C
    4 IF(ABS(BETR-BETL)+ABS(ALFR-ALFL).LT.-1.0D-4*DF)RETURN
      R=T
      IF(BETR-BETL.NE.ZERO)T=MIN((ALFR-ALFL)/(BETR-BETL),TP)
      IF(ABS(T-R).LT.1.0D-3)RETURN
      AF(MA+L)=-ONE
      W=T*BET-ALF
      L=-1
      DO 5 J=1,K
      ALF=AF(MA+J)
      IF(ALF.LT.ZERO)GOTO5
      BET=AF(MA+MA+J)
      R=T*BET-ALF
      IF(R.GT.W) THEN
        W=R
        L=J
      ENDIF
    5 CONTINUE
      IF(L.LT.0)RETURN
C
C     NEW INTERVAL SELECTION
C
      BET=AF(MA+MA+L)
      ALF=AF(MA+L)
      IF(BET.LE.ZERO)THEN
        ALFL=ALF
        BETL=BET
      ELSE
        ALFR=ALF
        BETR=BET
      ENDIF
      GOTO 4
      END
