* SUBROUTINE UO2FU3                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* PRINT OF ITERATIONS IN UNCONSTRAINED MINIMIZATION.
*
* PARAMETERS:
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE MATRIX.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  NC  NUMBER OF CONSTRAINTS
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  H(M)  HESSIAN MATRIX OF THE OBJECTIVE FUNCTION.
*  RI  AF(NA)  VALUES OF THE APPROXIMATED FUNCTIONS.
*  RI  CF(NC)  VALUES OF THE CONSTRAINT FUNCTIONS.
*  RI  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RI  DMAX  MAXIMUM RELATIVE DIFFERENCE OF VARIABLES.
*  RO  GMAX  NORM OF THE TRANSFORMED GRADIENT.
*
* COMMON DATA :
*  II  KD  DEGREE OF REQUIRED DERIVATIVES.
*  II  ISNA  LEVEL OF PRINT. IF ISNA>0 VALUES OF THE APPROXIMATED
*         FUNCTIONS ARE PRINTED.
*  II  ISNC  LEVEL OF PRINT. IF ISNC>0 VALUES OF CONSTRAINTS ARE
*         PRINTED.
*  II  JOUT  SYSTEM VARIABLE WHICH SPECIFIES PRINT OF INFORMATION
*         DURING THE OPTIMIZATION PROCESS. JOUT=0-NO PRINT.
*         JOUT=1-PRINT IS REQUIRED.
*  II  KOUT  LEVEL OF PRINT. KOUT=0-NO PRINT. KOUT=1-PRINT OF INITIAL
*         VALUES AND FINAL RESULTS TOGETHER WITH PRINT OF ITERATIONS.
*         KOUT=2-ADDITIONAL PRINT OF STEPSIZE SELECTION.
*         KOUT=3-ADDITIONAL PRINT OF DIRECTION DETERMINATION.
*         KOUT=4-ADDITIONAL PRINT OF MATRIX CORRECTION.
*         KOUT=5-ADDITIONAL PRINT OF NUMERICAL DIFFERENTIATION.
*         KOUT=6-ADDITIONAL PRINT OF DEBUGGING.
*  II  LOUT  PRINT OF ITERATIONS. LOUT=0-STANDARD PAPER SAVING PRINT.
*         LOUT=1-ADDITIONAL PRINT OF VARIABLES. LOUT=2-ADDITIONAL
*         PRINT OF THE GRADIENT. LOUT=3-ADDITIONAL PRINT OF THE HESSIAN
*         MATRIX.
*  II  MOUT  LEVEL OF PRINT ON THE TERMINAL DEVICE.
*         MOUT=0 - NO PRINT.
*         MOUT=1 - PRINT OF FINAL RESULTS.
*         MOUT=2 - ADDITIONAL PRINT OF ITERATIONS.
*  II  LIT  NUMBER OF THE FIRST ITERATION.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  RI  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RI  DMAX  MAXIMUM RELATIVE DIFFERENCE OF VARIABLES.
*  RO  GMAX  NORM OF THE TRANSFORMED GRADIENT.
*
      SUBROUTINE UO2FU3(NF,M,NA,MA,NC,MC,X,G,H,AF,AG,CF,CG,F,DMAX,GMAX)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NDEM
      COMMON /UMCOMD/ NDEM
      INTEGER NF,M,NA,MA,NC,MC
      REAL*8 X(NF),G(NF),H(*),AF(*),AG(*),CF(*),CG(*),F,DMAX,GMAX
      INTEGER I
      LOGICAL LL
      NDEM=NF
      LL=KOUT.EQ.0.AND.LOUT.GT.0
      IF (ABS(MOUT).GT.1) THEN
      IF (KD.LE.0) THEN
      WRITE(ITR,15) NIT,NFV,F
      IF(LL) WRITE(IWR,15) NIT,NFV,F
      ELSE
      WRITE(ITR,16) NIT,NFV,NFG,F,GMAX
      IF(LL) WRITE(IWR,16) NIT,NFV,NFG,F,GMAX
      ENDIF
      ENDIF
      IF(JOUT.EQ.0) RETURN
      IF (ABS(KOUT).LE.1.AND.LOUT.EQ.0) THEN
      IF (KD.LE.0) THEN
      WRITE (IWR,11) NIT,NFV,F,DMAX,TXFU,TDXX,TSXX,TPXX
   11 FORMAT (' ','NIT=',I5,1X,'NFV=',I5,1X,'F=',
     & D 9.3,1X,'D=',D 9.3,1X,A8,1X,A8,1X,A8,1X,A8)
      ELSE
      WRITE (IWR,12) NIT,NFV,NFG,F,GMAX,DMAX,TXFU,TDXX,TSXX,
     & TPXX
   12 FORMAT (' ','NIT=',I5,1X,'NFV=',I5,1X,'NFG=',I5,1X,
     & 'F=',D 9.3,1X,'G=',D 9.3,1X,'D=',D 9.3,1X,A8,
     & 1X,A8,1X,A8,1X,A8)
      ENDIF
      ELSE
      IF (KD.LE.0) THEN
      WRITE (IWR,1) NIT,NFV,F,DMAX,TXFU,TDXX,TSXX,TPXX
    1 FORMAT (' ','NIT=',I5,1X,'NFV=',I5,1X,'F=',D 9.3,
     & 1X,'D=',D 9.3/1X,A8,2X,A8,2X,A8,2X,A8)
      ELSE
      WRITE (IWR,2) NIT,NFV,NFG,F,GMAX,DMAX,TXFU,TDXX,TSXX,TPXX
    2 FORMAT (' ','NIT=',I5,1X,'NFV=',I5,1X,'NFG=',I5,1X,
     & 'F=',D 9.3,1X,'G=',D 9.3,1X,'D=',D 9.3/1X,A8,2X,A8,2X,A8,2X,
     & A8)
      ENDIF
      ENDIF
      IF (ABS(LOUT).GE.1) THEN
      WRITE ( IWR, 3)  (X(I), I=1,MIN(NF,25))
    3 FORMAT ( ' ', 'X   =', 5D 14.5:/( 6X, 5D 14.5))
      IF (LOUT.LT.0.AND.ISNA.GT.0)
     *WRITE ( IWR, 4)  (AF(I), I=1,MIN(NA,25))
    4 FORMAT ( ' ', 'AF  =', 5D 14.5:/( 6X, 5D 14.5))
      IF (LOUT.LT.0.AND.ISNC.GT.0)
     *WRITE ( IWR, 5)  (CF(I), I=1,MIN(NC,25))
    5 FORMAT ( ' ', 'CF  =', 5D 14.5:/( 6X, 5D 14.5))
      ENDIF
      IF (ABS(LOUT).GE.2) THEN
      IF (KD.GT.0) WRITE(IWR,6) (G(I),I=1,MIN(NF,25))
    6 FORMAT ( ' ', 'G   =', 5D 14.5:/( 6X, 5D 14.5))
      ENDIF
      IF (ABS(LOUT).GE.3) THEN
      IF (ISNF.GT.0) WRITE(IWR,7) (H(I),I=1,MIN(M,50))
    7 FORMAT ( ' ', 'H   =', 5D 14.5:/( 6X, 5D 14.5))
      IF (LOUT.LT.0.AND.ISNA.GT.1)
     *WRITE ( IWR,24)  (AG(I), I=1,MIN(MA,50))
   24 FORMAT ( ' ', 'AG  =', 5D 14.5:/( 6X, 5D 14.5))
      IF (LOUT.LT.0.AND.ISNC.GT.1)
     *WRITE ( IWR,25)  (CG(I), I=1,MIN(MC,50))
   25 FORMAT ( ' ', 'CG  =', 5D 14.5:/( 6X, 5D 14.5))
      ENDIF
      IF(LOUT.NE.0.AND.ABS(KOUT).LE.1) WRITE(IWR,8)
      IF(ABS(KOUT).GT.1) WRITE(IWR,9)
    8 FORMAT(' ')
    9 FORMAT(' ',43('-'))
   15 FORMAT(5X,'NIT=',I5,1X,'NFV=',I5,1X,'F=',G17.10)
   16 FORMAT(5X,'NIT=',I5,1X,'NFV=',I5,1X,'NFG=',I5,2X,'F=',
     *       G17.10,1X,'G=',D9.3)
      RETURN
      END
