* SUBROUTINE UP1AF4             ALL SYSTEMS                 97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE OBJECTIVE FUNCTION.
* COMPUTATION OF THE VALUES AND THE GRADIENTS OF THE CONSTRAINT
* FUNCTIONS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  NAL  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD AG.
*  II  KA  INDEX OF THE APPROXIMATED FUNCTION.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE SPARSE JACOBIAN MATRIX.
*  IU  KC  INDEX OF THE CONSTRAINT.
*  RI  GA(NF)  GRADIENT OF THE APPROXIMATED FUNCTION.
*  RI  IAG(NA+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD AG.
*  RI  JAG(MA)  COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RI  GC(NF)  GRADIENT OF THE SELECTED FUNCTION.
*  II  ICG(NC+1)  POSITIONS OF THE FIRST ROWS ELEMENTS IN THE SPARSE
*         JACOBIAN STRUCTURE.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE SPARSE JACOBIAN
*         STRUCTURE.
*  RO  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  CZ(NC)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RI  RPF1  PENALTY COEFFICIENT.
*  RI  FF  VALUE OF THE MODEL FUNCTION.
*  RO  FA  VALUE OF THE APPROXIMATED FUNCTION.
*  RO  FC  VALUE OF THE CONSTRAINT FUNCTION.
*  RO  F  VALUE OF THE PENALTY FUNCTION.
*  RO  CMAX  MAXIMUM CONSTRAINT VIOLATION.
*
* COMMON DATA :
*  II  KD  DEGREE OF REQUIRED DERIVATIVES.
*  II  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  II  IEXT  TYPE OF EXTREMUM. IEXT=0-MINIMUM. IEXT=1-MAXIMUM.
*  IU  NAV  NUMBER OF APPROXIMATED FUNCTION VALUES COMPUTED.
*  IU  NAG  NUMBER OF APPROXIMATED FUNCTION GRADIENTS COMPUTED.
*  IU  NFV  NUMBER OF OBJECTIVE FUNCTION VALUES COMPUTED.
*  IU  NFG  NUMBER OF OBJECTIVE FUNCTION GRADIENTS COMPUTED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UXVSET  INITIATION OF A VECTOR.
*
      SUBROUTINE UP1AF4(NF,NA,NAL,MA,KA,NC,MC,KC,GA,IAG,JAG,GC,ICG,JCG,
     & G,CZ,RPF1,FF,FA,FC,F,CMAX)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NA,NAL,MA,KA,NC,MC,KC,IAG(NA+1),JAG(MA),ICG(NC+1),
     & JCG(MC)
      REAL*8 GA(NF),GC(NF),G(NF),CZ(NC),RPF1,FF,FA,FC,F,CMAX
      INTEGER J,K,L,JP
      IF (TSS(NS).NE.'UFXX') CALL UYPRO1('UFXX',-1)
      GO TO (1,4,6,14,16) ISP(NS)
    1 CONTINUE
      IF (KD.LE.LD) GO TO 21
      IF (KD.GE.0.AND.LD.LT.0) THEN
      CMAX=ZERO
      F=ZERO
      ENDIF
      IF (KD.GE.1.AND.LD.LT.1) CALL UXVSET(NF,ZERO,G)
      KA=1
    2 IF (KA.GT.NA) GO TO 11
      IF (LD.GE.0) GO TO 5
      CALL UYSET0(1,2)
      RETURN
    4 CONTINUE
      F=F+FA
    5 IF (KD.LT.1) GO TO 9
      IF (LD.GE.1) GO TO 9
      CALL UYSET0(2,3)
      RETURN
    6 CONTINUE
      K=IAG(KA)
      L=IAG(KA+1)-K
      DO 3 J=1,L
      JP=ABS(JAG(K))
      G(JP)=G(JP)+GA(JP)
      K=K+1
    3 CONTINUE
    9 KA=KA+1
      GO TO 2
   11 IF (NA.GT.NAL) THEN
      NFV=NAV/(NA-NAL)
      NFG=NAG/(NA-NAL)
      ELSE
      NFV=NFV+1
      NFG=NFG+1
      ENDIF
      IF (KD.GE.0.AND.LD.LT.0) FF=F
      IF (KD.GE.1.AND.LD.LT.1) CALL UXVCOP(NF,G,GA)
      KC=1
   12 IF (KC.GT.NC) GO TO 21
      IF (LD.GE.0) GO TO 15
      CALL UYSET0(3,4)
      RETURN
   14 CONTINUE
      F=F+(CZ(KC)+HALF*RPF1*FC)*FC
      CMAX=MAX(CMAX,ABS(FC))
   15 IF (KD.LT.1) GO TO 19
      IF (LD.GE.1) GO TO 19
      CALL UYSET0(4,5)
      RETURN
   16 CONTINUE
      K=ICG(KC)
      L=ICG(KC+1)-K
      DO 13 J=1,L
      JP=ABS(JCG(K))
      G(JP)=G(JP)+(CZ(KC)+RPF1*FC)*GC(JP)
      K=K+1
   13 CONTINUE
   19 KC=KC+1
      GO TO 12
   21 CALL UYEPI1(-1)
      RETURN
      END
