* SUBROUTINE UP1EF7             ALL SYSTEMS                 01/12/01
C PORTABILITY : ALL SYSTEMS
C 01/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE OBJECTIVE FUNCTION.
* COMPUTATION OF THE VALUES AND THE GRADIENTS OF THE CONSTRAINT
* FUNCTIONS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE SPARSE JACOBIAN MATRIX.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  II  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NX)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NX)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XZL(NX)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NX)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  GF(NF)  GRADIENT OF THE MODEL FUNCTION.
*  RI  CF(NC+1)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  RI  GC(NF)  GRADIENT OF THE SELECTED FUNCTION.
*  II  ICG(NC+1)  POSITIONS OF THE FIRST ROWS ELEMENTS IN THE SPARSE
*         JACOBIAN STRUCTURE.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE SPARSE JACOBIAN
*         STRUCTURE.
*  IO  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  S(NF+NCA)  VECTOR CONTAINING NEWTON INCREMENTS.
*  RO  XO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR LOWER INACTIVE
*         MULTIPLIERS.
*  RO  GO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR UPPER INACTIVE
*         MULTIPLIERS.
*  RO  G(NF)  GRADIENT OF THE LAGRANGIAN FUNCTION.
*  RI  R  STEPSIZE  PARAMETER.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  EPS7  SMOOTHING PARAMETER.
*
* COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  IEXT  TYPE OF EXTREMUM. IEXT=0-MINIMUM. IEXT=1-MAXIMUM.
*
* SUBPROGRAMS USED :
*  S   UXVCOP  COPYING OF A VECTOR.
*  RF  UXUDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXWDIR  VECTOR AUGMENTED BY THE SCALED VECTOR (PACKED CASE).
*
      SUBROUTINE UP1EF7(NF,NX,NC,MC,NCA,X,IX,XL,XU,XZL,XZU,GF,CF,IC,
     & CL,CU,CZL,CZU,CG,ICG,JCG,ICA,S,XO,GO,G,R,F,EPS7)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NX,NC,MC,NCA,IX(NF),IC(NC),ICG(NC+1),JCG(MC),ICA(NCA)
      REAL*8 X(NF),XL(NX),XU(NX),XZL(NX),XZU(NX),GF(NF),CF(NC+1),CL(NC),
     & CU(NC),CZL(NC),CZU(NC),CG(MC),S(NF+NCA),XO(NX+NC),GO(NX+NC),
     & G(NF),R,F,EPS7
      INTEGER I,K,KK,L
      REAL*8 TEMP,TEMP1,TEMP2,UXUDOT
      F=CF(NF+1)
      IF (R.GT.ZERO) THEN
      IF (IEXT.LE.0) THEN
      CALL UXVCOP(NF,GF,G)
      ELSE
      CALL UXVNEG(NF,GF,G)
      ENDIF
      ENDIF
      DO 1 I=1,NCA
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      KK=-IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP1=XL(K)-X(K)
      TEMP2=XZL(K)+R*S(NF+I)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      IF (R.GT.ZERO) G(K)=G(K)-TEMP2
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP1=X(K)-XU(K)
      TEMP2=XZU(K)+R*S(NF+I)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      IF (R.GT.ZERO) G(K)=G(K)+TEMP2
      ENDIF
      ELSE
      KK=-IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP1=CL(K)-CF(K)
      TEMP2=CZL(K)+R*S(NF+I)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      L=ICG(K)
      IF (R.GT.ZERO) CALL UXWDIR(ICG(K+1)-L,NF,JCG(L),-TEMP2,CG(L),G,G,
     & 1)
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP1=CF(K)-CU(K)
      TEMP2=CZU(K)+R*S(NF+I)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      L=ICG(K)
      IF (R.GT.ZERO) CALL UXWDIR(ICG(K+1)-L,NF,JCG(L), TEMP2,CG(L),G,G,
     & 1)
      ELSE IF (KK.EQ.5) THEN
      F=F+CF(K)**2
      L=ICG(K)
      IF (R.GT.ZERO) CALL UXWDIR(ICG(K+1)-L,NF,JCG(L), TEMP2,CG(L),G,G,
     & 1)
      ENDIF
      ENDIF
    1 CONTINUE
      IF (KBF.GT.0) THEN
      DO 2 K=1,NX
      KK=IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP1=XL(K)-X(K)
      TEMP2=XZL(K)+R*XO(K)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      IF (R.GT.ZERO) G(K)=G(K)-TEMP2
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP1=X(K)-XU(K)
      TEMP2=XZU(K)+R*GO(K)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      IF (R.GT.ZERO) G(K)=G(K)+TEMP2
      ENDIF
    2 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 3 K=1,NC
      KK=IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP1=CL(K)-CF(K)
      TEMP2=CZL(K)+R*XO(NX+K)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      L=ICG(K)
      IF (R.GT.ZERO) CALL UXWDIR(ICG(K+1)-L,NF,JCG(L),-TEMP2,CG(L),G,G,
     & 1)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP1=CF(K)-CU(K)
      TEMP2=CZU(K)+R*GO(NX+K)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      L=ICG(K)
      IF (R.GT.ZERO) CALL UXWDIR(ICG(K+1)-L,NF,JCG(L), TEMP2,CG(L),G,G,
     & 1)
      ENDIF
    3 CONTINUE
      ENDIF
      F=HALF*(F+UXUDOT(NF,G,G,IX,KBF))
      RETURN
      END
