* SUBROUTINE US1L07                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SPECIAL LINE SEARCH FOR NONSMOOTH CONVEX VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  MB  DECLARED NUMBER OF LINEAR APPROXIMATED FUNCTIONS
*  II  NB  CURRENT NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  RU  X(N)  VECTOR OF VARIABLES.
*  RO  G(N)  SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  S(N)  DIRECTION VECTOR.
*  RU  U(N)  PREVIOUS VECTOR OF VARIABLES.
*  RI  V(N)  PREVIOUS SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  AF(4*MB)  VECTOR OF BUNDLE FUNCTIONS VALUES.
*  RI  AG(N*MB)  MATRIX WHOSE COLUMNS ARE BUNDLE SUBGRADIENTS.
*  RI  AY(N*MB)  MATRIX WHOSE COLUMNS ARE VARIABLE VECTORS.
*  RO  T  VALUE OF THE STEPSIZE PARAMETER.
*  RO  TB  BUNDLE PARAMETER FOR MATRIX SCALING.
*  RO  FO  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RU  PO  PREVIOUS DIRECTIONAL DERIVATIVE.
*  RU  P  DIRECTIONAL DERIVATIVE.
*  RI  TMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  TMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  GNORM  NORM OF THE GRADIENT VECTOR.
*  RI  SNORM  EUCLIDEAN NORM OF THE DIRECTION VECTOR.
*  RI  D1  AUXILIARY VALUE FOR NULL/DESCENT STEP TEST.
*  RU  DF  AUXILIARY VALUE FOR TEST ON FUNCTION DECREASE.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  IO  JL  TERMINATION INDICATOR.
*  IO  NE  EXTRAPOLATION COUNTER.
*  IO  NK  NULL STEP INDICATOR. NK=0-DESCENT STEP. NK=2-NULL STEP.
*  IU  NV  AUXILIARY NUMBER OF FUNCTION EVALUATIONS.
*
* COMMON DATA :
*  RI  TOLF  LOWER BOUND FOR FUNCTION DECREASE.
*  IU  NTESF  ACTUAL NUMBER OF TESTS ON FUNCTION DECREASE.
*  II  MTESF  MAXIMUM NUMBER OF TESTS ON FUNCTION DECREASE.
*  IO  ITERS  NULL STEP INDICATOR. ITERS=0-NULL STEP. ITERS=1-DESCENT
*         STEP.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UXVCOP  COPYING OF A VECTOR.
*  RF  UXVMX2  L-INFINITY NORM OF VECTOR DIFFERENCE.
*  S   UNSTP2  STEPSIZE DETERMINATION FOR DESCENT STEPS.
*  S   UNSTP3  STEPSIZE DETERMINATION FOR NULL STEPS.
*
* METHOD :
* SPECIAL METHOD OF STEP LENGTH DETERMINATION.
*
      SUBROUTINE US1L07(N,MB,NB,X,G,S,U,V,AF,AG,AY,T,TB,FO,F,PO,P,
     & TMIN,TMAX,GNORM,SNORM,D1,DF,ETA9,JL,NE,NV)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,MB,NB,JL,NE,NV
      REAL*8 G(N),S(N),U(N),V(N),X(N),AF(MB*4),AG(N*MB),AY(N*MB),
     & T,TB,FO,F,PO,P,TMIN,TMAX,GNORM,SNORM,D1,DF,ETA9
      REAL*8 DT,W,UXVMX2
      SAVE DT
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',1)
      GOTO (1,8) ISP(NS)
    1 CONTINUE
      CALL UOS1L1(ZERO,FO,PO,GNORM,SNORM,TMAX)
      T=MIN(ONE,TMAX)
      IF (PO.EQ.ZERO) TSXX='P - ZERO'
      IF (PO.NE.ZERO) THEN
        IF (ITERS.EQ.1) THEN
          CALL UNSTP2(N,MB,NB,X,AF,AG,AY,S,F,PO,T,TB,ETA9)
          TSXX='INIT-DES'
        ELSE
          CALL UNSTP3(N,MB,NB,X,AF,AG,AY,S,F,PO,T,TB,ETA9)
          TSXX='INIT-ZER'
        ENDIF
      ENDIF
      T=MIN(MAX(T,TMIN),TMAX)
      DT=T
      NE=0
C
C     FUNCTION AND GRADIENT EVALUATION AT A NEW POINT
C
    7 CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    8 CONTINUE
      CALL UOS1L3(T,F,P,SNORM,NE)
      NV=NV+1
C
C     NULL/DESCENT STEP TEST (ITERS=0/1)
C
      ITERS=1
      TSXX='DESCENT '
      IF (F.GT.FO-T*D1) THEN
        ITERS=0
        TSXX='NULL  ST'
      ENDIF
      W=DF
      IF (ABS(FO-F).GE.DF*1.0D-5) W=ABS(FO-F)
      IF (ITERS.EQ.1) DF=W
      IF (W/MAX(ABS(F),ONE).LE.TOLF.OR.FO.EQ.F) THEN
        NTESF=NTESF+1
        JL=1
        IF(NTESF.GE.MTESF)RETURN
      ELSE
        NTESF=0
      ENDIF
      W=UXVMX2(N,G,V)
      T=DT
      DT=DT+T
      IF (W.EQ.ZERO.AND.ITERS.EQ.1.AND.NE.LE.9.AND.DT.LE.TMAX) THEN
C
C     EXTRAPOLATION
C
        TSXX='EXTRAPOL'
        NE=NE+1
        FO=F
        CALL UXVCOP(N,X,U)
        GO TO 7
      ENDIF
      JL=0
      IF (T.GE.TMAX) NMAX=NMAX+1
      CALL UOS0L2(T,F,SNORM)
      CALL UYEPI1(1)
      RETURN
      END
