* SUBROUTINE UUDBM2                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* VARIABLE METRIC UPDATE OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX.
*
* PARAMETERS :
*  II  NF DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  H(M)  POSITIVE SEMIDEFINITE MATRIX OF THE NORMAL EQUATION.
*  RU  HH(M) UPDATED APPROXIMATION OF THE SECOND ORDER TERM
*  RA  S(NF)  AUXILIARY VECTOR.
*  RU  XO(NF)  VECTORS OF VARIABLES DIFFERENCE.
*  RI  GO(NF)  GRADIENTS DIFFERENCE OR INTERMEDIATE GRADIENT.
*  RA  SO(NF)  AUXILIARY VECTOR.
*  RI  FO  OLD VALUE OF THE OBJECTIVE FUNCTION.
*  RI  ETA9  MAXIMUM MACHINE NUMBER.
*  RO  PAR  SCALING PARAMETER.
*  II  MET  METHOD SELECTION. MET=1-BFGS UPDATE. MET=2-DFP UPDATE.
*         MET=3-HOSHINO UPDATE.
*  II  MET1  SELECTION OF SELF SCALING.  MET1=1-SELF SCALING SUPPRESSED.
*         MET1=2-INITIAL SELF SCALING.
*  II  MET2  SELECTION OF THE LINE SEARCH MODEL. MET2=1-QUADRATIC MODEL.
*         MET2=2 USE OF TAYLOR EXPANSION.
*  II  MET3  METHOD CORRECTION. MET3=1-NO CORRECTION.
*         MET3=2-POWELL'S CORRECTION.
*  II  MET4  TYPE OF QN CONDITION. MET4=1-BASIC QN CONDITION. MET4=2 QN
*         CONDITION WITH INTERMEDIATE GRADIENT.
* COMMON DATA :
*  II  ITERD  TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*  II  ITRAN  TRANSFORMATION INDICATOR. ITRAN=0 OR ITRAN=1 IF
*         TRANSFORMATION IS NOT OR IS USED.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXDSMM  MATRIX-VECTOR PRODUCT.
*  S   UXDSMU  CORRECTION OF A DENSE SYMMETRIC MATRIX.
*  S   UXDSMS  SCALING OF A DENSE SYMMETRIC MATRIX.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVNEG  COPYING OF A VECTOR WITH THE CHANGE OF THE SIGN.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UOU1D1  PRINT OF ENTRY TO VARIABLE METRIC UPDATE.
*  S   UOU1D2  PRINT OF EXIT FROM VARIABLE METRIC UPDATE.
*
* METHOD :
* BASIC VARIABLE METRIC METHODS.
*
      SUBROUTINE UUDBM2(NF,N,H,HH,S,XO,GO,SO,FO,ETA9,PAR,MET,MET1,MET2,
     & MET3,MET4)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,N,MET,MET1,MET2,MET3,MET4
      REAL*8 H(NF*(NF+1)/2),HH(NF*(NF+1)/2),S(NF),XO(NF),GO(NF),SO(NF),
     & FO,ETA9,PAR
      REAL*8 DEN,DIS,A,B,C,GAM,RHO,POM,UXVDOT
      LOGICAL L1
      REAL*8 CON,CON3,CON6,CON7
      PARAMETER (CON=1.0D-8,CON3=0.2D 0,CON6=0.5D 0,CON7=2.5D 0)
      IF (MET.LE.0) RETURN
      CALL UOU1D1(N,XO,GO)
      IF(IDECF.NE.0) THEN
      ITERH=-1
      TUXX='BAD DEC0'
      CALL UOERR1('UUDBM2',1)
      GO TO 6
      ENDIF
      IF (MET.GT.4) MET=1
      L1=MET1.GE.3.OR.MET1.EQ.2.AND.NIT.EQ.KIT
C
C     DETERMINATION OF THE PARAMETERS B, C
C
      IF (MET4.EQ.1) THEN
      CALL UXDSMM(N,H,XO,S)
      CALL UXVDIF(N,GO,S,SO)
      ENDIF
      IF (MET3.EQ.2) CALL UXVSCL(N,ONE/SQRT(FO),SO,SO)
      B=UXVDOT(N,XO,SO)
      IF (B.LE.ZERO) L1=.FALSE.
      IF (MET.LT.4.AND.B.LE.ZERO) THEN
      ITERH=2
      TUXX='B - NEG.'
      GO TO 6
      ENDIF
      A=ZERO
      CALL UXDSMM(N,HH,XO,S)
      C=UXVDOT(N,XO,S)
      IF (C.LE.ZERO) L1=.FALSE.
      IF (MET.LT.4.AND.C.LT.ZERO) THEN
      ITERH=3
      TUXX='C - NEG.'
      GO TO 6
      ENDIF
      IF (L1) THEN
C
C     DETERMINATION OF THE PARAMETER GAM (SELF SCALING)
C
      IF (MET2.LE.1) THEN
      GAM=MAX(ONE,C/B)
      ELSE
C      GAM=ONE/MAX(ONE/ETA9,ABS(PAR))
      GAM=ONE/PAR
      ENDIF
      ELSE
      GAM=ONE
      ENDIF
      PAR=GAM
      GO TO (1,2,3,4)  MET
C
C     BFGS UPDATE
C
    1 POM=ZERO
      CALL UXDSMU(N,HH,PAR/B,SO)
      IF (C.GT.ZERO) CALL UXDSMU(N,HH,-ONE/C,S)
      TUXX='BFGS    '
      IF (MET.EQ.1) NUPD=NUPD+1
      GO TO 5
C
C     DFP UPDATE
C
    2 DEN=PAR*B+C
      DIS=PAR+C/B
      POM=ONE
      CALL UXVDIR(N,-DIS,SO,S,SO)
      CALL UXDSMU(N,HH,ONE/DEN,SO)
      IF (C.GT.ZERO) CALL UXDSMU(N,HH,-ONE/DEN,S)
      TUXX='DFP     '
      NUPD=NUPD+1
      GO TO 5
C
C     HOSHINO UPDATE
C
    3 DEN=PAR*B+C
      DIS=HALF*B
      POM=PAR*B/DEN
      CALL UXVDIR(N,PAR,SO,S,S)
      CALL UXDSMU(N,HH,PAR/DIS,SO)
      CALL UXDSMU(N,HH,-ONE/DEN,S)
      TUXX='HOSHINO '
      NUPD=NUPD+1
      GO TO 5
C
C     RANK ONE UPDATE
C
    4 DEN=PAR*B-C
      IF (ABS(DEN).LE.CON*MAX(CON,ABS(PAR*B),ABS(C))) THEN
      IF (B.GT.ZERO.AND.C.GT.ZERO) GO TO 1
      ITERH=4
      TUXX='ONE FAIL'
      GO TO 6
      ENDIF
      POM=PAR*B/DEN
      CALL UXVDIR(N,-PAR,SO,S,S)
      CALL UXDSMU(N,HH,ONE/DEN,S)
      TUXX='RANK ONE'
      NUPD=NUPD+1
    5 ITERH=0
      IF (GAM.EQ.ONE) GO TO 6
      NSCL=NSCL+1
      CALL UXDSMS(N,HH,ONE/GAM)
    6 CALL UOU1D2(N,HH,S,RHO,GAM,PAR,A,B,C,POM,ETA9)
      RETURN
      END
