* SUBROUTINE UUDFG1                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* VARIABLE METRIC UPDATE OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX
* USING THE FACTORIZATION H=L*D*TRANS(L).
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RU  H(M)  FACTORIZATION H=L*D*TRANS(L) OF A POSITIVE
*         SEMIDEFINITE HESSIAN MATRIX.
*  RU  B(M)  FACTORIZATION B=L*D*TRANS(L) OF A POSITIVE
*         DEFINITE APPROXIMATION OF THE HESSIAN MATRIX.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RU  XO(NF)  VECTORS OF VARIABLES DIFFERENCE.
*  RI  GO(NF)  GRADIENTS DIFFERENCE.
*  RI  F  CURRENT VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RI  ETA2  TOLERANCE FOR POSITIVE DEFINITENESS.
*  RI  ETA5  TOLERANCE FOR HYBRID METHOD.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  RI  PAR1 PARAMETER FOR CONTROLLED SCALING.
*  RI  PAR2 PARAMETER FOR CONTROLLED SCALING.
*  II  IPOM1  METHOD INDICATOR.
*  IO  IPOM2  INDICATOR FOR SCALING.
*  II  MET  METHOD SELECTION. MET=0-NO UPDATE. MET=1-BFGS UPDATE.
*  II  MET1  SELECTION OF SELF SCALING.  MET1=1-SELF SCALING SUPPRESSED.
*         MET1=2 SELF SCALING IN THE FIRST ITERATION AFTER RESTART.
*         MET1=3-SELF SCALING IN EACH ITERATION.
*  II  MOT2  SELECTED COMBINATION OF NEWTON AND VARIABLE METRIC
*         METHODS. MOT1=1-SIMPLE COMBINATION. MOT1=2-FLETCHER'S
*         FIRST COMBINATION. MOT1=2-FLETCHER'S SECOND COMBINATION.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*  IU  NDECF  NUMBER OF DECOMPOSITIONS.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXDPGF  FACTORIZATION B=L*D*TRANS(L) OF A DENSE SYMMETRIC
*         POSITIVE DEFINITE MATRIX.
*  S   UXDPGM  MATRIX-VECTOR PRODUCT USING THE FACTORIZATION
*         B=L*D*TRANS(L).
*  S   UXDPGU  CORRECTION OF A DENSE SYMMETRIC POSITIVE DEFINITE
*         MATRIX IN THE FACTORED FORM B=L*D*TRANS(L).
*  S   UXDPGS  SCALING OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX
*         IN THE FACTORED FORM B=L*D*TRANS(L).
*  S   UXVCOP  COPYING OF A VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UOERR1  ERROR MESAGES.
*  S   UOU1D1  PRINT OF ENTRY TO VARIABLE METRIC UPDATE.
*  S   UOU1D2  PRINT OF EXIT FROM VARIABLE METRIC UPDATE.
*
* METHOD :
*  FLETCHER'S COMBINATION OF THE GAUSS-NEWTON AND THE BFGS METHODS.
*
      SUBROUTINE UUDFG1(N,H,B,S,XO,GO,F,FO,ETA2,ETA5,ETA9,PAR1,PAR2,
     & IPOM1,IPOM2,MET,MET1,MOT2)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,IPOM1,IPOM2,MET,MET1,MOT2
      REAL*8 H(N*(N+1)/2),B(N*(N+1)/2),S(N),XO(N),GO(N),F,FO,ETA2,ETA5,
     & ETA9,PAR1,PAR2
      REAL*8 UXVDOT,UXDPGP
      REAL*8 AH,BH,CH,DH,AB,BB,CB,DB,GAM,PAR
      INTEGER INF
      LOGICAL L1,L2
      REAL*8 CON,CON3,CON5,CON6
      PARAMETER (CON=1.0D 6,CON3=0.1D 0,CON5=1.0D 6,CON6=0.4D 0)
      IF (MET.LE.0) RETURN
      CALL UOU1D1(N,XO,GO)
      IF (IDECF.EQ.0) THEN
C
C     GILL-MURRAY DECOMPOSITION
C
      BH=ETA2
      CALL UXDPGF(N,H,INF,BH,DH)
      NDECF=NDECF+1
      IDECF=1
      ELSE IF (IDECF.NE.1) THEN
      ITERH=-1
      TDXX='BAD DEC1'
      CALL UOERR1('UUDFG1',1)
      GOTO 2
      ENDIF
      PAR=ZERO
C
C     DETERMINATION OF THE PARAMETERS A,B,C
C
      BH=UXVDOT(N,XO,GO)
      IF(BH.LE.ZERO) THEN
      ITERH=2
      TUXX='B - NEG.'
      IPOM1=0
      GOTO 2
      ENDIF
      BB=BH
      L1=MET1.EQ.4.OR.MET1.EQ.3.AND.IPOM2.GE.1.OR.MET1.EQ.2
     & .AND.IPOM2.EQ.1
      L2=ABS(PAR1).LE.CON3.AND.PAR2.LE.ZERO
      IF (.NOT.L2) THEN
      CALL UXVCOP(N,GO,S)
      CALL UXDPGB(N,H,S,1)
      AH=UXDPGP(N,H,S,S)
      CALL UXVCOP(N,XO,S)
      CALL UXDPGM(N,H,S,0)
      CH=UXVDOT(N,XO,S)
      IF (CH.LE.ZERO) THEN
      DH=ETA9
      ELSE
      DH=(AH/BH)**2-TWO*(BH/CH)+ONE
      ENDIF
      ENDIF
      CALL UXVCOP(N,GO,S)
      CALL UXDPGB(N,B,S,1)
      AB=UXDPGP(N,B,S,S)
      CALL UXVCOP(N,XO,S)
      CALL UXDPGM(N,B,S,0)
      CB=UXVDOT(N,XO,S)
      DB=(AB/BB)**2-TWO*(BB/CB)+ONE
      IF (L2) THEN
      ELSE IF (MOT2.EQ.1) THEN
      IF(DH.LE.MIN(CON6*DB,CON)) THEN
      IPOM1=0
      ELSE
      IPOM1=1
      ENDIF
      ELSE
      IF (FO-F.GE.ETA5*FO.AND.DH.LE.CON5*DB.OR.
     * MOT2.EQ.3.AND.DH.LE.MIN(CON6*DB,CON)) THEN
      IPOM1=0
      ELSE
      IPOM1=1
      ENDIF
      ENDIF
      IF(L1) THEN
C
C     DETERMINATION OF THE PARAMETER GAM (SELF SCALING)
C
      GAM=CB/BB
      ELSE
      GAM=ONE
      ENDIF
C
C     BFGS UPDATE
C
      CALL UXDPGU(N,B,GAM/BB,GO,XO)
      CALL UXDPGU(N,B,-ONE/CB,S,XO)
      TUXX='BFGS    '
      NUPD=NUPD+1
      ITERH=0
      IPOM2=0
      IF(L1) THEN
      NSCL=NSCL+1
      CALL UXDPGS(N,B,ONE/GAM)
      ENDIF
    2 CALL UOU1D2(N,B,S,DB,GAM,PAR,AB,BB,CB,DH,ETA9)
      RETURN
      END
