* SUBROUTINE UULBV3                ALL SYSTEMS                31/12/13
C PORTABILITY : ALL SYSTEMS
C 31/12/13 VL :                                       ORIGINAL VERSION
*
* PURPOSE :
* CORRECTION TRANSFORMATION OF USED VECTORS AND MATRICES.
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF  MATRICES XM, GM.
*  IU  M  NUMBER OF COLUMNS OF MATRICES XM, GM.
*  II  MM  MAXIMUM NUMBER OF COLUMNS OF MATRICES XM, GM.
*  RU  XR(MM,MM) MATRIX  TRANS(XO)*GO.
*  RU  GR(M*(M+1)/2)  SYMMETRIC MATRIX TRANS(GO)*GO STORED COLUMNWISE.
*  RU  XM(N*MM)  TRANSFORMED RECTANGULAR MATRIX OF VARIABLES DIFFERENCES.
*  RU  GM(N*MM)  TRANSFORMED RECTANGULAR MATRIX OF GRADIENTS DIFFERENCES.
*  RU  RV(M*(M+1)/2)  UPPER TRIANGULAR PART OF XR STORED COLUMNWISE.
*  RU  DV(M)  DIAGONAL OF MATRIX XR.
*  RA  WW(6*MM)  AUXILIARY VECTOR.
*  IA  IND(M)  AUXILIARY ARRAY.
*  IA  IC(M)  AUXILIARY ARRAY.
*  IA  IN(M)  AUXILIARY ARRAY.
*  IA  INC(MM+M)  AUXILIARY ARRAY.
*  RI  B  INPUT PARAMETER B = TRANS(XO)*GO.
*  RI  AH  INPUT PARAMETER  AH = TRANS(GO)*GO.
*
* SUBPROGRAMS USED :
*  S   UULBV2  COMPUTATION OF TRANS(GO)*HH*GO, WHERE HH IS THE LAST
*       BUT ONE VARIABLE METRIC AUXILIARY MATRIX.
*  S   UXDCMS  VECTOR MULTIPLIED BY A COLUMNWISE STORED RECTANGULAR
*         MATRIX AND ADDED TO A VECTOR.
*  S   UXDSMM  MULTIPLICATION OF A DENSE SYMMETRIC MATRIX BY A VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVSUM  ADDITION OF TWO VECTORS.
*
* METHOD :
*
      SUBROUTINE UULBV3(N,M,MM,XR,GR,XM,GM,RV,DV,WW,IND,IC,IN,INC,B,
     & AH)
      INTEGER N,M,MM,IND(M),IC(M),IN(M),INC(MM+M)
      REAL*8 XR(MM,MM),GR(M*(M+1)/2),XM(N*MM),GM(N*MM),RV(M*(M+1)/2),
     & DV(M),WW(MM*6),B,AH
      REAL*8 BV,BVP,DELQ,Q,W,A,AV,UXVDOT
      INTEGER J,K,L
C
C     NEW CORRECTIONS
C
      L=M*(M-1)/2
      DV(M)=B
      DO J=1,M
        RV(L+J)=XR(J,M)
        IC(J)=J
        IN(J)=J
        INC(J)=1
      ENDDO
C
C     COMPUTATION OF A = TRANS(GO)*HH*GO
C
      CALL UULBV2(M,MM,XR,GR,RV,DV,WW(3*MM+1),B/AH,AH,A)
      AV=A
      BVP=B
      DO J=M-1,1,-1
        INC(J)=IND(J)
        Q=XR(J,M)*XR(M,J)/XR(J,J)
        DELQ=(XR(J,M)-XR(M,J))**2/(B*XR(J,J))
        W=ABS(BVP-AV)/BVP*(B/BVP-1.0D 0)
        IF (J.EQ.M-1.AND.DELQ.GT.1.0D-2) INC(J)=0
        IF (BVP-Q*INC(J).GT.B+B) INC(J)=0
        IF (J.LE.M-1.AND.AV-XR(J,M)**2/XR(J,J)*INC(J).LE.A*1.0D-5)
     &   INC(J)=0
        IF (J.LT.M-1.AND.DELQ.GT.1.0D-5.AND.W.LT.1.0D0) INC(J)=0
        IF (BVP-Q*INC(J).LT.B*1.0D-5) INC(J)=0
        IF (J.LT.M-1.AND.DELQ.GT.MIN(1.0D-2,(1-BVP/B)**4/2+1.0D-5))
     &   INC(J)=0
        BVP=BVP-Q*INC(J)
        AV=AV-XR(J,M)**2/XR(J,J)*INC(J)
      ENDDO
      DO J=1,M
        IF (INC(J).EQ.1) IN(IC(J))=J
        IF (J.LT.M) IC(J+1)=IC(J)+INC(J)
      ENDDO
      IF (IC(M).LE.1) RETURN
      CALL UXVSET(M,0.0D 0,WW)
      CALL UXVSET(M,0.0D 0,WW(M+1))
      BV=B
      DO J=1,IC(M)-1
        K=IN(J)
        WW(K)=-XR(K,M)/DV(K)
        WW(M+K)=-XR(M,K)/DV(K)
      ENDDO
      DELQ=GR(L+M)
      DO J=1,M-1
        Q=XR(J,M)
        W=XR(M,J)
       DO K=1,M-1
          Q=Q+XR(J,K)*WW(K)
          XR(M,J)=XR(M,J)+XR(K,J)*WW(M+K)
        ENDDO
        XR(J,M)=Q
        RV(L+J)=Q
        DELQ=DELQ+WW(J)*GR(L+J)*2.0D 0
      ENDDO
      IF(M.GT.1)THEN
        CALL UXDSMM(M-1,GR,WW,WW(2*M+1))
        W=UXVDOT(M-1,WW,WW(2*M+1))
        CALL UXVSUM(M-1,WW(2*M+1),GR(L+1),GR(L+1))
        GR(L+M)=DELQ+W
C
C     COMPUTATION OF XO, GO, B
C
        CALL UXDCMS(N,M-1,XM,WW(M+1),XM(M*N-N+1))
        CALL UXDCMS(N,M-1,GM,WW,GM(M*N-N+1))
      ENDIF
      BV=0.0D 0
      DO J=1,N
        BV=BV+XM(M*N-N+J)*GM(M*N-N+J)
      ENDDO
      IF (BV.LE.0.0D 0) STOP56
      RV(M*(M+1)/2)=BV
      DV(M)=BV
      XR(M,M)=BV
      RETURN
      END
