* SUBROUTINE UULCI1                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE INVERSE COLUMN UPDATE METHOD.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  MF  MAXIMUM NUMBER OF VARIABLE METRIC STEPS.
*  II  MMAX  SIZE OF AUXILIARY ARRAYS FOR COMPLETE LU DECOMPOSITION.
*  RA  XMEM(MMAX)  REAL AUXILIARY ARRAY FOR COMPLETE LU DECOMPOSITION.
*  IA  IMEM(MMAX)  INTEGER AUXILIARY ARRAY FOR COMPLETE LU DECOMPOSITION.
*  IO  PMEM(7,2)  STATUS OF AUXILIARY ARRAYS.
*  IO  LU(2,2)  POINTERS OF LU FACTOR IN AUXILIARY ARRAYS.
*  RU  XM(NF*MF)  SET OF VECTOR OF VARIABLES DIFFERENCES.
*  RU  GM(MF)  SET OF GRADIENT DIFFERENCES.
*  IU  IM(MF)  VECTOR OF INDICES.
*  RI  XO(NF)  VECTOR OF VARIABLES DIFFERENCE.
*  RI  AFO(NF)  GRADIENTS DIFFERENCE.
*  RA  S(NF)  AUXILIARY.
*
* COMMON DATA :
*  IU  IREST  RESTART INDICATOR.
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXLIDM  MATRIX MULTIPLICATION FOR LIMITED STORAGE INVERSE
*         COLUMN UPDATE METHOD. COMPLETE LU DECOMPOSITION VARIANT.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXVMX1  L-INFINITY NORM OF A VECTOR.
*
* METHOD :
* INVERSE COLUMN UPDATE LIMITED STORAGE VARIABLE METRIC METHOD.
*
      SUBROUTINE UULCI1(NF,MF,MMAX,XMEM,IMEM,JMEM,LU,XM,GM,IM,XO,AFO,S)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,MF,MMAX,IMEM(MMAX),JMEM(7,2),LU(2,2),IM(MF)
      REAL*8 XMEM(MMAX),XM(NF*MF),GM(MF),XO(NF),AFO(NF),S(2*NF)
      INTEGER II,MM
      REAL*8 TEMP,UXVMX1
      MM=MIN(NIT-KIT,MF)
      IF (MM.GE.MF) THEN
      IREST=1
      ELSE
      II=NF*MM+1
      CALL UXLIDM(NF,MM,MMAX,XMEM,IMEM,JMEM,LU,XM,GM,IM,AFO,XM(II),S)
      CALL UXVDIR(NF,-ONE,XM(II),XO,XM(II))
      MM=MM+1
      TEMP=UXVMX1(NF,AFO,II)
      IF (TEMP.LE.ZERO) THEN
      ITERH=2
      TUXX='ZERO DEN'
      CALL UOERR1('UULCI1',1)
      ELSE
      IM(MM)=II
      GM(MM)=AFO(II)
      TUXX='INV COL '
      NUPD=NUPD+1
      ITERH=0
      ENDIF
      ENDIF
      RETURN
      END
