* SUBROUTINE UULVP1                ALL SYSTEMS                02/12/01
C PORTABILITY : ALL SYSTEMS
C 02/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* RANK-TWO LIMITED-STORAGE VARIABLE-METRIC METHODS IN THE PRODUCT FORM.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES (NUMBER OF ROWS OF XM).
*  II  M  NUMBER OF COLUMNS OF XM.
*  RI  XM(N*M)  RECTANGULAR MATRIX IN THE PRODUCT FORM SHIFTED BROYDEN
*         METHOD (STORED COLUMNWISE): H-SIGMA*I=XM*TRANS(XM)
*  RO  XR(M)  VECTOR TRANS(XM)*H**(-1)*XO.
*  RO  GR(M)  MATRIX TRANS(XM)*GO.
*  RA  S(N)  AUXILIARY VECTORS (H**(-1)*XO AND U).
*  RU  XO(N)  VECTORS OF VARIABLES DIFFERENCE XO AND VECTOR XO-TILDE.
*  RU  GO(N)  GRADIENT DIFFERENCE GO AND VECTOR XM*TRANS(XM)*GO.
*  RA  XS(3*N) AUXILIARY ARRAY.
*  RA  GS(N) AUXILIARY ARRAY.
*  RI  R  STEPSIZE PARAMETER.
*  RI  F  NEW FUNCTION VALUE.
*  RI  FO  OLD FUNCTION VALUE.
*  RI  P  NEW DIRECTIONAL DERIVATIVE (MULTIPLIED BY R)
*  RI  PO  OLD DIRECTIONAL DERIVATIVE (MULTIPLIED BY R)
*  RU  SIG  SCALING PARAMETER (ZETA AND SIGMA).
*  RI  PAR1  PARAMETER FOR SCALING.
*  RI  PAR2  PARAMETER FOR SCALING.
*  II  MET  METHOD SELECTION. MET=1-BFGS UPDATE.
*  II  MET1  SCALING OPTION. MET1=1-NO SCALING.
*  II  MET2  CORRECTION OPTION. MET1=1-NO CORRECTION.
*  II  MET3  CHOICE OF SIGMA (1-CONSTANT, 2-QUADRATIC EQUATION).
*  II  MET4  SCALING OPTION. MET4=1 - RHO/GAM=A/B.
*  II  MET5  CHOICE OF Q. (1 AND 2 SIMPLE CHOICE, 3 - APPROXIMATION
*         OF THE SHIFTED BROYDEN CLASS.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXDRMM  MULTIPLICATION OF A ROWWISE STORED DENSE RECTANGULAR
*         MATRIX BY A VECTOR.
*  S   UXDCMM  MULTIPLICATION OF A COLUMNWISE STORED DENSE RECTANGULAR
*         MATRIX BY A VECTOR.
*  S   UXDCMU  UPDATE OF A COLUMNWISE STORED DENSE RECTANGULAR MATRIX.
*         WITH CONTROLLING OF POSITIVE DEFINITENESS.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVDIR  VECTOR AUGMENTED BY A SCALED VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  RF  UXVNOR  EUCLIDEAN NORM OF A VECTOR.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*
* METHOD :
* RANK-TWO LIMITED-STORAGE VARIABLE-METRIC METHODS IN THE PRODUCT FORM.
*
      SUBROUTINE UULVP1(N,M,XM,XR,GR,S,XO,GO,XS,GS,R,F,FO,P,PO,SIG,
     & PAR1,PAR2,MET,MET1,MET2,MET3,MET4,MET5)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,M,MET,MET1,MET2,MET3,MET4,MET5
      REAL*8 XM(N*M),XR(M),GR(M),S(N),XO(N),GO(N),XS(3*N),GS(N),
     & R,F,FO,P,PO,SIG,PAR1,PAR2
      REAL*8 A,B,C,AA,BB,CC,AH,BH,CH,POM,GAM,RHO,PAR,ZET,UXVDOT
      INTEGER METS,IER
      IF (MET.LE.0) RETURN
      CALL UOU1D1(N,XO,GO)
      METS=MET
      IF (MET.GT.5) MET=1
      ZET=SIG
C
C     COMPUTATION OF B
C
      B=UXVDOT(N,XO,GO)
      IF (B.LE.ZERO) THEN
      ITERH=2
      TUXX='B - NEG.'
      GO TO 22
      ENDIF
C
C     COMPUTATION OF GR=TRANS(XM)*GO, XR=TRANS(XM)*H**(-1)*XO
C     AND S=H**(-1)*XO, SO=(H-SIGMA*I)*H**(-1)*XO. COMPUTATION
C     OF AA=GR*GR, BB=GR*XR, CC=XR*XR. COMPUTATION OF A AND C.
C
      CALL UXDRMM(N,M,XM,GO,GR)
C      CALL UXVDIF(N,GO,G,S)
      CALL UXVSCL(N,R,S,S)
      CALL UXDRMM(N,M,XM,S,XR)
      AH=UXVDOT(N,GO,GO)
      BH=UXVDOT(N,GO,S)
      CH=UXVDOT(N,S,S)
      AA=UXVDOT(M,GR,GR)
      BB=UXVDOT(M,GR,XR)
      CC=UXVDOT(M,XR,XR)
      A=AA+AH*SIG
      C=-R*PO
C
C     DETERMINATION OF THE PARAMETER RHO (NONQUADRATIC PROPERTIES)
C
      CALL UURHO1(F,FO,P,PO,B,RHO,MET2)
C
C     DETERMINATION OF THE PARAMETER GAM (SELF SCALING)
C
      CALL UUGAM1(A,B,C,RHO,GAM,PAR,PAR1,PAR2,MET1,MET4)
C
C     DETERMINATION OF THE PARAMETER SIG (SHIFT)
C
      CALL UUSIG1(N,XO,A,B,C,AA,AH,PAR,SIG,SIG,2.5D-1,MET3)
C
C     DETERMINATION OF THE BROYDEN PARAMETER POM=ETA
C
      CALL UUPOM1(A,B,PAR,POM,MET)
C
C     COMPUTATION OF THE VALUE AA*CC-BB*BB WITH TEST ON POSITIVITY
C
      IF (ABS(MET5).GE.2) CALL UXDCMM(N,M,XM,GR,GS)
      CALL UULVPS(N,M,XM,XS,XO,GS,S,GO,GR,XR,ZET,SIG,POM,GAM,RHO,
     & A,AH,AA,B,BB,C,CC,ABS(MET5),IER)
      SIG=ZET
      TUXX='SHIFT VM'
      NUPD=NUPD+1
      IF (GAM.EQ.ONE) GO TO 22
C
C     SCALING
C
      CALL UXVSCL(N*M,SQRT(GAM),XM,XM)
      NSCL=NSCL+1
   22 CONTINUE
      ITERH=0
C      CALL UOU1D3(N,H,S,RHO,GAM,PAR,A,B,C,POM,ETA9)
      RETURN
      END
