* SUBROUTINE UUSAMG                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* GROUP FRACTIONED UPDATE OPERATION - M VARIANT.
*
* PARAMETERS :
*  II  NF DECLARED NUMBER OF VARIABLES.
*  II  MMAX  NUMBER OF NONZERO ELEMENTS OF THE MATRIX.
*  RU  H(MMAX)  POSITIVE DEFINITE APPROXIMATION OF THE SPARSE HESSIAN
*         MATRIX.
*  II  IH(NF+1)  POINTERS OF THE DIAGONAL ELEMENTS OF H.
*  II  JH(MMAX)  INDICES OF THE NONZERO ELEMENTS OF H.
*  II  COL(NF) GROUPS OF THE SAME COLOUR FORMED BY THE INDICES
*           OF THE COLUMNS. LAST ONE HAS NEGATIVE SIGN.
*  RI  XO(NF) DIFFERENCE OF THE NEW ITERATE AND THE OLD ONE.
*  II  WN01(NF) AUXILIARY VECTOR.
*  RI  WR01(NF)  AUXILIARY VECTOR USED TO STORE y - Vs.
*  II  IVAR BEGINNING OF THE GROUP IN COL.
*  II  JVAR END OF THE GROUP IN COL.
*
* METHOD :
*  MARWILL GROUP UPDATE.
*
      SUBROUTINE UUSAMG(NF,MMAX,H,IH,JH,COL,XO,WN01,WR01,WR02,IVAR,JVAR)
      INTEGER NF,MMAX,IH(NF+1),JH(MMAX),COL(NF),WN01(NF),IVAR,JVAR
      REAL*8  H(MMAX),XO(NF),WR01(NF),WR02(NF)
      INTEGER ISTRT,ISTOP,I,J,K,L,I1
      REAL*8 TEMP,D,EPSQ,ALF
      PARAMETER(EPSQ=1.0D-20)
C
C INITIATE DOT PRODUCT AND GROUP INCIDENCE VECTOR
C
      D=0.0D 0
      DO 50 J=1,NF
        WN01(J)=0
        WR02(J)=0.0D 0
   50 CONTINUE
      ALF=1.0D 0/(2.0D 0)
      DO 60 J=IVAR,JVAR
        L=ABS(COL(J))
        WN01(L)=1
   60 CONTINUE
C
C COMPUTE SUM OF QUADRATS OF DIRECTION COMPONENTS IN HESSIAN ROWS
C
      DO 70 I1=1,NF
      ISTRT=IH(I1)
      ISTOP=IH(I1+1)-1
      DO 80 I=ISTRT,ISTOP
        J=JH(I)
        IF(WN01(J).EQ.1) THEN
          WR02(I1)=WR02(I1)+XO(J)*XO(J)
        ENDIF
   80 CONTINUE
   70 CONTINUE
C
C MAIN CYCLE
C
      DO 400 I1=IVAR,JVAR
      L=ABS(COL(I1))
C
C FIRST PART OF THE WIDTHENED SPARSE STRUCTURE IS UPDATED
C
      ISTRT=IH(L)
      ISTOP=IH(L+1)-1
      DO 100 I=ISTRT,ISTOP
        J=JH(I)
        IF(J.GE.L) GO TO 200
        IF(ABS(WR02(J)).GT.EPSQ) THEN
          TEMP=ALF*(WR01(J)*XO(L)+WR01(L)*WN01(J)*XO(J))/WR02(J)
          K=H(I)
          H(K)=H(K)+TEMP
        ENDIF
  100 CONTINUE
C
C END OF THE UPPER TRIANGLE UPDATE
C
  200 CONTINUE
C
C DIAGONAL UPDATE
C
      IF(ABS(WR02(J)).GT.EPSQ) THEN
        TEMP=2.0D 0*ALF*WR01(J)*XO(J)/WR02(J)
        H(I)=H(I)+TEMP
      ENDIF
      ISTRT=I+1
      DO 300 I=ISTRT,ISTOP
        J=JH(I)
        IF(WN01(J).EQ.1) GO TO 300
        IF(ABS(WR02(J)).GT.EPSQ) THEN
          TEMP=ALF*(WR01(J)*XO(L)+WR01(L)*WN01(J)*XO(J))/WR02(J)
          H(I)=H(I)+TEMP
        ENDIF
  300 CONTINUE
  400 CONTINUE
      DO 500 J=IVAR,JVAR
        L=ABS(COL(J))
        TEMP=ABS(XO(L))
        WN01(L)=0
  500 CONTINUE
      RETURN
      END
