* SUBROUTINE UUSSD7                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF QUADRATIC TERMS FOR THE HYBRID METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE HESSIAN MATRIX.
*  II  MB  NUMBER OF NONZERO ELEMENTS OF THE PARTITIONED HESSIAN
*         MATRIX.
*  II  IA(NA)  TYPES OF APPROXIMATED FUNCTIONS.
*  RI  AF(NA)  VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RU  H(M)  POSITIVE DEFINITE APPROXIMATION OF THE SPARSE HESSIAN
*         MATRIX
*  II  IH(NF+1)  POINTERS OF THE DIAGONAL ELEMENTS OF THE SPARSE
*         HESSIAN MATRIX.
*  II  JH(M)  INDICES OF THE NONZERO ELEMENTS OF THE SPARSE HESSIAN
*             MATRIX IN THE PACKED ROW FORM.
*  RU  AH(MB)  POSITIVE DEFINITE APPROXIMATION OF THE PARTITIONED
*         HESSIAN MATRIX.
*  RI  AG(MA)  NONZERO ELEMENTS OF THE SPARSE JACOBIAN MATRIX.
*  II  IAG(NA+1)  POINTERS OF THE SPARSE JACOBIAN MATRIX.
*  II  JAG(MA)  COLUMN INDICES OF THE SPARSE JACOBIAN MATRIX.
*  RI  S(NF)  DIRECTION VECTOR.
*  RO  PP  QUADRATIC PART OF THE PREDICTED FUNCTION VALUE.
*  RU  PPJ  QUADRATIC TERM WITH THE NORMAL EQUATION MATRIX.
*  RU  PPB  QUADRATIC TERM WITH A SECOND ORDER INFORMATION.
*  II  IPOM  INDICATION FOR SECOND ORDER INFORMATION.
*  II  MOT2  SWITCH FOR THE HYBRID METHOD. MOT2=0-FLETCHER AND XU
*         SWITCH WITH GENERAL UPDATE. MOT2=1-FLETCHER AND XU SWITCH
*         WITH SPECIAL UPDATE. MOT2=2-DENNIS AND WELSCH SWITCH.
*         MOT2=3-RAMSIN AND WEDIN SWITCH.
*
* SUBPROGRAMS USED :
*  RF  UXSSMQ  VALUE OF SPARSE QUADRATIC FORM.
*  RF  UXBSBQ  VALUE OF PARTITIONED QUADRATIC FORM.
*  RF  UXWDOT  DOT PRODUCT OF SPARSE VECTORS.
*
      SUBROUTINE UUSSD7(N,NA,M,MA,MB,IA,AF,H,IH,JH,AH,AG,IAG,JAG,S,PP,
     & PPJ,PPB,IPOM,MOT2)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,NA,M,MA,MB,IA(NA),IH(N+1),JH(M),IAG(NA+1),JAG(MA),IPOM,
     & MOT2
      REAL*8 AF(NA),H(M),AH(MB),AG(MA),S(N),PP,PPJ,PPB
      REAL*8 UXBSBQ,UXSSMQ,UXWDOT
      REAL*8 RHO
      INTEGER K,L,LL,KA,NB
      RHO=PPJ
      IF (IPOM.EQ.0) THEN
      PPJ=UXSSMQ(N,M,H,IH,JH,S,S)
      IF (MOT2.LE.2) THEN
      PPB=PPJ
      NB=0
      DO 1 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      LL=L*(L+1)/2
      IF (IA(KA).GT.0) THEN
      PPB=PPB+AF(KA)*UXBSBQ(L,N,AH(NB+1),JAG(K),S,S,1)
      ENDIF
      NB=NB+LL
    1 CONTINUE
      ENDIF
      ELSE
      PPJ=ZERO
      DO 2 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      IF (IA(KA).GT.0) THEN
      PPJ=PPJ+UXWDOT(L,N,JAG(K),AG(K),S,2)
      ENDIF
    2 CONTINUE
      PPB=UXSSMQ(N,M,H,IH,JH,S,S)
      ENDIF
      PPJ=PPJ*HALF
      PPB=PPB*HALF
      IF (IPOM.EQ.0) THEN
      PP=PPJ
      ELSE
      PP=PPB
      ENDIF
      IF (MOT2.EQ.3) PPB=RHO
      RETURN
      END
