* SUBROUTINE UXDCMG               ALL SYSTEMS                06/12/01
C PORTABILITY : ALL SYSTEMS
C 06/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* GRAM-SCHMIDT ORTHOGONALIZATION OF COLUMNS OF A RECTANGULAR MATRIX.
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF THE MATRIX A.
*  II  M  NUMBER OF COLUMNS OF THE MATRIX A.
*  RU  A(N*M)  RECTANGULAR MATRIX STORED COLUMNWISE IN THE
*         ONE-DIMENSIONAL ARRAY.
*  RA  Z(N)  AUXILIARY ARRAY.
*  II  JOB  TYPE OF GRAM-SCHMIDT ORTOGONALIZATION. JOB=1-CLASSIC
*         ORTHOGONALIZATION. JOB=2-MODIFIED ORTHOGONALIZATION.
*
* SUBPROGRAMS USED :
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  RF  UXVNOR  EUCLIDEAN NORM OF A VECTOR.
*  S   UXVSCL  SCALING OF A VECTOR.
*
      SUBROUTINE UXDCMG(N,M,A,Z,JOB)
      INTEGER N,M,JOB,I,J
      REAL*8 A(N*M),Z(N),TEMPR,UXVDOT,UXVNOR
      IF (JOB.EQ.1) THEN
C
C     CLASSIC ORTHOGONALIZATION
C
      DO I=1,M
      CALL UXVCOP(N,A((I-1)*N+1),Z)
      DO J=1,I-1
      TEMPR=UXVDOT(N,A((J-1)*N+1),A((I-1)*N+1))
      CALL UXVDIR(N,-TEMPR,A((J-1)*N+1),Z,Z)
      ENDDO
      TEMPR=UXVNOR(N,Z)
      CALL UXVSCL(N,1.0D 0/TEMPR,Z,A((I-1)*N+1))
      ENDDO
      ELSE
C
C     MODIFIED ORTHOGONALIZATION
C
      DO I=1,M
      TEMPR=UXVNOR(N,A((I-1)*N+1))
      CALL UXVSCL(N,1.0D 0/TEMPR,A((I-1)*N+1),A((I-1)*N+1))
      DO J=I+1,M
      TEMPR=UXVDOT(N,A((I-1)*N+1),A((J-1)*N+1))
      CALL UXVDIR(N,-TEMPR,A((I-1)*N+1),A((J-1)*N+1),A((J-1)*N+1))
      ENDDO
      ENDDO
      ENDIF
      END
