* SUBROUTINE UXDCPU               ALL SYSTEMS                16/12/01
C PORTABILITY : ALL SYSTEMS
C 16/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* UPDATE OF A QR DECOMPOSITION. THIS QR DECOMPOSITION IS UPDATED
* BY THE RULE Q*R:=Q*R+ALF*X*TRANS(Y).
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE MATRIX Q.
*  II  M  NUMBER OF ROWS OF THE MATRIX Q.
*  RU  Q(M*N)  RECTANGULAR MATRIX STORED COLUMNWISE IN THE
*         ONE-DIMENSIONAL ARRAY (PART OF THE ORTHOGONAL MATRIX).
*  RU  R(N*(N+1)/2)  UPPER TRIANGULAR MATRIX STORED IN A PACKED FORM.
*  RI  ALF  SCALAR PARAMETER.
*  RI  X(M)  INPUT VECTOR.
*  RI  Y(N)  INPUT VECTOR.
*  RA  Z(N)  AUXILIARY VECTOR.
*  IO  INF  INFORMATION. IF INF=0 THEN X LIES IN THE COLUMN SPACE OF Q.
*         IF INF=1 THEN X DOES NOT LIE IN THE COLUMN SPACE OF Q.
*
* SUBPROGRAMS USED :
*  RF  UXVNOR  EUCLIDEAN NORM OF A VECTOR.
*  S   UXVORT  COMPUTATION OF THE PLANE ROTATION MATRIX.
*  S   UXVROT  PLANE ROTATION IS APPLIED TO TWO NUMBERS.
*  S   UXVRTV  PLANE ROTATION IS APPLIED TO TWO VECTORS.
*
* METHOD :
* J.W.DANIEL, W.B.GRAGG, L.KAUFMAN, G.W.STEWARD : REORTHOGONALIZATION
* AND STABLE ALGORITHMS FOR UPDATING THE GRAM-SCHMIDT QR FACTORIZATION.
* MATHEMATICS OF COMPUTATION 30 (1976) 772-795.
      SUBROUTINE UXDCPU(N,M,Q,R,ALF,X,Y,Z,INF)
      INTEGER N,M,INF
      REAL*8 Q(M*N),R(N*(N+1)/2),ALF,X(M),Y(N),Z(N)
      REAL*8 CK,CL,ZK,ZL,UXVNOR
      INTEGER J,K,L,KJ,KK,KL,IER
      REAL*8 ONE,CON
      PARAMETER (ONE=1.0D 0,CON=1.0D-10)
      INF=0
      KK=N*(N+1)/2
C
C     COMPUTATION OF THE VECTOR TRANS(Q)*X
C
      CALL UXDRMM(M,N,Q,X,Z)
      IF (M.GT.N) THEN
C
C     IF X DOES NOT LIE IN THE COLUMN SPACE OF Q WE HAVE TO USE
C     A SUBPROBLEM WHOSE DIMENSION IS BY ONE GREATER (INF=1).
C
      ZK=UXVNOR(M,X)
      CALL UXDCMD(M,N,Q,Z,-ONE,X,X)
      ZL=UXVNOR(M,X)
      IF (ZL.GT.CON*ZK) THEN
      INF=1
      CALL UXVSCL(M,-ONE/ZL,X,X)
      CALL UXVORT(Z(N),ZL,CK,CL,IER)
      IF (IER.EQ.0.OR.IER.EQ.1) THEN
      CALL UXVROT(R(KK),ZL,CK,CL,IER)
      CALL UXVRTV(M,Q((N-1)*M+1),X,CK,CL,IER)
      ENDIF
      ENDIF
      ENDIF
C
C     APPLICATION OF PLANE ROTATIONS TO THE VECTOR Z SO THAT
C     TRANS(Q1)*Z=E1 WHERE Q1 IS AN ORTHOGONAL MATRIX (ACCUMULATION OF
C     THE PLANE ROTATIONS) AND E1 IS THE FIRST COLUMN OF THE UNIT
C     MATRIX. AT THE SAME TIME BOTH THE UPPER HESSENBERG MATRIX
C     TRANS(Q1)*R AND THE ORTHOGONAL MATRIX Q*Q1 ARE CONSTRUCTED SO THAT
C     Q*Q1*R1=Q*Q1*(TRANS(Q1)*R+ALF*E1*TRANS(Y)) WHERE R1 IS AN UPPER
C     HESSENBERG MATRIX.
C
      KL=(N-1)*M+1
      DO 4 L=N,2,-1
      K=L-1
      KK=KK-L
      CALL UXVORT(Z(K),Z(L),CK,CL,IER)
      IF (IER.EQ.0.OR.IER.EQ.1) THEN
      CALL UXVROT(R(KK),Z(L),CK,CL,IER)
      KJ=KK
      DO 2 J=L,N
      KJ=KJ+J-1
      CALL UXVROT(R(KJ),R(KJ+1),CK,CL,IER)
    2 CONTINUE
      CALL UXVRTV(M,Q(KL-M),Q(KL),CK,CL,IER)
      ENDIF
      KL=KL-M
    4 CONTINUE
      Z(1)=ALF*Z(1)
      KJ=1
      DO 5 J=1,N
      R(KJ)=R(KJ)+Z(1)*Y(J)
      KJ=KJ+J
    5 CONTINUE
C
C     APPLICATION OF PLANE ROTATIONS TO THE UPPER HESSENBERG MATRIX R1
C     GIVEN ABOVE SO THAT R2=TRANS(Q2)*R1 WHERE Q2 IS AN ORTHOGONAL
C     MATRIX (ACCUMULATION OF THE PLANE ROTATIONS) AND R2 IS AN UPPER
C     TRIANGULAR MATRIX. WE OBTAIN THE NEW QR DECOMPOSITION Q*Q1*Q2*R2.
C
      KK=1
      KL=1
      DO 8 L=2,N
      K=L-1
      CALL UXVORT(R(KK),Z(L),CK,CL,IER)
      IF (IER.EQ.0.OR.IER.EQ.1) THEN
      KJ=KK
      DO 6 J=L,N
      KJ=KJ+J-1
      CALL UXVROT(R(KJ),R(KJ+1),CK,CL,IER)
    6 CONTINUE
      CALL UXVRTV(M,Q(KL),Q(KL+M),CK,CL,IER)
      ENDIF
      KK=KK+L
      KL=KL+M
    8 CONTINUE
C
C     BACK TRANSFORMATION OF THE GREATER SUBPROBLEM IF INF=1.
C
      IF (INF.EQ.1) THEN
      CALL UXVORT(R(KK),ZL,CK,CL,IER)
      IF (IER.EQ.0.OR.IER.EQ.1) THEN
      CALL UXVRTV(M,Q((N-1)*M+1),X,CK,CL,IER)
      ENDIF
      ENDIF
      RETURN
      END
