* SUBROUTINE UXDPRH               ALL SYSTEMS                02/12/01
C PORTABILITY : ALL SYSTEMS
C 02/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* ORTHOGONAL TANSFORMATION OF THE TRIANGULAR DECOMPOSITION OF THE
* REDUCED HESSIAN MATRIX.
*
* PARAMETERS :
*  II  N  ORDER OF THE MATRIX A.
*  RU  A(N*(N+1)/2) FACTORIZATION A=TRANS(R)*R STORED IN THE PACKED
*         FORM.
*  RU  B(N*(N+1)/2) UPPER TRIANGULAR MATRIX WHICH REPRESENTS ORTHOGONAL
*         TRANSFORMATIONS IN LIMITED MEMORY REDUCED HESSIAN METHOD.
*  II  X(N)  REDUCED GRADIENT.
*  II  Y(N)  REDUCED DIRECTION VECTOR.
*  II  Z(N)  REDUCED GRADIENT DIFFERENCE.
*  IU  IER  INFORMATION ON THE TRANSFORMATION. IER<0-K OR L OUT OF
*         RANGE. IER=0-GENERAL PLANE ROTATION. IER=1-PERMUTATION.
*         IER=2-TRANSFORMATION SUPPRESSED.
*  II  JOB  NUMBER OF VECTORS TRANSFORMED.
*
* SUBPROGRAMS USED :
*  S   UXDPRR  PLANE ROTATION OF THE CHOLESKI FACTOR OF A DENSE
*         SYMMETRIC POSITIVE DEFINITE MATRIX.
*  S   UXVORT  COMPUTATION OF THE PLANE ROTATION MATRIX.
*  S   UXVROT  PLANE ROTATION IS APPLIED TO TWO NUMBERS.
*
      SUBROUTINE UXDPRH(N,A,B,X,Y,Z,IER,JOB)
      INTEGER N,IER,JOB
      REAL*8 A(N*(N+1)/2),B(N*(N+1)/2),X(N),Y(N),Z(N)
      REAL*8 CK,CL
      INTEGER J,JJ,K,KK,L
      KK=0
      DO 2 L=2,N
      KK=KK+L
      CALL UXVORT(B(KK),B(KK+1),CK,CL,IER)
      CALL UXVROT(X(L-1),X(L),CK,CL,IER)
      IF (JOB.GE.2) CALL UXVROT(Y(L-1),Y(L),CK,CL,IER)
      IF (JOB.GE.3) CALL UXVROT(Z(L-1),Z(L),CK,CL,IER)
      JJ=KK
      DO 1 J=L,N-1
      JJ=JJ+J
      CALL UXVROT(B(JJ),B(JJ+1),CK,CL,IER)
    1 CONTINUE
      CALL UXDPRR(N,A,L-1,L,CK,CL,IER)
    2 CONTINUE
      KK=0
      DO 4 K=1,N-1
      DO 3 J=1,K
      KK=KK+1
      B(KK)=B(KK+K)
    3 CONTINUE
    4 CONTINUE
      RETURN
      END
