* SUBROUTINE UXDRHF               ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* QR DECOMPOSITION OF A ROWWISE STORED DENSE RECTANGULAR MATRIX A USING
* HOUSEHOLDER TRANSFORMATIONS WITH PIVOTING.
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE MATRIX A.
*  II  M  NUMBER OF ROWS OF THE MATRIX A.
*  RU  A(M*N)  RECTANGULAR MATRIX STORED ROWWISE IN THE
*         ONE-DIMENSIONAL ARRAY.
*  IO  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX.
*  RO  D(N)  VECTOR CONTAINING INFORMATION ABOUT HOUSEHOLDER
*         TRANSFORMATIONS.
*  RA  E(N)  AUXILIARY VECTOR.
*  IO  NM  ASSUMED RANK OF THE MATRIX A.
*
* SUBPROGRAMS USED :
*  S   UXDRMN  EUCLIDEAN NORM OF A PART OF THE ROWWISE STORED
*         RECTANGULAR MATRIX COLUMN.
*
* METHOD :
* P.A.BUSSINGER, G.H.GOLUB : LINEAR LEAST SQUARES SOLUTION BY
* HOUSEHOLDER TRANSFORMATION. NUMER. MATH. 7 (1965) 269-276.
*
      SUBROUTINE UXDRHF(N,M,A,IA,D,E,NM)
      INTEGER N,M,IA(N),NM
      REAL*8 A(M*N),D(N),E(N)
      REAL*8 ALF,BET,POM,UXDRMN
      INTEGER I,J,K,L,IP,JP,KP
      REAL*8 ZERO,ONE,CON
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0,CON=5.0D-2)
      NM=MIN(N,M)
C
C     COMPUTATION OF COLUMN NORMS
C
      DO 1 I=1,N
      IA(I)=I
      D(I)=UXDRMN(N,M,A,I,1)
      E(I)=D(I)
    1 CONTINUE
      L=0
      DO 8 K=1,NM
C
C     COLUMN PIVOTING
C
      J=0
      POM=ZERO
      DO 2 I=K,N
      IP=IA(I)
      IF (POM.LT.D(IP)) THEN
      POM=D(IP)
      J=I
      KP=IP
      ENDIF
    2 CONTINUE
      IF (J.EQ.0) THEN
      NM=K-1
      RETURN
      ENDIF
      IA(J)=IA(K)
      IA(K)=KP
      POM=UXDRMN(N,M,A,KP,K)
      IF (A(L+KP).NE.ZERO) POM=SIGN(POM,A(L+KP))
      JP=L
C
C     HOUSEHOLDER TRANSFORMATION
C
      DO 3 J=K,M
      A(JP+KP)=A(JP+KP)/POM
      JP=JP+N
    3 CONTINUE
      A(L+KP)=A(L+KP)+ONE
      DO 6 I=K+1,N
      IP=IA(I)
      ALF=ZERO
      JP=L
      DO 4 J=K,M
      ALF=ALF+A(JP+KP)*A(JP+IP)
      JP=JP+N
    4 CONTINUE
      ALF=ALF/A(L+KP)
      JP=L
      DO 5 J=K,M
      A(JP+IP)=A(JP+IP)-ALF*A(JP+KP)
      JP=JP+N
    5 CONTINUE
C
C     COLUMN NORMS UPDATE
C
      IF (D(IP).EQ.ZERO) GO TO 6
      ALF=ONE-(A(L+IP)/D(IP))**2
      ALF=MAX(ALF,ZERO)
      BET=ONE+CON*ALF*(D(IP)/E(IP))**2
      IF (BET.GT.ONE) THEN
      D(IP)=D(IP)*SQRT(ALF)
      ELSE
      D(IP)=UXDRMN(N,M,A,IP,K+1)
      E(IP)=D(IP)
      ENDIF
    6 CONTINUE
      D(KP)=A(L+KP)
      A(L+KP)=-POM
      L=L+N
    8 CONTINUE
      RETURN
      END
