* SUBROUTINE UXDSGD                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF A DIRECTION OF NEGATIVE CURVATURE WITH RESPECT
* TO A DENSE SYMMETRIC MATRIX A USING THE FACTORIZATION
* A+E=P*D*TRANS(L)*TRANS(P) OBTAINED BY THE SUBROUTINE UXDSGF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2) FACTORIZATION A+E=L*D*TRANS(L) OBTAINED BY THE
*         SUBROUTINE UXDPGF.
*  II  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RO  X(N)  COMPUTED DIRECTION OF NEGATIVE CURVATURE (I.E.
*         TRANS(X)*A*X<0) IF IT EXISTS.
*  II  INF  INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. THE
*         DIRECTION OF NEGATIVE CURVATURE EXISTS ONLY IF INF>0.
*
* METHOD :
* P.E.GILL, W.MURRAY : NEWTON TYPE METHODS FOR UNCONSTRAINED AND
* LINEARLY CONSTRAINED OPTIMIZATION, MATH. PROGRAMMING 28 (1974)
* PP. 311-350.
*
      SUBROUTINE UXDSGD(N,A,IA,X,INF)
      INTEGER N, IA(N), INF
      REAL*8  A(N*(N+1)/2), X(N)
      INTEGER  I, J, IP, JP, IJ, IND
      REAL*8 ZERO, ONE
      PARAMETER (ZERO = 0.0D0, ONE = 1.0D0)
      IND(I,J) = MAX(I,J) * (MAX(I,J) - 1) / 2 + MIN(I,J)
C
C     RIGHT HAND SIDE FORMATION
C
      DO 1 I=1,N
      IP=IA(I)
      X(IP) = ZERO
    1 CONTINUE
      IF (INF .LE. 0) RETURN
      IP = IA(INF)
      X(IP) = ONE
C
C     BACK SUBSTITUTION
C
      DO  3  I = INF-1, 1, -1
      IP=IA(I)
      DO  2  J = I+1, INF
      JP=IA(J)
      IJ=IND(IP,JP)
      X(IP) = X(IP) - A(IJ)*X(JP)
    2 CONTINUE
    3 CONTINUE
      RETURN
      END
