* SUBROUTINE UXLRMM                ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE QUASI-NEWTON UPDATE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  MF  NUMBER OF QUASI-NEWTON STEPS.
*  II  NA NUMBER OF COLUMNS OF THE MATRIX A.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD A.
*  RI  A(MA)  RECTANGULAR MATRIX STORED AS A TWO-DIMENSIONAL ARRAY.
*  II  IA(NA+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD A.
*  II  JA(MA) COLUMN INDICES OF ELEMENTS IN THE FIELD A.
*  RU  XM(NF*MF)  SET OF VECTOR OF VARIABLES DIFFERENCES.
*  RU  GM(NF*MF)  SET OF GRADIENT DIFFERENCES.
*  II  IM(MF)  VECTOR OF INDICES.
*  RU  XR(MF*(MF+1)/2)  MATRIX TRANS(XM)*XM STORED IN PACKED FORM.
*  RA  UM(MF)  AUXILIARY VECTOR.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RI  U(NF)  INPUT VECTOR.
*  RO  V(NA)  OUTPUT VECTOR.
*  II  JOB  OPTION. JOB=1 - SCHUBERT UPDATE. JOB=2 - COLUMN UPDATE.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION.
*  S   UXDCMM  VECTOR MULTIPLIED BY A COLUMNWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXDCMD  VECTOR MULTIPLIED BY A COLUMNWISE STORED RECTANGULAR
*         MATRIX AND ADDED TO A SCALED VECTOR.
*  S   UXDRMM  VECTOR MULTIPLIED BY A ROWWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXSRMD  VECTOR MULTIPLIED BY A ROWWISE STORED SPARSE RECTANGULAR
*         MATRIX AND ADDED TO A SCALED VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH THE CHANGE OF THE SIGN.
*
* METHOD :
* LIMITED STORAGE QUASI-NEWTON METHODS.
*
      SUBROUTINE UXLRMM(NF,MF,NA,MA,A,IA,JA,XM,GM,IM,XR,UM,S,U,V,JOB)
      INTEGER NF,MF,NA,MA,IA(NA+1),JA(MA),IM(MF),JOB
      REAL*8 A(MA),XM(NF*MF),GM(NF*MF),XR(MF*(MF+1)/2),UM(MF),S(NF),
     & U(NF),V(NA)
      INTEGER I
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      IF (MF.LE.0) THEN
      CALL UXSRMM(NF,NA,MA,A,IA,JA,U,V)
      ELSE
      IF (JOB.EQ.1) THEN
      CALL UXDRMM(NF,MF,XM,U,UM)
      ELSE
      DO 1 I=1,MF
      UM(I)=U(IM(I))
    1 CONTINUE
      ENDIF
      CALL UXDPRB(MF,XR,UM,-1)
      CALL UXDCMD(NF,MF,XM,UM,-ONE,U,S)
      CALL UXDCMM(NA,MF,GM,UM,V)
      CALL UXSRMD(NF,NA,MA,A,IA,JA,S,-ONE,V,V)
      CALL UXVNEG(NA,V,V)
      ENDIF
      RETURN
      END
