* SUBROUTINE UXLSIG                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE VARIABLE METRIC DIRECTION DETERMINATION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  MF  NUMBER OF QUASI-NEWTON STEPS.
*  RU  XM(2*NF*MF)  SET OF DIFFERENCES.
*  RO  HR(MF*(2*MF+1))  MATRIX IN THE UPDATE REPRESENTATION.
*  RU  U(2*MF)  VECTOR TRANS(XM)*G.
*  RA  V(2*MF)  AUXILIARY VECTOR.
*  RI  X(NF)  INPUT VECTOR.
*  RO  Y(NF)  OUTPUT VECTOR.
*  RI  PAR  SCALING PARAMETER.
*  II  MET1  SCALING OPTION. MET1=1-NO SCALING. MET1=2-PRELIMINARY
*         SCALING.
*  II  MET5  REALIZATION FORM. MET5=1-THE USE OF 6*MF*NF OPERATIONS.
*         MET5=2-THE USE OF 4*MF*NF OPERATIONS.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION.
*  S   UXDCMM  VECTOR MULTIPLIED BY A COLUMNWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXDCMD  VECTOR MULTIPLIED BY A COLUMNWISE STORED RECTANGULAR
*         MATRIX AND ADDED TO A SCALED VECTOR.
*  S   UXDRMM  VECTOR MULTIPLIED BY A ROWWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  S   UXVMUL  VECTOR MULTIPLIED BY A DIAGONAL MATRIX STORED AS VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH THE CHANGE OF THE SIGN.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*
* METHOD :
* LIMITED STORAGE VARIABLE METRIC METHODS.
*
      SUBROUTINE UXLSIG(NF,MF,XM,HR,U,V,X,Y,PAR,MET1,MET5)
      INTEGER NF,MF,MET1,MET5
      REAL*8 XM(2*NF*MF),HR(MF*(2*MF+1)),U(2*MF),V(2*MF),X(NF),Y(NF),
     & PAR
      INTEGER I
      IF (MF.LE.0) THEN
      CALL UXVCOP(NF,X,Y)
      ELSE
      IF (MET5.EQ.1) CALL UXDRMM(NF,2*MF,XM,X,U)
      IF (MET1.GT.1) THEN
      DO 1 I=1,MF
      U(2*I)=PAR*U(2*I)
    1 CONTINUE
      ENDIF
      CALL UXDSMM(2*MF,HR,U,V)
      IF (MET1.GT.1) THEN
      DO 2 I=1,MF
      V(2*I)=PAR*V(2*I)
    2 CONTINUE
      ENDIF
      CALL UXDCMD(NF,2*MF,XM,V,PAR,X,Y)
      ENDIF
      RETURN
      END
