* SUBROUTINE UXLSIV                ALL SYSTEMS                31/12/13
C PORTABILITY : ALL SYSTEMS
C 31/12/13 VL :                                       ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE DIRECTION VECTOR S = -HH*G BY THE ADAPTED BNS METHOD,
* WHERE HH IS TRANSFORMED VARIABLE METRIC MATRIX.
*
* PARAMETERS :
*  IU  N  NUMBER OF ROWS OF  MATRICES XM, GM.
*  IU  M  NUMBER OF COLUMNS OF MATRICES XM, GM.
*  II  MM  MAXIMUM NUMBER OF COLUMNS OF MATRICES XM, GM.
*  RI  XR(MM,MM) MATRIX  TRANS(XM)*GM.
*  RI  GR(M*(M+1)/2)  SYMMETRIC MATRIX TRANS(GM)*GM STORED COLUMNWISE.
*  RI  XM(N*MM)  TRANSFORMED RECTANGULAR MATRIX OF VARIABLES DIFFERENCES.
*  RI  GM(N*MM)  TRANSFORMED RECTANGULAR MATRIX OF GRADIENTS DIFFERENCES.
*  RI  G(N)  GRADIENT VECTOR.
*  RU  S(N)  DIRECTION VECTOR.
*  RI  RV(M*(M+1)/2)  UPPER TRIANGULAR PART OF XR STORED COLUMNWISE.
*  RI  DV(M)  DIAGONAL OF MATRIX XR.
*  RI  SG(M)  VECTOR TRANS(XM)*G.
*  RI  YG(M)  VECTOR TRANS(GM)*G.
*  RA  WW(3*M)  AUXILIARY ARRAY.
*  RI  BB  INPUT PARAMETER  BB = TRANS(GO)*XO/(TRANS(GO)*GO).
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION.
*  S   UXDSMM  MULTIPLICATION OF A DENSE SYMMETRIC MATRIX BY A VECTOR.
*  S   UXVCOP  COPYING OF A VECTOR.
*
* METHOD :
*
      SUBROUTINE UXLSIV(N,M,MM,XR,GR,XM,GM,G,S,RV,DV,SG,YG,WW,BB)
      INTEGER M,N,MM
      REAL*8 XR(MM,MM),GR(M*(M+1)/2),XM(N*M),GM(N*M),G(N),S(N),
     & RV(M*(M+1)/2),DV(M),SG(M),YG(M),WW(M*3),BB
      REAL*8 W,W1,W2
      INTEGER I,J,L
      CALL UXVCOP(M,SG,WW)
      CALL UXDPRB(M,RV,WW,-1)
      CALL UXDSMM(M,GR,WW,WW(2*M+1))
      DO J=1,M
        WW(M+J)=WW(J)*DV(J)+BB*(WW(2*M+J)-YG(J))
      ENDDO
      CALL UXDPRB(M,RV,WW(M+1),1)
      DO I=1,N
        S(I)=-BB*G(I)
      ENDDO
      L=0
      DO J=1,M
        W1=WW(J)
        W2=WW(M+J)
        DO I=1,N
          S(I)=S(I)-XM(L+I)*W2+(BB*GM(L+I))*W1
        ENDDO
        L=L+N
        W=0.0D 0
        DO I=1,M
          W=W+XR(I,J)*WW(M+I)
        ENDDO
        WW(J)=W+BB*(YG(J)-WW(2*M+J))
      ENDDO
      RETURN
      END
