* SUBROUTINE UXSGFD                 ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SPARSE FACTORIZATION DRIVER.
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF THE SPARSE MATRIX
*  II  M  NUMBER OF PROCESSED COLUMNS THAT FORM THE COLUMS OF THE MATRIX
*         THAT DESCRIBES THE ACTIVE SET. M <= N.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NZA SIZE OF THE INDEX ARRAY RNR
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  II  NEL NUMBER OF L-UPDATES IN THE ARRAYS LI,LJ,CM.
*  II  P  NUMBER OF ACTIVE BOUNDS.
*  II  Q  NUMBER OF COLUMNS OF THE MATRIX OF CONSTRAINTS.
*  IO  PTL(N) POINTER VECTOR OF THE COLUMNS OF THE FACTOR L.
*  IO  LENL(N) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  IO  PTC(N) POINTER VECTOR OF THE COLUMN SCHEME OF THE
*             FACTOR U (ONLY IF IFL.LT.0).
*  IO  LENC(N) LENGTHS OF THE COLUMNS OF U IF ( ONLY IF IFL.LT.0).
*  IO  PTR(N)  POINTER VECTOR OF THE ROWS OF THE FACTORS U.
*  IO  LENR(N) LENGTHS OF THE ROWS OF THE FACTOR U OF THE SPARSE MATRIX.
*  IO  CNR(NZ) CONTAINS THE INDICES OF THE COLUMN AND ROW SCHEME
*              FOR THE FACTORS OF THE SPARSE MATRIX.
*  IO  ORDC(N) ORDER OF THE COLUMNS THAT WILL BE FURTHER UPDATED.
*  IO  ORDR(N) ORDER OF THE ROWS THAT WILL BE FURTHER UPDATED.
*  IO  ORDL(N) ORDER OF THE ROWS OF THIS REFACTORIZATION.
*  II  RNR(NZA) COLUMNS OF THE MATRIX OF CONSTRAINTS.
*  II  PRNR(Q+1) POINTER VECTOR FOR RNR.
*  II  BCOL(N) NUMBERS OF THE BASIC COLUMNS.
*  IO  RIGHT(N) SIZE OF THE ROW SPIKE USED DURING COLUMNS EXCHANGE.
*  RA  WR01(N) AUXILIARY VECTOR
*  RI  A(NZA) NUMERICAL VALUES OF THE MATRIX OF CONSTRAINTS.
*  RU  AU(NAU) ON INPUT : ELEMENTS OF THE PROCESSED MATRIX
*
*  RI  STFL  STABILITY FLAG.
*  RO  MMFL  MAXIMUM MULTIPLIER FOUND.
*  RO  MXU   MAXIMUM ELEMENT OF THE U-DIAGONAL.
*  RO  MNU   MINIMUM ELEMENT OF THE U-DIAGONAL.
*  IO  IBEG  BEGINNING OF THE ROW SCHEME OF THE FACTOR U IN CNR.
*  IO  IBGR  BEGINNING OF THE COLUMN SCHEME IN CNR(IF IFL.EQ.0)
*  IO  IENDLU  END OF THE ROW INDICES OF THE FACTOR U IN CNR.
*  IO  LACK  LACK OF SPACE IN WORDS IF FLAG IS 1 OR 2.
*  RO  GRFL  GROWTH OF THE ELEMENTS IN THE NONFACTORIZED SUBMATRIX.
*  RI  GSFL  BOUND ON THE ELEMENT GROWTH.
*  RI  DRFL  DROPPING PARAMETER.
*  IO  FLAG OUTPUT INFORMATION:
*                              =1 IF NAU IS DECLARED TOO SMALL
*                              =2 IF NZ IS TO BE INCREASED
*                              =3 IF MATRIX IS TOO UNSTABLE
*                              =4 IF MATRIX IS SINGULAR
*                              =5 IF NZ IS TOO SMALL FOR THE FACTORS
*                              =6 IF NAU IS TOO SMALL FOR FACTORS
*                              =7 IF NZ IS TOO SMALL FOR THE DATA
*                                 MANIPULATION AFTER FACTORIZATION
*                              =8 IF COLUMN SCHEME FOR THE FACTOR U
*                                  IS NOT CREATED DUE TO LACK OF SPACE.
*  II  IJOB =1 FOR SOLUTION OF SPARSE LINEAR SYSTEMS, =0 OTHERWISE.
*
      SUBROUTINE UXSGFD(N,M,NZ,NZA,NAU,NEL,P,Q,PTL,LENL,PTC,LENC,PTR,
     & LENR,CNR,ORDC,ORDR,ORDL,RNR,PRNR,BCOL,RIGHT,WR01,A,AU,STFL,MMFL,
     & MXU,MNU,IBEG,IBGR,IENDLU,LACK,GSFL,DRFL,FLAG,IJOB)
      INTEGER N,M,NZ,NZA,NAU,NEL,P,Q,IBEG,IBGR,IENDLU,LACK,IFL,FLAG
      INTEGER PTL(N),LENL(N),PTC(N),LENC(N),PTR(N),LENR(N),IJOB
      INTEGER ORDC(N),ORDR(N),ORDL(N),BCOL(N)
      INTEGER CNR(NZ)
      INTEGER RNR(NZA)
      INTEGER PRNR(Q+1)
      INTEGER RIGHT(*)
      INTEGER NFL,JOB
      REAL*8 STFL,MMFL,MXU,MNU
      REAL*8 A(NZA)
      REAL*8 AU(NAU)
      REAL*8 WR01(N)
      REAL*8 GSFL,DRFL
C
C     NUMBER OF L-TRIPLES IS UPDATED
C
      NEL=0
C
C     FLAGS ARE INITIALIZED
C
      FLAG=0
      IFL=0
      DRFL=1.0D-16
C
C     INITIALIZE THE ARRAYS
C     LENCH IS REPLACED BY PTL
C
      CALL UXSGFI(N,M,Q,NZA,NZ,NAU,RNR,PRNR,LENC,PTC,PTR,LENR,CNR,
     *            BCOL,A,AU,PTL,IBGR,IENDLU,IFL,NFL,IJOB)
      IF(IFL.GT.0) THEN
                   LACK=NFL
                   IF(IFL.EQ.1) THEN
                                FLAG=1
                                ELSE
                                FLAG=2
                   END IF
                   RETURN
      END IF
C
C     SET THE ORDER OF ROWS AND COLUMNS
C     LENCH IS REPLACED BY PTL,PORDR IS REPLACED BY LENL,
C     PORDC IS REPLACED BY RIGHT.
C
      CALL UXSGFS(N,M,P,LENC,PTL,LENR,BCOL,LENL,ORDR,RIGHT,ORDC,IJOB)
C
C     JOB SPECIFICATION 2
C
      JOB=2
C
C     FACTORIZATION
C     LENCH IS REPLACED BY PTL,PORDR IS REPLACED BY LENL,
C     PORDC IS REPLACED BY RIGHT,DIAG IS REPLACED BY WR01.
C
      CALL UXSGFF(N,M,NZ,NAU,P,PTC,LENC,PTL,CNR,PTR,LENR,LENL,ORDR,
     *            RIGHT,ORDC,AU,WR01,IBGR,IENDLU,STFL,GSFL,DRFL,
     *            MMFL,JOB,IFL)
      IF(IFL.GT.0) THEN
                   IF(IFL.EQ.1) FLAG=3
                   IF(IFL.EQ.2) FLAG=4
                   IF(IFL.EQ.3.OR.IFL.EQ.5) FLAG=5
                   IF(IFL.EQ.4) FLAG=6
                   RETURN
      END IF
C
C     FIND VECTOR OF INVERSE PERMUTATION
C     INVC IS REPLACED BY RIGHT,PERM IS REPLACED BY ORDC.
C
      CALL UXVSIP(N,ORDC,RIGHT)
C
C     OUTPUT OF THE FACTORIZATION
C     DIAG IS REPLACED BY WR01.
C
      CALL UXSGFO(N,M,NZ,NAU,PTC,LENC,CNR,PTR,LENR,PTL,LENL,ORDR,ORDC,
     *            RIGHT,AU,ORDL,WR01,IENDLU,IBGR,IFL)
      IF (IFL.GT.0) THEN
                    FLAG=7
                    RETURN
      END IF
      IF (IFL.LT.0) THEN
                    FLAG=8
                    RETURN
      END IF
C
C     INTERFACE FOR THE PRESENT ALGORITHMS FOR COLUMN/ROW UPDATE
C
      CALL UXSGUI(N,NZ,PTR,CNR,IBEG,IBGR)
C
C     END OF THE FACTORIZATION
C
      CALL UXSGFE(N,M,NAU,PTR,AU,MXU,MNU)
      RETURN
      END
