* SUBROUTINE UXSGFT                 ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A TRANSPOSE OF THE
* SPARSE NONSYMMETRIC MATRIX A USING THE FACTORIZATION P A Q = L*U.
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF THE SPARSE MATRIX
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  II  NEL  NUMBER OF TRIPLES IN ARRAYS (LI,LJ,CM).
*  II  NUP  LENGTH OF THE ARRAYS LI,LJ,CM.
*  II  PTL(N) POINTER VECTOR OF THE COLUMNS OF THE L-FACTOR.
*  II  LENL(N) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  II  CNR(NZ) CONTAINS THE INDICES OF THE FACTORS.
*  II  PTR(N) POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  II  LENR(N) LENGTHS OF THE ROWS OF THE FACTOR U.
*  II  ORDC(N) COLUMN PERMUTATION.
*  II  ORDR(N) ROW PERMUTATION.
*  II  ORDL(N) ROW PERMUTATION OF THE ORIGINAL FACTOR L.
*  II  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  II  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RI  CM(NUP) UPDATE MULTIPLIERS.
*  RI  AU(NZ) NUMERICAL VALUES OF THE FACTORS.
*  RU  RHS(N) ON INPUT : RIGHT HAND SIDE OF THE SYSTEM
*             ON OUTPUT: SOLUTION VECTOR.
*  RA  WR01(N)  AUXILIARY VECTOR.
*  II  JOB  OPTION. IF JOB=0 THEN RHS:=TRANS(A)**(-1)*RHS. IF JOB>0 THEN
*         RHS:=TRANS(U)**(-1)*RHS. IF JOB<0 THEN
*         RHS:=TRANS(L)**(-1)*RHS.
*
* METHOD :
* BACK SUBSTITUTION
*
      SUBROUTINE UXSGFT(N,NZ,NAU,NEL,NUP,PTL,LENL,CNR,PTR,
     *                  LENR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,RHS,WR01,JOB)
      INTEGER N,NZ,NAU,NEL,NUP,JOB
      INTEGER PTL(N),PTR(N),LENL(N),LENR(N),ORDR(N),ORDC(N),ORDL(N)
      INTEGER CNR(NZ)
      INTEGER LI(NUP),LJ(NUP)
      REAL*8 CM(NUP)
      REAL*8 AU(NAU)
      REAL*8 RHS(N),WR01(N)
      INTEGER I,J,K,L,JSTRT
C
C     FORM TRANS(Q)*RHS
C
      DO 100 I=1,N
      WR01(I)=RHS(ORDC(I))
100   CONTINUE
C
C     FTRANU
C
C
      IF(JOB.LT.0) GO TO 400
      DO 300 I=1,N
      K=ORDR(I)
      JSTRT=PTR(K)
      IF(WR01(I).EQ.0.0D 0) THEN
                      RHS(I)=0.0D 0
                      ELSE
                      IF(AU(JSTRT).EQ.0.0D 0) AU(JSTRT)=1.0D-20
                      RHS(I)=WR01(I)/AU(JSTRT)
            DO 200 J=JSTRT+1,JSTRT+LENR(K)-1
            L=CNR(J)
            WR01(L)=WR01(L)-RHS(I)*AU(J)
200         CONTINUE
      END IF
300   CONTINUE
C
C     BACKWARD PERMUTATION IN ORDER TO PREPARE INPUT FOR
C     L-UPDATES.
C
      DO 350 I=1,N
      WR01(ORDR(I))=RHS(I)
350   CONTINUE
C
C     L-UPDATES ARE USED
C
      DO 500 I=NEL,1,-1
      WR01(LJ(I))=WR01(LJ(I))-CM(I)*WR01(LI(I))
500   CONTINUE
      IF(JOB.GT.0) GO TO 900
C
C     TRANSFORMATION AFTER L-UPDATES
C
      DO 550 I=1,N
      RHS(I)=WR01(ORDL(I))
550   CONTINUE
      DO 570 I=1,N
      WR01(I)=RHS(I)
570   CONTINUE
C
C     JUMP LABEL IF JOB.LT.0
C
400   CONTINUE
C
C     BTRANU
C
      DO 800 I=N,1,-1
            L=ORDL(I)
            DO 700 J=PTL(I),PTL(I)+LENL(I)-1
            K=CNR(J)
            WR01(I)=WR01(I)-AU(J)*RHS(K)
700         CONTINUE
            RHS(L)=WR01(I)
800         CONTINUE
C
C     JUMP LABEL IF JOB.GT.0
C
900   CONTINUE
      RETURN
      END
