* SUBROUTINE UXSGUM                 ALL SYSTEMS                90/12/01
C PORTABILITY : ALL SYSTEMS
C 90/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SPARSE COLUMN EXCHANGE MAIN.
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF THE SPARSE MATRIX
*  II  M  NUMBER OF PROCESSED COLUMNS THAT FORM THE COLUMS OF THE MATRIX
*         THAT DESCRIBES THE ACTIVE SET. M <= N.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NZA SIZE OF THE INDEX ARRAY RNR
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  II  NEL NUMBER OF L-UPDATES IN THE ARRAYS LI,LJ,CM.
*  II  NUP SIZE OF THE L-UPDATE ARRAYS (LI,LJ,CM)
*  IU  PTR(N) POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  IU  LENR(N) LENGTHS OF THE ROWS OF THE FACTOR U.
*  IU  CNR(NZ) CONTAINS THE INDICES OF THE FACTORS.
*  II  PRNR(N+1) POINTER VECTOR FOR RNR.
*  II  RNR(NZA) COLUMNS OF THE MATRIX OF CONSTRAINTS.
*  IU  ORDR(N) ROW PERMUTATION.
*  II  IDEB  =1 FOR LUSOL UPDATE - OTHERWISE USES PERMUTATION
*            OF THE BUMP.
*  IU  ORDC(N) COLUMN PERMUTATION.
*  II  BCOL(N) COLUMN ORDER IN BLOCKS.
*  II  PTL(N) POINTER VECTOR OF THE COLUMNS OF THE L-FACTOR.
*  II  LENL(N) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  IU  ORDL(N) ROW PERMUTATION OF THE ORIGINAL FACTOR L.
*  RA  WR01(N) AUXILIARY VECTOR
*  RA  WR01(N) AUXILIARY VECTOR
*  IU  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  IU  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RI  CM(NUP) UPDATE MULTIPLIERS.
*  IU  RIGHT(N) RIGHTMOST POSITION OF THE ROW SPIKE FOR BASIS.
*  IU  UP(N) UPPERMOST POSITION OF THE COLUMN SPIKE FOR BASIS.
*  RI  A(NZA) NUMERICAL VALUES OF THE MATRIX OF CONSTRAINTS.
*  RU  AU(NAU) NUMERICAL VALUES OF THE FACTORS.
*  RI  STFL  STABILITY FLAG.
*  RU  MMFL  MAXIMUM MULTIPLIER FOUND.
*  RI  DRFL  DROPPING PARAMETER.
*  RU  MXU   MAXIMUM ELEMENT OF THE U-DIAGONAL.
*  RU  MNU   MINIMUM ELEMENT OF THE U-DIAGONAL.
*  II  CIN INPUT COLUMN NUMBER
*  II  COUT POSITION OF THE OUTPUT COLUMN IN BCOL.
*  IO  IBGR  BEGINNING OF THE COLUMN SCHEME IN CNR(IF IFL.EQ.0 AFTER
*            FACTORIZATION).
*  IO  IENDLU  END OF THE ROW INDICES OF THE FACTOR U IN CNR.
*  II  IBEG  BEGINNING OF THE FACTOR U IN CNR.
*  IO  FLAG OUTPUT INFORMATION:
*                              =11 IF NAU IS DECLARED TOO SMALL
*                              =12 IF MATRIX IS SINGULAR
*                              =13 IF FACTORS ARE BADLY CONDITIONED.
*                              =14 LITTLE SPACE FOR L-UPDATES.
*                              =15 MATRIX SINGULARITY IS FACED.
*
      SUBROUTINE UXSGUM(N,M,NZ,NZA,NAU,NEL,NUP,PTR,LENR,CNR,PRNR,RNR,
     *               ORDR,IDEB,ORDC,BCOL,PTL,LENL,ORDL,WR01,WR02,LI,LJ,
     *               CM,RIGHT,UP,A,AU,STFL,MMFL,DRFL,MXU,MNU,CIN,
     *               COUT,IBGR,IENDLU,IBEG,FLAG)
      INTEGER N,M,NZ,NZA,NAU,NEL,NUP,IDEB,CIN,COUT,COLUP
      INTEGER IBGR,IENDLU,IBEG,FLAG
      INTEGER RIGHT(N),PTR(N),LENR(N),ORDR(N),PRNR(*),UP(N)
      INTEGER ORDC(N),BCOL(N),PTL(N),LENL(N),ORDL(N)
      INTEGER CNR(NZ),RNR(NZA),LI(NUP),LJ(NUP)
      REAL*8 STFL,MMFL,DRFL,MXU,MNU
      REAL*8 WR01(N),WR02(N)
      REAL*8 CM(NUP),A(NZA),AU(NAU)
      INTEGER I,J,K,L,ISTRT,ISTOP,LAST,IQCMPR,J1,J2,J3
C
C     INITIALIZATION
C
      IBGR=NAU
      DRFL=1.0D-25
      FLAG=0
C
C     BCOL UPDATE
C     CIN .....COLUMN NUMBER
C     COUT.....POSITION IN BCOL.
C
      BCOL(COUT)=CIN
      COLUP=COUT
C
C     FIND COLUP
C
      DO 250 I=1,N
      J=ORDC(I)
      IF(COLUP.EQ.J) THEN
                     COLUP=I
                     GO TO 300
      END IF
250   CONTINUE
300   CONTINUE
C
C     WR01 INITIALIZATION
C
      DO 400 I=1,N
      WR01(I)=0
400   CONTINUE
      ISTRT=PRNR(CIN)
      ISTOP=PRNR(CIN+1)-1
      DO 500 I=ISTRT,ISTOP
      J=RNR(I)
      WR01(J)=A(I)
500   CONTINUE
C
C     FIND SPIKE=L**(-1)*CIN
C
      CALL UXSGFL(N,NZ,NAU,NEL,NUP,PTL,LENL,CNR,ORDR,ORDL,LI,LJ,CM,AU,
     *            WR01,WR02)
C
C     FIND LAST
C
      DO 600 I=N,1,-1
      IF(ABS(WR02(I)).GT.DRFL) THEN
                               LAST=I
                               GO TO 700
      END IF
600   CONTINUE
      FLAG=12
      RETURN
700   CONTINUE
      DO 650  I=LAST-1,1,-1
      IF(ABS(WR02(I)).GT.DRFL) THEN
                               UP(COUT)=I
                               GO TO 730
      END IF
650   CONTINUE
      UP(COUT)=0
730   CONTINUE
C
C     COLUMN COUT IS REPLACED BY THE COLUMN CIN
C
      DO 1100 I=1,MAX(LAST,COLUP)
      L=ORDR(I)
      ISTRT=PTR(L)
      ISTOP=ISTRT+LENR(L)-1
      DO 800 J=ISTRT,ISTOP
      IF(CNR(J).EQ.COLUP) THEN
          IF(ABS(WR02(I)).GT.DRFL) THEN
                      AU(J)=WR02(I)
                                   ELSE
              IF(I.EQ.COLUP) THEN
                             AU(J)=WR02(I)
                             ELSE
                      CNR(J)=CNR(ISTOP)
                      AU(J)=AU(ISTOP)
                      CNR(ISTOP)=0
                      LENR(L)=LENR(L)-1
                      IF(RIGHT(L).GE.COLUP) THEN
                      RIGHT(L)=N+1
                      DO 750 K=ISTRT+1,ISTOP-1
                      RIGHT(L)=MIN(RIGHT(L),CNR(K))
750                   CONTINUE
                      END IF
              END IF
          END IF
          GO TO 1100
      END IF
800   CONTINUE
C
C     TEST OF END OF INSERTING
C
      IF(ABS(WR02(I)).GT.DRFL) THEN
900   IF(CNR(ISTOP+1).EQ.0) THEN
                            IQCMPR=0
                            ISTOP=ISTOP+1
                            CNR(ISTOP)=COLUP
                            AU(ISTOP)=WR02(I)
                            IENDLU=MAX(IENDLU,ISTOP)
                            LENR(L)=LENR(L)+1
                            ELSE
                            J1=ISTOP-ISTRT+1
          IF((IENDLU+J1).LT.NAU-1)         THEN
                                  DO 1000 J2=1,J1
                                  J3=ISTRT+J2-1
                                  CNR(J2+IENDLU)=CNR(J3)
                                  AU(J2+IENDLU)=AU(J3)
                                  CNR(J3)=0
1000                               CONTINUE
                                  IQCMPR=0
                                  ISTRT=IENDLU+1
                                  PTR(L)=ISTRT
                                  ISTOP=IENDLU+J1
                                  GO TO 900
                                           ELSE
              IF(IQCMPR.EQ.1) THEN
                              FLAG=11
                              RETURN
              END IF
              IQCMPR=1
              CALL UXSGUG(N,M,NZ,NAU,PTR,LENR,CNR,
     *                    AU,IBEG,IENDLU)
              ISTRT=PTR(L)
              ISTOP=ISTRT+LENR(L)-1
              GO TO 900
          END IF
      END IF
      RIGHT(L)=MIN(COLUP,RIGHT(L))
      END IF
1100  CONTINUE
C
C     UPDATE UP
C
      IF(LAST.NE.COLUP) THEN
                        UP(COUT)=LAST
      END IF
C
C     TEST OF COLUMN EXCHANGE
C
C
C     CALL TEST
C
      IF(LAST.LT.COLUP) THEN
                        FLAG=12
                        RETURN
      END IF
      IF(LAST.GT.COLUP) CALL UXSGUC(N,M,NZ,NAU,NEL,NUP,PTR,LENR,CNR,
     *                       ORDR,IDEB,ORDC,RIGHT,UP,WR01,WR02,LI,LJ,CM,
     *                       AU,STFL,MMFL,DRFL,MXU,MNU,COLUP,LAST,
     *                       IENDLU,IBEG,FLAG)
      RETURN
      END
