* SUBROUTINE UXSIF2                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DU : ORIGINAL VERSION
*
* PURPOSE :
* AUXILIARY SUBROUTINE FOR UXSIBF.
* FACTORIZATION SUBROUTINE
*
      SUBROUTINE UXSIF2(N, M, A, LA, IW, MW, PERM, NSTK, NS,
     & MF, NELIM, IW2, INF)
C
C     THIS SUBROUTINE OPERATES ON THE INPUT MATRIX ORDERED BY UXSIF1
C     AND PRODUCES THE FACTORS OF U AND D ('A'=UTRANSPOSE*D*U) FOR USE
C     IN SUBSEQUENT SOLUTIONS.  GAUSSIAN ELIMINATION IS USED WITH PIVOTS
C     CHOSEN FROM THE DIAGONAL.  TO ENSURE STABILITY, BLOCK PIVOTS OF
C     ORDER 2 WILL BE USED IF THE DIAGONAL ENTRY IS NOT LARGE ENOUGH.
C
      INTEGER N, M, LA, MW, MF, INF
      INTEGER NS, POSFAC, ASTK, ASTK2, APOS, APOS1, APOS2, APOS3,
     & AINPUT, PIVSIZ, POSPV1, POSPV2, AZERO, IW2(N)
      INTEGER IW(MW),PERM(N),NSTK(NS),NELIM(NS)
      DOUBLE PRECISION A(LA), ZERO, TMAX, AMAX, RMAX, SWOP, DETPIV,
     & ONE, AMULT, AMULT1, AMULT2, THRESH, HALF
C     N - MUST BE SET TO THE ORDER OF THE MATRIX. IT IS NOT ALTERED.
C     M - MUST BE SET TO THE NUMBER OF NON-ZEROS IN UPPER TRIANGLE OF
C         PERMUTED MATRIX.  NOT ALTERED BY UXSIF2.
C     A - MUST BE SET ON INPUT TO MATRIX HELD BY ROWS REORDERED BY
C         PERMUTATION FROM UXSIBS IN A(LA-M+I),I=1,M.   ON
C         EXIT FROM UXSIF2, THE FACTORS OF U AND D ARE HELD IN
C         POSITIONS 1 TO POSFAC-1.
C     LA- LENGTH OF ARRAY A.  A VALUE FOR LA
C         SUFFICIENT FOR DEFINITE SYSTEMS
C         WILL HAVE BEEN PROVIDED BY UXSIBS. NOT ALTERED BY UXSIF2.
C     IW- MUST BE SET SO THAT,ON INPUT, IW(MW-M+I),I=1,M
C         HOLDS THE COLUMN INDEX OF THE ENTRY IN A(LA-M+I).  ON EXIT,
C         IW HOLDS INTEGER INDEXING INFORMATION ON THE FACTORS.
C         THE ABSOLUTE VALUE OF THE FIRST ENTRY IN IW WILL BE SET TO
C         THE NUMBER OF BLOCK PIVOTS ACTUALLY USED.  THIS MAY BE
C         DIFFERENT FROM NS SINCE NUMERICAL CONSIDERATIONS
C         MAY PREVENT US CHOOSING A PIVOT AT EACH STAGE.  IF THIS ENTRY
C         IN IW IS NEGATIVE, THEN AT LEAST ONE TWO BY TWO
C         PIVOT HAS BEEN USED DURING THE DECOMPOSITION.
C         INTEGER INFORMATION ON EACH BLOCK PIVOT ROW FOLLOWS.  FOR
C         EACH BLOCK PIVOT ROW THE COLUMN INDICES ARE PRECEDED BY A
C         COUNT OF THE NUMBER OF ROWS AND COLUMNS IN THE BLOCK PIVOT
C         WHERE, IF ONLY ONE ROW IS PRESENT, ONLY THE NUMBER OF
C         COLUMNS TOGETHER WITH A NEGATIVE FLAG IS HELD.  THE FIRST
C         COLUMN INDEX FOR A TWO BY TWO PIVOT IS FLAGGED NEGATIVE.
C     MW- LENGTH OF ARRAY IW.  A VALUE FOR MW SUFFICIENT FOR
C         DEFINITE SYSTEMS
C         WILL HAVE BEEN PROVIDED BY UXSIBS.  NOT ALTERED BY UXSIF2
C     PERM- PERM(I) MUST BE SET TO POSITION OF ROW/COLUMN I IN THE
C         TENTATIVE PIVOT ORDER GENERATED BY UXSIBS.
C         IT IS NOT ALTERED BY UXSIF2.
C     NSTK- MUST BE LEFT UNCHANGED SINCE OUTPUT FROM UXSIBS. NSTK(I)
C         GIVES THE NUMBER OF GENERATED STACK ELEMENTS ASSEMBLED AT
C         STAGE I.  IT IS NOT ALTERED BY UXSIF2.
C     NS- LENGTH OF ARRAYS NSTK AND NELIM. VALUE IS GIVEN ON OUTPUT
C         FROM UXSIBS (WILL NEVER EXCEED N). IT IS NOT ALTERED BY
C         UXSIF2.
C     MF- NEED NOT BE SET ON INPUT.  ON OUTPUT
C         MF WILL BE SET TO THE MAXIMUM FRONT SIZE ENCOUNTERED
C         DURING THE DECOMPOSITION.
C     NELIM- MUST BE UNCHANGED SINCE OUTPUT FROM UXSIBS. NELIM(I)
C         GIVES THE NUMBER OF ORIGINAL ROWS ASSEMBLED AT STAGE I.
C         IT IS NOT ALTERED BY UXSIF2.
C     IW2 - INTEGER ARRAY OF LENGTH N. USED AS WORKSPACE BY UXSIF2.
C         ALTHOUGH WE COULD HAVE USED A SHORT WORD INTEGER IN THE IBM
C         VERSION, WE HAVE NOT DONE SO BECAUSE THERE IS A SPARE
C         FULL INTEGER ARRAY (USED IN THE SORT BEFORE UXSIF2)
C         AVAILABLE WHEN UXSIF2 IS CALLED FROM UXSIBF.
C     INF - INTEGER VARIABLE.  DIAGNOSTIC FLAG.  A ZERO VALUE ON EXIT
C         INDICATES SUCCESS.  POSSIBLE NEGATIVE VALUES ARE ...
C         -3  INSUFFICIENT STORAGE FOR IW.
C         -4  INSUFFICIENT STORAGE FOR A.
C         -5  ZERO PIVOT FOUND IN FACTORIZATION OF DEFINITE MATRIX.
C
      INTEGER I, IASS, IBEG,IDUMMY, IELL, IEND, IERROR,  IEXCH,
     &        IFR, IINPUT, IOLDPS, IPMNP,
     &        IORG, IPIV, IPOS,  IROW, ISNPIV,  ISTK, ISTK2,
     &        ISWOP, IWPOS,
     &        J, J1, J2, JCOL, JDUMMY, JFIRST, JJ, JJ1,JJJ,JLAST, JMAX,
     &        JMXMIP, JNEW, JNEXT, JPIV, JPOS, K, KDUMMY,  KROW,
     &        K1, K2, KMAX, KK,
     &        LAELL, LAPOS2, LDIAG, LIELL,  LP, LNASS, LNPIV,
     &        LT, LTOPST,  MP,
     &        NASS, NBLK, NCMPA, NCMPBI, NCMPBR, NEIG,
     &        NEWEL,  NF,
     &        NIRADU, NIRBDU, NIRNEC, NIRTOT,  NPIV,
     &        NPIVP1, NRLADU,  NRLBDU,
     &        NRLNEC, NRLTOT,  NTOTPV, NTWO, NUMASS, NUMORG, NUMSTK
      DOUBLE PRECISION U, UU, OPS
      INTRINSIC MIN0, MAX0, DABS, DMAX1, DMIN1
      COMMON /UMMAT1/ U, LP, MP, LDIAG
      COMMON /UMMAT2/ OPS, IERROR, NRLTOT, NIRTOT, NRLNEC, NIRNEC,
     & NRLADU, NIRADU, NRLBDU, NIRBDU, NCMPA, NCMPBR, NCMPBI, NTWO,NEIG
      INTEGER  IX,IY,IDIAG
      DATA ZERO /0.0D0/, HALF /0.5D0/, ONE /1.0D0/
C
C     THE FOLLOWING ARITHMETIC FUNCTION GIVES THE DISPLACEMENT FROM
C     THE START OF THE ASSEMBLED MATRIX(OF ORDER IX) OF THE DIAGONAL
C     ENTRY IN ITS ROW IY.
C
      IDIAG(IX,IY) = ((IY-1)*(2*IX-IY+2))/2
C
C     INITIALIZATION.
C     NBLK IS THE NUMBER OF BLOCK PIVOTS USED.
C
      NBLK = 0
      NTWO = 0
      NEIG = 0
      NCMPBI = 0
      NCMPBR = 0
      MF = 0
C
C     A PRIVATE VARIABLE UU IS SET TO U, SO THAT U WILL REMAIN
C     UNALTERED.
C
      UU = DMIN1(U,HALF)
      UU = DMAX1(UU,-HALF)
      DO 10 I=1,N
        IW2(I) = 0
   10 CONTINUE
C
C     IWPOS IS POINTER TO FIRST FREE POSITION FOR FACTORS IN IW.
C     POSFAC IS POINTER FOR FACTORS IN A. AT EACH PASS THROUGH THE
C     MAJOR LOOP POSFAC INITIALLY POINTS TO THE FIRST FREE LOCATION
C     IN A AND THEN IS SET TO THE POSITION OF THE CURRENT PIVOT IN A.
C     ISTK IS POINTER TO TOP OF STACK IN IW.
C     ISTK2 IS POINTER TO BOTTOM OF STACK IN IW (NEEDED BY COMPRESS).
C     ASTK IS POINTER TO TOP OF STACK IN A.
C     ASTK2 IS POINTER TO BOTTOM OF STACK IN A (NEEDED BY COMPRESS).
C     IINPUT IS POINTER TO CURRENT POSITION IN ORIGINAL ROWS IN IW.
C     AINPUT IS POINTER TO CURRENT POSITION IN ORIGINAL ROWS IN A.
C     AZERO IS POINTER TO LAST POSITION ZEROED IN A.
C     NTOTPV IS THE TOTAL NUMBER OF PIVOTS SELECTED. THIS IS USED
C     TO DETERMINE WHETHER THE MATRIX IS SINGULAR.
C
      IWPOS = 2
      POSFAC = 1
      ISTK = MW - M + 1
      ISTK2 = ISTK - 1
      ASTK = LA - M + 1
      ASTK2 = ASTK - 1
      IINPUT = ISTK
      AINPUT = ASTK
      AZERO = 0
      NTOTPV = 0
C
C     NUMASS IS THE ACCUMULATED NUMBER OF ROWS ASSEMBLED SO FAR.
C
      NUMASS = 0
C
C     EACH PASS THROUGH THIS MAIN LOOP PERFORMS ALL THE OPERATIONS
C     ASSOCIATED WITH ONE SET OF ASSEMBLY/ELIMINATIONS.
C
      DO 760 IASS=1,NS
C
C     NASS WILL BE SET TO THE NUMBER OF FULLY ASSEMBLED VARIABLES IN
C     CURRENT NEWLY CREATED ELEMENT.
C
        NASS = NELIM(IASS)
C
C     NEWEL IS A POINTER INTO IW TO CONTROL OUTPUT OF INTEGER
C     INFORMATION FOR NEWLY CREATED ELEMENT.
C
        NEWEL = IWPOS + 1
C
C     SYMBOLICALLY ASSEMBLE INCOMING ROWS AND GENERATED STACK ELEMENTS
C     ORDERING THE RESULTANT ELEMENT ACCORDING TO PERMUTATION PERM.  WE
C     ASSEMBLE THE STACK ELEMENTS FIRST BECAUSE THESE WILL ALREADY BE
C     ORDERED.
C     SET HEADER POINTER FOR MERGE OF INDEX LISTS.
C
        JFIRST = N + 1
C
C     INITIALIZE NUMBER OF VARIABLES IN CURRENT FRONT.
C
        NF = 0
        NUMSTK = NSTK(IASS)
        LTOPST = 1
        LNASS = 0
C
C     JUMP IF NO STACK ELEMENTS ARE BEING ASSEMBLED AT THIS STAGE.
C
        IF (NUMSTK.EQ.0) GO TO 80
        J2 = ISTK - 1
        LNASS = NASS
        LTOPST = ((IW(ISTK)+1)*IW(ISTK))/2
        DO 70 IELL=1,NUMSTK
C
C     ASSEMBLE ELEMENT IELL PLACING
C     THE INDICES INTO A LINKED LIST IN IW2 ORDERED
C     ACCORDING TO PERM.
C
          JNEXT = JFIRST
          JLAST = N + 1
          J1 = J2 + 2
          J2 = J1 - 1 + IW(J1-1)
C
C     RUN THROUGH INDEX LIST OF STACK ELEMENT IELL.
C
          DO 60 JJ=J1,J2
            J = IW(JJ)
C
C     JUMP IF ALREADY ASSEMBLED
C
            IF (IW2(J).GT.0) GO TO 60
            JNEW = PERM(J)
C
C     IF VARIABLE WAS PREVIOUSLY FULLY SUMMED BUT WAS NOT PIVOTED ON
C     EARLIER BECAUSE OF NUMERICAL TEST, INCREMENT NUMBER OF FULLY
C     SUMMED ROWS/COLUMNS IN FRONT.
C
            IF (JNEW.LE.NUMASS) NASS = NASS + 1
C
C     FIND POSITION IN LINKED LIST FOR NEW VARIABLE.  NOTE THAT WE START
C     FROM WHERE WE LEFT OFF AFTER ASSEMBLY OF PREVIOUS VARIABLE.
C
            DO 20 IDUMMY=1,N
              IF (JNEXT.EQ.N+1) GO TO 30
              IF (PERM(JNEXT).GT.JNEW) GO TO 30
              JLAST = JNEXT
              JNEXT = IW2(JLAST)
   20       CONTINUE
   30       IF (JLAST.NE.N+1) GO TO 40
            JFIRST = J
            GO TO 50
   40       IW2(JLAST) = J
   50       IW2(J) = JNEXT
            JLAST = J
C
C     INCREMENT NUMBER OF VARIABLES IN THE FRONT.
C
            NF = NF + 1
   60     CONTINUE
   70   CONTINUE
        LNASS = NASS - LNASS
C
C     NOW INCORPORATE ORIGINAL ROWS.  NOTE THAT THE COLUMNS IN THESE
C     ROWS NEED NOT BE IN ORDER. WE ALSO PERFORM
C     A SWOP SO THAT THE DIAGONAL ENTRY IS THE FIRST IN ITS
C     ROW.  THIS ALLOWS US TO AVOID STORING THE INVERSE OF ARRAY PERM.
C
   80   NUMORG = NELIM(IASS)
        J1 = IINPUT
        DO 150 IORG=1,NUMORG
          J = -IW(J1)
          DO 140 IDUMMY=1,MW
            JNEW = PERM(J)
C
C     JUMP IF VARIABLE ALREADY INCLUDED.
C
            IF (IW2(J).GT.0) GO TO 130
C
C     HERE WE MUST ALWAYS START OUR SEARCH AT THE BEGINNING.
C
            JLAST = N + 1
            JNEXT = JFIRST
            DO 90 JDUMMY=1,N
              IF (JNEXT.EQ.N+1) GO TO 100
              IF (PERM(JNEXT).GT.JNEW) GO TO 100
              JLAST = JNEXT
              JNEXT = IW2(JLAST)
   90       CONTINUE
  100       IF (JLAST.NE.N+1) GO TO 110
            JFIRST = J
            GO TO 120
  110       IW2(JLAST) = J
  120       IW2(J) = JNEXT
C
C     INCREMENT NUMBER OF VARIABLES IN FRONT.
C
            NF = NF + 1
  130       J1 = J1 + 1
            IF (J1.GT.MW) GO TO 150
            J = IW(J1)
            IF (J.LT.0) GO TO 150
  140     CONTINUE
  150   CONTINUE
C
C     NOW RUN THROUGH LINKED LIST IW2 PUTTING INDICES OF VARIABLES IN
C     NEW ELEMENT INTO IW AND SETTING IW2 ENTRY TO POINT TO THE RELATIVE
C     POSITION OF THE VARIABLE IN THE NEW ELEMENT.
C
        IF (NEWEL+NF.LT.ISTK) GO TO 160
C
C     COMPRESS IW.
C
        CALL UXSIF3(A, IW, ISTK, ISTK2, IINPUT, 2)
        IF (NEWEL+NF.LT.ISTK) GO TO 160
        IERROR = MW + 1 + NEWEL + NF - ISTK
        GO TO 770
  160   J = JFIRST
        DO 170 IFR=1,NF
          NEWEL = NEWEL + 1
          IW(NEWEL) = J
          JNEXT = IW2(J)
          IW2(J) = NEWEL - (IWPOS+1)
          J = JNEXT
  170   CONTINUE
C
C     ASSEMBLE REALS INTO FRONTAL MATRIX.
C
        MF = MAX0(MF,NF)
        IW(IWPOS) = NF
C
C     FIRST ZERO OUT FRONTAL MATRIX AS APPROPRIATE FIRST CHECKING TO SEE
C     IF THERE IS SUFFICIENT SPACE.
C
        LAELL = ((NF+1)*NF)/2
        APOS2 = POSFAC + LAELL - 1
        IF (NUMSTK.NE.0) LNASS = LNASS*(2*NF-LNASS+1)/2
        IF (POSFAC+LNASS-1.GE.ASTK) GO TO 180
        IF (APOS2.LT.ASTK+LTOPST-1) GO TO 190
C
C     COMPRESS A.
C
  180   CALL UXSIF3(A, IW, ASTK, ASTK2, AINPUT, 1)
        IF (POSFAC+LNASS-1.GE.ASTK) GO TO 780
        IF (APOS2.GE.ASTK+LTOPST-1) GO TO 780
  190   IF (APOS2.LE.AZERO) GO TO 220
        APOS = AZERO + 1
        LAPOS2 = MIN0(APOS2,ASTK-1)
        IF (LAPOS2.LT.APOS) GO TO 210
        DO 200 K=APOS,LAPOS2
          A(K) = ZERO
  200   CONTINUE
  210   AZERO = APOS2
C
C     JUMP IF THERE ARE NO STACK ELEMENTS TO ASSEMBLE.
C
  220   IF (NUMSTK.EQ.0) GO TO 260
C
C     PLACE REALS CORRESPONDING TO STACK ELEMENTS IN CORRECT POSITIONS IN A.
C
        DO 250 IELL=1,NUMSTK
          J1 = ISTK + 1
          J2 = ISTK + IW(ISTK)
          DO 240 JJ=J1,J2
            IROW = IW(JJ)
            IROW = IW2(IROW)
            APOS = POSFAC + IDIAG(NF,IROW)
            DO 230 JJJ=JJ,J2
              J = IW(JJJ)
              APOS2 = APOS + IW2(J) - IROW
              A(APOS2) = A(APOS2) + A(ASTK)
              A(ASTK) = ZERO
              ASTK = ASTK + 1
  230       CONTINUE
  240     CONTINUE
C
C     INCREMENT STACK POINTER.
C
          ISTK = J2 + 1
  250   CONTINUE
C
C     INCORPORATE REALS FROM ORIGINAL ROWS.
C
  260   DO 280 IORG=1,NUMORG
          J = -IW(IINPUT)
C
C     WE CAN DO THIS BECAUSE THE DIAGONAL IS NOW THE FIRST ENTRY.
C
          IROW = IW2(J)
          APOS = POSFAC + IDIAG(NF,IROW)
C
C     THE FOLLOWING LOOP GOES FROM 1 TO M BECAUSE THERE MAY BE DUPLICATES.
C
          DO 270 IDUMMY=1,M
            APOS2 = APOS + IW2(J) - IROW
            A(APOS2) = A(APOS2) + A(AINPUT)
            AINPUT = AINPUT + 1
            IINPUT = IINPUT + 1
            IF (IINPUT.GT.MW) GO TO 280
            J = IW(IINPUT)
            IF (J.LT.0) GO TO 280
  270     CONTINUE
  280   CONTINUE
C
C     RESET IW2 AND NUMASS.
C
        NUMASS = NUMASS + NUMORG
        J1 = IWPOS + 2
        J2 = IWPOS + NF + 1
        DO 290 K=J1,J2
          J = IW(K)
          IW2(J) = 0
  290   CONTINUE
C
C     PERFORM PIVOTING ON ASSEMBLED ELEMENT.
C     NPIV IS THE NUMBER OF PIVOTS SO FAR SELECTED.
C     LNPIV IS THE NUMBER OF PIVOTS SELECTED AFTER THE LAST PASS THROUGH
C     THE THE FOLLOWING LOOP.
C
        LNPIV = -1
        NPIV = 0
        DO 650 KDUMMY=1,NASS
          IF (NPIV.EQ.NASS) GO TO 660
          IF (NPIV.EQ.LNPIV) GO TO 660
          LNPIV = NPIV
          NPIVP1 = NPIV + 1
C
C     JPIV IS USED AS A FLAG TO INDICATE WHEN 2 BY 2 PIVOTING HAS
C     OCCURRED SO THAT IPIV IS INCREMENTED CORRECTLY.
C
          JPIV = 1
C
C     NASS IS MAXIMUM POSSIBLE NUMBER OF PIVOTS.
C     WE EITHER TAKE THE DIAGONAL ENTRY OR THE 2 BY 2 PIVOT WITH THE
C     LARGEST OFF-DIAGONAL AT EACH STAGE.
C     EACH PASS THROUGH THIS LOOP TRIES TO CHOOSE ONE PIVOT.
C
          DO 640 IPIV=NPIVP1,NASS
            JPIV = JPIV - 1
C
C     JUMP IF WE HAVE JUST PROCESSED A 2 BY 2 PIVOT.
C
            IF (JPIV.EQ.1) GO TO 640
            APOS = POSFAC + IDIAG(NF-NPIV,IPIV-NPIV)
C
C     IF THE USER HAS INDICATED THAT THE MATRIX IS DEFINITE, WE
C     DO NOT NEED TO TEST FOR STABILITY BUT WE DO CHECK TO SEE IF THE
C     PIVOT IS NON-ZERO OR HAS CHANGED SIGN.
C     IF IT IS ZERO, WE EXIT WITH AN ERROR. IF IT HAS CHANGED SIGN
C     AND U WAS SET NEGATIVE, THEN WE AGAIN EXIT IMMEDIATELY.  IF THE
C     PIVOT CHANGES SIGN AND U WAS ZERO, WE CONTINUE WITH THE
C     FACTORIZATION BUT PRINT A WARNING MESSAGE ON UNIT MP.
C     ISNPIV HOLDS A FLAG FOR THE SIGN OF THE PIVOTS TO DATE SO THAT
C     A SIGN CHANGE WHEN DECOMPOSING AN ALLEGEDLY DEFINITE MATRIX CAN
C     BE DETECTED.
C
            IF (UU.GT.ZERO) GO TO 320
            IF (A(APOS).EQ.ZERO) GO TO 790
C
C     JUMP IF THIS IS NOT THE FIRST PIVOT TO BE SELECTED.
C
            IF (NTOTPV.GT.0) GO TO 300
C
C     SET ISNPIV.
C
            IF (A(APOS).GT.ZERO) ISNPIV = 1
            IF (A(APOS).LT.ZERO) ISNPIV = -1
  300       IF (A(APOS).GT.ZERO .AND. ISNPIV.EQ.1) GO TO 560
            IF (A(APOS).LT.ZERO .AND. ISNPIV.EQ.-1) GO TO 560
            IF (INF.NE.2) IERROR = 0
            IERROR = IERROR + 1
            INF = 2
            I = NTOTPV + 1
            IF (IERROR.LE.10) CALL UOERR1('UXSIF2',74)
            ISNPIV = -ISNPIV
            IF (UU.EQ.ZERO) GO TO 560
            GO TO 800
  320       AMAX = ZERO
            TMAX = AMAX
C
C     FIND LARGEST ENTRY TO RIGHT OF DIAGONAL IN ROW OF PROSPECTIVE
C     PIVOT IN THE FULLY-SUMMED PART. ALSO RECORD COLUMN OF THIS
C     LARGEST ENTRY.
C
            J1 = APOS + 1
            J2 = APOS + NASS - IPIV
            IF (J2.LT.J1) GO TO 340
            DO 330 JJ=J1,J2
              IF (DABS(A(JJ)).LE.AMAX) GO TO 330
              JMAX = IPIV + JJ - J1 + 1
              AMAX = DABS(A(JJ))
  330       CONTINUE
C
C     DO SAME AS ABOVE FOR NON-FULLY-SUMMED PART ONLY HERE WE DO NOT
C     NEED TO RECORD COLUMN SO LOOP IS SIMPLER.
C
  340       J1 = J2 + 1
            J2 = APOS + NF - IPIV
            IF (J2.LT.J1) GO TO 360
            DO 350 JJ=J1,J2
              TMAX = DMAX1(DABS(A(JJ)),TMAX)
  350       CONTINUE
C
C     NOW CALCULATE LARGEST ENTRY IN OTHER PART OF ROW.
C
  360       RMAX = DMAX1(TMAX,AMAX)
            APOS1 = APOS
            KK = NF - IPIV
            LT = IPIV - (NPIV+1)
            IF (LT.EQ.0) GO TO 380
            DO 370 K=1,LT
              KK = KK + 1
              APOS1 = APOS1 - KK
              RMAX = DMAX1(RMAX,DABS(A(APOS1)))
  370       CONTINUE
C
C     JUMP IF STABILITY TEST SATISFIED.
C
  380       IF (DABS(A(APOS)).GT.UU*RMAX) GO TO 450
C
C     CHECK BLOCK PIVOT OF ORDER 2 FOR STABILITY.
C
            IF (AMAX.EQ.ZERO) GO TO 640
            APOS2 = POSFAC + IDIAG(NF-NPIV,JMAX-NPIV)
            DETPIV = A(APOS)*A(APOS2) - AMAX*AMAX
            THRESH = DABS(DETPIV)
C
C     SET THRESH TO U TIMES THE RECIPROCAL OF THE MAX-NORM OF THE
C     INVERSE OF THE PROSPECTIVE BLOCK.
C
            THRESH = THRESH/(UU*DMAX1(DABS(A(APOS))+AMAX,DABS(A(APOS2))
     &       +AMAX))
C
C     CHECK 2 BY 2 PIVOT FOR STABILITY.
C     FIRST CHECK AGAINST ROW IPIV.
C
            IF (THRESH.LE.RMAX) GO TO 640
C
C     FIND LARGEST ENTRY IN ROW JMAX.
C     FIND MAXIMUM TO THE RIGHT OF THE DIAGONAL.
C
            RMAX = ZERO
            J1 = APOS2 + 1
            J2 = APOS2 + NF - JMAX
            IF (J2.LT.J1) GO TO 400
            DO 390 JJ=J1,J2
              RMAX = DMAX1(RMAX,DABS(A(JJ)))
  390       CONTINUE
C
C     NOW CHECK TO THE LEFT OF THE DIAGONAL.
C     WE USE TWO LOOPS TO AVOID TESTING FOR ROW IPIV INSIDE THE LOOP.
C
  400       KK = NF - JMAX + 1
            APOS3 = APOS2
            JMXMIP = JMAX - IPIV - 1
            IF (JMXMIP.EQ.0) GO TO 420
            DO 410 K=1,JMXMIP
              APOS2 = APOS2 - KK
              KK = KK + 1
              RMAX = DMAX1(RMAX,DABS(A(APOS2)))
  410       CONTINUE
  420       IPMNP = IPIV - NPIV - 1
            IF (IPMNP.EQ.0) GO TO 440
            APOS2 = APOS2 - KK
            KK = KK + 1
            DO 430 K=1,IPMNP
              APOS2 = APOS2 - KK
              KK = KK + 1
              RMAX = DMAX1(RMAX,DABS(A(APOS2)))
  430       CONTINUE
  440       IF (THRESH.LE.RMAX) GO TO 640
            PIVSIZ = 2
            GO TO 460
  450       PIVSIZ = 1
  460       IROW = IPIV - NPIV
C
C     PIVOT HAS BEEN CHOSEN.  IF BLOCK PIVOT OF ORDER 2, PIVSIZ IS EQUAL
C     TO TWO OTHERWISE PIVSIZ EQUALS ONE..
C     THE FOLLOWING LOOP MOVES THE PIVOT BLOCK TO THE TOP LEFT HAND
C     CORNER OF THE FRONTAL MATRIX.
C
            DO 550 KROW=1,PIVSIZ
C
C     WE JUMP IF SWOP IS NOT NECESSARY.
C
              IF (IROW.EQ.1) GO TO 530
              J1 = POSFAC + IROW
              J2 = POSFAC + NF - (NPIV+1)
              IF (J2.LT.J1) GO TO 480
              APOS2 = APOS + 1
C
C     SWOP PORTION OF ROWS WHOSE COLUMN INDICES ARE GREATER THAN LATER
C     ROW.
C
              DO 470 JJ=J1,J2
                SWOP = A(APOS2)
                A(APOS2) = A(JJ)
                A(JJ) = SWOP
                APOS2 = APOS2 + 1
  470         CONTINUE
  480         J1 = POSFAC + 1
              J2 = POSFAC + IROW - 2
              APOS2 = APOS
              KK = NF - (IROW+NPIV)
              IF (J2.LT.J1) GO TO 500
C
C     SWOP PORTION OF ROWS/COLUMNS WHOSE INDICES LIE BETWEEN THE TWO
C     ROWS.
C
              DO 490 JJJ=J1,J2
                JJ = J2 - JJJ + J1
                KK = KK + 1
                APOS2 = APOS2 - KK
                SWOP = A(APOS2)
                A(APOS2) = A(JJ)
                A(JJ) = SWOP
  490         CONTINUE
  500         IF (NPIV.EQ.0) GO TO 520
              APOS1 = POSFAC
              KK = KK + 1
              APOS2 = APOS2 - KK
C
C     SWOP PORTION OF COLUMNS WHOSE INDICES ARE LESS THAN EARLIER ROW.
C
              DO 510 JJ=1,NPIV
                KK = KK + 1
                APOS1 = APOS1 - KK
                APOS2 = APOS2 - KK
                SWOP = A(APOS2)
                A(APOS2) = A(APOS1)
                A(APOS1) = SWOP
  510         CONTINUE
C
C     SWOP DIAGONALS AND INTEGER INDEXING INFORMATION
C
  520         SWOP = A(APOS)
              A(APOS) = A(POSFAC)
              A(POSFAC) = SWOP
              IPOS = IWPOS + NPIV + 2
              IEXCH = IWPOS + IROW + NPIV + 1
              ISWOP = IW(IPOS)
              IW(IPOS) = IW(IEXCH)
              IW(IEXCH) = ISWOP
  530         IF (PIVSIZ.EQ.1) GO TO 550
C
C     SET VARIABLES FOR THE SWOP OF SECOND ROW OF BLOCK PIVOT.
C
              IF (KROW.EQ.2) GO TO 540
              IROW = JMAX - (NPIV+1)
              JPOS = POSFAC
              POSFAC = POSFAC + NF - NPIV
              NPIV = NPIV + 1
              APOS = APOS3
              GO TO 550
C
C     RESET VARIABLES PREVIOUSLY SET FOR SECOND PASS.
C
  540         NPIV = NPIV - 1
              POSFAC = JPOS
  550       CONTINUE
            IF (PIVSIZ.EQ.2) GO TO 600
C
C     PERFORM THE ELIMINATION USING ENTRY (IPIV,IPIV) AS PIVOT.
C     WE STORE U AND DINVERSE.
C
  560       A(POSFAC) = ONE/A(POSFAC)
            IF(A(POSFAC).LT.ZERO) NEIG=NEIG+1
            J1 = POSFAC + 1
            J2 = POSFAC + NF - (NPIV+1)
            IF (J2.LT.J1) GO TO 590
            IBEG = J2 + 1
            DO 580 JJ=J1,J2
              AMULT = -A(JJ)*A(POSFAC)
              IEND = IBEG + NF - (NPIV+JJ-J1+2)
              DO 570 IROW=IBEG,IEND
                JCOL = JJ + IROW - IBEG
                A(IROW) = A(IROW) + AMULT*A(JCOL)
  570         CONTINUE
              IBEG = IEND + 1
              A(JJ) = AMULT
  580       CONTINUE
  590       NPIV = NPIV + 1
            NTOTPV = NTOTPV + 1
            JPIV = 1
            POSFAC = POSFAC + NF - NPIV + 1
            GO TO 640
C
C     PERFORM ELIMINATION USING BLOCK PIVOT OF ORDER TWO.
C     REPLACE BLOCK PIVOT BY ITS INVERSE.
C     SET FLAG TO INDICATE USE OF 2 BY 2 PIVOT IN IW.
C
  600       IPOS = IWPOS + NPIV + 2
            NTWO = NTWO + 1
            IW(IPOS) = -IW(IPOS)
            POSPV1 = POSFAC
            POSPV2 = POSFAC + NF - NPIV
            SWOP = A(POSPV2)
            IF(DETPIV.LT.ZERO) NEIG=NEIG+1
            IF(DETPIV.GT.ZERO .AND. SWOP.LT.ZERO) NEIG=NEIG+2
            A(POSPV2) = A(POSPV1)/DETPIV
            A(POSPV1) = SWOP/DETPIV
            A(POSPV1+1) = -A(POSPV1+1)/DETPIV
            J1 = POSPV1 + 2
            J2 = POSPV1 + NF - (NPIV+1)
            IF (J2.LT.J1) GO TO 630
            JJ1 = POSPV2
            IBEG = POSPV2 + NF - (NPIV+1)
            DO 620 JJ=J1,J2
              JJ1 = JJ1 + 1
              AMULT1 = -(A(POSPV1)*A(JJ)+A(POSPV1+1)*A(JJ1))
              AMULT2 = -(A(POSPV1+1)*A(JJ)+A(POSPV2)*A(JJ1))
              IEND = IBEG + NF - (NPIV+JJ-J1+3)
              DO 610 IROW=IBEG,IEND
                K1 = JJ + IROW - IBEG
                K2 = JJ1 + IROW - IBEG
                A(IROW) = A(IROW) + AMULT1*A(K1) + AMULT2*A(K2)
  610         CONTINUE
              IBEG = IEND + 1
              A(JJ) = AMULT1
              A(JJ1) = AMULT2
  620       CONTINUE
  630       NPIV = NPIV + 2
            NTOTPV = NTOTPV + 2
            JPIV = 2
            POSFAC = POSPV2 + NF - NPIV + 1
  640     CONTINUE
  650   CONTINUE
C
C     END OF MAIN ELIMINATION LOOP.
C
  660   IF (NPIV.NE.0) NBLK = NBLK + 1
        IOLDPS = IWPOS
        IWPOS = IWPOS + NF + 2
        IF (NPIV.EQ.0) GO TO 690
        IF (NPIV.GT.1) GO TO 680
        IW(IOLDPS) = -IW(IOLDPS)
        DO 670 K=1,NF
          J1 = IOLDPS + K
          IW(J1) = IW(J1+1)
  670   CONTINUE
        IWPOS = IWPOS - 1
        GO TO 690
  680   IW(IOLDPS+1) = NPIV
C
C     COPY REMAINDER OF ELEMENT TO TOP OF STACK
C
  690   LIELL = NF - NPIV
        IF (LIELL.EQ.0 .OR. IASS.EQ.NS) GO TO 750
        IF (IWPOS+LIELL.LT.ISTK) GO TO 700
        CALL UXSIF3(A, IW, ISTK, ISTK2, IINPUT, 2)
  700   ISTK = ISTK - LIELL - 1
        IW(ISTK) = LIELL
        J1 = ISTK
        KK = IWPOS - LIELL - 1
        DO 710 K=1,LIELL
          J1 = J1 + 1
          KK = KK + 1
          IW(J1) = IW(KK)
  710   CONTINUE
C
C     WE COPY IN REVERSE DIRECTION TO AVOID OVERWRITE PROBLEMS.
C
        LAELL = ((LIELL+1)*LIELL)/2
        KK = POSFAC + LAELL
        IF (KK.NE.ASTK) GO TO 720
        ASTK = ASTK - LAELL
        GO TO 740
C
C     THE MOVE AND ZEROING OF ARRAY A IS PERFORMED WITH TWO LOOPS SO
C     THAT THE CRAY-1 WILL VECTORIZE THEM SAFELY.
C
  720   KMAX = KK - 1
        DO 730 K=1,LAELL
          KK = KK - 1
          ASTK = ASTK - 1
          A(ASTK) = A(KK)
  730   CONTINUE
        KMAX = MIN0(KMAX,ASTK-1)
        DO 735 K=KK,KMAX
          A(K) = ZERO
  735   CONTINUE
  740   AZERO = MIN0(AZERO,ASTK-1)
  750   IF (NPIV.EQ.0) IWPOS = IOLDPS
  760 CONTINUE
C
C     END OF LOOP ON TREE NODES.
C
      IW(1) = NBLK
      IF (NTWO.GT.0) IW(1) = -NBLK
      NRLBDU = POSFAC - 1
      NIRBDU = IWPOS - 1
      IF (NTOTPV.EQ.N) GO TO 810
      INF = 3
      IERROR = NTOTPV
      GO TO 810
C
C     ERROR RETURNS
C
  770 INF = -3
      GO TO 810
  780 INF = -4
      IERROR = LA + MAX0(POSFAC+LNASS,APOS2-LTOPST+2) - ASTK
      GO TO 810
  790 INF = -5
      IERROR = NTOTPV + 1
      GO TO 810
  800 INF = -6
      IERROR = NTOTPV + 1
  810 RETURN
      END
