* SUBROUTINE UXSIIS                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SYMBOLIC TRANSFORMATION OF THE PARTITIONED KARUSH-KUHN-TUCKER MATRIX
* INTO THE COMPLETE SYMMETRIC STRUCTURE.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE HESSIAN MATRIX.
*  II  IH(N+1)  POINTERS OF THE HESSIAN MATRIX.
*  II  JH(M)  COLUMN INDICES OF THE HESSIAN MATRIX.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  ICG(NC+1)  POINTERS OF THE JACOBIAN MATRIX.
*  II  JCG(MC)  COLUMN INDICES OF THE JACOBIAN MATRIX.
*  IO  KCG(MC+N)  INDICES OF ELEMENTS OF THE ARRAY CG IN THE ARRAY B.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  IO  IB(N+NC+1)  POINTERS OF THE INDEFINITE KARUSH-KUHN-TUCKER MATRIX.
*  IO  JB(M+MC+NC)  COLUMN INDICES OF THE INDEFINITE KARUSH-KUHN-TUCKER
*         MATRIX.
*
      SUBROUTINE UXSIIS(N,M,IH,JH,NC,MC,ICG,JCG,KCG,NCA,ICA,IB,JB)
      INTEGER N,M,IH(N+1),JH(M),NC,MC,ICG(NC+1),JCG(MC),KCG(MC+N),NCA,
     & ICA(NCA),IB(N+NCA+1),JB(M+MC+N+NCA)
      INTEGER I,J,K,KC
C
C     INITIAL COMPUTATION OF THE POINTERS
C
      IB(1)=1
      DO 1 I=1,N
      IB(I+1)=IH(I+1)-IH(I)
    1 CONTINUE
      DO 5 I=1,NCA
      KC=ICA(I)
      IF (KC.LT.0) THEN
      K=-KC
      IB(K+1)=IB(K+1)+1
      ELSE
      DO 4 J=ICG(KC),ICG(KC+1)-1
      K=ABS(JCG(J))
      IB(K+1)=IB(K+1)+1
    4 CONTINUE
      ENDIF
    5 CONTINUE
      DO 6 I=1,N
      IB(I+1)=IB(I+1)+IB(I)
    6 CONTINUE
C
C     ELEMENTS OF THE MATRIX H
C
      DO 11 I=1,N
      DO 10 J=IH(I),IH(I+1)-1
      JB(IB(I))=JH(J)
      IB(I)=IB(I)+1
   10 CONTINUE
   11 CONTINUE
C
C     ELEMENTS OF THE MATRIX CG
C
      DO 13 I=1,NCA
      KC=ICA(I)
      IF (KC.LT.0) THEN
      K=-KC
      JB(IB(K))=I+N
      KCG(MC+K)=IB(K)
      IB(K)=IB(K)+1
      ELSE
      DO 12 J=ICG(KC),ICG(KC+1)-1
      K=ABS(JCG(J))
      JB(IB(K))=I+N
      KCG(J)=IB(K)
      IB(K)=IB(K)+1
   12 CONTINUE
      ENDIF
   13 CONTINUE
C
C     FINAL COMPUTATION OF THE POINTERS
C
      DO 14 I=N,1,-1
      IB(I+1)=IB(I)
   14 CONTINUE
      IB(1)=1
C
C     DIAGONAL SUBMATRIX
C
      K=IB(N+1)
      DO 15 I=1,NCA
      JB(K)=N+I
      K=K+1
      IB(N+I+1)=K
   15 CONTINUE
      RETURN
      END
