* SUBROUTINE UXSKIN                ALL SYSTEMS                96/12/01
C PORTABILITY : ALL SYSTEMS
C 96/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL TRANSFORMATION OF THE PARTITIONED KARUSH-KUHN-TUCKER MATRIX
* INTO THE COMPLETE SYMMETRIC STRUCTURE.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE HESSIAN MATRIX.
*  RI  H(M)  NONZERO ELEMENTS OF THE HESSIAN MATRIX.
*  II  IH(N+1)  POINTERS OF THE HESSIAN MATRIX.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  RI  CG(MC)  NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  JCG(MC)  COLUMN INDICES OF THE JACOBIAN MATRIX.
*  II  KCG(MC)  INDICES OF ELEMENTS OF THE ARRAY CG IN THE ARRAY B.
*  RO  B(2*(M+MC)-N)  NONZERO ELEMENTS OF THE GENERAL KARUSH-KUHN-TUCKER
*         MATRIX.
*  II  IB(N+NC+1)  POINTERS OF THE GENERAL KARUSH-KUHN-TUCKER MATRIX.
*
      SUBROUTINE UXSKIN(N,M,H,IH,NC,MC,CG,JCG,KCG,B,IB)
      INTEGER N,M,IH(N+1),NC,MC,JCG(MC),KCG(MC),IB(N+NC+1)
      REAL*8 H(M),CG(MC),B(M+MC+NC)
      INTEGER I,J,L
      REAL ZERO
      PARAMETER (ZERO=0.0D 0)
C
C     ELEMENTS OF THE MATRIX H
C
      DO 2 I=1,N
      L=IB(I)
      DO 1 J=IH(I),IH(I+1)-1
      B(L)=H(J)
      L=L+1
    1 CONTINUE
    2 CONTINUE
C
C     ELEMENTS OF THE MATRIX CG
C
      DO 3 J=1,MC
      IF (JCG(J).GT.0) THEN
      B(KCG(J))=CG(J)
      ELSE
      B(KCG(J))=ZERO
      ENDIF
    3 CONTINUE
C
C     ZERO SUBMATRIX
C
      DO 4 J=1,NC
      B(IB(N+J))=ZERO
    4 CONTINUE
      RETURN
      END
