* SUBROUTINE UXSLCN                ALL SYSTEMS                08/12/01
C PORTABILITY : ALL SYSTEMS
C 08/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL TRANSFORMATION OF THE PARTITIONED KARUSH-KUHN-TUCKER MATRIX
* INTO THE COMPLETE SYMMETRIC STRUCTURE.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE HESSIAN MATRIX.
*  RI  H(M)  NONZERO ELEMENTS OF THE HESSIAN MATRIX.
*  II  IH(N+1)  POINTERS OF THE HESSIAN MATRIX.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  RI  CG(MC)  NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  ICG(N+1)  POINTERS OF THE JACOBIAN MATRIX.
*  II  JCG(MC)  COLUMN INDICES OF THE JACOBIAN MATRIX.
*  II  KCG(MC)  INDICES OF ELEMENTS OF THE ARRAY CG IN THE ARRAY B.
*  II  KX(NX)  INDICES OF THE UNIT ELEMENTS IN THE ARRAY B.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  II  ICA(NCA)  INDICES OF ACTIVE CONSTRAINTS.
*  II  D(MD)  ELEMENTS OF THE TRIDIAGONAL MATRIX.
*  II  ID(NX+NC+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD D.
*  RO  B(2*(M+MC)-N)  NONZERO ELEMENTS OF THE GENERAL KARUSH-KUHN-TUCKER
*         MATRIX.
*  II  IB(N+NCA+1)  POINTERS OF THE GENERAL KARUSH-KUHN-TUCKER MATRIX.
*
      SUBROUTINE UXSLCN(N,M,H,IH,NX,NC,MC,CG,ICG,JCG,KCG,KX,NCA,ICA,
     & D,ID,B,IB)
      INTEGER N,M,IH(N+1),NX,NC,MC,ICG(NC+1),JCG(MC),KCG(MC),KX(NX),
     & NCA,ICA(NCA),ID(NX+NC+1),IB(N+NCA+1)
      REAL*8 H(M),CG(MC),D(2*(NX+NC)),B(M+MC+NX+NCA)
      INTEGER I,J,K,L
      REAL ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
C
C     ELEMENTS OF THE MATRIX H
C
      DO 2 I=1,N
      L=IB(I)
      DO 1 J=IH(I),IH(I+1)-1
      B(L)=H(J)
      L=L+1
    1 CONTINUE
    2 CONTINUE
C
C     ELEMENTS OF THE MATRIX CG
C
      DO 4 I=1,NCA
      K=ICA(I)
      IF (K.LT.0) THEN
      L=-K
      B(KX(L))=ONE
      ELSE
      DO 3 J=ICG(K),ICG(K+1)-1
      IF (JCG(J).GT.0) THEN
      B(KCG(J))=CG(J)
      ELSE
      B(KCG(J))=ZERO
      ENDIF
    3 CONTINUE
      ENDIF
    4 CONTINUE
C
C     TRIDIAGONAL SUBMATRIX
C
      DO 5 I=1,NCA
      B(IB(N+I))=D(ID(I))
      IF (ID(I+1)-ID(I).GT.1) THEN
      B(IB(N+I)+1)=D(ID(I)+1)
      ENDIF
    5 CONTINUE
      RETURN
      END
