* SUBROUTINE UXSPCI                ALL SYSTEMS                89/12/01
C PORTABILITY : ALL SYSTEMS
C 89/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SYMBOLIC FACTORIZATION OF A SPARSE SYMMETRIC MATRIX GIVEN IN THE
* NORMAL SCHEME PA,SA. ON OUTPUT WE HAVE POINTER VECTOR OF THE FACTOR
* PSL AND VECTOR OF COLUMN INDICES SL. ML IS THE NUMBER OF THE INDICES
* USED FOR THE VECTOR SL, WHERE WE HAVE FACTOR IN THE COMPACT FORM.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  IO  ML NUMBER OF THE NONZERO ELEMENTS IN THE FACTOR'S COMPACT SCHEME
*  II  MMAX  LENGTH OF THE ARRAY SL. IN THE CASE OF THE
*            INSUFFICIENT SPACE IT IS TO BE INCREASED.
*  II  PA(N+1) POINTER VECTOR OF THE INPUT MATRIX
*  II  SA(MMAX) VECTOR OF THE COLUMN INDICES OF THE INPUT MATRIX
*  IO  PSL(N+1) POINTER VECTOR OF THE FACTOR
*  RO  SL(MMAX) COMPACT SCHEME OF THE INDICES OF THE FACTOR
*  II  PERM(N) PERMUTATION VECTOR
*  II  INVP(N) INVERSE PERMUTATION VECTOR
*  IA  WN11(N+1) WORK VECTOR OF THE LENGTH N+1
*  IA  WN12(N+1) WORK VECTOR OF THE LENGTH N+1
*  IA  WN13(N+1) WORK VECTOR OF THE LENGTH N+1
*  IO  ISPACE AN INFORMATION ON SPACE OBTAINED DURING THE PROCESS
*       OF THE FACTORIZATION.
*      ISPACE=0    THE FACTORIZATION HAS TERMINATED NORMALLY
*      ISPACE=1    INSUFFICIENT SPACE AVAILABLE
*
* METHOD :
* S.C.EISENSTAT,M.C.GURSKY,M.H.SCHULTZ,A.H.SHERMAN:YALE SPARSE MATRIX
* PACKAGE I. THE SYMMETRIC CODES,ACM TRANS. ON MATH. SOFTWARE.
*
* NOTE: TYPE OF SL CHANGED FOR THE UFO APPLICATION.
*
      SUBROUTINE UXSPCI(N,ML,MMAX,PA,SA,PSL,SL,PERM,INVP,
     *            WN11,WN12,WN13,ISPACE)
      INTEGER N,MMAX,PA(N+1),SA(MMAX),PSL(N+1)
      INTEGER PERM(N),INVP(N),WN11(N+1),WN12(N+1),WN13(N+1)
      INTEGER ISPACE,I,INZ,J,JSTOP,JSTRT,K,KNZ,KXSUB,MRGK,LMAX,ML
      INTEGER NABOR,NODE,NP1,NZBEG,NZEND,RCHM,MRKFLG,M
      REAL*8 SL(*)
      NZBEG=1
      NZEND=0
      PSL(1)=1
      DO 100 K=1,N
      WN11(K)=0
      WN13(K)=0
100   CONTINUE
      NP1=N+1
      DO 1500 K=1,N
      KNZ=0
      MRGK=WN11(K)
      MRKFLG=0
      WN13(K)=K
      IF(MRGK.NE.0) WN13(K)=WN13(MRGK)
      SL(K)=NZEND
      NODE=PERM(K)
      JSTRT=PA(NODE)
      JSTOP=PA(NODE+1)-1
      IF(JSTRT.GT.JSTOP) GO TO 1500
      WN12(K)=NP1
      DO 300 J=JSTRT,JSTOP
      NABOR=SA(J)
      IF(NABOR.EQ.NODE) GO TO 300
      NABOR=INVP(NABOR)
      IF(NABOR.LE.K) GO TO 300
      RCHM=K
200   M=RCHM
      RCHM=WN12(M)
      IF(RCHM.LE.NABOR) GO TO 200
      KNZ=KNZ+1
      WN12(M)=NABOR
      WN12(NABOR)=RCHM
      IF(WN13(NABOR).NE.WN13(K)) MRKFLG=1
300   CONTINUE
      LMAX=0
      IF(MRKFLG.NE.0.OR.MRGK.EQ.0) GO TO 350
      IF(WN11(MRGK).NE.0) GO TO 350
      SL(K)=SL(MRGK)+1
      KNZ=PSL(MRGK+1)-(PSL(MRGK)+1)
      GO TO 1400
350   I=K
400   I=WN11(I)
      IF(I.EQ.0) GO TO 800
      INZ=PSL(I+1)-(PSL(I)+1)
      JSTRT=SL(I)+1
      JSTOP=SL(I)+INZ
      IF(INZ.LE.LMAX) GO TO 500
      LMAX=INZ
      SL(K)=JSTRT
500   RCHM=K
      DO 700 J=JSTRT,JSTOP
      NABOR=SL(N+1+J)
600   M=RCHM
      RCHM=WN12(M)
      IF(RCHM.LT.NABOR) GO TO 600
      IF(RCHM.EQ.NABOR) GO TO 700
      KNZ=KNZ+1
      WN12(M)=NABOR
      WN12(NABOR)=RCHM
      RCHM=NABOR
700   CONTINUE
      GO TO 400
800   IF(KNZ.EQ.LMAX) GO TO 1400
      IF(NZBEG.GT.NZEND) GO TO 1200
      I=WN12(K)
      DO 900 JSTRT=NZBEG,NZEND
      IF(SL(N+1+JSTRT)-I .GE.0) THEN
        IF(SL(N+1+JSTRT).EQ.I) THEN
          GO TO 1000
        ELSE
          GO TO 1200
        END IF
      END IF
900   CONTINUE
      GO TO 1200
1000  SL(K)=JSTRT
      DO 1100 J=JSTRT,NZEND
      IF (SL(N+1+J).NE.I) GO TO 1200
      I=WN12(I)
      IF(I.GT.N) GO TO 1400
1100  CONTINUE
      NZEND=JSTRT-1
1200  NZBEG=NZEND+1
      NZEND=NZEND+KNZ
C
C A VARIANT IS USED WHEN CALLED SO THAT SL(X)=A(NB+X)
C
      IF(NZEND.GE.MMAX-N-1) GO TO 1600
      I=K
      DO 1300 J=NZBEG,NZEND
      I=WN12(I)
      SL(N+1+J)=I
      WN13(I)=K
1300  CONTINUE
      SL(K)=NZBEG
      WN13(K)=K
1400     IF(KNZ.GT.1) THEN
            KXSUB=SL(K)
            I=SL(N+1+KXSUB)
            WN11(K)=WN11(I)
            WN11(I)=K
      END IF
1500  PSL(K+1)=PSL(K)+KNZ
      SL(N)=SL(N)+1
      SL(N+1)=SL(N)
      ML=N+SL(N+1)
      ISPACE=0
      RETURN
1600  ISPACE=1
      RETURN
      END
