* SUBROUTINE UXSRBD               ALL SYSTEMS               91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* FIND THE ORDER OF THE VERTICES FOR THE BLOCK QR SCHEME USING
* MINIMUM DEGREE VARIANT OF THE LOCAL GIVENS RULE.
*
*  PARAMETERS :
*  II  NN NUMBER OF COLUMNS OF THE MATRIX.
*  II  M NUMBER OF ROWS OF THE MATRIX.
*  II  NADJ MAXIMUM NUMBER OF NONZEROS IN ADJNCY.
*  II  NR SIZE OF THE ARRAY FOR THE SPARSE TRIANGULAR FACTOR.
*  II  P(NN) VECTOR OF WIDTHS OF THE BLOCKS.
*  II  BJA1(NN) BEGINNINGS OF THE BLOCK COLUMNS IN A.
*  II  BJA2(NN) ENDING INDICES OF THE BLOCK COLUMNS IN A.
*  II  XADJ(NN+1) POINTER VECTOR TO THE SPARSE ADJACENCY MATRIX.
*  II  ADJNCY(NADJ)  INDICES OF THE SYMMETRIC ADJACENCY MATRIX.
*  IA  DEG(NN) THE DEGREE VECTOR.  DEG(I) LT 0 MEANS THE NODE
*              BELONGS TO THE FINAL SUBSET.
*  IA  RCHSET(NN) INDICES OF THE NODES IN REACHABLE SET.
*  IA  RCHST2(NN) INDICES OF THE REACHSETS.
*  IA  MARKER(NN) THE MARKER VECTOR FOR REACH AND NBRHD SETS.
*  IA  NBRHD(NN) VERTICES OF THE NEIGHBORHOOD.
*  IA  MASK1(M) MASK FOR THE PARTS OF THE MATRIX NOT ACTIVE.
*  IA  BEG(M) BEGINNINGS OF THE COLUMNS.
*  IO  PERM(NN) THE LOCAL GIVENS MINIMUM DEGREE ORDERING.
*  IO  INVP(NN) INVERSE OF THE OUTPUT ORDERING.
*  IO  IR(NN+1) POINTER VECTOR TO THE STRUCTURE OF R.
*  IO  JR(NR) INDICES OF THE SPARSE VECTOR OF R.
*  II  ITYPE TYPE OF ORDER ON OUTPUT: IF ITYPE=0 THEN ORDER IS
*        PERMUTED, ELSE ORDER IS CONSECUTIVE.
*
      SUBROUTINE UXSRBD(NN,M,NADJ,NR,P,BJA1,BJA2,XADJ,ADJNCY,DEG,
     *                 RCHSET,RCHST2,MARKER,NBRHD,MASK1,BEG,PERM,
     *                 INVP,IR,JR,ITYPE)
C
      INTEGER NN,M,NADJ,NR,P(NN),ITYPE
      INTEGER ADJNCY(NADJ),PERM(NN),INVP(NN),DEG(NN),MARKER(NN)
      INTEGER RCHSET(NN),RCHST2(NN),BJA1(NN),BJA2(NN)
      INTEGER XADJ(NN+1),MASK1(M),BEG(M),NBRHD(NN)
      INTEGER MINDEG,ISUM,J,L,IP,ISTRT,ISTOP,RCHSZ2
      INTEGER I,INODE,IRCH
      INTEGER NHDSZE,NODE,NOFSUB,NP,NUM,NUMP1
      INTEGER RCHSZE,SEARCH,THRESH
      INTEGER PTR,JR(NR),IR(NN+1)
      INTEGER IBEG,IER,LEN
C
C INITIALIZE BEG, MASK1 - NBRHD USED AS THE WORKING VECTOR
C
      CALL UXSRO5(M,NN,BJA1,IBEG,BEG,NBRHD)
      CALL UXSRO6(M,IBEG,BEG,MASK1)
C
C INITIALIZE DEGREE VECTOR AND OTHER WORKING VARIABLES.
C
      PTR=1
      MINDEG = NN*M
      NOFSUB = 0
      DO 100 NODE = 1, NN
        PERM(NODE) = NODE
        INVP(NODE) = NODE
        MARKER(NODE) = 0
C
C COMPUTE DEGREE OF THE NODE
C
        ISUM=0
        DO 110 J=XADJ(NODE),XADJ(NODE+1)-1
          L=ADJNCY(J)
          CALL UXSRO0(NN,NODE,L,BJA1,BJA2,LEN)
C
C WITDTH OF COLUMN L IS P(L)
C
          ISUM=ISUM+P(L)*LEN
  110   CONTINUE
        DEG(NODE)=ISUM
C
C MINIMUM DEGREE
C
        IF(ISUM.LT.MINDEG) MINDEG=ISUM
  100 CONTINUE
      NUM = 0
C
C PERFORM THRESHOLD SEARCH TO GET A NODE OF MIN DEGREE.
C VARIABLE SEARCH POINTS TO WHERE SEARCH SHOULD START.
C
  200 SEARCH = 1
      THRESH = MINDEG
      MINDEG = NN*M
  300 NUMP1 = NUM + 1
      IF ( NUMP1 .GT. SEARCH )  SEARCH = NUMP1
      DO 400 J = SEARCH, NN
        NODE = PERM(J)
        IF ( MARKER(NODE) .LT. 0 )  GOTO 400
        ISUM = DEG(NODE)
        IF ( ISUM .LE. THRESH )  GO TO 500
        IF ( ISUM .LT. MINDEG )  MINDEG =  ISUM
  400 CONTINUE
      GO TO 200
 500  CONTINUE
C
C  CHANGE TUMA 13.10.91 - ORDER PERMUTED OR CONSECUTIVE
C
      IF(ITYPE.NE.0) NODE=NUM+1
C
C NODE HAS MINIMUM DEGREE. FIND ITS REACHABLE SETS BY
C CALLING QMDRCH.
C
      SEARCH = J
      NOFSUB = NOFSUB + DEG(NODE)
      MARKER(NODE) = 1
      CALL UXSRB3(M,NN,NADJ,NODE,BJA1,BJA2,XADJ,ADJNCY,DEG,MARKER,
     *            RCHSZE,RCHSET,NHDSZE,NBRHD,MASK1)
C
C ELIMINATE NODE
C
      NUM = NUM + 1
      NP  = INVP(NODE)
      IP  = PERM(NUM)
      PERM(NP) = IP
      INVP(IP) = NP
      PERM(NUM) = NODE
      INVP(NODE) = NUM
      IF ( RCHSZE .LE. 0 )  GO TO 800
C
C FIRST PART OF THE UPDATE THE DEGREES OF THE NODES IN THE REACHABLE
C SET
C
      CALL UXSRB8(NN,M,NADJ,BJA1,BJA2,RCHSZE,P,RCHSET,MASK1,
     *           RCHSZ2,RCHST2,DEG,MARKER,XADJ,ADJNCY,NODE)
C
C UPDATE MASK1
C
      ISTRT=BJA1(NODE)
      ISTOP=BJA2(NODE)
      CALL UXSRO7(M,P(NODE),ISTRT,ISTOP,MASK1,LEN)
      CALL UXSRO2(M,BEG,IBEG,MASK1,ISTRT,ISTOP)
C
C UPDATE LABELS
C
      CALL UXSRO8(NN,NODE,BJA1,BJA2,RCHSZE,RCHSET)
C
C SECOND PART OF THE UPDATE THE DEGREES OF THE NODES IN THE REACHABLE
C SET
C
      CALL UXSRB9(NN,M,NADJ,BJA1,BJA2,RCHSZE,P,RCHSET,MASK1,
     *           RCHSZ2,RCHST2,DEG,MARKER,XADJ,ADJNCY,NODE)
C
C RESET MARKER FOR NODE AND SET ITS DEGREE TO MINUS ONE
C
      MARKER(NODE) = 0
      DEG(NODE)=-1
C
C RESET MARKER, SEARCH, MINDEG, THRESH
C
      DO 700 IRCH = 1, RCHSZE
        INODE = RCHSET(IRCH)
        IF ( MARKER(INODE) .LT. 0 )  GOTO 700
        MARKER(INODE) = 0
        ISUM = DEG(INODE)
        IF ( ISUM .LT. MINDEG )  MINDEG = ISUM
        IF ( ISUM .GT. THRESH )  GOTO 700
        MINDEG = THRESH
        THRESH = ISUM
        SEARCH = INVP(INODE)
  700 CONTINUE
C
C JOINING THE SUPERNODES
C
      IF(NHDSZE.GT.0) THEN
        CALL UXSRB0(NODE,XADJ,ADJNCY,MARKER,RCHSZE,RCHSET,NBRHD)
      END IF
C
C TEST ON THE END
C
  800 CONTINUE
C
C REWRITE STRUCTURE OF THE ROW OF THE VERTEX NODE INTO THE
C STRUCTURE OF THE FACTOR R
C
      IR(NUM)=PTR
C
C IF NOT SPACE THEN BREAK
C
      IF(PTR+RCHSZE.GE.NR) THEN
        IER=103
        RETURN
      END IF
C
      DO 900 IRCH=1,RCHSZE
        I=RCHSET(IRCH)
        JR(PTR)=I
        PTR=PTR+1
 900  CONTINUE
C
C TEST ON COMPLETENESS
C
      IF(NUM.LT.NN) GO TO 300
C
C END OF THE SPARSE FACTOR STRUCTURE
C
      IR(NN+1)=PTR
      RETURN
      END
