* SUBROUTINE UXSRBI                ALL SYSTEMS               93/12/01
C PORTABILITY : ALL SYSTEMS
C 93/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
*  INITIALIZE SPARSE QR DECOMPOSITION.
*  MULTIFRONTAL ALGORITHM. USES SPARSPAK'S QUOTIENT MINIMUM DEGREE.
*
*  PARAMETERS :
*
*  II  M NUMBER OF ROW OF THE MATRIX.
*  II  N NUMBER OF COLUMN OF THE MATRIX.
*  II  NJA NUMBER OF NONZEROES MEMBERS IN THE MATRIX.
*  IO  IER  ERROR MESSAGE
*  II  MAXNJA MAXIMUM SIZE OF NJA.
*  II  MAXNADJ MAXIMUM SIZE OF NADJ.
*  II  IB(M+1) INDICES OF THE FIRST MEMBERS IN ROWS OF THE MATRIX.
*  II  JA(MAXNJA) COLUMN INDICES OF THE MATRIX.
*  II  IA(N+1) INDICES OF THE FIRST MEMBERS IN COLUMNS OF THE MATRIX.
*  IA  XADJ(N+1) POINTERS TO THE BEGINNINGS OF THE ROWS (COLUMNS)
*         IN THE SPARSE STRUCTURE OF THE PRODUCT.
*  IA  SON(N+1) THE DEGREE VECTOR. SONG(I).LT.0 MEANS THE NODE
*              BELONGS TO THE FINAL SUBSET.
*  IA  LABEL2(N+1) THE MARKER VECTOR FOR REACH AND NBRHD SETS.
*  IA  LABEL(N+1) VERTICES OF THE NEIGHBORHOOD.
*  IA  BJA1(N+1) ROW INDICES WITH BEGINNINGS OF THE COLUMNS.
*  IA  EJA2(N+1) ROW INDICES WITH ENDINGS OF THE COLUMNS.
*  IA  BIA(N+1) BEGINNING BLOCKS IN VALUE BA
*  IA  BLEN(N+1)  LENGTH BLOCKS IN VALUE BA
*  IA  P(N+1) WINDTH OF THE BLOCK COLUMNS.
*  IO  INVP(N+1) INVERSE OF THE OUTPUT ORDERING.
*  IO  PERMC(N+1) INDIVIDUAL ORDERING BEFORE UXSRBD.
*  IA  PERMR(M+1)  PERMUTATION OF ROWS.
*  IA  IPERMR(M+1) THE LOCAL GIVENS MINIMUM DEGREE ORDERING.
*  IA  WN01(M+1) AUXILIARY ARRAY.
*  IA  WN02(M+1) AUXILIARY ARRAY.
*  IA  WN03(M+1) AUXILIARY ARRAY.
*  IA  WN04(M+1) AUXILIARY ARRAY.
*
      SUBROUTINE UXSRBI(M,N,NJA,IER,MAXNJA,MAXNADJ,
     *        IB,JA,IA,XADJ,SON,LABEL2,LABEL,BJA1,BJA2,BIA,BLEN,
     *        P,INVP,PERMC,PERMR,IPERMR,WN01,WN02,WN03,WN04)
      INTEGER M,N,NJA,IER,MAXNJA,MAXNADJ,NN,NADJ,FTOP,IND
      INTEGER NONZ,ITYPE,NR,MAXLNZ
      INTEGER NRHS
      INTEGER IA(N+1),IB(M+1),XADJ(N+1),JA(MAXNJA)
      INTEGER SON(N+1),LABEL2(N+1),LABEL(N+1)
      INTEGER BJA1(N+1),BJA2(N+1),BIA(N+1),BLEN(N+1),P(N+1)
      INTEGER PERMR(M+1),IPERMR(M+1),INVP(N+1),PERMC(N+1)
      INTEGER WN01(M+1),WN02(M+1),WN03(M+1),WN04(M+1),I
      NRHS = 1
      FTOP = 2 * NJA
C  ----------------------
C  FORM MATRIX TRANS(A)*A
C  ----------------------
      CALL UXSRQC(M,N,NJA,IA,JA,IB,WN01)
      CALL UXSRBS(M,N,NJA,IA,IB,XADJ,JA,NADJ,MAXNADJ,WN01,IER)
      IF (IER.GT.0) GO TO 26
C  ------------
C  WORKING PART
C  ------------
      CALL UXSRBN(N,XADJ,JA(FTOP+1),WN02,PERMC,IPERMR,WN04,WN03,
     + LABEL,SON)
      CALL UXVETI(N,LABEL,SON,WN01,WN02,LABEL2)
      DO 33 I=1,N
        PERMC(I)=I
 33   CONTINUE
C
C  ROW ORDERING FOR MODIFIED WIDTH-2 CLIQUE SEPARATION SCHEME
C
      CALL UXVSR6(N,LABEL2,PERMC,1,WN01,WN02)
      CALL UXVSIP(N,PERMC,WN01)
      CALL UXSRBP(M,N,IA,IB,JA(NJA+1),
     *            JA,LABEL,WN02,LABEL2,PERMC,PERMR,
     *            IPERMR,WN03,WN04,SON,WN01)
      CALL UXVETC(N,LABEL,SON,WN01,NN)
      DO 17 I=1,N
        PERMC(I)=I
 17   CONTINUE
      CALL UXVSR6(N,LABEL,PERMC,1,WN01,WN02)
C
C  INITIALIZE COLUMN POINTERS
C
      CALL UXSRBC(M,N,IA,JA(NJA+1),IPERMR,LABEL,PERMC,BJA1,BJA2,NN,P)
      CALL UXSRO9(M,NN,N,P,BIA,BJA1,BJA2,BLEN)
C
C REDEFINE XADJ/ADJNCY GIVING THERE STRUCTURE OF BLOCK TRANS(A)*A
C ADJNCY IS IN JA AT THE POSITIONS NJA+1, ...
C
       CALL UXSRBO(NN,NADJ,NONZ,BJA1,BJA2,XADJ,JA(FTOP+1),WN02)
C
C CHECK WHETHER WE CAN GIVE FACTOR R BEHIND XADJ/ADJNCY.
C
      NR=MAXNJA-FTOP-1-NONZ-1-M-1
      IF(NR.LE.1) THEN
        IER=50
        CALL UOERR1('UXSRBI',50)
        GO TO 26
      END IF
      ITYPE=1
C
C ON OUTPUT IPERM APPLIES TO THE BLOCK ORDERING
C
      CALL UXSRBD(NN,M,NADJ,NR,P,BJA1,BJA2,XADJ,JA(FTOP+1),SON,
     *                 WN03,WN02,LABEL,LABEL2,WN04,WN01,IPERMR,INVP,
     *                 JA(FTOP+1+NONZ),JA(FTOP+1+NONZ+M+1),ITYPE)
C
C COMPUTE ELIMINATION TREE FOR THE BLOCK DECOMPOSITION
C
C PARENT - WN02, ISON - WN03, PSTORD - LABEL, IA - PERM
C
      CALL UXSRBG(NN,JA(FTOP+1+NONZ),JA(FTOP+1+NONZ+M+1),WN02)
      CALL UXVETS(NN,WN02,WN01,SON,WN03,LABEL)
C
C IPERM - BLOCK ORDERING
C
      CALL UXVQRP(N,NN,P,PERMC,INVP,LABEL,WN01,IPERMR)
C
C POINTWISE FACTOR, ELIMINATION TREE, POSTORDER AND PERMUTATION.
C
C RENEW THE STRUCTURE XADJ/ADJNCY AT POSITIONS FTOP+1, ... IN JA.
C
      CALL UXSRBS(M,N,NJA,IA,IB,XADJ,JA,NADJ,MAXNADJ,WN01,IER)
      IF(IER.GT.0) GO TO 26
      NONZ=NADJ
C
C CHECK WHETHER WE CAN GIVE FACTOR R BEHIND XADJ/ADJNCY.
C
      NR=MAXNJA- FTOP-1-NONZ-1-M-1
      IF(NR.LE.1) THEN
        IER=50
        CALL UOERR1('UXSRBI',50)
        GO TO 26
      END IF
C
C BIA = LABEL2 (POLE MUSI BYT DEKLAROVANO JAKO N+1)
C
      CALL UXSRBE(N,XADJ,JA(FTOP+1),PERMC,INVP,
     1            JA(FTOP+1+NONZ),MAXLNZ,BIA,JA(FTOP+1+NONZ+M+1),
     1            NR,WN01,WN02,WN03,IER)
      IF (IER.GT.0) GO TO 26
C
C TRANSFORMATION FROM COMPACT SCHEME
C
      CALL UXSRBH(N,JA(FTOP+1+NONZ),BIA,JA(FTOP+1+NONZ+M+1),
     *             WN02,IND,WN04)
C
C WN04  =  MASK - AUXILIARY
C
      CALL UXVETP(N,WN02,WN01,SON,WN03,LABEL,IND,WN04)
C
C IPERM - BLOCK ORDERING
C
      CALL UXVQRQ(N,PERMC,INVP,LABEL,WN01)
C
C PERMUTE SOLUTION VECTOR
C
      CALL UXVSIP(M,PERMR,WN01)
 26   CONTINUE
      RETURN
      END
