* SUBROUTINE UYABC2                ALL SYSTEMS                05/12/01
C PORTABILITY : ALL SYSTEMS
C 05/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SIMPLIFIED AGGREGATION FOR NONSMOOTH VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE HESSIAN MATRIX.
*  RI  H(M)  POSITIVE DEFINITE APPROXIMATION OF THE SPARSE HESSIAN
*         MATRIX.
*  IO  JH(M)  INDICES OF THE NONZERO ELEMENTS OF H.
*  II  PSL(NF+1) POINTER ARRAY OF THE FACTORIZED SPARSE MATRIX
*  II  PERM(NF)  PERMUTATION VECTOR
*  RI  G(NF)  ACTUAL SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GV(NF)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RA  S(NF)  DIRECTION VECTOR.
*  RA  V(NF)  AUXILIARY VECTOR.
*  RO  ALF  LINEARIZATION TERM.
*  RU  ALFV  AGGREGATED LINEARIZATION TERM.
*  RI  RHO  CORRECTION PARAMETER.
*  II  JC  CORRECTION INDICATOR.
*
      SUBROUTINE UYABC2(NF,M,IX,H,JH,PSL,PERM,G,GV,S,V,ALF,ALFV,
     & RHO,JC)
      INTEGER NF,M,IX(NF),JH(M),PSL(NF+1),PERM(NF),JC
      REAL*8 H(M),G(NF),GV(NF),S(NF),V(NF),ALF,ALFV,RHO
      REAL*8 P,Q,W,LAM
      INTEGER I
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      P=ALFV-ALF
      DO 1 I=1,NF
      IF (IX(I).GE.0) THEN
      W=S(I)
      P=P+W*S(I)
      S(I)=G(I)-GV(I)
      ENDIF
    1 CONTINUE
      Q=ZERO
      IF (JC.GE.1) THEN
      DO 2 I=1,NF
      IF (IX(I).GE.0) Q=Q+RHO*S(I)**2
    2 CONTINUE
      ENDIF
      CALL UXVSFP(NF,PERM,S,V)
      CALL UXSPCB(NF,M,H,PSL,JH,S,1)
      DO 4 I=1,NF
        IF (IX(I).GE.0) THEN
        W=ONE/H(PSL(I)+I-1)
        Q=Q+W*S(I)**2
        ENDIF
    4 CONTINUE
      LAM=0.5D 0+SIGN(0.5D 0,P)
      IF (Q.GT.ZERO) LAM=MIN(ONE,MAX(ZERO,P/Q))
      P=ONE-LAM
      ALFV=LAM*ALF+P*ALFV
      DO 5 I=1,NF
      GV(I)=LAM*G(I)+P*GV(I)
    5 CONTINUE
      RETURN
      END
