* SUBROUTINE UYADP0             ALL SYSTEMS                   93/12/01
C PORTABILITY : ALL SYSTEMS
C 93/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NEW LINEAR CONSTRAINTS OR NEW SIMPLE BOUNDS ARE ADDED TO THE ACTIVE
* SET.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  IU  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF LINEARIZED CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS IN THE FIELD CG.
*  II  MMAX  MAXIMUM NUMBER OF NONZERO ELEMENTS IN THE FIELD CR.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  CF(NC)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  RI  CFD(NC) VECTOR CONTAINING INCREMENTS OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CG(MC)  SPARSE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC) COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  RI  CR(MMAX)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  II  ICR(NF+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CR.
*  II  JCR(MMAX) COLUMN INDICES OF ELEMENTS IN THE FIELD CR.
*  RO  CZ(NF)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RI  EPS9  TOLERANCE FOR ACTIVE CONSTRAINTS.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  II  MLC1  SPECIFICATION OF THE CONSTRAINT ADDITION. MLC1=1-ONLY ONE
*         ACTIVE CONSTRAINT IS ADDED TO THE ACTIVE SET. MLC1=2-ALL
*         ACTIVE CONSTRAINTS ARE ADDED TO THE ACTIVE SET.
*  II  INEW  INDEX OF THE NEW ACTIVE CONSTRAINT.
*  IO  IER  ERROR INDICATOR.
*
* COMMON DATA :
*  II  KBF  TYPE OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS. KBF=1-ONE
*         SIDED SIMPLE BOUNDS. KBF=2-TWO SIDED SIMPLE BOUNDS.
*  II  KBC  TYPE OF CONSTRAINTS. KBC=0-NO CONSTRAINTS. KBC=1-CONSTRAINTS
*         WITH ONE SIDED BOUNDS. KBC=2-CONSTRAINTS WITH TWO SIDED
*         BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  IU  NADD  NUMBER OF CONSTRAINT ADDITIONS.
*  IO  ITERM  TERMINATION INDICATOR.
*  TO  TXFU  TEXT INFORMATION ON THE CAUSE OF TERMINATION.
*
* SUBPROGRAMS USED :
*  S   UMADR0  ADDITION OF A NEW ACTIVE CONSTRAINT.
*  S   ULNEWS  IDENTIFICATION OF ACTIVE UPPER BOUNDS.
*  S   ULNEWL  IDENTIFICATION OF ACTIVE LINEAR CONSTRAINRS.
*  S   ULDIRL  NEW VALUES OF CONSTRAINT FUNCTIONS.
*  S   UXVIND  CHANGE OF THE INTEGER VECTOR FOR CONSTRAINT ADDITION.
*  S   UOYAD1  PRINT OF ENTRY TO CONSTRAINT ADDITION.
*  S   UOYAD5  PRINT OF INFORMATION ON CONSTRAINT ADDITION.
*  S   UOERR1  ERROR MESSAGES.
*
      SUBROUTINE UYADP0(NF,N,NC,MC,MMAX,X,IX,XN,XL,XU,CF,CFD,IC,ICA,
     & CL,CU,CG,ICG,JCG,CR,ICR,JCR,CZ,S,R,EPS7,EPS9,GMAX,UMAX,MLC1,
     & INEW,IER)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,N,NC,MC,MMAX,IX(NF),IC(NC),ICA(NF),ICG(NC+1),JCG(MC),
     & ICR(NF+1),JCR(MMAX),MLC1,INEW,IER
      REAL*8 X(NF),XN(NF),XL(NF),XU(NF),CF(NC),CFD(NC),CL(NC),CU(NC),
     & CG(MC),CR(MMAX),CZ(NF),S(NF),R,EPS7,EPS9,GMAX,UMAX
      INTEGER I,KC
      IF (R.NE.ZERO) CALL ULDIRL(NC,CF,CFD,IC,R,KBC)
      IF (MLC1.EQ.2.AND.INEW.NE.0) THEN
      IF (KBF.GT.0) THEN
      DO 1 I=1,NF
      INEW=0
      CALL ULNEWS(NF,X,IX,XN,XL,XU,EPS9,NORMF,I,INEW)
      CALL UMADR0(NF,N,NC,MC,MMAX,ICA,CG,ICG,JCG,CR,ICR,JCR,S,EPS7,
     & GMAX,UMAX,INEW,IER)
      CALL UXVIND(NF,IX,I,IER)
      IF (ITERM.NE.0) RETURN
      CALL UOYAD1(IER,'UYADP0')
      CALL UOYAD6(NF,N,NC,MMAX,IX,IC,ICA,CR,ICR,JCR,CZ,UMAX,GMAX,INEW,
     & IER)
      IF (IER.LT.0) THEN
      ITERM=-15
      TXFU='BAD ADD.'
      CALL UOERR1('UYADP0',15)
      RETURN
      ENDIF
    1 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 2 KC=1,NC
      INEW=0
      CALL ULNEWL(NC,KC,CF,IC,CL,CU,EPS9,INEW)
      CALL UMADR0(NF,N,NC,MC,MMAX,ICA,CG,ICG,JCG,CR,ICR,JCR,S,EPS7,
     & GMAX,UMAX,INEW,IER)
      CALL UXVIND(NC,IC,KC,IER)
      IF (ITERM.NE.0) RETURN
      CALL UOYAD1(IER,'UYADP0')
      CALL UOYAD6(NF,N,NC,MMAX,IX,IC,ICA,CR,ICR,JCR,CZ,UMAX,GMAX,INEW,
     & IER)
      IF (IER.LT.0) THEN
      ITERM=-15
      TXFU='BAD ADD.'
      CALL UOERR1('UYADP0',15)
      RETURN
      ENDIF
    2 CONTINUE
      ENDIF
      ELSE IF (MLC1.EQ.1.AND.INEW.GT.0) THEN
      KC=INEW
      INEW=0
      CALL ULNEWL(NC,KC,CF,IC,CL,CU,EPS9,INEW)
      CALL UMADR0(NF,N,NC,MC,MMAX,ICA,CG,ICG,JCG,CR,ICR,JCR,S,EPS7,
     & GMAX,UMAX,INEW,IER)
      CALL UXVIND(NC,IC,KC,IER)
      IF (ITERM.NE.0) RETURN
      CALL UOYAD1(IER,'UYADP0')
      CALL UOYAD6(NF,N,NC,MMAX,IX,IC,ICA,CR,ICR,JCR,CZ,UMAX,GMAX,INEW,
     & IER)
      IF (IER.LT.0) THEN
      ITERM=-15
      TXFU='BAD ADD.'
      CALL UOERR1('UYADP0',15)
      RETURN
      ENDIF
      ELSE IF (MLC1.EQ.1.AND.INEW.LT.0) THEN
      I=-INEW
      INEW=0
      CALL ULNEWS(NF,X,IX,XN,XL,XU,EPS9,NORMF,I,INEW)
      CALL UMADR0(NF,N,NC,MC,MMAX,ICA,CG,ICG,JCG,CR,ICR,JCR,S,EPS7,
     & GMAX,UMAX,INEW,IER)
      CALL UXVIND(NF,IX,I,IER)
      IF (ITERM.NE.0) RETURN
      CALL UOYAD1(IER,'UYADP0')
      CALL UOYAD6(NF,N,NC,MMAX,IX,IC,ICA,CR,ICR,JCR,CZ,UMAX,GMAX,INEW,
     & IER)
      IF (IER.LT.0) THEN
      ITERM=-15
      TXFU='BAD ADD.'
      CALL UOERR1('UYADP0',15)
      RETURN
      ENDIF
      ENDIF
      RETURN
      END
