* SUBROUTINE UYADU2                ALL SYSTEMS                04/12/01
C PORTABILITY : ALL SYSTEMS
C 04/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SIMPLIFIED AGGREGATION FOR NONSMOOTH VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  HH(N*(N+1)/2)  GILL-MURRAY DECOMPOSITION OF THE AGGREGATED
*         HESSIAN MATRIX.
*  RI  G(N)  SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GV(N)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  H(N*(N+1)/2)  HESSIAN MATRIX OF THE OBJECTIVE FUNCTION.
*  RI  HV(N*(N+1)/2)  AGGREGATED HESSIAN MATRIX OF THE OBJECTIVE
*         FUNCTION.
*  RA  S(N)  AUXILIARY VECTOR.
*  RO  ALF  LINEARIZATION TERM.
*  RU  ALFV  AGGREGATED LINEARIZATION TERM.
*
      SUBROUTINE UYADU2(N,HH,G,GV,H,HV,S,ALF,ALFV)
      INTEGER N
      REAL*8 HH(N*(N+1)/2),G(N),GV(N),H(N*(N+1)/2),HV(N*(N+1)/2),S(N),
     & ALF,ALFV
      REAL*8 LAM,P,Q,W
      INTEGER I
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      P=ALFV-ALF
      DO 1 I=1,N
      W=S(I)
      P=P+W*S(I)
      S(I)=G(I)-GV(I)
    1 CONTINUE
      Q=ZERO
      CALL UXDPGB(N,HH,S,1)
      DO 4 I=1,N
        W=ONE/H(I*(I+1)/2)
        Q=Q+W*S(I)**2
    4 CONTINUE
      LAM=0.5D 0+SIGN(0.5D 0,P)
      IF (Q.GT.ZERO) LAM=MIN(ONE,MAX(ZERO,P/Q))
      P=ONE-LAM
      ALFV=LAM*ALF+P*ALFV
      DO 5 I=1,N
      GV(I)=LAM*G(I)+P*GV(I)
    5 CONTINUE
      DO 6 I=1,N*(N+1)/2
      HV(I)=LAM*H(I)+P*HV(I)
    6 CONTINUE
      RETURN
      END
