* SUBROUTINE UYALU2                ALL SYSTEMS                04/12/01
C PORTABILITY : ALL SYSTEMS
C 04/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SIMPLIFIED AGGREGATION FOR NONSMOOTH VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  M  NUMBER OF COLUMNS OF XM.
*  RI  XM(N*M)  RECTANGULAR MATRIX IN THE PRODUCT FORM SHIFTED BROYDEN
*         METHOD (STORED COLUMNWISE): H-SIGMA*I=XM*TRANS(XM)
*  RA  GR(M)  AUXILIARY VECTOR.
*  RI  G(N)  SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GV(N)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RA  S(N)  AUXILIARY VECTOR.
*  RI  PAR  SHIFT PARAMETER.
*  RO  ALF  LINEARIZATION TERM.
*  RU  ALFV  AGGREGATED LINEARIZATION TERM.
*
      SUBROUTINE UYALU2(N,M,XM,GR,G,GV,S,PAR,ALF,ALFV)
      INTEGER N,M
      REAL*8 XM(N*M),GR(M),G(N),GV(N),S(N),PAR,ALF,ALFV
      REAL*8 LAM,P,Q,W
      INTEGER I
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      P=ALFV-ALF
      DO 1 I=1,N
      W=S(I)
      P=P+W*S(I)
      S(I)=G(I)-GV(I)
    1 CONTINUE
      Q=ZERO
      CALL UXDRMM(N,M,XM,S,GR)
      DO 4 I=1,N
        Q=Q+PAR*S(I)**2
    4 CONTINUE
      DO 2 I=1,M
        Q=Q+GR(I)**2
    2 CONTINUE
      LAM=0.5D 0+SIGN(0.5D 0,P)
      IF (Q.GT.ZERO) LAM=MIN(ONE,MAX(ZERO,P/Q))
      P=ONE-LAM
      ALFV=LAM*ALF+P*ALFV
      DO 5 I=1,N
      GV(I)=LAM*G(I)+P*GV(I)
    5 CONTINUE
      RETURN
      END
