* SUBROUTINE UYFUT4                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* TERMINATION CRITERIA AND TEST ON RESTART.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  OLD VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FP  MINIMUM VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FUB  INTERMEDIATE VALUE OF THE OBJECTIVE FUNCTION.
*  RI  DMAX  MAXIMUM RELATIVE DIFFERENCE OF VARIABLES.
*  II  ITES  SYSTEM VARIBLE WHICH SPECIFIES TERMINATION. IF ITES=0
*         THEN TERMINATION IS SUPPRESSED.
*  II  IRES1  RESTART SPECIFICATION. RESTART IS PERFORMED AFTER
*         IRES1*N+IRES2 ITERATIONS.
*  II  IRES2  RESTART SPECIFICATION. RESTART IS PERFORMED AFTER
*         IRES1*N+IRES2 ITERATIONS.
*
* COMMON DATA :
*  II  ITERS  TERMINATION INDICATOR FOR STEPLENGTH DETERMINATION.
*         ITERS=0 FOR ZERO STEP.
*  IO  ITERM  TERMINATION INDICATOR. ITERM=1-TERMINATION AFTER MTESX
*         UNSUFFICIENT STEPLENGTHS. ITERM=2-TERMINATION AFTER MTESF
*         UNSUFFICIENT FUNCTION DECREASES. ITERM=3-TERMINATION ON LOWER
*         BOUND FOR FUNCTION VALUE. ITERM=4-TERMINATION ON LOWER BOUND
*         FOR GRADIENT. ITERM=11-TERMINATION AFTER MAXIMUM NUMBER OF
*         ITERATIONS. ITERM=12-TERMINATION AFTER MAXIMUM NUMBER OF
*         COMPUTED FUNCTION VALUES.
*  IU  IREST  RESTART INDICATOR. RESTART IS PERFORMED IF IREST>0.
*  II  MTESX  MAXIMUM NUMBER OF TESTS ON STEPLENGTH.
*  IU  NTESX  ACTUAL NUMBER OF TESTS ON STEPLENGTH.
*  II  MTESF  MAXIMUM NUMBER OF TESTS ON FUNCTION DECREASE.
*  IU  NTESF  ACTUAL NUMBER OF TESTS ON FUNCTION DECREASE.
*  II  LIT  NUMBER OF THE FIRST ITERATION.
*  II  MIT  MAXIMUM NUMBER OF ITERATIONS.
*  IU  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  LFV  NUMBER OF THE FIRST COMPUTED FUNCTION VALUE.
*  II  MFV  MAXIMUM NUMBER OF COMPUTED FUNCTION VALUES.
*  IU  NFV  ACTUAL NUMBER OF COMPUTED FUNCTION VALUES.
*  II  LFG  NUMBER OF THE FIRST COMPUTED GRADIENT VALUE.
*  II  MFG  MAXIMUM NUMBER OF COMPUTED GRADIENT VALUES.
*  IU  NFG  ACTUAL NUMBER OF COMPUTED GRADIENT VALUES.
*  II  KD  DEGREE OF REQUIRED DERIVATIVES.
*  II  JOUT  SYSTEM VARIABLE WHICH SPECIFIES PRINT OF INFORMATION
*         DURING THE OPTIMIZATION PROCESS. JOUT=0-NO PRINT.
*         JOUT=1-PRINT IS REQUIRED.
*  II  KOUT  LEVEL OF PRINT. KOUT=0-NO PRINT. KOUT=1-PRINT OF INITIAL
*         VALUES AND FINAL RESULTS TOGETHER WITH PRINT OF ITERATIONS.
*         KOUT=2-ADDITIONAL PRINT OF STEPSIZE SELECTION.
*         KOUT=3-ADDITIONAL PRINT OF DIRECTION DETERMINATION.
*         KOUT=4-ADDITIONAL PRINT OF MATRIX CORRECTION.
*         KOUT=5-ADDITIONAL PRINT OF NUMERICAL DIFFERENTIATION.
*         KOUT=6-ADDITIONAL PRINT OF DEBUGGING.
*  II  KOUT1  LOWER ITERATION IN WHICH INFORMATION IS PRINTED.
*  II  KOUT2  UPPER ITERATION IN WHICH INFORMATION IS PRINTED.
*  II  KOUT3  NUMBER OF ITERATIONS AFTER THAT INFORMATION IS PRINTED.
*  RI  TOLX  LOWER BOUND FOR STEPLENGTH.
*  RI  TOLF  LOWER BOUND FOR FUNCTION DECREASE.
*  RI  TOLB  LOWER BOUND FOR FUNCTION VALUE.
*  RI  TOLG  LOWER BOUND FOR GRADIENT.
*  TU  TPXX  TEXT INFORMATION ON THE CAUSE OF RESTART.
*  TO  TXFU  TEXT INFORMATION ON THE CAUSE OF TERMINATION.
*
* SUBPROGRAMS USED :
*  S   UYRES1  DEFINITION OF THE RESTART VARIABLES.
*
      SUBROUTINE UYFUT4(N,F,FO,FP,FUB,DMAX,ITES,IRES1,IRES2)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,ITES,IRES1,IRES2
      REAL*8  F,FO,FP,FUB,DMAX
      REAL*8  TEMP
      IF (ITERM.LT.0) RETURN
      IF (ITES .LE.0) GOTO 2
      IF (ITERS.EQ.0) GOTO 1
      IF (ITERL.LT.0) THEN
      TXFU='NON FEAS'
      ITERM=ITERL-10
      ENDIF
      IF (NIT.LE.LIT) FO=F+MIN(SQRT(ABS(F)),ABS(F)/TEN)
      IF ( F .LE. TOLB)  THEN
      ITERM = 3
      TXFU = 'FV BOUND'
      RETURN
      ENDIF
      IF ( NIT .LE. LIT)  THEN
      NTESX = 0
      NTESF = 0
      ENDIF
      IF ( DMAX .LE. TOLX)  THEN
      ITERM = 1
      TXFU = 'STEP TOL'
      NTESX = NTESX + 1
      IF ( NTESX .GE. MTESX) GOTO3
      ELSE
      NTESX = 0
      ENDIF
      TEMP = ABS( FO-FUB) / MAX( ABS(FUB), ONE)
      IF ( TEMP .LE. TOLF )  THEN
        ITERM = 2
        TXFU = 'FV   TOL'
        NTESF = NTESF + 1
        IF ( NTESF .GE. MTESF) GOTO3
      ELSE
        NTESF = 0
      ENDIF
    1 IF ( NIT-LIT .GE. MIT)  THEN
      ITERM = 11
      TXFU = 'IT LIMIT'
      CALL UOERR1('UYFUT4',-1)
      GOTO3
      ENDIF
      IF ( NFV-LFV .GE. MFV)  THEN
      ITERM = 12
      TXFU = 'FV LIMIT'
      CALL UOERR1('UYFUT4',-2)
      GOTO3
      ENDIF
      IF ( NFG-LFG .GE. MFG)  THEN
      ITERM = 13
      TXFU = 'FG LIMIT'
      CALL UOERR1('UYFUT1',-3)
      RETURN
      ENDIF
    2 ITERM = 0
      TPXX='        '
      IF ( N .GT. 0 .AND. NIT-KIT .GE. IRES1*N + IRES2)   THEN
      CALL UYRES1('NORMAL  ')
      ENDIF
      NIT = NIT + 1
      JOUT=0
      IF(KOUT.EQ.0.OR.KOUT3.LE.0) RETURN
      IF(KOUT1.LE.KOUT2) THEN
        IF(NIT-LIT.GE.KOUT1.AND.NIT-LIT.LE.KOUT2) JOUT=1
      ELSE
        IF(NIT-LIT.GE.KOUT1.OR. NIT-LIT.LE.KOUT2) JOUT=1
      ENDIF
      IF(NIT-LIT-(INT((NIT-LIT)/KOUT3)*KOUT3).NE.0) JOUT=0
      RETURN
  3   F=FP
      RETURN
      END
