* SUBROUTINE UYSFIB                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SYMBOLIC FACTORIZATION OF THE SYMMETRIC (HESSIAN) MATRIX.
*
* PARAMETERS :
*  II  NF DECLARED NUMBER OF VARIABLES.
*  II  M  LENGTH OF THE PACKED FORM OF THE HESSIAN APPROXIMATION.
*  II  NC  NUMBER OF CURRENT LINEAR CONSTRAINTS.
*  II  MC  NUMBER OF ELEMENTS IN THE FIELD CG.
*  II  MMAX LENGTH OF THE ARRAYS IW.
*  II  IH(NF+1) POINTERS OF THE HESSIAN MATRIX STORED
*           IN A COMPACT FORM.
*  II  JH(M) COLUMN INDICES OF NONZEROES IN THE HESSIAN MATRIX.
*  II  ICG(NF+1)  POINTERS OF THE JACOBIAN MATRIX.
*  II  JCG(MC)  COLUMN INDICES OF THE JACOBIAN MATRIX.
*  IO  KCG(MC)  INDICES OF ELEMENTS OF THE ARRAY CG IN THE ARRAY B.
*  IO  IB(NF+NC+1)  POINTERS OF THE INDEFINITE KARUSH-KUHN-TUCKER
*         MATRIX.
*  IO  JB(MF+MC+NC)  COLUMN INDICES OF THE INDEFINITE KARUSH-KUHN-TUCKER
*         MATRIX.
*  II  KB(MF+MC+NC)  ROW INDICES OF THE INDEFINITE KARUSH-KUHN-TUCKER
*         MATRIX.
*  II  IW(MMAX) WORKING ARRAY.
*  IA  IW1(2*(NF+NC)) WORKING ARRAY.
*  IA  IW2(3*(NF+NC)) WORKING ARRAY.
*  IO  NAU  NUMBER OF MAJOR STEPS.
*  IO  INF  ERROR INDICATOR. INF=0-NO ERRORS.
*  II  JOB  CHOICE OF SYMMETRIC MATRIX. JOB=1-HESSIAN MATRIX.
*         JOB=2-POSITIVE DEFINITE DIAGONAL MATRIX.
*
* COMMON DATA :
*  IO  ITERM  TERMINATION INDICATOR.
*  TO  TXFU  TEXT INFORMATION ON THE CAUSE OF TERMINATION.
*
* SUBPROGRAMS USED :
*  S   UXSKIS  SYMBOLIC TRANSFORMATION OF THE PARTITIONED
*         KARUSH-KUHN-TUCKER MATRIX INTO THE COMPLETE SYMMETRIC
*         STRUCTURE.
*  S   UXSIBS  MINIMUM DEGREE ORDERING.
*
      SUBROUTINE UYSFIB(NF,M,NC,MC,MMAX,IH,JH,ICG,JCG,KCG,IB,JB,KB,
     & IW,IW1,IW2,NAU,INF,JOB)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,M,NC,MC,MMAX,NAU,INF,JOB
      INTEGER IH(NF+1),JH(M),ICG(NC+1),JCG(MC),KCG(MC),IB(NF+NC+1),
     & JB(M+MC+NC),KB(M+MC+NC),IW(MMAX),IW1(2*(NF+NC)),IW2(3*(NF+NC))
      INTEGER I,J,NN,MM
      IF (JOB.EQ.1) THEN
      M=IH(NF+1)-1
      CALL UXSKIS(NF,M,IH,JH,NC,MC,ICG,JCG,KCG,IB,JB)
      MM=M+MC+NC
      ELSE
      CALL UXVINP(NF+1,IW1)
      CALL UXVINP(NF+1,IW2)
      CALL UXSKIS(NF,NF,IW1,IW2,NC,MC,ICG,JCG,KCG,IB,JB)
      MM=NF+MC+NC
      ENDIF
      NN=NF+NC
      DO 2 I=1,NN
      DO 1 J=IB(I),IB(I+1)-1
      KB(J)=I
    1 CONTINUE
    2 CONTINUE
      CALL UXSIBS(NN,MM,KB,JB,IW,MMAX,IW1,IW2,NAU,INF)
      IF (INF.LT.0) THEN
      ITERM=INF
      TXFU='LACK SPC'
      ENDIF
      RETURN
      END
