* SUBROUTINE UYSORE               ALL SYSTEMS                08/10/01
C PORTABILITY : ALL SYSTEMS
C 08/10/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* LSTRS METHOD: COMPUTES SMALLEST EIGENPAIRS OF BORDERED MATRIX.
*
* PARAMETERS :
*  II  NN  NUMBER OF VARIABLES (N OR N+1).
*  II  M  NUMBER OF NONZERO ELEMENTS IN HESSIAN MATRIX.
*  II  NCV  NUMBER OF LANCZOS VECTORS.
*  RI  H(M)  ELEMENTS OF THE SPARSE SYMMETRIC MATRIX STORED IN THE
*      PACKED FORM.
*  II  IH(NN) POINTERS OF THE HESSIAN MATRIX STORED IN A COMPACT FORM.
*  II  JH(M) COLUMN INDICES OF NONZEROES IN THE MATRIX H.
*  RI  G(NN)  GRADIENT.
*  RI  ALPHA  SCALAR PARAMETER IN LSTRS METHOD.
*  RO  LAMBDA1  THE 1-ST SMALLEST EIGENVALUE OF THE BORDERED MATRIX.
*  RO  LAMBDA2  THE 2-ND SMALLEST EIGENVALUE OF THE BORDERED MATRIX.
*  RO  NU1  THE FIRST COMPONENT OF THE 1-ST EIGENVECTOR.
*  RO  NU2  THE FIRST COMPONENT OF THE 2-ND EIGENVECTOR.
*  RO  U1(NN-1)  COMPONENTS (2,...,N+1) OF THE 1-ST EIGENVECTOR.
*  RO  U2(NN-1)  COMPONENTS (2,...,N+1) OF THE 2-ND EIGENVECTOR.
*  RO  U1NORM  THE NORM OF THE VECTOR U1.
*  RO  U2NORM  THE NORM OF THE VECTOR U2.
*  IO  NCONV  NUMBER OF CONVERGED EIGENVALUES OF THE BORDERED MATRIX.
*  IU  NMV  NUMBER OF MATRIX-VECTOR MULTIPLICATIONS.
*  RA  RESID(NN)  AUXILIARY ARRAY.
*  RA  V(NN,NCV)  AUXILIARY ARRAY.
*  RA  WORKD(3*NN)  AUXILIARY ARRAY.
*  RA  WORKL(NCV*(NCV+8))  AUXILIARY ARRAY.
*  LA  SELECT(NCV)  AUXILIARY ARRAY.
*  RI  ETA4  STOPPING TOLERANCE FOR EIGENVALUES OF THE BORDERED MATRIX.
*  II  NEV  NUMBER OF WANTED EIGENPAIRS (1 OR 2)
*  IO  INFO  INFO<>0 - WRONG OUTPUT FROM ARPACK.
*
* SUBPROGRAMS USED :
*  S   UXSSMM  MULTIPLICATION OF A SPARSE SYMMETRIC MATRIX BY A VECTOR.
*  S   UXSSMD  MULTIPLICATION OF A SPARSE SYMMETRIC MATRIX BY A VECTOR
*              AND ADDITION OF A SCALED VECTOR.
*  S   UXVCOP  COPYING OF A VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  RF  UXVNOR  EUCLIDEAN NORM OF A VECTOR.
*  S   DSAUPD  ARPACK SUBROUTINE FOR COMPUTATION OF EIGENPAIRS.
*  S   DSEUPD  ARPACK SUBROUTINE FOR COMPUTATION OF EIGENPAIRS.
*
      SUBROUTINE UYSORE(NN,M,NCV,H,IH,JH,G,ALPHA,LAMBDA1,LAMBDA2,
     & NU1,NU2,U1,U2,U1NORM,U2NORM,NCONV,NMV,RESID,V,WORKD,WORKL,
     & SELECT,ETA4,NEV,INFO)
      REAL*8 ONE
      PARAMETER (ONE=1.0D0)
      INTEGER NN,M,NCV,IH(NN),JH(M),NCONV,NMV,NEV,INFO
      REAL*8 H(M),G(NN-1),ALPHA,LAMBDA1,LAMBDA2,NU1,NU2,U1(NN-1),
     & U2(NN-1),U1NORM,U2NORM,RESID(NN),V(NN,NCV),WORKD(3*NN),
     & WORKL(NCV*(NCV+8)),ETA4
      LOGICAL SELECT(NCV)
      INTEGER IDO,LDV,IPARAM(11),IPNTR(11),LWORKL
      REAL*8 D(2),SIGMA,UXVDOT,UXVNOR
      CHARACTER BMAT*1,WHICH*2,HOWMNY*1
      LOGICAL RVEC
      IDO = 0
      BMAT = 'I'
      WHICH = 'SA'
      LDV = NN
      IPARAM(1) = 1
      IPARAM(3) = 20  ! MAX NUMBER OF ARNOLDI STEPS
      IPARAM(4) = 1
      IPARAM(7) = 1
      LWORKL = NCV*(NCV+8)
      INFO = 0
      DO WHILE (IDO.NE.99)
      CALL DSAUPD(IDO,BMAT,NN,WHICH,NEV,ETA4,RESID,NCV,V,LDV,IPARAM,
     & IPNTR,WORKD,WORKL,LWORKL,INFO)
      IF (ABS(IDO).EQ.1) THEN
      IF (NEV.EQ.1) THEN
      CALL UXSSMM(NN,M,H,IH,JH,WORKD(IPNTR(1)),WORKD(IPNTR(2)))
      ELSE
      WORKD(IPNTR(2)) = ALPHA*WORKD(IPNTR(1)) +
     & UXVDOT(NN-1,G,WORKD(IPNTR(1)+1))
      CALL UXSSMD(NN-1,M,H,IH,JH,WORKD(IPNTR(1)+1),WORKD(IPNTR(1)),
     & G,WORKD(IPNTR(2)+1))
      ENDIF
      ENDIF
      ENDDO
      IF (INFO.LT.0) RETURN
      RVEC = .TRUE.
      HOWMNY = 'A'
      CALL DSEUPD(RVEC,HOWMNY,SELECT,D,V,LDV,SIGMA,BMAT,NN,WHICH,NEV,
     & ETA4,RESID,NCV,V,LDV,IPARAM,IPNTR,WORKD,WORKL,LWORKL,INFO)
      IF (INFO.NE.0) RETURN
      NMV = NMV + IPARAM(9)
      IF (NEV.EQ.2) THEN
      NCONV = IPARAM(5)
      LAMBDA1 = D(1)
      LAMBDA2 = D(2)
      NU1 = V(1,1)
      NU2 = V(1,2)
      CALL UXVCOP(NN-1,V(2,1),U1)
      CALL UXVCOP(NN-1,V(2,2),U2)
      IF (ABS(NU1).GT.ONE) THEN
      U1NORM = UXVNOR(NN-1,U1)
      ELSE
      U1NORM = SQRT(ONE - NU1**2)
      ENDIF
      IF (ABS(NU2).GT.ONE) THEN
      U2NORM = UXVNOR(NN-1,U2)
      ELSE
      U2NORM = SQRT(ONE - NU2**2)
      ENDIF
      ENDIF
      RETURN
      END
