* SUBROUTINE UYSSEQ                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF A CONSISTENT PARTITION OF THE COLUMNS OF A BY A
* SEQUENTIAL ALGORITHM.
*
* PARAMETERS :
*  II  NF DECLARED NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  IA(NA+1) ROW POINTERS OF THE SPARSE JACOBIAN MATRIX.
*  II  JA(MA) COLUMN INDICES OF THE SPARSE JACOBIAN MATRIX.
*  II  IB(NF+1) COLUMN POINTERS OF THE SPARSE JACOBIAN MATRIX.
*  II  JB(MA) ROW INDICES OF THE SPARSE JACOBIAN MATRIX.
*  II  LIST(NF) VECTOR SPECIFYING INCIDENCE-DEGREE ORDERING.
*  IO  IG(NF) VECTOR INDICATING GROUPS FOR THE JACOBIAN COLUMNS.
*  IA  IW(NF)  AUXILIARY VECTOR.
*  IO  MAXG  NUMBER OF GROUPS IN THE PARTITION OF THE COLUMNS OF A.
*
      SUBROUTINE UYSSEQ(NF,NA,MA,IA,JA,IB,JB,LIST,IG,IW,MAXG)
      INTEGER MAXG
      INTEGER NF,NA,MA,IA(NA+1),JA(MA),IB(NF+1),JB(MA),LIST(NF),IG(NF),
     & IW(NF)
      INTEGER IC,IP,IR,J,JC,JP
C
C     INITIALIZATION BLOCK
C
      MAXG = 0
      DO 10 JP = 1, NF
      IG(JP) = NF
      IW(JP) = 0
   10 CONTINUE
C
C     BEGINNING OF ITERATION LOOP
C
      DO 60 J = 1, NF
      JC = LIST(J)
C
C     FIND ALL COLUMNS ADJACENT TO COLUMN JC.
C
C     DETERMINE ALL POSITIONS (IR,JC) WHICH CORRESPOND
C     TO NON-ZEROES IN THE MATRIX
C
      DO 30 JP = IB(JC), IB(JC+1)-1
      IR = JB(JP)
C
C     FOR EACH ROW IR, DETERMINE ALL POSITIONS (IR,IC)
C     WHICH CORRESPOND TO NON-ZEROES IN THE MATRIX
C
      DO 20 IP = IA(IR), IA(IR+1)-1
      IC = JA(IP)
C
C     ARRAY IW MARKS THE GROUP NUMBERS OF THE
C     COLUMNS WHICH ARE ADJACENT TO COLUMN JC
C
      IW(IG(IC)) = J
   20 CONTINUE
   30 CONTINUE
C
C     ASSIGN THE SMALLEST UN-MARKED GROUP NUMBER TO JC
C
      DO 40 JP = 1, MAXG
      IF (IW(JP) .NE. J) GO TO 50
   40 CONTINUE
      MAXG = MAXG + 1
   50 CONTINUE
      IG(JC) = JP
   60 CONTINUE
C
C     END OF ITERATION LOOP
C
      RETURN
      END
