* SUBROUTINE UYTLUD                ALL SYSTEMS                02/12/01
C PORTABILITY : ALL SYSTEMS
C 02/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE REDUCED HESSIAN VARIABLE METRIC TRANSFORMATIONS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  IU  N  CURRENT NUMBER OF VARIABLE METRIC STEPS.
*  II  MF  MAXIMUM NUMBER OF VARIABLE METRIC STEPS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RU  XM(NF*(MF+1))  SET OF VECTOR OF VARIABLES DIFFERENCES.
*  RU  XR((MF+1)*(MF+2)/2)  CHOLESKI DECOMPOSITION OF THE REDUCED
*         HESSIAN MATRIX.
*  RU  HR((MF+1)*(MF+2)/2)  TRIANGULAR TRANFORMATION MATRIX.
*  RI  G(NF) GRADIENT VECTOR.
*  RO  GR(MF+1) REDUCED GRADIENT VECTOR.
*  RI  SR(MF+1) REDUCED DIRECTION VECTOR.
*  RU  XO(NF)  REDUCED VECTOR OF VARIABLES DIFFERENCE.
*  RU  GO(NF)  REDUCED GRADIENT DIFFERENCE.
*  RI  R  STEPSIZE PARAMETER.
*  RO  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  OLD VALUE OF THE OBJECTIVE FUNCTION.
*  RO  P  NEW VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  PO  OLD VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  DMAX  MAXIMUM RELATIVE DIFFERENCE OF VARIABLES.
*  RI  GNORM  NORM OF THE GRADIENT VECTOR.
*  RI  EPS7  TOLERANCE FOR LINEAR DEPENDENCE.
*  RI  SIG  SCALING COEFFICIENT.
*  II  MET5  SCALING OPTION. MET5=1-STANDARD SCALING. MET5=2-SIMPLIFIED
*         SCALING.
*  II  MOS2  COMPUTATION OF THE PROJECTED GRADIENT NORM.
*         MOS2=1-DIFFERENCE OF TWO GRADIENT NORMS. MOS2=2-PROJECTED
*         GRADIENT NORM
*  IO  INF  LINEAR DEPENDENCE INDICATOR. INF=0 IF THE CURRENT GRADIENT
*         IS A LINEAR COMBINATION OF BASIC VECTORS. INF=1 OTHERWISE.
*  CO  TEXT  TYPE OF DECOMPOSITION. TEXT='R'-RECURSIVE QR DECOMPOSITION
*         WITHOUT PERMUTATIONS. TEXT='I'-INVERSION.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXDRMM  VECTOR MULTIPLIED BY A ROWWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*
      SUBROUTINE UYTLUD(NF,N,MF,X,XN,XM,XR,HR,G,GR,SR,XO,GO,R,F,FO,
     & P,PO,DMAX,GNORM,EPS7,SIG,MET5,MOS2,INF,TEXT)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,N,MF,MET5,MOS2,INF
      REAL*8 X(NF),XN(NF),XM(NF*(MF+1)),XR((MF+1)*(MF+2)/2),
     & HR((MF+1)*(MF+2)/2),G(NF),GR(MF+1),SR(MF+1),XO(NF),GO(NF),
     & R,F,FO,P,PO,DMAX,GNORM,EPS7,SIG
      CHARACTER*1 TEXT
      REAL*8 RHO,UXVDOT
      INTEGER I,KK
      IF (ITERS.GT.0) THEN
      CALL UXVDIF(NF,X,XO,XO)
      PO=R*PO
      P=R*P
      ELSE
      F = FO
      P = PO
      CALL UXVSAV(NF,X,XO)
      CALL UYSET1
      ENDIF
      DMAX = ZERO
      DO 1 I=1,NF
      IF ( NORMF .GT. 0) THEN
      DMAX=MAX(DMAX,ABS(XO(I))/MAX(ABS(X(I)),XN(I)))
      XO(I)=XO(I)/XN(I)
      ELSE
      DMAX=MAX(DMAX,ABS(XO(I))/MAX(ABS(X(I)),ONE))
      ENDIF
    1 CONTINUE
      IF (ITERS.LE.0) RETURN
      CALL UXVSCL(N,R,SR,XO)
      CALL UXVCOP(N,GR,GO)
      CALL UXDRMM(NF,N,XM,G,GR)
      CALL UXDPRB(N,HR,GR,1)
      CALL UXVDIF(N,GR,GO,GO)
      IF (MOS2.EQ.1) THEN
      RHO=SQRT(ABS(UXVDOT(NF,G,G)-UXVDOT(N,GR,GR)))
      ELSE
      CALL UXVCOP(N,GR,HR(N*(N+1)/2+1))
      CALL UXDPRB(N,HR,HR(N*(N+1)/2+1),-1)
      CALL UXDCMD(NF,N,XM,HR(N*(N+1)/2+1),-ONE,G,XM(N*NF+1))
      RHO=SQRT(UXVDOT(NF,XM(N*NF+1),XM(N*NF+1)))
      ENDIF
      IF (RHO.LE.EPS7*GNORM) THEN
      INF=0
      ELSE
      INF=1
      N=N+1
      XO(N)=ZERO
      GO(N)=RHO
      IF (MET5.EQ.2) THEN
      SIG=UXVDOT(N,XO,GO)
      IF (SIG.LE.ZERO) THEN
      SIG=ONE
      ELSE IF (TEXT.EQ.'R') THEN
      SIG=SQRT(UXVDOT(N,GO,GO)/SIG)
      ELSE
      SIG=SIG/UXVDOT(N,GO,GO)
      ENDIF
      ENDIF
      KK=N*(N-1)/2+1
      CALL UXVSET(N-1,ZERO,XR(KK))
      XR(KK+N-1)=SIG
      GR(N)=RHO
      CALL UXVCOP(NF,G,XM((N-1)*NF+1))
      CALL UXVCOP(N,GR,HR(KK))
      ENDIF
      RETURN
      END
