* SUBROUTINE UYTRPG               ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* GRADIENT OF THE OBJECTIVE FUNCTION IS SCALED AND REDUCED.
* TEST VALUES GMAX AND UMAX ARE COMPUTED.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS IN THE FIELD CG.
*  II  MMAX  MAXIMUM NUMBER OF NONZERO ELEMENTS IN THE FIELD CR.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CG(MC)  SPARSE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(MCL+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC) COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  RI  CR(MMAX)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  II  ICR(NF+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CR.
*  II  JCR(MMAX) COLUMN INDICES OF ELEMENTS IN THE FIELD CR.
*  RO  CZ(NF)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  GN(NF)  TRANSFORMED GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  UMAX  MAXIMUN ABSOLUTE VALUE OF THE NEGATIVE LAGRANGE MULTIPLIER.
*  RI  GMAX  NORM OF THE TRANSFORMED GRADIENT.
*  II  IOLD  INDEX OF THE REMOVED CONSTRAINT.
*
* COMMON DATA :
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* SUBPROGRAMS USED :
*  S   UMSLAG  NEGATIVE PROJECTED GRADIENT IS DETERMINED.
*  S   ULTLAG  THE OPTIMUM LAGRANGE MULTIPLIER IS DETERMINED.
*  S   UMVLAG  AN AUXILIARY VECTOR IS DETERMINED.
*  S   UXSPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXVSET  INITIATION OF A VECTOR.
*
      SUBROUTINE UYTRPG(NF,N,NC,MC,MMAX,IX,XN,IC,ICA,CG,ICG,JCG,CR,ICR,
     & JCR,CZ,G,GN,EPS7,UMAX,GMAX,IOLD)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,N,NC,MC,MMAX,IX(NF),IC(NC),ICA(NF),ICG(NC+1),JCG(MC),
     & ICR(NF+1),JCR(MMAX),IOLD
      REAL*8 XN(NF),CG(MC),CR(MMAX),CZ(NF),G(NF),GN(NF),EPS7,UMAX,GMAX
      IF (NORMF.GT.0) THEN
      CALL UXVMUL(NF,XN,G,GN,1)
      ELSE
      CALL UXVCOP(NF,G,GN)
      ENDIF
      IF (NF.GT.N) THEN
      CALL UMVLAG(NF,N,NC,NC,NC,MC,MC,ICA,CG,ICG,JCG,CG,ICG,JCG,GN,CZ)
      CALL UXSPRB(NF-N,MMAX,CR,ICR,JCR,CZ,0)
      CALL ULTLAG(NF,N,NC,NC,NC,IX,IC,ICA,CZ,IC,EPS7,UMAX,IOLD)
      ELSE
      IOLD=0
      UMAX=0.0D 0
      ENDIF
      IF (N.GT.0) THEN
      CALL UXVNEG(NF,GN,GN)
      CALL UMSLAG(NF,N,NC,NC,NC,MC,MC,ICA,CG,ICG,JCG,CZ,CG,ICG,JCG,GN,
     & EPS7,GMAX)
      ELSE
      CALL UXVSET(NF,0.0D 0,GN)
      GMAX=0.0D 0
      ENDIF
      RETURN
      END
