$INTERFACE UA0GS6                                             97/12/01

  $HELP
    PRECISION OF THE COMPUTED VALUES
  $CHECK(ETA1,ETA0,REAL,4)
  CALL UA0GS6(NF,NA,$X,$IX,$XN,$D,$DL,$DU,$AF,$AFO,$ETA1)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$D($NF)\$DL($NF)\$DU($NF)\$AF($NA)&
  \$AFO($NA)')
  $ADD(INTEGER,'\$IX($NX)')
$REM  $ADD(INTEGER,'\NF\NA\NAL')
  $ADD(INIT)
    NAGX='UA0GS6'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UA0GS6             ALL SYSTEMS                 97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* NUMERICAL COMPUTATION OF THE GRADIENT OF THE APPROXIMATED
* FUNCTION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF FUNCTIONS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  D(NF)  DIAGONAL OF THE JACOBIAN MATRIX.
*  RO  DL(NF)  SUBDIAGONAL OF THE JACOBIAN MATRIX.
*  RO  DU(NF)  SUPERDIAGONAL OF THE JACOBIAN MATRIX.
*  RI  AF(NA)  VECTOR OF FUNCTION VALUES.
*  RA  AFO(NA)  AUXILIARY VECTOR.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*
* COMMON DATA :
*  II  KBF  TYPE OF BOUNDS. KBF=0-BOUNDS ARE NOT USED. KBF=1-ONE SIDED
*         BOUNDS. KBF=1-TWO SIDED BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*
      SUBROUTINE UA0GS6(NF,NA,X,IX,XN,D,DL,DU,AF,AFO,ETA1)
      $INCLUDE('UMCOMM')
      INTEGER NF,NA,IX(NF)
      $FLOAT X(NF),XN(NF),D(NF),DL(NF),DU(NF),AF(NA),AFO(NA),ETA1
      $FLOAT XSTEP,XTEMP,FTEMP,ETA
      INTEGER IVAR,KA,KAP
      SAVE XSTEP,XTEMP,FTEMP,ETA
      SAVE IVAR
      IF (TSS(NS).NE.'UAGX') CALL UYPRO1('UAGX',-1)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      ETA=SQRT(ETA1)
      CALL UXVCOP(NA,AF,AFO)
      IVAR=0
    2 CONTINUE
      IVAR=IVAR+1
      IF (IVAR.GT.NF) GO TO 4
      IF (KBF.GT.0) THEN
      IF (IX(IVAR).LE.-5) GO TO 2
      ENDIF
C
C     STEP SELECTION
C
      IF (NORMF.GT.0) THEN
      XSTEP=XN(IVAR)
      ELSE
      XSTEP=ONE
      ENDIF
      XSTEP=ETA*MAX(ABS(X(IVAR)),XSTEP)*SIGN(ONE,X(IVAR))
      XTEMP=X(IVAR)
      X(IVAR)=X(IVAR)+XSTEP
      XSTEP=X(IVAR)-XTEMP
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
C
C     NUMERICAL DIFFERENTIATION
C
      KAP=0
      DO 33 KA=1,NA
      FTEMP=(AF(KA)-AFO(KA))/XSTEP
      IF (IVAR.LT.KA) THEN
      DL(KA-1)=FTEMP
      ELSE IF (IVAR.EQ.KA) THEN
      D(KA) =FTEMP
      ELSE
      DU(KA)=FTEMP
      ENDIF
      KAP=KAP+NF
   33 CONTINUE
      X(IVAR)=XTEMP
      GO TO 2
    4 CALL UXVCOP(NA,AFO,AF)
      CALL UYEPI1(-1)
      RETURN
      END
