* SUBROUTINE UCESD1             ALL SYSTEMS                   98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE SPARSE SCHUR COMPLEMENT STRUCTURE FOR THE
* KARUSH-KUHN-TUCKER MATRIX WITH THE UNIT HESSIAN BLOCK.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CONSTRAINT FUNCTIONS.
*  II  MC  NUMBER OF NONZERO ELEMENTS IN THE FIELD CG.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE SCHUR COMPLEMENT.
*  IO  ND  ACTUAL NUMBER OF DENSE ROWS.
*  IO  MD  MAXIMUM NUMBER OF DENSE ROWS.
*  IO  MDE  MINIMUM NUMBER OF NONZERO ELEMENTS IN DENSE ROWS.
*  IO  IB(NF+1)  POINTERS OF DIAGONAL ELEMENTS OF THE SPARSE SCHUR
*         COMPLEMENT.
*  IO  JB(M)  COLUMN INDICES OF NONZERO ELEMENTS OF THE SPARSE SCHUR
*         COMPLEMENT.
*  II  ICG(NC+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  IO  KCG(NF)  INDICATION OF DENSE ROWS.
*  IO  ITERM  TERMINATION INDICATOR.
*
      SUBROUTINE UCESD1(NF,NC,MC,M,ND,MD,MDE,IB,JB,ICG,JCG,KCG,ITERM)
      INTEGER NF,NC,MC,M,ND,MD,MDE,IB(NC+1),JB(M),ICG(NC+1),JCG(MC),
     & KCG(NF),ITERM
      INTEGER K,KK,K1,K2,L,LL,L2,MM,KC,KCP
      CALL UXVINS(NF,0,KCG)
      K2=0
      DO 2 KC=1,NC
      K1=K2+1
      K2=ICG(KC+1)-1
      DO 1 KK=K1,K2
      K=ABS(JCG(KK))
      KCG(K)=KCG(K)+1
    1 CONTINUE
    2 CONTINUE
      ND=0
      DO 3 K=1,NF
      IF (KCG(K).GE.MDE) THEN
      ND=ND+1
      KCG(K)=ND
      ELSE
      KCG(K)=0
      ENDIF
    3 CONTINUE
      IF (ND.GT.MD) THEN
      CALL UOERR2('UCESD1',85,ND,MD)
      ITERM=-85
      RETURN
      ENDIF
      MM=0
      IB(1)=1
      DO 6 KC=1,NC
      MM=MM+1
      JB(MM)=KC
      K1=ICG(KC)
      K2=ICG(KC+1)-1
      DO 5 KCP=KC+1,NC
      KK=K1
      LL=ICG(KCP)
      L2=ICG(KCP+1)-1
      K=ABS(JCG(KK))
      L=ABS(JCG(LL))
    4 IF (KCG(K).NE.0.OR.K.LT.L) THEN
      IF (KK.GE.K2) GO TO 5
      KK=KK+1
      K=ABS(JCG(KK))
      GO TO 4
      ELSE IF (KCG(L).NE.0.OR.L.LT.K) THEN
      IF (LL.GE.L2) GO TO 5
      LL=LL+1
      L=ABS(JCG(LL))
      GO TO 4
      ELSE
      MM=MM+1
      JB(MM)=KCP
      ENDIF
    5 CONTINUE
      IB(KC+1)=MM+1
    6 CONTINUE
      RETURN
      END
