* SUBROUTINE UCFDER             ALL SYSTEMS                 01/12/01
C PORTABILITY : ALL SYSTEMS
C 01/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE DIRECTIONAL DERIVATIVE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  II  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC+1)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  RI  CFD(NC)  VECTOR CONTAINING INCREMENTS OF CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  II  S(NF+NCA)  VECTOR CONTAINING NEWTON INCREMENTS.
*  RI  XO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR LOWER INACTIVE
*         MULTIPLIERS.
*  RI  GO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR UPPER INACTIVE
*         MULTIPLIERS.
*  RI  STEPSIZE  PARAMETER.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UCFDER(NF,NX,NC,NCA,X,IX,XL,XU,XSL,XSU,XZL,XZU,CF,CFD,
     & IC,CL,CU,CSL,CSU,CZL,CZU,ICA,S,XO,GO,RPF2,P)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,NCA,IX(NX),IC(NC),ICA(NCA)
      $FLOAT X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CF(NC),CFD(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),
     & S(NF+NCA),XO(NX+NC),GO(NX+NC),RPF2,P
      INTEGER I,K,KK
      $FLOAT TEMP
      DO 1 I=1,NCA
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      KK=-IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP=RPF2*(XSL(K)+XL(K)-X(K)-XZL(K))/XSL(K)
      P=P-TEMP*(S(NF+I)*(XSL(K)-XZL(K))+S(K)*(XSL(K)+XL(K)-X(K)))-
     & S(NF+I)*S(K)
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP=RPF2*(XSU(K)+X(K)-XU(K)-XZU(K))/XSU(K)
      P=P-TEMP*(S(NF+I)*(XSU(K)-XZU(K))-S(K)*(XSU(K)+X(K)-XU(K)))+
     & S(NF+I)*S(K)
      ENDIF
      ELSE
      KK=-IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP=RPF2*(CSL(K)+CL(K)-CF(K)-CZL(K))/CSL(K)
      P=P-TEMP*(S(NF+I)*(CSL(K)-CZL(K))+CFD(K)*(CSL(K)+CL(K)-CF(K)))-
     & S(NF+I)*CFD(K)
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP=RPF2*(CSU(K)+CF(K)-CU(K)-CZU(K))/CSU(K)
      P=P-TEMP*(S(NF+I)*(CSU(K)-CZU(K))-CFD(K)*(CSU(K)+CF(K)-CU(K)))+
     & S(NF+I)*CFD(K)
      ELSE IF (KK.EQ.5) THEN
      P=P+CFD(K)*(S(NF+I)+RPF2*CF(K))
      ENDIF
      ENDIF
    1 CONTINUE
      IF (KBF.GT.0) THEN
      DO 2 K=1,NX
      KK=IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP=RPF2*(XSL(K)+XL(K)-X(K)-XZL(K))/XSL(K)
      P=P-TEMP*(XO(K)*(XSL(K)-XZL(K))+S(K)*(XSL(K)+XL(K)-X(K)))-
     & XO(K)*S(K)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP=RPF2*(XSU(K)+X(K)-XU(K)-XZU(K))/XSU(K)
      P=P-TEMP*(GO(K)*(XSU(K)-XZU(K))-S(K)*(XSU(K)+X(K)-XU(K)))+
     & GO(K)*S(K)
      ENDIF
    2 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 3 K=1,NC
      KK=IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP=RPF2*(CSL(K)+CL(K)-CF(K)-CZL(K))/CSL(K)
      P=P-TEMP*(XO(NX+K)*(CSL(K)-CZL(K))+CFD(K)*(CSL(K)+CL(K)-CF(K)))-
     & XO(NX+K)*CFD(K)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP=RPF2*(CSU(K)+CF(K)-CU(K)-CZU(K))/CSU(K)
      P=P-TEMP*(GO(NX+K)*(CSU(K)-CZU(K))-CFD(K)*(CSU(K)+CF(K)-CU(K)))+
     & GO(NX+K)*CFD(K)
      ENDIF
    3 CONTINUE
      ENDIF
      RETURN
      END
