* SUBROUTINE UCIDEL             ALL SYSTEMS                 03/12/01
C PORTABILITY : ALL SYSTEMS
C 03/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF ACTIVE AND INACTIVE INCREMENTS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC+1)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  RI  CFD(NC)  VECTOR CONTAINING INCREMENTS OF CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  S(NF+NCA)  VECTOR CONTAINING NEWTON INCREMENTS.
*  RU  GS(NCA)  VECTOR CONTAINING ACTIVE SLACK INCREMENTS.
*  RO  XO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR LOWER INACTIVE
*         MULTIPLIERS.
*  RO  GO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR UPPER INACTIVE
*         MULTIPLIERS.
*  RI  RPF3  BARRIER COEFFICIENT.
*  II  IGS  CHOICE OF ACTIVE SLACK INCREMENTS. IGS=0-ACTIVE SLACK
*         INCREMENTS ARE GIVEN BEFOREHAND. IGS=1-ACTIVE SLACK
*         INCREMENTS ARE COMPUTED IN THIS SUBROUTINE.
*  II  JOB  CHOICE OF THE IP FORMULATION. JOB=1-PRIMAL FORMULATION.
*         JOB=2-PRIMAL-DUAL FORMULATION.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UCIDEL(NF,NX,NC,NCA,X,IX,XL,XU,XSL,XSU,XZL,XZU,CF,CFD,
     & IC,CL,CU,CSL,CSU,CZL,CZU,ICA,S,GS,XO,GO,RPF3,IGS,JOB)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,NCA,IX(NX),IC(NC),ICA(NCA),IGS,JOB
      $FLOAT X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CF(NC),CFD(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),
     & S(NF+NCA),GS(NCA),XO(NX+NC),GO(NX+NC),RPF3
      INTEGER I,K,KK
      DO 1 I=1,NCA
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      KK=-IX(K)
      IF (JOB.EQ.1) THEN
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      S(NF+I)=-S(NF+I)
      GS(I)=-GS(I)
C      IF (IGS.NE.0) GS(I)=D(I)*(XZL(K)+S(NF+I))+XSL(K)
      IF (IGS.NE.0) GS(I)=XSL(K)*(ONE-(XSL(K)/RPF3)*(XZL(K)+S(NF+I)))
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
C      IF (IGS.NE.0) GS(I)=D(I)*(XZU(K)+S(NF+I))+XSU(K)
      IF (IGS.NE.0) GS(I)=XSU(K)*(ONE-(XSU(K)/RPF3)*(XZU(K)+S(NF+I)))
      ENDIF
      ELSE
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      S(NF+I)=-S(NF+I)
      GS(I)=-GS(I)
C      IF (IGS.NE.0) GS(I)=D(I)*S(NF+I)-XSL(K)+RPF3/XZL(K)
      IF (IGS.NE.0) GS(I)=(RPF3-XSL(K)*S(NF+I))/XZL(K)-XSL(K)
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
C      IF (IGS.NE.0) GS(I)=D(I)*S(NF+I)-XSU(K)+RPF3/XZU(K)
      IF (IGS.NE.0) GS(I)=(RPF3-XSU(K)*S(NF+I))/XZU(K)-XSU(K)
      ENDIF
      ENDIF
      ELSE
      KK=-IC(K)
      IF (JOB.EQ.1) THEN
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      S(NF+I)=-S(NF+I)
      GS(I)=-GS(I)
C      IF (IGS.NE.0) GS(I)=D(I)*(CZL(K)+S(NF+I))+CSL(K)
      IF (IGS.NE.0) GS(I)=CSL(K)*(ONE-(CSL(K)/RPF3)*(CZL(K)+S(NF+I)))
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
C      IF (IGS.NE.0) GS(I)=D(I)*(CZU(K)+S(NF+I))+CSU(K)
      IF (IGS.NE.0) GS(I)=CSU(K)*(ONE-(CSU(K)/RPF3)*(CZU(K)+S(NF+I)))
      ENDIF
      ELSE
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      S(NF+I)=-S(NF+I)
      GS(I)=-GS(I)
C      IF (IGS.NE.0) GS(I)=D(I)*S(NF+I)-CSL(K)+RPF3/CZL(K)
      IF (IGS.NE.0) GS(I)=(RPF3-CSL(K)*S(NF+I))/CZL(K)-CSL(K)
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
C      IF (IGS.NE.0) GS(I)=D(I)*S(NF+I)-CSU(K)+RPF3/CZU(K)
      IF (IGS.NE.0) GS(I)=(RPF3-CSU(K)*S(NF+I))/CZU(K)-CSU(K)
      ENDIF
      ENDIF
      ENDIF
    1 CONTINUE
      IF (KBF.GT.0) THEN
      DO 2 K=1,NX
      KK=IX(K)
      IF (JOB.EQ.1) THEN
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      XO(K)=RPF3*(TWO+(XL(K)-X(K)-S(K))/XSL(K))/XSL(K)-XZL(K)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      GO(K)=RPF3*(TWO+(X(K)+S(K)-XU(K))/XSU(K))/XSU(K)-XZU(K)
      ENDIF
      ELSE
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      XO(K)=(RPF3+XZL(K)*(XL(K)-X(K)-S(K)))/XSL(K)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      GO(K)=(RPF3+XZU(K)*(X(K)+S(K)-XU(K)))/XSU(K)
      ENDIF
      ENDIF
    2 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 3 K=1,NC
      KK=IC(K)
      IF (JOB.EQ.1) THEN
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      XO(NX+K)=RPF3*(TWO+(CL(K)-CF(K)-CFD(K))/CSL(K))/CSL(K)-CZL(K)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      GO(NX+K)=RPF3*(TWO+(CF(K)+CFD(K)-CU(K))/CSU(K))/CSU(K)-CZU(K)
      ENDIF
      ELSE
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      XO(NX+K)=(RPF3+CZL(K)*(CL(K)-CF(K)-CFD(K)))/CSL(K)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      GO(NX+K)=(RPF3+CZU(K)*(CF(K)+CFD(K)-CU(K)))/CSU(K)
      ENDIF
      ENDIF
    3 CONTINUE
      ENDIF
      RETURN
      END
