* SUBROUTINE UCIDIR             ALL SYSTEMS                 98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* STEP FOR LAGRANGIAN MULTIPLIERS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NX)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NX)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NX)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NX)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NX)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC+1)  VECTOR CONTAINING VALUES OF CONSTRAINT FUNCTIONS.
*  RI  CFD(NC)  VECTOR CONTAINING INCREMENTS OF CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  S(NF+NCA)  VECTOR CONTAINING NEWTON INCREMENTS.
*  RI  GS(NCA)  VECTOR CONTAINING ACTIVE SLACK INCREMENTS.
*  RI  XO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR LOWER INACTIVE
*         MULTIPLIERS.
*  RI  GO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR UPPER INACTIVE
*         MULTIPLIERS.
*  RI  R  STEPSIZE  PARAMETER.
*  RI  RP  PRIMAL STEPSIZE  PARAMETER.
*  RI  RD  DUAL STEPSIZE  PARAMETER.
*  RI  RAT  RATIO OF TRUST REGION RADIUS.
*  II  ICON  OPTION. ABS(ICON)=1-SLACK VARIABLES ARE DETERMINED BY
*         LINE SEARCH. ABS(ICON)=2-SLACK VARIABLES ARE DETERMINED FROM
*         CONSTRAINTS. ICON>0-BOX CONSTRAINTS ARE DETERMINED BY LINE
*         SEARCH. ICON<0-BOX CONSTRAINTS ARE ALWAYS FEASIBLE.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UCIDIR(NF,NX,NC,NCA,X,IX,XL,XU,XSL,XSU,XZL,XZU,CF,CFD,
     & IC,CL,CU,CSL,CSU,CZL,CZU,ICA,S,GS,XO,GO,R,RP,RD,RAT,ICON)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,NCA,IX(NX),IC(NC),ICA(NCA),ICON
      $FLOAT X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CF(NC+1),CFD(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),
     & S(NF+NCA),GS(NCA),XO(NF+NC),GO(NF+NC),R,RP,RD,RR,RAT
      INTEGER I,K,KK
      $FLOAT RQ
      RQ=RAT*RP
      RR=ONE-RQ
      DO 1 I=1,NCA
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      KK=-IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      XZL(K)=XZL(K)+RD*S(NF+I)
      IF (ABS(ICON).EQ.2.AND.X(K).GT.XL(K)) THEN
      XSL(K)=X(K)-XL(K)
      ELSE
      XSL(K)=XSL(K)+RP*GS(I)
      ENDIF
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      XZU(K)=XZU(K)+RD*S(NF+I)
      IF (ABS(ICON).EQ.2.AND.X(K).LT.XU(K)) THEN
      XSU(K)=XU(K)-X(K)
      ELSE
      XSU(K)=XSU(K)+RP*GS(I)
      ENDIF
      ENDIF
      ELSE
      KK=-IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      CZL(K)=CZL(K)+RD*S(NF+I)
      IF (ABS(ICON).EQ.2.AND.CF(K).GT.CL(K)) THEN
      CSL(K)=CF(K)-CL(K)
      ELSE
      CSL(K)=CSL(K)+RP*GS(I)
      ENDIF
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      CZU(K)=CZU(K)+RD*S(NF+I)
      IF (ABS(ICON).EQ.2.AND.CF(K).LT.CU(K)) THEN
      CSU(K)=CU(K)-CF(K)
      ELSE
      CSU(K)=CSU(K)+RP*GS(I)
      ENDIF
      ELSE IF (KK.EQ.5) THEN
      CZL(K)=CZL(K)+R*S(NF+I)
      ENDIF
      ENDIF
    1 CONTINUE
      IF (KBF.GT.0) THEN
      DO 2 K=1,NX
      KK=IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      XZL(K)=XZL(K)+RD*XO(K)
      IF (ABS(ICON).EQ.2.AND.X(K).GT.XL(K)) THEN
      XSL(K)=X(K)-XL(K)
      ELSE
      XSL(K)=RR*XSL(K)+RQ*(X(K)+S(K)-XL(K))
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      XZU(K)=XZU(K)+RD*GO(K)
      IF (ABS(ICON).EQ.2.AND.X(K).LT.XU(K)) THEN
      XSU(K)=XU(K)-X(K)
      ELSE
      XSU(K)=RR*XSU(K)+RQ*(XU(K)-X(K)-S(K))
      ENDIF
      ENDIF
    2 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 3 K=1,NC
      KK=IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      CZL(K)=CZL(K)+RD*XO(NX+K)
      IF (ABS(ICON).EQ.2.AND.CF(K).GT.CL(K)) THEN
      CSL(K)=CF(K)-CL(K)
      ELSE
      CSL(K)=RR*CSL(K)+RQ*(CFD(K)-CL(K))
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      CZU(K)=CZU(K)+RD*GO(NX+K)
      IF (ABS(ICON).EQ.2.AND.CF(K).LT.CU(K)) THEN
      CSU(K)=CU(K)-CF(K)
      ELSE
      CSU(K)=RR*CSU(K)+RQ*(CU(K)-CFD(K))
      ENDIF
      ENDIF
    3 CONTINUE
      ENDIF
      RETURN
      END
