$INTERFACE UCNRL3                                             92/12/01

  CALL UCNRL3(NCL,MC,$CW,$CF,$CL,$CU,$CG,$ICG,$JCG,$KBC,$NORMC,$JOB)
  $ADD(REAL,'\$CW($NC)\$CF($NC)\$CL($NC)\$CU($NC)\$CG($MC)')
  $ADD(INTEGER,'\$ICG($NC+1)\$JCG($MC)')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UCNRL3             ALL SYSTEMS                   92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* INITIAL SCALING OF LINEAR FUNCTIONS.
*
* PARAMETERS :
*  II  NC  NUMBER OF LINEAR FUNCTIONS.
*  II  MC  NUMBER OF ELEMENTS IN THE FIELD AG.
*  RI  CW(NC)  VECTOR OF SCALING PARAMETERS.
*  RU  CF(NC)  VECTOR CONTAINING FUNCTION VALUES.
*  RU  CL(NC)  VECTOR CONTAINING LOWER BOUNDS.
*  RU  CU(NC)  VECTOR CONTAINING UPPER BOUNDS.
*  RI  CG(MC)  SPARSE JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC) COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  II  KBC  SPECIFICATION OF MEASURED VALUES. KBA=0-ZERO MEASURED
*         VALUES. KBA=1 MEASURED VALUES ARE GIVEN IN THE FIELD AM.
*  II  NORMC  SCALING SPECIFICATION. NORMC=0-NO SCALING IS PERFORMED.
*         NORMC=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMC=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  II  JOB  SCALING SPECIFICATION. IF JOB=1 THEN FORWARD SCALING
*         IS PERFORMED. IF JOB=-1 THEN BACKWARD SCALING IS PERFORMED.
*
      SUBROUTINE UCNRL3(NCL,MC,CW,CF,CL,CU,CG,ICG,JCG,KBC,NORMC,JOB)
      INTEGER NCL,MC,ICG(NCL+1),JCG(MC),KBC,NORMC,JOB
      $FLOAT CW(NCL),CF(NCL),CL(NCL),CU(NCL),CG(MC)
      $FLOAT POM
      INTEGER J,JP,K,L,KC
      IF (NORMC.LE.0) RETURN
      IF (JOB.EQ.0) RETURN
      DO 2 KC=1,NCL
      POM=CW(KC)
      IF (JOB.LT.0) POM=1.0$P 0/POM
      K=ICG(KC)
      L=ICG(KC+1)-K
      DO 1 J=1,L
      JP=ABS(JCG(K))
      CG(K)=CG(K)*POM
      K=K+1
    1 CONTINUE
      IF (KBC.GT.0) CL(KC)=POM*CL(KC)
      IF (KBC.GT.1) CU(KC)=POM*CU(KC)
      IF (JOB.LT.0) CF(KC)=POM*CF(KC)
    2 CONTINUE
      RETURN
      END
