$INTERFACE UD1TM1                                             98/12/01

  CALL UD1TM1(NF,$N,$GN,$SN,P,GNORM,SNORM,RMAX,XMAX,INITD)
  $ADD(REAL,'\$GN($NF)\$SN($NF)')
$REM  $ADD(INTEGER,'\NF\$N\INITD')
$REM  $ADD(REAL,'\P\GNORM\SNORM\RMAX\XMAX')
  $HELP
    MAXIMUM STEPSIZE
  $CHECK(XMAX,'1.0$P 3',REAL,2)
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UD1TM1                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* TEST ON LOCALLY CONSTRAINED STEP AND PREPARATION OF SIMPLE SEARCH.
*
* PARAMETERS :
*  II  NDIM DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  G(NDIM) GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  S(NDIM) DIRECTION VECTOR.
*  RO  P  DIRECTIONAL DERIVATE.
*  RI  GNORM  EUCLIDEAN NORM OF THE GRADIENT.
*  RI  SNORM  EUCLIDEAN NORM OF THE DIRECTION VECTOR.
*  RO  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  XMAX  MAXIMUM STEPSIZE.
*  II  INITD  CHOICE OF THE RESTART CRITERION. INITD<0-CHANGE OF THE
*         SIGN IF P<0. INITD>0-NO CHANGE IF P<0. ABS(INITD)=0-NO
*         RESTART CRITERION. ABS(INITD)=1-UNIFORM DESCENT CRITERION.
*         ABS(INITD)=2-SHANNO'S CRITERION. ABS(INITD)=3-POWELL'S
*         CRITERION.
*
* COMMON DATA :
*  II  ITERD  TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  IU  IREST  RESTART SPECIFICATION. IF IREST>0 THEN RESTART IS
*         PERFORMED.
*  IU  ITERM  TERMINATION INDICATOR. TERMINATION IF ITERM .NE. 0.
*  TO  TDXX  TEXT INFORMATION ON THE DIRECTION VECTOR OBTAINED.
*
* SUBPROGRAMS USED :
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UYRES1  DEFINITION OF THE RESTART VARIABLES.
*  S   UOD1D8  INTERMEDIATE PRINT.
*
      SUBROUTINE UD1TM1(NDIM,N,G,S,P,GNORM,SNORM,RMAX,XMAX,INITD)
      $INCLUDE('UMCOMM')
      $INCLUDE('UMCOMD')
      INTEGER NDIM,N,INITD
      $FLOAT G(NDIM),S(NDIM),P,GNORM,SNORM,RMAX,XMAX
      $FLOAT UXVDOT
      $FLOAT EPS,TOL
      PARAMETER (EPS=1.0$P -2,TOL=1.0$P 2)
      NDEM=NDIM
      IF(ITERD.LT.0) THEN
        ITERM=ITERD
        RETURN
      ENDIF
      IF (KD.GT.0) P=UXVDOT(N,G,S)
      IF (INITD.LT.0.AND.P.GT.ZERO) THEN
C
C     CHANGE OF THE SIGN
C
        CALL UXVNEG(N,S,S)
        P=-P
      ENDIF
C
C     TEST ON LOCALLY CONSTRAINED STEP
C
      IF(SNORM.LE.ZERO) THEN
        CALL UYRES1('ZERO DIR')
      ELSE
        IREST=0
      ENDIF
      IF (IREST.EQ.0) THEN
C
C     PREPARATION OF SIMPLE SEARCH
C
        RMAX=XMAX/SNORM
      ENDIF
      CALL UOD1D8(N,G,S,GNORM,SNORM,P)
      RETURN
      END
