$INTERFACE UF0GS1                                             97/12/01

  $HELP
    PRECISION OF THE COMPUTED VALUES
  $CHECK(ETA1,ETA0,REAL,4)
  CALL UF0GS1(NF,$X,$IX,$XN,$GF,FF,TWO/THREE,$ETA1,KBF,NORMF)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$GF($NF)')
$REM  $ADD(REAL,'\FF')
  $ADD(INTEGER,'\$IX($NX)')
$REM  $ADD(INTEGER,'\NF\KBF\NORMF')
  $ADD(INIT)
    NFGX='UF0GS1'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UF0GS1             ALL SYSTEMS                 97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE:
* NUMERICAL COMPUTATION OF THE GRADIENT OF THE MODEL FUNCTION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  GF(NF)  GRADIENT OF THE MODEL FUNCTION.
*  RI  FF  VALUE OF THE MODEL FUNCTION.
*  RI  PREC  PARAMETER FOR THE DIFFERENCE DETERMINATION.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*  II  KB  TYPE OF BOUNDS. KB=0-BOUNDS ARE NOT USED. KB=1-ONE SIDED
*         BOUNDS. KB=1-TWO SIDED BOUNDS.
*  II  NORM  SCALING SPECIFICATION. NORM=0-NO SCALING IS PERFORMED.
*         NORM=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORM=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*
      SUBROUTINE UF0GS1(NF,X,IX,XN,GF,FF,PREC,ETA1,KB,NORM)
      $INCLUDE('UMCOMM')
      INTEGER NF,IX(NF),KB,NORM
      $FLOAT X(NF),XN(NF),GF(NF),FF,PREC,ETA1
      $FLOAT XSTEP,XTEMP,FTEMP,ETA
      INTEGER IVAR
      SAVE XSTEP,XTEMP,FTEMP,ETA
      SAVE IVAR
      IF (TSS(NS).NE.'UFGX') CALL UYPRO1('UFGX',-1)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      ETA=ETA1**PREC
      FTEMP=FF
      IVAR=0
    2 CONTINUE
      IVAR=IVAR+1
      IF (IVAR.GT.NF) GO TO 4
      IF (KB.GT.0) THEN
      IF (IX(IVAR).LE.-5) THEN
      GF(IVAR)=ZERO
      GO TO 2
      ENDIF
      ENDIF
C
C     STEP SELECTION
C
      IF (NORM.GT.0) THEN
      XSTEP=XN(IVAR)
      ELSE
      XSTEP=1.0$P 0
      ENDIF
      XSTEP=ETA*MAX(ABS(X(IVAR)),XSTEP)*SIGN(1.0$P 0,X(IVAR))
      XTEMP=X(IVAR)
      X(IVAR)=X(IVAR)+XSTEP
      XSTEP=X(IVAR)-XTEMP
      CALL UYSET0(1,2)
      RETURN
C
C     NUMERICAL DIFFERENTIATION
C
    3 CONTINUE
      GF(IVAR)=(FF-FTEMP)/XSTEP
      X(IVAR)=XTEMP
      GO TO 2
    4 FF=FTEMP
      CALL UYEPI1(-1)
      RETURN
      END
