$INTERFACE UF0GS2                                             97/12/01

  $HELP
    PRECISION OF THE COMPUTED VALUES
  $CHECK(ETA1,ETA0,REAL,4)
  $HELP
    DETERMINATION OF THE SECOND DIRIVATIVES APPROXIMATIONS
      1 - VARIABLE METRIC METHOD
      2 - NUMERICAL APPROXIMATION
  $CHECK(MCG1,1,'1\2',4)
  CALL UF0GS2(NF,$X,$IX,$XN,$GF,FF,$HD,R,SNORM,$ETA0,$ETA1,$MCG,$MCG1)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$GF($NF)\$HD($NF)')
$REM  $ADD(REAL,'\FF\R\SNORM')
  $ADD(INTEGER,'\$IX($NX)')
$REM  $ADD(INTEGER,'\NF')
  $ADD(INIT)
    NFGX='UF0GS2'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UF0GS2              ALL SYSTEMS                  97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL COMPUTATION OF THE GRADIENT OF THE MODEL FUNCTION
* WITH AUTOMATIC STEP SELECTION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  GF(NF)  GRADIENT OF THE MODEL FUNCTION.
*  RI  FF  VALUE OF THE MODEL FUNCTION.
*  RA  HD(NF)  AUXILIARY VECTOR.
*  RI  R  VALUE OF THE STEPSIZE PARAMETER.
*  RO  SNORM  EUCLIDEAN NORM OF THE DIRECTION VECTOR.
*  RI  ETA0  MACHINE PRECISION.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*  II  MCG  METHOD OF STEP SELECTION. MCG=0-CONSTANT STEP.
*         MCG=1-EASY COMPARATIVE METHOD. MCG=2-STEWART'S METHOD.
*  II  MCG1  DETERMINATION OF THE SECOND DERIVATIVES. MCG1=1-VARIABLE
*         METRIC METHOD. MCG1=2-NUMERICAL APPROXIMATION.
*
* COMMON DATA :
*  II  LIT  NUMBER OF THE FIRST ITERATION.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KBF  TYPE OF BOUNDS. KBF=0-BOUNDS ARE NOT USED. KBF=1-ONE SIDED
*         BOUNDS. KBF=1-TWO SIDED BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UOF0G1  PRINT OF ENTRY TO NUMERICAL DIFFERENTIATION.
*  S   UOF0G2  PRINT OF EXIT FROM NUMERICAL DIFFERENTIATION.
*  S   UOF0G3  PRINT OF INFORMATION DURING NUMERICAL DIFFERENTIATION.
*
      SUBROUTINE UF0GS2(NF,X,IX,XN,GF,FF,HD,R,SNORM,ETA0,ETA1,MCG,MCG1)
      $INCLUDE('UMCOMM')
      INTEGER NF,IX(NF),MCG,MCG1
      $FLOAT X(NF),XN(NF),GF(NF),FF,HD(NF),R,SNORM,ETA0,ETA1
      $FLOAT XSTEP,XTEMP,FTEMP,GTEMP,HTEMP,EPS,ETA
      INTEGER IVAR,MODE
      $FLOAT CON
      PARAMETER (CON=1.0$P-2)
      SAVE XSTEP,XTEMP,FTEMP,GTEMP,HTEMP,ETA
      SAVE IVAR,MODE
      IF (TSS(NS).NE.'UFGX') CALL UYPRO1('UFGX',-1)
      GOTO (1,3,4) ISP(NS)
    1 CONTINUE
      CALL UOF0G1
      ETA=ETA1**(TWO/THREE)
      FTEMP=FF
      IVAR=0
    2 CONTINUE
      IVAR=IVAR+1
      IF(IVAR.GT.NF) GOTO 5
      IF (KBF.GT.0) THEN
      IF (IX(IVAR).LE.-5) THEN
      GF(IVAR)=ZERO
      GO TO 2
      ENDIF
      ENDIF
      XTEMP=X(IVAR)
      GTEMP=GF(IVAR)
      HTEMP=ABS(HD(IVAR))
      IF (HTEMP.EQ.ZERO) HTEMP=ONE
C
C     STEP SELECTION
C
      IF (MCG.EQ.0.OR.NIT.LE.LIT.OR.FTEMP.EQ.ZERO) THEN
C
C     CONSTANT STEP
C
      MODE=0
      IF(NORMF.GT.0) THEN
      XSTEP=XN(IVAR)
      ELSE
      XSTEP=ONE
      ENDIF
      XSTEP=ETA*MAX(ABS(XTEMP),XSTEP)*SIGN(ONE,XTEMP)
      ELSEIF(MCG.EQ.1.OR.GTEMP.EQ.ZERO) THEN
C
C     EASY COMPARATIVE METHOD
C
      MODE=1
      EPS=MAX(ETA1,ETA0*ABS(XTEMP*GTEMP/FTEMP))
      XSTEP=TWO*SQRT(EPS*ABS(FTEMP/HTEMP))*SIGN(ONE,XTEMP)
      IF(XSTEP*HTEMP.GE.CON*ABS(GTEMP)) THEN
      MODE=2
      XSTEP=ABS(XSTEP)**(TWO/THREE)
      ENDIF
      ELSEIF(MCG.EQ.2) THEN
C
C     STEWART'S METHOD
C
      MODE=1
      EPS=MAX(ETA1,ETA0*ABS(XTEMP*GTEMP/FTEMP))
      IF(GTEMP*GTEMP.GE.EPS*ABS(FTEMP*HTEMP)) THEN
      XSTEP=TWO*SQRT(EPS*ABS(FTEMP/HTEMP))
      XSTEP=XSTEP*(ONE-XSTEP*HTEMP/(THREE*XSTEP*HTEMP+FOUR*ABS(GTEMP)))
      ELSE
      XSTEP=TWO*(EPS*ABS((FTEMP/HTEMP)*(GTEMP/HTEMP)))**(ONE/THREE)
      XSTEP=XSTEP*(ONE-TWO*ABS(GTEMP)/(THREE*XSTEP*HTEMP
     &      +FOUR*ABS(GTEMP)))
      ENDIF
      XSTEP=SIGN(XSTEP,XTEMP)
      IF(ABS(XSTEP*HTEMP/GTEMP).GE.CON) THEN
      MODE=2
      XSTEP=SQRT(GTEMP*GTEMP+(TWO/CON)*EPS*ABS(FTEMP*HTEMP))
      XSTEP=(TWO/CON)*EPS*ABS(FTEMP)/(ABS(GTEMP)+XSTEP)
      ENDIF
      ENDIF
      CALL UOF0G3(HD(IVAR),XSTEP,MODE,IVAR)
C
C     NUMERICAL DIFFERENTIATION
C
      X(IVAR)=XTEMP+XSTEP
      XSTEP=X(IVAR)-XTEMP
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
C
C     ONE SIDED FORMULA
C
      IF(MODE.LE.1) THEN
      GF(IVAR)=(FF-FTEMP)/XSTEP
      X(IVAR)=XTEMP
      IF(MCG1.EQ.2) THEN
      IF(MODE.EQ.0) THEN
      HD(IVAR)=ONE
      ELSE IF(R*SNORM.NE.ZERO) THEN
      HD(IVAR)=ABS(GF(IVAR)-GTEMP)/(R*SNORM)
      ENDIF
      ENDIF
      GOTO 2
      ENDIF
      GTEMP=FF
      X(IVAR)=XTEMP-XSTEP
      XSTEP=XTEMP-X(IVAR)
      CALL UYSET0(1,3)
      RETURN
    4 CONTINUE
C
C     TWO SIDED (CENTRAL DIFFERENCE) FORMULA
C
      IF (XSTEP.EQ.ZERO) THEN
      GF(IVAR)=ZERO
      ELSE
      GF(IVAR)=HALF*(GTEMP-FF)/XSTEP
      ENDIF
      X(IVAR)=XTEMP
      IF(MCG1.EQ.2) THEN
      HD(IVAR)=ABS(GTEMP+FF-FTEMP-FTEMP)/(XSTEP*XSTEP)
      ENDIF
      GOTO 2
    5 FF=FTEMP
      CALL UOF0G2
      CALL UYEPI1(-1)
      RETURN
      END
