$INTERFACE UF1FQ4                                             92/12/01

  $HELP
    TYPE OF EXTREMUM
      0 - MINIMUM
      1 - MAXIMUM
  $CHECK(IEXT,0,'0\1',1)
  $HELP
    LOWER BOUND FOR FUNCTION VALUE
  $CHECK(FMIN,'-$ETA9',REAL,2)
  $ADD(REAL,'\$X($MCOLS)\$GF($MCOLS)\$G($MCOLS)\$AH($MAH)')
$REM  $ADD(REAL,'\FF\F')
  $ADD(INTEGER,'\$IH($MCOLS+1)\$JH($M)')
$REM  $ADD(INTEGER,'\NF\NA\MA\NB\NAP')
  CALL UF1FQ4(NF,NA,MA,MAH,$X,$GF,$G,$AH,$IH,$JH,FF,F,NAP)
  $ADD(INIT)
    NFXX='UF1FQ4'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UF1FQ4                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE QUADRATIC OBJECTIVE
* FUNCTION - SPARSE CASE.
*
* PARAMETERS:
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF BLOCKS IN THE PARTITIONED STRUCTURE.
*  II  MA  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  MB  NUMBER OF NONZERO ELEMENTS THE PARTITIONED HESSIAN MATRIX.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  GF(NF)  GRADIENT OF THE QUADRATIC MODEL FUNCTION.
*  RO  G(NF)  GRADIENT OF THE QUADRATIC OBJECTIVE FUNCTION.
*  RI  HF(M)  HESSIAN ELEMENTS OF THE QUADRATIC MODEL FUNCTION.
*  II  IH(NA+1)  POINTERS OF THE DIAGONAL ELEMENTS OF HESSIAN MATRIX.
*  II  JH(MA)  INDICES OF THE NONZERO ELEMENTS OF HESSIAN MATRIX.
*  RI  FF  CONSTANT TERM OF THE QUADRATIC MODEL FUNCTION.
*  RO  F  VALUE OF THE QUADRATIC OBJECTIVE FUNCTION.
*  II  NAP  POINTER.
*
* COMMON DATA :
*  II  KD  DEGREE OF REQUIRED DERVATIVES.
*  II  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  II  IEXT  TYPE OF EXTREMUM. IEXT=0-MINIMUM. IEXT=1-MAXIMUM.
*
* SUBPROGRAMS USED :
*  S   UXBSMM  MATRIX VECTOR PRODUCT.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXVSUM  SUM OF TWO VECTORS.
*
      SUBROUTINE UF1FQ4(NF,NA,MA,MB,X,GF,G,HF,IH,JH,FF,F,NAP)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NA,MA,MB,IH(NA+1),JH(MA),NAP
      $FLOAT X(NF),GF(NF),G(NF),HF(MB),FF,F
      $FLOAT UXVDOT
      INTEGER NP
      IF (KD.LT.0) GO TO 1
      IF (LD.LT.0) THEN
      NP=NF-NAP
      CALL UXBSMM(NP,MB,NA,MA,HF,IH,JH,X,G)
      CALL UXVSUM(NF,GF,G,G)
      F=FF+HALF*(UXVDOT(NF,X,GF)+UXVDOT(NF,X,G))
      IF (IEXT.GT.0) F=-F
      ENDIF
      IF (KD.LT.1) GO TO 1
      IF (LD.LT.1) THEN
      IF (LD.GE.0) THEN
      CALL UXBSMM(NP,MB,NA,MA,HF,IH,JH,X,G)
      CALL UXVSUM(NF,GF,G,G)
      ENDIF
      IF (IEXT.GT.0) CALL UXVNEG(NF,G,G)
      ENDIF
    1 CONTINUE
      RETURN
      END

