$INTERFACE UF1HS2                                             99/12/01

  $HELP
    PRECISION OF THE COMPUTED GRADIENTS
  $CHECK(ETA1,ETA0,REAL,4)
  CALL UF1HS2(NF,MH,$MMAX,$X,$IX,$XN,$S,$HF,$IH,$JH,$GF,$GO,$COL,$WN11&
,$WN12,$XS,FF,$ETA1)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$S($NF)\$XS($NF)')
  $ADD(REAL,'\$HF($MMAX)\$GF($NF)\$GO($NF)')
$REM  $ADD(REAL,'\FF')
  $ADD(INTEGER,'\$IX($NF)\$COL($NF)')
  $ADD(INTEGER,'\$WN11($NF+1)\$WN12($NF+1)')
  $ADD(INTEGER,'\$IH($NF+1)\$JH($MMAX)')
$REM  $ADD(INTEGER,'\NF\MH\MMAX')
  $ADD(INIT)
    NFHX='UF1HS2'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UF1HS2                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL COMPUTATION OF THE HESSIAN MATRIX OF THE MODEL FUNCTION
* USING ITS GRADIENTS - SPARSE VERSION USING DIRECT COLOURING METHOD.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  ML SIZE OF THE COMPACT FACTOR.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE HESSIAN MATRIX.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RA  XO(NF)  AUXILIARY VECTOR.
*  RO  HF(M)  HESSIAN MATRIX OF THE MODEL FUNCTION.
*  IU  IH(NF+1)  POINTER VECTOR OF SPARSE HESSIAN MATRIX.
*  IU  JH(M)  INDEX VECTOR OF THE HESSIAN MATRIX.
*  RI  GF(NF)  GRADIENT OF THE MODEL FUNCTION.
*  RA  GO(NF)  AUXILIARY VECTOR.
*  II  COL(NF)  VECTOR DISCERNING GROUPS OF THE HESSIAN COLUMN OF THE
*         SAME COLOUR.
*  IA  WN11(NF+1)  AUXILIARY VECTOR.
*  IA  WN12(NF+1)  AUXILIARY VECTOR.
*  RA  XS(NF)  AUXILIARY VECTOR USED FOR STEP SIZES.
*  RI  FF  VALUE OF THE MODEL FUNCTION.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*
* COMMON DATA :
*  II  KBF  TYPE OF BOUNDS. KBF=0-BOUNDS ARE NOT USED. KBF=1-ONE SIDED
*         BOUNDS. KBF=1-TWO SIDED BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  IU  ITERM  TERMINATION INDICATOR.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UOERR1  ERROR MESAGES.
*
      SUBROUTINE UF1HS2(NF,ML,M,X,IX,XN,XO,HF,IH,JH,GF,GO,COL,WN11,
     & WN12,XS,FF,ETA1)
      $INCLUDE('UMCOMM')
      INTEGER NF,ML,M,IX(NF),IH(NF+1),JH(M),COL(NF),WN11(NF+1),
     & WN12(NF+1)
      $FLOAT X(NF),XN(NF),XO(NF),HF(M),GF(NF),GO(NF),XS(NF),FF,ETA1
      $FLOAT  XSTEP,XTEMP,FTEMP,ETA
      INTEGER I,J,J1,K,K1,L,MX,MM,IVAR,JVAR
      SAVE MX,MM,IVAR,JVAR
      SAVE  XSTEP,XTEMP,FTEMP,ETA
      IF(ITERM.NE.0) GO TO 12
      IF (TSS(NS).NE.'UFHX') CALL UYPRO1('UFHX',-1)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      MM=IH(NF+1)-1
      IF(3*MM-NF+ML.GE.M) THEN
      ITERM=-44
      TDXX='LACK SPC'
      CALL UOERR1('UF1HS2',44)
      GO TO 6
      ENDIF
      ETA=SQRT(ETA1)
      FTEMP=FF
      CALL UXVCOP(NF,GF,GO)
      CALL UXVCOP(NF,X,XO)
      IVAR=1
C
C     NOTE: TERMINATION DUE TO THE LACK OF SPACE IS DOMINATED BY USE OF
C     UUSSL1, ...
C
C     WIDTHEN THE STRUCTURE
C
      K=2*MM-NF
      DO 50 I=ML+MM,1,-1
        JH(K+I)=JH(MM+I)
   50 CONTINUE
      CALL UXSTG1(NF,MX,M,IH,JH,WN12,WN11)
      CALL UXVSET(K,ZERO,HF)
    2 CONTINUE
      IF(IVAR.GT.NF) GO TO 870
      DO 200 J=IVAR,NF
        IF(COL(J).GE.1) THEN
          GO TO 200
        ELSE
          JVAR=J
          GO TO 300
        END IF
 200  CONTINUE
 300  CONTINUE
      DO 400 J=IVAR,JVAR
        L=ABS(COL(J))
        IF(KBF.GT.0) THEN
          IF(IX(L).LE.-7) GO TO 400
        END IF
C
C     STEP SELECTION
C
        IF (NORMF.GT.0) THEN
          XSTEP=ABS(XN(L))
        ELSE
          XSTEP=ONE
        ENDIF
        XS(L)=ETA*MAX(ABS(X(L)),XSTEP)*SIGN(ONE,X(L))
        XTEMP=X(L)
        X(L)=XTEMP+XS(L)
        XS(L)=X(L)-XTEMP
 400  CONTINUE
        CALL UYSET0(1,2)
        RETURN
    3 CONTINUE
C
C     NUMERICAL DIFFERENTIATION
C
C      IF (KBF.GT.0) THEN
C      IF (IX(J).LE.-7) GO TO 4
C      ENDIF
C
C     SET AUXILIARY VECTOR DISCERNING THE SINGLETONS IN A GROUP TO ZERO
C
      DO 450 J1=1,NF
        WN11(J1)=0
  450 CONTINUE
C
C     DISCERN SINGLETONS OF THE GROUP OF THE SAME COLOR.
C
      DO 600 J1=IVAR,JVAR
        L=ABS(COL(J1))
        DO 500 K=IH(L),IH(L+1)-1
          K1=ABS(JH(K))
          IF(WN11(K1).EQ.0) THEN
            WN11(K1)=J1
          ELSE
            WN11(K1)=-1
          END IF
 500  CONTINUE
 600  CONTINUE
C
C     NUMERICAL VALUES COMPUTATION
C
      DO 800 J1=IVAR,JVAR
        L=ABS(COL(J1))
        DO 700 K=IH(L),IH(L+1)-1
          K1=ABS(JH(K))
          IF(WN11(K1).GT.0) THEN
            HF(K)=(GF(K1)-GO(K1))/XS(L)
          END IF
 700    CONTINUE
 800  CONTINUE
C    4 CONTINUE
C
C     SET THE ORIGINAL VALUE OF X FOR THE COMPONENTS OF THE ACTUAL COLOR.
C
      DO 850 J=IVAR,JVAR
        L=ABS(COL(J))
        X(L)=XO(L)
 850  CONTINUE
      IVAR=JVAR+1
      GO TO  2
 870  CONTINUE
C
C     MOVE THE ELEMENTS OF THE HESSIAN APPROXIMATION INTO THE UPPER
C     TRIANGULAR PART
C
      DO 900 I=1,NF
        WN11(I)=WN12(I)+1
 900  CONTINUE
      DO 1100 I=1,NF
        IVAR=IH(I)
        JVAR=WN12(I)-1
        DO 1000 J=IVAR,JVAR
          K=ABS(JH(J))
          L=WN11(K)
          IF(HF(L).EQ.0) THEN
            HF(L)=HF(J)
          ELSEIF(HF(L).NE.0.AND.HF(J).NE.0) THEN
            HF(L)=HALF*(HF(J)+HF(L))
          ENDIF
          WN11(K)=WN11(K)+1
 1000   CONTINUE
 1100 CONTINUE
      FF=FTEMP
C
C     SHRINK THE STRUCTURE
C
      CALL UXSTL2(NF,MX,M,HF,IH,JH,WN12)
      K=2*MM-NF
      DO 1200 I=1,ML+MM
        JH(MM+I)=JH(K+I)
 1200 CONTINUE
C
C     RETRIEVE VALUES
C
      CALL UXVCOP(NF,GO,GF)
      CALL UXVCOP(NF,XO,X)
    6 CALL UYEPI1(-1)
   12 CONTINUE
      RETURN
      END
