$INTERFACE UF1HS5                                             09/12/01

  $HELP
    CHOICE OF DIFFERENCES
      1 - VARIABLE DIFFERENCES DERIVED FROM VALUES OF VARIABLES
      2 - CONSTANT DIFFERENCES DERIVED FROM THE DIRECTION VECTOR
  $CHECK(MOT5,1,'\1\2',3)
  $HELP
    PRECISION OF THE COMPUTED GRADIENTS
  $CHECK(ETA1,ETA0,REAL,4)
  CALL UF1HS5(NF,$X,$IX,$XN,$D,$DU,$DV,$GF,$XS,$GS,$S,FF,$ETA1,MOT5)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$D($NF)\$DU($NF)\$DV($NF)\$GF($NF)&
  \$XS($NF)\$GS($NF)\$S($NF)')
$REM  $ADD(REAL,'\FF')
  $ADD(INTEGER,'\$IX($NX)')
$REM  $ADD(INTEGER,'\NF\MOT5')
  $ADD(INIT)
    NFHX='UF1HS5'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UF1HS5                ALL SYSTEMS                09/12/01
C PORTABILITY : ALL SYSTEMS
C 09/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL COMPUTATION OF THE HESSIAN MATRIX OF THE MODEL FUNCTION
* USING ITS GRADIENTS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RU  D(NF)  DIAGONAL OF THE PENTADIAGONAL PRECONDITIONER.
*  RU  DU(NF)  THE FIRST SUPERDIAGONAL OF THE PENTADIAGONAL
*         PRECONDITIONER.
*  RU  DV(NF)  THE SECOND SUPERDIAGONAL OF THE PENTADIAGONAL
*         PRECONDITIONER.
*  RI  GF(NF)  GRADIENT OF THE MODEL FUNCTION.
*  RA  XS(NF)  AUXILIARY VECTOR.
*  RA  GS(NF)  AUXILIARY VECTOR.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RI  FF  VALUE OF THE MODEL FUNCTION.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*  II  MOT5  CHOICE OF DIFFERENCES. MOT5=1-VARIABLE DIFFERENCES.
*         MOT5=2-CONSTANT DIFFERENCES.
*
* COMMON DATA :
*  II  KBF  TYPE OF BOUNDS. KBF=0-BOUNDS ARE NOT USED. KBF=1-ONE SIDED
*         BOUNDS. KBF=1-TWO SIDED BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*
      SUBROUTINE UF1HS5(NF,X,IX,XN,D,DU,DV,GF,XS,GS,S,FF,ETA1,MOT5)
      $INCLUDE('UMCOMM')
      INTEGER NF,IX(NF),MOT5
      $FLOAT X(NF),XN(NF),D(NF),DU(NF),DV(NF),GF(NF),XS(NF),GS(NF),
     & S(NF),FF,ETA1
      $FLOAT XSTEP,XTEMP,FTEMP,ETA
      INTEGER I,J,IVAR
      SAVE FTEMP,ETA
      SAVE IVAR
      IF (TSS(NS).NE.'UFHX') CALL UYPRO1('UFHX',-1)
      GO TO (1,4) ISP(NS)
    1 CONTINUE
      ETA=SQRT(ETA1)
      FTEMP=FF
      CALL UXVCOP(NF,GF,GS)
      CALL UXVCOP(NF,X,XS)
      IVAR=0
    2 CONTINUE
      IVAR=IVAR+1
      IF (IVAR.GT.3) GO TO 6
C
C     STEP SELECTION
C
      J=IVAR
      IF (J.EQ.3) J=0
      DO 3 I=1,NF
      IF (MOD(I,3).EQ.J) THEN
C      IF (KBF.GT.0) THEN
C      IF (IX(I).LE.-5) GO TO 3
C      ENDIF
      XTEMP=X(I)
      IF (MOT5.EQ.1) THEN
      IF (NORMF.GT.0) THEN
      XSTEP=XN(I)
      ELSE
      XSTEP=ONE
      ENDIF
      S(I)=SIGN(ETA*MAX(ABS(XTEMP),XSTEP),XTEMP)
      ELSE
      S(I)=SIGN(SQRT(THREE*ETA/$DBLE(NF)),XTEMP)
      ENDIF
      X(I)=XTEMP+S(I)
      S(I)=X(I)-XTEMP
      ENDIF
    3 CONTINUE
      CALL UYSET0(1,2)
      RETURN
    4 CONTINUE
C
C     NUMERICAL DIFFERENTIATION
C
      DO 5 I=1,NF
C      IF (KBF.GT.0) THEN
C      IF (IX(I).LE.-5) GO TO 5
C      ENDIF
      IF (IVAR.EQ.1) THEN
      D(I)=GF(I)-GS(I)
      ELSE IF (IVAR.EQ.2) THEN
      DU(I)=GF(I)-GS(I)
      ELSE
      DV(I)=GF(I)-GS(I)
      ENDIF
      X(I)=XS(I)
    5 CONTINUE
      GO TO 2
C
C     COMPUTATION OF THE PENTADIAGONAL MATRIX
C
    6 DO 7 I=1,NF
      IF (MOD(I,3).EQ.1) THEN
      ELSE IF (MOD(I,3).EQ.2) THEN
      XTEMP=D(I)
      D(I)=DU(I)
      DU(I)=DV(I)
      DV(I)=XTEMP
      ELSE
      XTEMP=D(I)
      D(I)=DV(I)
      DV(I)=DU(I)
      DU(I)=XTEMP
      ENDIF
      D(I)=D(I)/S(I)
      IF (I.EQ.1) THEN
      DU(I)=DU(I)/S(I+1)
      DV(I)=DV(I)/S(I+2)
      ELSE IF (I.EQ.2) THEN
      DU(I)=DU(I)/S(I+1)
      DV(I)=(DV(I)-DU(I-1)*S(I-1))/S(I+2)
      ELSE IF (I.LT.NF-1) THEN
      DU(I)=(DU(I)-DV(I-2)*S(I-2))/S(I+1)
      DV(I)=(DV(I)-DU(I-1)*S(I-1))/S(I+2)
      ELSE IF (I.LT.NF) THEN
      DU(I)=(DU(I)-DV(I-2)*S(I-2))/S(I+1)
      ENDIF
    7 CONTINUE
      FF=FTEMP
      CALL UXVCOP(NF,GS,GF)
      CALL UYEPI1(-1)
      RETURN
      END
