* SUBROUTINE UKQGLC                 ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 TU : ORIGINAL VERSION
*
* PURPOSE:
* TESTING THE CONVERGENCE USING THE PRESENT ACTIVE SET.
*
* PARAMETERS :
*  RI  DEL3 PARAMETER CONDITIONING ACCURACY OF THE CONVERGENCE.
*  RI  SNORM NORM OF THE SUPERBASICS.
*  RI  GNORM REDUCED GRADIENT NORM.
*  RI  XNORM NORM OF THE STEP.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  INITIAL VALUE OF THE OBJECTIVE FUNCTION.
*  II  TO  PARAMETER SIGNALIZING INCREASED ACCURACY FOR POSSIBLE
*            LAST UPDATE : =1  FOR POSSIBLE LAST UPDATE
*                          =0  OTHERWISE.
*  RI  E SPARSE QUADRATIC PROGRAMMING ACCURACY.
*  RI  EL  NORM OF THE REDUCED GRADIENT IN PREVIOUS ITERATION.
*  RI  EU  NORM OF THE REDUCED GRADIENT IN PREVIOUS ITERATION.
*  II  MLP1 NUMBER OF ITERATIONS IN SEQUENCE WITH NO PROGRESS
*             IN OPTIMIZATION.
*  II  MINP  PARAMETER IS SET UP WHEN FUNCTION AND/OR GRADIENT VALUE
*            ARE NOT TOO EXACTLY COMPUTED.
*  IO  IER  TYPE OF ABANDONING OF THE ACTIVE SET :
*           = 0 FOR FURTHER OPTIMIZATION WITH THE PRESENT ACTIVE SET
*           = 1 ABANDON ACTIVE SET FOR THE GRADIENT IS SMALLER THAN
*               SOME LOOSE BOUND AND DECREASE IN FUNCTION AND CHANGE
*               IN VARIABLES ARE BOTH TOO SMALL.
*           = 2 ABANDON ACTIVE SET FOR THE GRADIENT IS SMALLER THAN
*               SOME SHARP BOUND.
*           = 3 ABANDON ACTIVE SET WHEN NO PROGRESS IN OPTIMIZATION
*               IN CHNBND ITERATIONS IS NOT FACED.
*           = 4 ABANDON ACTIVE SET FOR THE ACCURACY OF THE FUNCTION
*               BEEING MINIMIZED IS SMALLER THAN CHANGES IN DIRECTION
*               AND/OR VARIABLES.
*
      SUBROUTINE UKQGLC(DEL3,SNORM,GNORM,XNORM,F,FO,TO,E,EL,EU,MLP1,
     & MINP,IER)
      INTEGER MLP1,MINP,IER
      $FLOAT DEL3,SNORM,GNORM,XNORM,F,FO,TO,E,EL,EU
      INTEGER CHNBND
      $FLOAT EPS,EPSF,EPSH,ETAG,RGFL,RSFL,TEMP1,TEMP2
      $FLOAT ONETEN,TWOTEN,ONEHUN,ONETHU
      $FLOAT FABS,FDABS
      $FLOAT ETAA,ETAB,ETAC,ETAD,EPSX
      PARAMETER (EPS=1.0D-16,ONETHU=1.0D-4)
      PARAMETER (ONETEN=0.1D0,TWOTEN=0.2D0,ONEHUN=0.01D0)
      PARAMETER(ETAA=1.0D-3,ETAB=1.0D-4,ETAC=1.0D-5,ETAD=1.0D-6)
      PARAMETER(CHNBND=5)
      IER=0
      ETAG=TWOTEN
      IF(TO.EQ.1) THEN
        RGFL=ETAC
        RSFL=ETAD
      ELSE
        RGFL=MAX(ETAG*DEL3*EU,ETAA)
        RSFL=MAX(0.1$P 0*RGFL,ETAB)
      END IF

      EPSH=EPS**0.5$P 0
      EPSX=ETAC
      EPSF=ETAC**2
C
C     DECIDE WHETHER WE WILL STAY ON THE SAME ACTIVE SET
C
      FDABS=ABS(F-FO)
      IF(GNORM.LE.RGFL) THEN
          FABS=ABS(F)
          TEMP1=(EPSX+EPSH)*(1.0$P 0+SNORM)
          TEMP2=(EPSF+EPS)*(1.0$P 0+FABS)
          IF(XNORM.LE.TEMP1.AND.FDABS.LE.TEMP2) THEN
             IER=1
             RETURN
          END IF
          IF(GNORM.LE.(RSFL*MAX(1.0$P 0,EL))) THEN
             IER=2
             RETURN
          END IF
      END IF
C
C NO PROGRESS IN THE OPTIMIZATION PROCESS
C
      IF(XNORM.EQ.0.0$P 0.AND.MLP1.GE.CHNBND) THEN
        MLP1=0
        IER=3
        RETURN
      END IF
C
C     ZERO DIRECTION IN MINOR OR MAJOR ITERATION.
C
      IF(MINP.EQ.1) THEN
        IER=4
        RETURN
      END IF
      IF(F-FO.GT.E*ONETEN) THEN
        IER=5
        RETURN
      END IF
      RETURN
      END
