* SUBROUTINE UKQGLI                 ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* INITIALIZE LINP,FUNCTIONAL VALUE,GRADIENT,REDUCED GRADIENT, COST AND
* USEFUL STATUS VARIABLES FOR THE SPARSE QUADRATIC PROGRAMMING PROBLEM.
*
* PARAMETERS :
*  II  NF  ORIGINAL NUMBER OF VARIABLES.
*  II  N  NUMBER OF VARIABLES.
*  II  M  NUMBER OF ROWS OF THE MATRIX OF CONSTRAINTS.
*  II  MCOLS DECLARED NUMBER OF VARIABLES.
*  II  MROWS DECLARED NUMBER OF ROWS.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NZA SIZE OF THE INDEX ARRAY RNR
*  II  NAU SIZE OF THE ARRAY AU.
*  II  NUP SIZE OF THE ARRAYS LI,LJ,CM.
*  II  NEL NUMBER OF L-UPDATES IN THE ARRAYS LI,LJ,CM.
*  II  PTL(MROWS) POINTER VECTOR OF THE COLUMNS OF THE L-FACTOR.
*  II  LENL(MROWS) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  II  CNR(NZ) CONTAINS THE INDICES OF THE FACTORS.
*  II  PTR(MROWS) POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  II  LENR(MROWS) LENGTHS OF THE ROWS OF THE FACTOR U.
*  II  ORDC(MROWS) COLUMN PERMUTATION.
*  II  ORDR(MROWS) ROW PERMUTATION.
*  II  ORDL(MROWS) ROW PERMUTATION OF THE ORIGINAL FACTOR L.
*  II  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  II  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RI  CM(NUP) UPDATE MULTIPLIERS.
*  RI  AU(NZ) NUMERICAL VALUES OF THE FACTORS.
*  RO  LAM(MCOLS) VECTOR OF REDUCED COST/REDUCED GRADIENT.
*  RA  WR01(MCOLS) AUXILIARY VECTOR.
*  RI  X(MCOLS) VECTOR OF VARIABLES.
*  RO  C(MCOLS) GRADIENT VECTOR.
*  RI  GF(M) GRADIENT OF THE MODEL FUNCTION.
*  RI  H(M) NUMERICAL ELEMENTS OF THE HESSIAN MATRIX.
*  II  IH(MCOLS+1) POINTER OF THE HESSIAN.
*  II  JH(M) INDICES OF THE HESSIAN.
*  II  BCOL(MCOLS) COLUMN ORDER IN BLOCKS, DEFINED AS OUTPUT FOR THE
*              QP PROBLEM.
*  II  RNR(NZA) COLUMNS OF THE MATRIX OF CONSTRAINTS.
*  II  PRNR(MCOLS+1) POINTER VECTOR FOR RNR.
*  RI  A(NZA) NUMERICAL VALUES OF THE MATRIX OF CONSTRAINTS.
*  IO  INBCOL(MCOLS) VECTOR WITH COMPONENTS :
*                INBCOL(K)=I WHEN EXISTS K=BCOL(I) AND I<=LINP,
*                INBCOL(K)=0 OTHERWISE.
*  II  TYPE(MCOLS) TYPES OF THE VARIABLES.
*  IA  WN01(MCOLS) AUXILIARY ARRAY.
*  RI  LB(MCOLS) LOWER BOUNDS ON VARIABLES.
*  RI  UB(MCOLS) UPPER BOUNDS ON VARIABLES.
*  RO  DEL3 CONTRACTING PARAMETER FOR CONVERGENCE WITH
*           THE PRESENT ACTIVE SET.
*  RO  GNORM NORM OF THE REDUCED GRADIENT.
*  RO  SNORM NORM OF THE VARIABLES CORRESPONDING TO THE ACTIVE SET.
*  RO  XNORM NORM OF THE STEP IN SUPERBASICS
*  RO  FF  ORIGINAL FUNCTIONAL VALUE.
*  RO  F  FUNCTIONAL VALUE.
*  RO  FO OLD FUNCTIONAL VALUE.
*  RO  TO  PARAMETER SIGNALIZING INCREASED ACCURACY FOR POSSIBLE
*            LAST UPDATE : =1  FOR POSSIBLE LAST UPDATE
*                          =0  OTHERWISE.
*  RO  EL NORM OF THE REDUCED COST.
*  RO  EU GNORM OF THE PREVIOUS ITERATION.
*  IO  MQP1 NUMBER OF REPEATED NULL ITERATIONS.
*  IO  NIQ MINOR CYCLE COUNT.
*  IO  NAQ MAJOR CYCLE COUNT.
*  IO  MRED MAXIMUM REDUCTIONS DURING LINESEARCH.
*  IO  KINP DEGENERACY INFORMATION PARAMETER.
*  IO  LINP  NUMBER OF SUPERBASICS.
*  IO  MINP SET UP WHEN FUNCTION/GRADIENT ARE INACCURATELY COMPUTED.
*  IO  ITERQ TERMINATION INDICATOR OF QP.
*
* COMMON DATA:
*
      SUBROUTINE UKQGLI(NF,N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,MMAX,NEL,PTL,
     & LENL,CNR,PTR,LENR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,LAM,WR01,X,C,GF,H,
     & IH,JH,BCOL,RNR,PRNR,A,INBCOL,TYPE,WN01,LB,UB,DEL3,GNORM,SNORM,
     & XNORM,FF,F,FO,TO,EL,EU,MQP1,NIQ,NAQ,NCGR,MRED,KINP,LINP,MINP,
     & ITERQ)
      INTEGER NF,N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,MMAX,NEL,INBCOL(MCOLS)
      INTEGER PTL(MROWS),LENL(MROWS),CNR(NZ),PTR(MROWS),LENR(MROWS)
      INTEGER ORDC(MROWS),ORDR(MROWS),MQP1,NIQ,NAQ,NCGR,MRED,KINP,
     & LINP,MINP,ITERQ
      INTEGER ORDL(MROWS),LI(NUP),LJ(NUP),TYPE(MCOLS),WN01(MCOLS)
      INTEGER BCOL(MCOLS),PRNR(MCOLS+1),RNR(NZA)
      INTEGER IH(MCOLS+1),JH(MMAX)
      $FLOAT UXVDOT,DEL3,GNORM,SNORM,XNORM,FF,F,FO,TO,EL,EU
      $FLOAT CM(NUP),AU(NAU),LAM(MCOLS),WR01(MCOLS),X(MCOLS),C(MCOLS)
      $FLOAT A(NZA),H(MMAX),GF(MCOLS),LB(MCOLS),UB(MCOLS)
      $FLOAT EPS15,TEMP,EPS19
      INTEGER I,J,K,L,TYPV,IND
      EPS15=1.0$P-6
      EPS19=1.0$P 9
C
C     MINP IS SET UP WHEN ZERO DIRECTION IS GENERATED IN
C     THE SUBROUTINE ULSGLX WHEN DEGENERACY IS NOT PRESENT.
C
      MINP=0
C
C     TO IS FLAG OF THE LAST OPTIMIZING CYCLE : =1 FOR THIS CYCLE
C     IT CONTROLS CONVERGENCE PARAMETERS, =0 OTHERWISE.
C
      TO=0.0$P 0
C
C     GNORM OF THE PREVIOUS ITERATION
C
      EU=1.0$P 0
C
C     NUMBER OF REPEATED NULL ITERATIONS
C
      MQP1=0
C
C     MAXIMUM NUMBER OF REDUCTIONS DURING QP LINE SEARCH.
C
      MRED=10
C
C     OLD FUNCTIONAL VALUE
C
      FO=0.0$P 0
C
C     DEGENERACY CONTROLLING PARAMETER
C
      KINP=0
C
C     TL GIVES THE STATE OF THE CYCLE:=1 FOR MINOR CYCLE,=2
C     FOR THE MAJOR ONE, MJST= STATE OF THE MAJOR CYCLE:=1 FOR
C     THE LAST ONE ,=0 OTHERWISE
C
C
C COMPUTE FUNCTIONAL VALUE,GRADIENT. HESSIAN IS GIVEN IN
C INPUT DATA STREAM.
C
      DO 50 I=1,N
        C(I)=0.0$P 0
   50 CONTINUE
      CALL UYSET2
      CALL UF1FQ2(NF,IH(NF+1)-1,X,GF,C,H,IH,JH,FF,F)
C
C     WIDTHEN BCOL. DEFINE INBCOL - ALL THAT IN CASE WHEN ITERQ=0.
C
      IF(ITERQ.EQ.0) THEN
      DO 100 I=1,N
        INBCOL(I)=0
100   CONTINUE
      DO 200 I=1,M
        J=BCOL(I)
        INBCOL(J)=I
200   CONTINUE
      K=M
      DO 300 I=1,N
        L=INBCOL(I)
        IF(L.EQ.0) THEN
          K=K+1
          BCOL(K)=I
        END IF
300   CONTINUE
      DO 400 I=M+1,LINP
        J=BCOL(I)
        INBCOL(J)=I
400   CONTINUE
      ENDIF
C
C EXCLUDE FIXED VARIABLES FROM SUPERBASIS - GIVEN THERE FOR INSTANCE
C DURING OUTER LINSEARCH
C
      IND=0
      DO 600 I=M+1,LINP
      J=BCOL(I)
      TEMP=X(J)
      TYPV=TYPE(J)
      IF((TYPV.EQ.1.OR.TYPV.EQ.3.OR.TYPV.EQ.5.OR.TYPV.EQ.3).AND.(
     * ABS(X(J)-LB(J)).LT.EPS15)) THEN
        IND=IND+1
        WN01(IND)=I
      ELSEIF((TYPV.EQ.2.OR.TYPV.EQ.4.OR.TYPV.EQ.3).AND.(ABS(
     * X(J)-UB(J)).LT.EPS15))  THEN
        IND=IND+1
        WN01(IND)=I
      ELSE
        GO TO 600
      END IF
600   CONTINUE
C
C     FIRST LEAVE ALL OUT FROM THE SUPERBASIS
C     ALL COLUMNS STORED IN WN01(1,...,IND) WILL BE SHIFTED.
C
      DO 500 I=1,IND
      J=WN01(I)
      K=BCOL(LINP)
      L=BCOL(J)
      BCOL(LINP)=L
      BCOL(J)=K
      INBCOL(K)=J
      INBCOL(L)=0
C
C CORRECTION OF THE TYPES 3,4
C
      IF(TYPE(L).EQ.4.OR.TYPE(L).EQ.3) THEN
        IF(ABS(X(L)-LB(L)).LE.EPS19)  THEN
          TYPE(L)=3
        ELSE
          TYPE(L)=4
        END IF
        IF(ABS(X(L)-LB(L)).GT.ABS(X(L)-UB(L))) TYPE(L)=4
      END IF
      LINP=LINP-1
500   CONTINUE
C
C     SET REDUCED COST AND GRADIENT
C
      CALL UKQGLL(M,MCOLS,BCOL,LAM,C)
      CALL UKQGLV(N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,PTL,LENL,CNR,PTR,
     & LENR,RNR,PRNR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,A,WR01,BCOL,LAM,C,LINP)
C
C     SET SOME INITIAL VALUES AND CONSTANTS
C
C
C     CONTRACTING PARAMETER FOR ACTIVE SET CONVERGENCE.
C
      DEL3=2.0$P 0
C
C     INITIALIZE CYCLE COUNTS, PCG-IT. COUNTS
C
      NIQ=0
      NAQ=0
      NCGR=0
C
C     COMPUTE SNORM,RGNORM,PINORM,DNORM,FMIN
C
      SNORM=0.0$P 0
      XNORM=0.0$P 0
      GNORM=0.0$P 0
      EL=SQRT(UXVDOT(M,LAM,LAM))
      RETURN
      END
