* SUBROUTINE ULADB5               ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NEW LINEAR CONSTRAINT OR A NEW SIMPLE BOUND IS ADDED TO THE
* ACTIVE SET. PRODUCT FORM OF THE INVERSE HESSIAN MATRIX APPROXIMATION
* IS UPDATED.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  IU  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF LINEARIZED CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RI  CR(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RI  CZ(NF*NF)  MATRIX WHOSE COLUMNS ARE BASIC VECTORS FROM THE
*         CURRENT REDUCED SUBSPACE AND WHOSE PRODUCT TRANS(CZ)*CZ
*         IS AN APPROXIMATION OF THE INVERSE HESSIAN MATRIX.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  II  INEW  INDEX OF THE NEW ACTIVE CONSTRAINT.
*  IU  NADD  NUMBER OF CONSTRAINT ADDITIONS.
*  IO  IER  ERROR INDICATOR.
*
* SUBPROGRAMS USED :
*  S   ULADR0  CORRECTION OF KERNEL OF THE ORTHOGONAL PROJECTION
*         AFTER CONSTRAINT ADDITION.
*  S   UXDRMM  PREMULTIPLICATION OF A VECTOR BY A ROWWISE STORED DENSE
*         RECTANGULAR MATRIX.
*  S   UXDRMV  COPY OF THE SELECTED COLUMN OF A ROWWISE STORED DENSE
*         RECTANGULAR MATRIX.
*  S   UXDCMM  PREMULTIPLICACION OF A VECTOR BY A DENSE RECTANGULAR
*         MATRIX.
*  S   UXDCMU  CORRECTION OF A DENSE RECTANGULAR MATRIX.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*
      SUBROUTINE ULADB5(NF,N,NC,ICA,CG,CR,CZ,S,EPS7,GMAX,UMAX,INEW,
     & NADD,IER)
      INTEGER NF,N,NC,ICA(NF),INEW,NADD,IER
      $FLOAT CG(NF*NC),CR(NF*(NF+1)/2),CZ(NF*NF),S(NF),EPS7,GMAX,UMAX
      $FLOAT CK,CL,UXVDOT
      INTEGER NCA,NCR,N1
      CALL ULADR0(NF,N,NC,ICA,CG,CR,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      IF (IER.NE.0) RETURN
      IF (N.GT.0) THEN
      N1=N+1
      IF (INEW.GT.0) THEN
      CALL UXDRMM(NF,N1,CZ,CG((INEW-1)*NF+1),S)
      ELSE
      CALL UXDRMV(NF,N1,CZ,S,-INEW)
      ENDIF
      CK=SQRT(UXVDOT(N1,S,S))
      CL=S(N1)
      IF (CL.GE.0.0$P 0) THEN
      CL=1.0$P 0/(CK+CL)
      CK=CL/CK
      ELSE
      CL=1.0$P 0/(CK-CL)
      CK=CL/CK
      CL=-CL
      ENDIF
      NCA=NF-N
      NCR=NCA*(NCA+1)/2
      CALL UXDCMM(NF,N1,CZ,S,CR(NCR+1))
      CALL UXDCMU(NF,N,CZ,-CK,CR(NCR+1),S)
      CALL UXDCMU(NF,N,CZ,-CL,CZ((N)*NF+1),S)
      ENDIF
      IER=0
      RETURN
      END
