* SUBROUTINE ULPLAG               ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* VECTOR OF LAGRANGE MULTIPLIERS IS DETERMINED.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF CURRENT LINEAR APPROXIMATED FUNCTIONS.
*  II  NC  NUMBER OF LINEARIZED CONSTRAINTS.
*  II  IA(2*NA)  VECTOR CONTAINING TYPES OF DEVIATIONS.
*  II  IAA(NF+1)  VECTOR CONTAINING INDICES OF ACTIVE FUNCTIONS.
*  RI  AR((NF+1)*(NF+2)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RO  AZ(NF+1)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RO  SNORM  INCREMENT FOR MAXIMUM APPROXIMATED FUNCTION VALUE.
*  RA  T  AUXILIARY VARIABLE.
*  IA  KOLD  AUXILIARY VARIABLE.
*
* SUBPROGRAMS USED:
*  S   UXDPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*
      SUBROUTINE ULPLAG(NF,N,NA,NC,IA,IAA,AR,AZ,S,SNORM,T,KOLD)
      INTEGER NF,N,NA,NC,IA(2*NA),IAA(NF+1),KOLD
      $FLOAT AR((NF+1)*(NF+2)/2),AZ(NF+1),S(NF+1),SNORM,T
      $FLOAT TEMP
      INTEGER NAA,J,L
      KOLD=0
      NAA=NF-N
      DO 1 J=1,NAA
      L=IAA(J)
      IF (L.GT.NC) THEN
      L=L-NC
      IF (IA(L).EQ.-1.OR.IA(L).EQ.-3) THEN
      S(J)= 1.0$P 0
      ELSE
      S(J)=-1.0$P 0
      ENDIF
      KOLD=1
      ELSE
      S(J)=0.0$P 0
      ENDIF
    1 CONTINUE
      IF (KOLD.EQ.0) RETURN
      CALL UXDPRB(NAA,AR,AZ,0)
      CALL UXDPRB(NAA,AR, S,0)
      SNORM=0.0$P 0
      TEMP=0.0$P 0
      DO 2 J=1,NAA
      L=IAA(J)
      IF (L.GT.NC) THEN
      L=L-NC
      IF (IA(L).EQ.-1.OR.IA(L).EQ.-3) THEN
      SNORM=SNORM+AZ(J)
      TEMP=TEMP+S(J)
      ELSE
      SNORM=SNORM-AZ(J)
      TEMP=TEMP-S(J)
      ENDIF
      ENDIF
    2 CONTINUE
      SNORM=(SNORM-1.0$P 0)/TEMP
      CALL UXVDIR(NAA,-SNORM,S,AZ,AZ)
      SNORM=SNORM+T
      RETURN
      END
