$INTERFACE ULQPA2                                             97/12/01

    $PAR=S
    $IF(JOB>0) $PAR=G
    $HELP
      TYPE OF EXTREMUM:
         0 - CHEBYSHEV APPROXIMATION
        -1 - MINIMAX
         1 - MAXIMIN
    $CHECK(IEXT,0,'-1\0\1',1)
    $HELP
      PRECISION FOR EITHER LINEAR OR QUADRATIC PROGRAMMING
      SUBROUTINE
    $CHECK(EPS7,'1.0$P-8',REAL,4)
    $HELP
      TOLERANCE FOR ACTIVE CONSTRAINTS
    $CHECK(EPS9,'1.0$P-8',REAL,4)
    CALL ULQPA2(NF,N,MAL,$X,$IX,$IXA,$XN,$XO,$XL,$XU,$AF,$AFD,$IA,$IAA&
,$AG,$AR,$AZ,$PAR,$H,$S,$ETA0,$ETA9,EPS7,EPS9,XDEL,XNORM,UMAX,GMAX&
,$JOB)
    $ADD(REAL,'\$X($NF)\$XN($NF)\$XO($NF)\$XL($NF)\$XU($NF)&
    \$AG($NF*($MAL))\$AR(($NF+1)*($NF+2)/2)\$AZ($NF+1)\$PAR($NF+1)&
    \$H($M)\$S($NF+1)')
$REM    $ADD(REAL,'\XDEL\XNORM\UMAX\GMAX\EPS7\EPS9')
$REM    $ADD(INTEGER,'\NF\N\MAL')
    $ADD(INTEGER,'\$IX($NX)\$IXA($NX)\$IAA($NF+1)')
    $IF(IEXT=0)
      $ADD(REAL,'\$AF(2*$MAL)\$AFD(2*$MAL)')
      $ADD(INTEGER,'\$IA(2*$MAL)')
    $ELSE
      $ADD(REAL,'\$AF($MAL)\$AFD($MAL)')
      $ADD(INTEGER,'\$IA($MAL)')
    $ENDIF
    $ADD(INIT)
      NQPX='ULQPA2'
    $ENDADD
    $EXIT

* IMPLEMENTATION

* SUBROUTINE ULQPA2             ALL SYSTEMS                   97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE INITIAL FEASIBLE POINT AND THE LINEAR PROGRAMMING
* SUBROUTINE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  IO  N  DIMENSION OF THE MANIFOLD DEFINED BY ACTIVE CONSTRAINTS.
*  II  NA  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  II  IXA(NF)  VECTOR CONTAINING INFORMATION ON TRUST REGION ACTIVITY.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  XO(NF)  SAVED VECTOR OF VARIABLES.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  AF(2*NA)  VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RO  AFD(2*NA)  VECTOR CONTAINING INCREMENTS OF THE APPROXIMATED
*         FUNCTIONS.
*  II  IA(2*NA)  VECTOR CONTAINING TYPES OF DEVIATIONS.
*  II  IAA(NF+1)  VECTOR CONTAINING INDICES OF ACTIVE FUNCTIONS.
*  RI  AG(NF*NA)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         APPROXIMATED FUNCTIONS.
*  RI  AR((NF+1)*(NF+2)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RO  AZ(NF+1)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RO  G(NF+1)  GRADIENT OF THE LAGRANGIAN FUNCTION.
*  RI  H(NF*(NF+1)/2)  THE GILL AND MURRAY DECOMPOSITION OF THE
*         HESSIAN MATRIX.
*  RO  S(NF)  DIRECTION VECTOR.
*  RI  ETA0  MACHINE PRECISION.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RI  EPS9  TOLERANCE FOR ACTIVITY OF CONSTRAINTS.
*  RU  XDEL  TRUST REGION BOUND.
*  RO  XNORM  VALUE OF LINEARIZED MINIMAX FUNCTION.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  II  JOB  SPECIFICATION OF THE PROBLEM. JOB=0-QUADRATIC PROGRAMMING
*         PROBLEM. JOB=1-QUADRATIC PROGRAMMING SUBPROBLEM FOR DIRECTION
*         DETERMINATION.
*
* COMMON DATA :
*  II  LCS  VARIABLE WHICH SPECIFIES TRUST REGION BOUNDS.
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  IU  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*         IDECF=1-GILL-MURRAY DECOMPOSITION. IDECF=9-INVERSION.
*         IDECF=10-DIAGONAL MATRIX.
*  IU  NDECF  NUMBER OF DECOMPOSITIONS.
*  IO  ITERQ  TYPE OF FEASIBLE POINT. ITERQ=1-ARBITRARY FEASIBLE POINT.
*         ITERQ=2-OPTIMUM FEASIBLE POINT. ITERQ=-1 FEASIBLE POINT DOES
*         NOT EXISTS. ITERQ=-2 OPTIMUM FEASIBLE POINT DOES NOT EXISTS.
*  II  IEXT  TYPE OF EXTREMUM. IEXT<0-MINIMAX. IEXT=0-CHEBYSHEV
*         APPROXIMATION, IEXT>0-MAXIMIN.
*
* SUBPROGRAMS USED :
*  S   ULINIT  DETERMINATION OF INITIAL POINT SATISFYING SIMPLE BOUNDS.
*  S   ULAMAX  DETERMINATION OF THE MAXIMUM DEVIATION.
*  S   ULMAXA  MAXIMUM STEPSIZE USING LINEAR APPROXIMATED FUNCTIONS.
*  S   ULMAXS  MAXIMUM STEPSIZE USING SIMPLE BOUNDS.
*  S   ULMAXT  MAXIMUM STEPSIZE USING TRUST REGION BOUNDS.
*  S   ULNEWA  IDENTIFICATION OF ACTIVE LINEAR APPROXIMATED FUNCTIONS.
*  S   ULNEWS  IDENTIFICATION OF ACTIVE SIMPLE BOUNDS.
*  S   ULNEWT  IDENTIFICATION OF ACTIVE TRUST REGION BOUNDS.
*  S   ULDIRA  NEW VALUES OF APPROXIMATED FUNCTIONS.
*  S   ULDIRS  NEW VALUES OF VARIABLES.
*  S   ULSETA  INITIAL VALUES OF APPROXIMATED FUNCTIONS.
*  S   ULSETG  DETERMINATION OF THE FIRST PHASE GRADIENT VECTOR.
*  S   ULGLAG  GRADIENT OF THE LAGRANGIAN FUNCTION IS DETERMINED.
*  S   ULPLAG  VECTOR OF LAGRANGE MULTIPLIERS IS DETERMINED.
*  S   ULSLAG  NEGATIVE PROJECTED GRADIENT IS DETERMINED.
*  S   ULTLAG  THE OPTIMUM LAGRANGE MULTIPLIER IS DETERMINED.
*  S   ULVLAG  AN AUXILIARY VECTOR IS DETERMINED.
*  S   ULADF1  CONSTRAINT ADDITION.
*  S   ULRMF0  CONSTRAINT DELETION.
*  S   UXDPGF  GILL-MURRAY DECOMPOSITION OF A DENSE SYMMETRIC MATRIX.
*  S   UXDPGB  BACK SUBSTITUTION AFTER GILL-MURRAY DECOMPOSITION.
*  S   UXDPGM  MATRIX-VECTOR PRODUCT AFTER GILL-MURRAY DECOMPOSITION.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVINA  ABSOLUTE VALUES OF ELEMENTS OF AN INTEGER VECTOR.
*  S   UXVINC  UPDATE OF AN INTEGER VECTOR.
*  S   UXVIND  CHANGE OF THE INTEGER VECTOR FOR CONSTRAINT ADDITION.
*  S   UXVINT  CHANGE OF THE INTEGER VECTOR FOR TRUST REGION BOUND
*         ADDITION.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UOLQP1  PRINT OF ENTRY TO QUADRATIC PROGRAMMING.
*  S   UOLQP2  PRINT OF EXIT FROM QUADRATIC PROGRAMMING.
*  S   UOLQP5  PRINT OF INFORMATION DURING QUADRATIC PROGRAMMING.
*  S   UOERR1  ERROR MESSAGES.
*
      SUBROUTINE ULQPA2(NF,N,NA,X,IX,IXA,XN,XO,XL,XU,AF,AFD,IA,IAA,AG,
     & AR,AZ,G,H,S,ETA0,ETA9,EPS7,EPS9,XDEL,XNORM,UMAX,GMAX,JOB)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N,NA,IX(NF),IXA(NF),IA(2*NA),IAA(NF+1),JOB
      $FLOAT X(NF),XN(NF),XO(NF),XL(NF),XU(NF),AF(2*NA),AFD(2*NA),
     & AG(NF*NA),AR((NF+1)*(NF+2)/2),AZ(NF+1),G(NF+1),H(NF*(NF+1)/2),
     & S(NF+1),ETA0,ETA9,EPS7,EPS9,XDEL,XNORM,UMAX,GMAX
      $FLOAT POM,E,T,DMAX,SNORM
      INTEGER NAD,NAA,NAR,I,L,IOLD,JOLD,KOLD,INEW,JNEW,KNEW,INF,MODE,
     & IER,KA,KREM
      $FLOAT CON
      CALL UOLQP1('ULQPA2')
      L=0
      N=NF
      T=ONE
      CON=ETA9
      IF (IEXT.EQ.0) THEN
      NAD=2*NA
      ELSE
      NAD=NA
      ENDIF
      IF (IDECF.LT.0) IDECF=1
      IF (IDECF.EQ.0) THEN
C
C     GILL-MURRAY DECOMPOSITION
C
      POM=ETA0
      CALL UXDPGF(NF,H,INF,POM,DMAX)
      NDECF=NDECF+1
      IDECF=1
      ENDIF
      IF (IDECF.NE.1) THEN
      ITERM=-1
      TDXX='BAD DEC1'
      CALL UOERR1('ULQPA2',1)
      GO TO 6
      ENDIF
C
C     INITIATION
C
      CALL UXVCOP(NF,X,XO)
      JOLD=0
      JNEW=0
      NRED=0
      KREM=0
      ITERQ=0
      DMAX=ZERO
      NAR=NF*(NF+1)/2
      IF (LCS.GT.0) CALL UXVINC(NF,IX,IXA)
      IF (KBF.GT.0) THEN
C
C     SHIFT OF VARIABLES FOR SATISFYING SIMPLE BOUNDS
C
      CALL UXVINA(NF,IX)
      CALL ULINIT(NF,X,IX,XN,XO,XL,XU,XDEL,EPS9,INEW,ITERQ)
      IF (ITERQ.LT.0) THEN
      CALL UOERR1('ULQPA2',10-ITERQ)
      GO TO 6
      ENDIF
      DO 1 I=1,NF
      IF (IX(I).LT.0) THEN
      INEW=-I
      CALL ULADF1(NF,N,NA,NA,0,IA,IAA,AG,AR,AG,H,AR(NAR+1),AZ,EPS7,
     & GMAX,UMAX,E,T,IDECF,INEW,KNEW,NADD,IER,0)
      ENDIF
    1 CONTINUE
      ENDIF
      CALL UXVINA(NAD,IA)
    3 IF (ITERQ.EQ.0) THEN
      ITERQ=1
      IF (NF.GT.N) CALL ULSETA(NF,NA,X,XN,XO,AF,IA,AG,S,NORMF)
C
C     INITIAL MINIMAX VARIABLE
C
      CALL ULAMAX(NA,NAD,AF,IA,XNORM,ETA9)
C
C     ADDITION OF ACTIVE FUNCTIONS
C
      DO 11 KA=1,NAD
      IF (IA(KA).GT.0) THEN
      INEW=0
      CALL ULNEWA(NA,0,KA,AF,IA,EPS9,XNORM,INEW,KNEW)
      CALL ULADF1(NF,N,NA,NA,0,IA,IAA,AG,AR,AG,H,AR(NAR+1),AZ,EPS7,
     & GMAX,UMAX,E,T,IDECF,INEW,KNEW,NADD,IER,0)
      CALL UXVIND(NAD,IA,KA,IER)
      ENDIF
   11 CONTINUE
      ENDIF
    5 IF (NF.LE.N) THEN
C
C     IMPOSSIBLE SITUATION
C
      ITERQ=-5
      CALL UOERR1('ULQPA2',10-ITERQ)
      GO TO 6
      ENDIF
C
C     LAGRANGE MULTIPLIERS DETERMINATION
C
      NAA=NF-N
      CALL UXVDIF(NF,X,XO,G)
      CALL ULVLAG(NF,N,NA,NA,0,IAA,AG,AG,G,AZ)
      CALL ULPLAG(NF,N,NA,0,IA,IAA,AR,AZ,S,SNORM,T,KOLD)
      IF (KOLD.EQ.0) THEN
C
C     IMPOSSIBLE SITUATION
C
      ITERQ=-6
      CALL UOERR1('ULQPA2',10-ITERQ)
      GO TO 6
      ENDIF
      CALL ULTLAG(NF,N,NA,NA,0,IX,IA,IAA,AZ,IA,EPS7,UMAX,IOLD)
C
C     PROJECTED GRADIENT DETERMINATION
C
      CALL UXVNEG(NF,G,S)
      CALL UXDPGM(NF,H,S,0)
      CALL ULSLAG(NF,N,NA,NA,0,IAA,AG,AZ,AG,S,EPS7,GMAX)
      MODE=1
      IF (L.GT.2) GMAX=ZERO
      CALL UOLQP5(NF,N,0,NA,X,IX,IA,IA,IAA,AF,AR,AZ,DMAX,GMAX,UMAX,
     & XNORM,JOLD,JNEW,MODE)
      INEW=0
      IF (GMAX.EQ.ZERO) THEN
C
C     OPTIMUM ON A LINEAR MANIFOLD OBTAINED
C
      IF (IOLD.EQ.0) THEN
C
C     OPTIMAL SOLUTION ACHIEVED
C
      IF (JOB.GT.0) THEN
      CALL UXVDIF(NF,X,XO,S)
      CALL UXVCOP(NF,XO,X)
      CALL UXVSET(NF,ZERO,G)
      CALL ULGLAG(NF,N,NA,NA,0,IAA,AG,AZ,AG,G)
      ENDIF
      ITERQ= 2
      GO TO 6
      ELSE
C
C     CONSTRAINT DELETION
C
      CALL ULRMF0(NF,N,NA,NA,0,IX,IXA,IA,IAA,AR,IA,S,LCS,IOLD,KREM,
     & NREM,IER)
      DMAX=ZERO
      JOLD=IOLD
      JNEW=0
      GO TO 5
      ENDIF
      ELSE
C
C     DIRECTION DETERMINATION
C
      CALL UXDPGB(NF,H,S,0)
C
C     STEPSIZE SELECTION
C
      POM=CON
      CALL ULMAXA(NF,NA,NAD,0,AF,AFD,IA,AG,S,POM,XNORM,SNORM,KREM,INEW)
      IF (NORMF.GT.0) CALL UXVMUL(NF,XN,S,S,1)
      IF (LCS.GT.0) CALL ULMAXT(NF,X,IX,IXA,XN,XO,S,POM,XDEL,NORMF,
     & KREM,INEW)
      CALL ULMAXS(NF,X,IX,XL,XU,S,POM,KBF,KREM,INEW)
      IF (POM.GT.ONE) THEN
C
C     OPTIMAL STEPSIZE
C
      INEW=0
      POM=ONE
      L=L+1
      ELSE
      L=0
      ENDIF
C
C     STEP REALIZATION
C
      CALL ULDIRS(NF,X,IX,S,POM,KBF)
      CALL ULDIRA(NA,NAD,AF,AFD,IA,POM,XNORM,SNORM)
C
C     CONSTRAINT ADDITION
C
      IF (INEW.EQ.0) THEN
      ELSE IF (INEW.GT.0) THEN
      KA=INEW
      INEW=0
      CALL ULNEWA(NA,0,KA,AF,IA,EPS9,XNORM,INEW,KNEW)
      CALL ULADF1(NF,N,NA,NA,0,IA,IAA,AG,AR,AG,H,AR(NAR+1),AZ,EPS7,
     & GMAX,UMAX,E,T,IDECF,INEW,KNEW,NADD,IER,0)
      CALL UXVIND(NAD,IA,KA,IER)
      ELSE IF (INEW+NF.GE.0) THEN
      I=-INEW
      INEW=0
      CALL ULNEWS(NF,X,IX,XN,XL,XU,EPS9,NORMF,I,INEW)
      CALL ULADF1(NF,N,NA,NA,0,IA,IAA,AG,AR,AG,H,AR(NAR+1),AZ,EPS7,
     & GMAX,UMAX,E,T,IDECF,INEW,KNEW,NADD,IER,0)
      CALL UXVIND(NF,IX,I,IER)
      ELSE
      I=-(INEW+NF)
      INEW=0
      CALL ULNEWT(NF,X,IX,IXA,XN,XO,XDEL,EPS9,NORMF,I,INEW)
      CALL ULADF1(NF,N,NA,NA,0,IA,IAA,AG,AR,AG,H,AR(NAR+1),AZ,EPS7,
     & GMAX,UMAX,E,T,IDECF,INEW,KNEW,NADD,IER,0)
      CALL UXVIND(NF,IX,I,IER)
C      CALL UXVINT(NF,IX,IXA,I,IER)
      ENDIF
      DMAX=POM
      NRED=NRED+1
      JNEW=INEW
      JOLD=0
      GO TO 3
      ENDIF
    6 CALL UOLQP2
      RETURN
      END


