* SUBROUTINE ULTRBG               ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* GRADIENT OF THE OBJECTIVE FUNCTION IS SCALED AND REDUCED. LAGRANGE
* MULTIPLIERS ARE DETERMINED. TEST VALUES GMAX AND UMAX ARE COMPUTED.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CURRENT LINEAR CONSTRAINTS.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RI  CR(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RU  CZ(NF*NF)  MATRIX WHOSE COLUMNS ARE BASIC VECTORS FROM THE
*         CURRENT REDUCED SUBSPACE. VECTOR CZ(1,NF) CONTAINS LAGRANGE
*         MULTIPLIERS BEING DETERMINED.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  GN(NF)  TRANSFORMED GRADIENT OF THE OBJECTIVE FUNCTION IF IT IS
*         NONZERO.
*  RI  EPS7  TOLERANCE FOR LINEAR AND QUADRATIC PROGRAMMING.
*  RO  GMAX  NORM OF THE TRANSFORMED GRADIENT.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF THE NEGATIVE LAGRANGE MULTIPLIER.
*  IO  IOLD  INDEX OF THE REMOVED CONSTRAINT.
*
* SUBPROGRAMS USED :
*  S   ULVLAG  GRADIENT IS PREMULTIPLIED BY THE MATRIX WHOSE COLUMNS
*         ARE NORMALS OF THE ACTIVE CONSTRAINTS.
*  S   ULTLAG  COMPUTATION OF THE MAXIMUM ABSOLUTE VALUE OF THE NEGATIVE
*         LAGRANGE MULTIPLIER.
*  S   UXDRMM  PREMULTIPLICATION OF A VECTOR BY A ROWWISE STORED DENSE
*         RECTANGULAR MATRIX.
*  S   UXDPRB  BACK SUBSTITUTION AFTER A CHOLESKI DECOMPOSITION.
*  RF  UXVMAX  L-INFINITY NORM OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*
      SUBROUTINE ULTRBG(NF,N,NC,IX,IC,ICA,CG,CR,CZ,G,GN,EPS7,GMAX,UMAX,
     & IOLD)
      INTEGER NF,N,NC,IX(NF),IC(NC),ICA(NF),IOLD
      $FLOAT CG(NF*NC),CR(NF*(NF+1)/2),CZ(NF*NF),G(NF),GN(NF),EPS7,
     & GMAX,UMAX
      $FLOAT UXVMAX
      INTEGER NCA,NCR,NCZ
      GMAX=0.0$P 0
      IF (N.GT.0) THEN
      CALL UXDRMM(NF,N,CZ,G,GN)
      GMAX=UXVMAX(N,GN)
      ENDIF
      IF (NF.GT.N.AND.GMAX.LE.EPS7) THEN
      NCA=NF-N
      NCZ=N*NF
      NCR=NCA*(NCA+1)/2
      CALL ULVLAG(NF,N,NC,NC,NC,ICA,CG,CG,G,CZ(NCZ+1))
      CALL UXDPRB(NCA,CR,CZ(NCZ+1),0)
      CALL ULTLAG(NF,N,NC,NC,NC,IX,IC,ICA,CZ(NCZ+1),IC,EPS7,UMAX,IOLD)
      IF (UMAX.LE.EPS7) IOLD=0
      CALL UXVSET(N,0.0$P 0,GN)
      GMAX=0.0$P 0
      ELSE
      IOLD=0
      UMAX=0.0$P 0
      ENDIF
      RETURN
      END
