* SUBROUTINE UMADR0               ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* TRIANGULAR DECOMPOSITION OF KERNEL OF THE ORTHOGONAL PROJECTION
* IS UPDATED AFTER CONSTRAINT ADDITION.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  IU  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF LINEARIZED CONSTRAINTS.
*  II  MC  NUMBER OF ELEMENTS IN THE FIELD CG.
*  II  MMAX  MAXIMUM NUMBER OF NONZERO ELEMENTS OF THE MATRIX CR.
*  IU  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CG(MC)  SPARSE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC) COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  RI  CR(MMAX)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  II  ICR(NF+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CR.
*  II  JCR(MMAX) COLUMN INDICES OF ELEMENTS IN THE FIELD CR.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  II  INEW  INDEX OF THE NEW ACTIVE CONSTRAINT.
*  IO  IER  ERROR INDICATOR.
*
* COMMON DATA :
*  IU  NADD  NUMBER OF CONSTRAINT ADDITIONS.
*  IO  ITERM  TERMINATION INDICATOR.
*  TO  TDXX  TEXT INFORMATION ON THE DIRECTION VECTOR OBTAINED.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION.
*  RF  UXSRMP  DOT PRODUCT OF TWO ROWS OF THE SPARSE MATRIX.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UOERR1  ERROR MESSAGES.
*
      SUBROUTINE UMADR0(NF,N,NC,MC,MMAX,ICA,CG,ICG,JCG,CR,ICR,JCR,S,
     & EPS7,GMAX,UMAX,INEW,IER)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N,NC,MC,MMAX,ICA(NF),ICG(NC+1),JCG(MC),ICR(NF+1),
     & JCR(MMAX),INEW,IER
      $FLOAT CG(MC),CR(MMAX),S(NF),EPS7,GMAX,UMAX
      $FLOAT UXVDOT,UXSRMP
      INTEGER NCA,J,K,L,JOB
      IER=0
      IF (N.LE.0) IER=2
      IF (INEW.EQ.0) IER=3
      IF (IER.NE.0) RETURN
      NCA=NF-N
      IF (INEW.GT.0) THEN
      K=ICG(INEW)
      L=ICG(INEW+1)-K
      GMAX=UXVDOT(L,CG(K),CG(K))
      ELSE
      GMAX=1.0$P 0
      ENDIF
      JOB=0
      DO 1 J=1,NCA
      S(J)=UXSRMP(NC,MC,CG,ICG,JCG,INEW,ICA(J))
      IF (JOB.EQ.0) THEN
      IF (S(J).NE.0.0$P 0) JOB=J
      ENDIF
    1 CONTINUE
      IF (NCA.EQ.0) THEN
      UMAX=GMAX
      ELSE
      CALL UXSPRB(NCA,MMAX,CR,ICR,JCR,S,JOB)
      UMAX=GMAX-UXVDOT(NCA,S,S)
      ENDIF
      IF (UMAX.LE.EPS7*GMAX) THEN
      IER=1
      RETURN
      ELSE
      N=N-1
      NCA=NCA+1
      ICA(NCA)=INEW
      S(NCA)=SQRT(UMAX)
      L=ICR(NCA)
      DO 5 J=1,NCA
      IF (S(J).NE.0.0$P 0) THEN
      CR(L)=S(J)
      JCR(L)=J
      L=L+1
      ENDIF
    5 CONTINUE
      IF (L-1.GT.MMAX) THEN
      ITERM=-17
      TDXX='LACK SPC'
      CALL UOERR1('UMADR0',17)
      RETURN
      ENDIF
      ICR(NCA+1)=L
      NADD=NADD+1
      ENDIF
      RETURN
      END
