* SUBROUTINE UNCUR1                ALL SYSTEMS                15/12/01
C PORTABILITY : ALL SYSTEMS
C 15/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* ACCEPTABLE ROOTS OF A CUBIC EQUATION.
*
* PARAMETERS :
*  RI  A0  COEFFICIENT OF THE CUBIC EQUATION.
*  RI  A1  COEFFICIENT OF THE CUBIC EQUATION.
*  RI  A2  COEFFICIENT OF THE CUBIC EQUATION.
*  RI  A3  COEFFICIENT OF THE CUBIC EQUATION.
*  RI  B  BOUND FOR ACCEPTABLE ROOTS.
*  RO  X  THE SMALLER ACCEPTABLE ROOT.
*  IO  INF  THE CASE INDICATOR. INF=0-ACCEPTABLE ROOTS DOES NOT EXIST.
*         INF=1-AN ACCEPTABLE ROOT OF THE LINEAR EQUATION, INF=2-AN
*         ACCEPTABLE ROOT OF THE QUADRATIC EQUATION. INF=3-THE FIRST
*         ACCEPTABLE ROOT OF THE CUBIC EQUATION. INF=4-THE SECOND
*         ACCEPTABLE ROOT OF THE CUBIC EQUATION. INF=-1-THE INITIAL
*         FUNCTION VALUE IS NONPOSITIVE, INF=-2-THE INITIAL DERIVATIVE
*         IS NONNEGATIVE.    
*
      SUBROUTINE UNCUR1(A0,A1,A2,A3,B,X,INF)
      REAL*8 A0,A1,A2,A3,B,X
      INTEGER INF
      REAL*8 C,D,E,F,FUN,DER
      REAL*8 ZERO,TWO,THREE,FOUR
      REAL*8 CON1,CON2
      PARAMETER (ZERO=0.0$P 0,TWO=2.0$P 0,THREE=3.0$P 0,FOUR=4.0$P 0)
      PARAMETER (CON1=1.0$P-4,CON2=1.0$P-8)
      FUN(X)=A0+X*(A1+X*(A2+X*A3))
      DER(X)=A1+X*(TWO*A2+X*THREE*A3)
      INF=ZERO
      IF (A0.LE.ZERO) THEN
      INF=-1
      RETURN 
      ELSE IF (A1.GE.ZERO) THEN
      INF=-2
      RETURN
      ELSE IF (A3.LE.ZERO) THEN
*
*     THE QUADRATIC CASE
*    
      IF (A2.EQ.ZERO) THEN
      X=-A0/A1
      IF (X.GT.B) INF=1
      RETURN
      ENDIF
      D=A1*A1-FOUR*A0*A2
      IF (D.LT.ZERO) RETURN
      D=SQRT(D)
      X=-(A1+D)/(TWO*A2)
      IF (X.GT.B) THEN
      INF=2
      RETURN
      ENDIF
      X=-(A1-D)/(TWO*A2)
      IF (X.GT.B) THEN
      INF=2
      RETURN
      ENDIF
      ELSE
*
*     THE CUBIC CASE
*     
      C=-(A2-SQRT(A2*A2-THREE*A1*A3))/(THREE*A3)
      F=FUN(C)
      IF (F.GT.ZERO) RETURN
      IF (C.GT.B) THEN
      X=ZERO
      F=A0
      D=A1
      IF (A2.LT.ZERO) THEN
*
*     POSITIVE INFLECTION POINT
*     
      X=-A2/(THREE*A3)
      F=FUN(X)
      D=DER(X)
      ENDIF
      IF (F.GT.ZERO.OR.X.GT.B) THEN 
*
*     THE FIRST.ROOT BY THE NEWTON METHOD
*     
    1 CONTINUE
      E=-F/D
      X=X+E
      F=FUN(X)
      D=DER(X)
      IF (ABS(E).GT.CON1*C.AND.ABS(F).GT.CON2*A0) GO TO 1
      IF (X.GT.B) THEN
      INF=3
      RETURN
      ENDIF
      ENDIF
      ENDIF
*
*     SEEKING A POSITIVE FUNCTION VALUE 
*     
      X=C
    2 CONTINUE
      X=TWO*X
      F=FUN(X)
      IF (F.LT.ZERO) GO TO 2
      D=DER(X)
*
*     THE SECOND ROOT BY THE NEWTON METHOD
*     
    3 CONTINUE
      E=-F/D
      X=X+E
      F=FUN(X)
      D=DER(X)
      IF (ABS(E).GT.CON1*C.AND.ABS(F).GT.CON2*A0) GO TO 3
      IF (X.GT.B) THEN
      INF=4
      RETURN
      ENDIF
      ENDIF
      RETURN
      END   
        
     