* SUBROUTINE UNINT4                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* EXTRAPOLATION OR INTERPOLATION FOR LINE SEARCH USED IN LEAST SQUARES
* APPROXIMATION.
*
* PARAMETERS :
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  RI  AF(NA)  VECTOR CONTAINING NEW VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RI  AFO(NA)  VECTOR CONTAINING OLD VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RO  AFD(NA)  VECTOR CONTAINING INCREMENTS OF THE APPROXIMATED
*         FUNCTIONS.
*  II  IA(NA)  VECTOR CONTAINING TYPES OF DEVIATIONS.
*  RI  ETA9  MAXIMUM MACHINE NUMBER.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RI  PAR   PARAMETER.
*  RI  PAR3  PARAMETER.
*  II  MES1  INTERPOLATION METHOD.
*  II  MODE  MODE OF INTERPOLATION.
*
* COMMON DATA :
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* METHOD :
* L.LUKSAN : AN IMPLEMENTATION OF RECURSIVE QUADRATIC PROGRAMMING
* VARIABLE METRIC METHODS FOR LINEARLY CONSTRAINED NONLINEAR MINIMAX
* APPROXIMATION, KYBERNETIKA 21,(1985),PP 22-40.
*
      SUBROUTINE UNINT4(NA,AF,AFO,AFD,IA,ETA9,R,PAR,PAR3,MES1,MODE)
      $INCLUDE ('UMCOMM')
      INTEGER NA,IA(NA),MES1,MODE
      $FLOAT AF(NA),AFO(NA),AFD(NA),ETA9,R,PAR,PAR3
      $FLOAT B1,B2,B3,B4,C1,C2,C3,C4,POM,DEN,DIS,RBEST,SIX
      INTEGER K,KA
      $FLOAT PI,CON1,CON2
      PARAMETER (PI=3.14159265358979323846$P 0)
      PARAMETER (CON1=1.2$P 2,CON2=1.0$P-5)
      IF (MODE.LE.0) THEN
      RETURN
      ELSE IF (MODE.EQ.1) THEN
      IF (MES1.EQ.1) THEN
      R=TWO*R
      TSXX='BISECT. '
      RETURN
      ELSE
      RBEST=R
      TSXX='LSQ EXT '
      ENDIF
      ELSE IF (MODE.EQ.2) THEN
      RBEST=ZERO
      TSXX='LSQ INT '
      ENDIF
      SIX=THREE+THREE
C
C     DETERMINATION OF COEFFICIENTS OF THE CUBIC EQUATION
C     C1 + C2*R + 3*C3*R**2 + 2*C4*R**3 = 0
C
      C1=PAR
      C2=ZERO
      C3=ZERO
      C4=ZERO
      DO 1 KA=1,NA
      IF (IA(KA).GT.0) THEN
      DIS=((AF(KA)-AFO(KA))/R-AFD(KA))/R
      C2=C2+DIS*AFO(KA)
      C3=C3+DIS*AFD(KA)
      C4=C4+DIS*DIS
      ENDIF
    1 CONTINUE
      C2=PAR3+TWO*C2
      B3=THREE*C3
      B4=TWO*C4
C
C     DECISION BETWEEN DIRECT AND ITERATIVE SOLUTION
C
      POM=C1+RBEST*(C2+RBEST*(B3+RBEST*B4))
      DEN=C2+SIX*RBEST*(C3+RBEST*C4)
      IF (DEN.EQ.ZERO) THEN
      DIS=ETA9
      ELSE
      DIS=ABS(POM/DEN)
      ENDIF
      B2=ABS(C3+TWO*C4*RBEST)+FOUR*C4*DIS
      IF (DEN.LE.CON1*B2*MAX(ONE,DIS)) THEN
C
C     DIRECT SOLUTION
C
      B4=TWO*C4
      B1=C1/B4
      B2=C2/B4
      B3=C3/B4
      POM=(B2-THREE*B3**2)/THREE
      DEN=(B1-B2*B3+TWO*B3**3)/TWO
      DIS=DEN**2+POM**3
      IF (DIS.LT.ZERO) THEN
C
C     THREE REAL ROOTS
C
      DIS=SQRT(-DIS)
      IF (DEN.EQ.ZERO) THEN
      B1=HALF*PI
      ELSE
      B1=ATAN(DIS/ABS(DEN))
      ENDIF
      B2=TWO*SQRT(-POM)
      IF (DEN.GT.ZERO) B2=-B2
      C1=B2*COS(B1/THREE)-B3
      C2=B2*COS((B1+ TWO*PI)/THREE)-B3
      C3=B2*COS((B1+FOUR*PI)/THREE)-B3
      IF (C2.LT.C1) THEN
      C4=C1
      C1=C2
      C2=C4
      ENDIF
      IF (C3.LT.C1) THEN
      C4=C1
      C1=C3
      C3=C4
      ENDIF
      IF (C3.LT.C2) THEN
      C4=C2
      C2=C3
      C3=C4
      ENDIF
      IF (RBEST.LE.C2) THEN
      R=C1
      ELSE
      R=C3
      ENDIF
      ELSE
C
C     ONE REAL ROOT
C
      DIS=SQRT(DIS)
      B1=-DEN-DIS
      B2=-DEN+DIS
      C1=SIGN(ONE,B1)
      C2=SIGN(ONE,B2)
      C3=ONE/THREE
      R=C1*ABS(B1)**C3+C2*ABS(B2)**C3-B3
      ENDIF
      ELSE
C
C     ITERATIVE SOLUTION BY THE NEWTON METHOD
C
      K=1
    2 DIS=-POM/DEN
      R=RBEST+DIS
      B1=B2*DIS*DIS/ABS(DEN)
      IF (B1.LE.CON2.OR.K.GE.3) RETURN
      K=K+1
      RBEST=R
      POM=C1+RBEST*(C2+RBEST*(B3+RBEST*B4))
      DEN=C2+SIX*RBEST*(C3+RBEST*C4)
      GO TO 2
      ENDIF
      RETURN
      END



