* SUBROUTINE UNSTP4                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  STEPSIZE SELECTION USING POLYHEDRAL APPROXIMATION
*  FOR DESCENT STEP IN NONCONVEX VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  MA  DECLARED NUMBER OF LINEAR APPROXIMATED FUNCTIONS
*  II  MAL  CURRENT NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  RU  X(N)  VECTOR OF VARIABLES.
*  RI  AF(4*MA)  VECTOR OF BUNDLE FUNCTIONS VALUES.
*  RI  AG(N*MA)  MATRIX WHOSE COLUMNS ARE BUNDLE SUBGRADIENTS.
*  RI  AY(N*MA)  MATRIX WHOSE COLUMNS ARE VARIABLE VECTORS.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RU  DF  DIRECTIONAL DERIVATIVE.
*  RO  T  VALUE OF THE STEPSIZE PARAMETER.
*  RO  TB  BUNDLE PARAMETER FOR MATRIX SCALING.
*  RI  ETA5  DISTANCE MEASURE PARAMETER.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  RI  MOS3  LOCALITY MEASURE PARAMETER.
*
      SUBROUTINE UNSTP4(N,MA,MAL,X,AF,AG,AY,S,F,DF,T,TB,ETA5,ETA9,MOS3)
      INTEGER N,MA,MAL,MOS3
      $FLOAT X(N),AF(MA*4),AG(N*MA),AY(N*MA),S(N),F,DF,T,TB,ETA5,ETA9
      $FLOAT ALF,ALFL,ALFR,BET,BETL,BETR,DX,Q,R,W
      INTEGER I,J,JN,K,L,LQ
      $FLOAT ZERO,ONE,HALF
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0,HALF=0.5$P 0)
      W=DF*T*(ONE-T*HALF)
C
C     INITIAL CHOICE OF POSSIBLY ACTIVE LINES
C
      K=0
      L=-1
      JN=0
      TB=SQRT(ETA9)
      BETR=-ETA9
      DO 3 J=1,MAL-1
      R=ZERO
      BET=ZERO
      ALFL=AF(J)-F
      DO 2 I=1,N
      DX=X(I)-AY(JN+I)
      Q=AG(JN+I)
      R=R+DX*DX
      ALFL=ALFL+DX*Q
    2 BET=BET+S(I)*Q
      IF(MOS3.NE.2)R=R**($DBLE(MOS3)*HALF)
      ALF=MAX(ABS(ALFL),ETA5*R)
      R=ONE-BET/DF
      IF(R*R+(ALF+ALF)/DF.GT.1.0$P-6)THEN
        K=K+1
        AF(MA+K)=ALF
        AF(MA+MA+K)=BET
        R=T*BET-ALF
        IF(R.GT.W) THEN
          W=R
          L=K
        ENDIF
      ENDIF
      IF(BET.GT.ZERO)TB=MIN(TB,ALF/(BET-DF))
      BETR=MAX(BETR,BET-ALF)
    3 JN=JN+N
      LQ=-1
      IF(BETR.LE.DF*HALF)RETURN
      LQ=1
      IF(L.LT.0)RETURN
      BETR=AF(MA+MA+L)
      IF(BETR.LE.ZERO)THEN
        IF(T.LT.ONE.OR.BETR.EQ.ZERO)RETURN
        LQ=2
      ENDIF
      ALFR=AF(MA+L)
C
C     ITERATION LOOP
C
    4 IF(LQ.GE.1)THEN
        Q=ONE-BETR/DF
        R=Q+SQRT(Q*Q+(ALFR+ALFR)/DF)
        IF(BETR.GE.ZERO)R=-(ALFR+ALFR)/(DF*R)
        R=MIN(1.95$P 0,MAX(ZERO,R))
      ELSE
        IF(ABS(BETR-BETL)+ABS(ALFR-ALFL).LT.-1.0$P-4*DF)RETURN
        R=(ALFR-ALFL)/(BETR-BETL)
      ENDIF
      IF(ABS(T-R).LT.1.0$P-4)RETURN
      T=R
      AF(MA+L)=-ONE
      W=T*BETR-ALFR
      L=-1
      DO 5 J=1,K
      ALF=AF(MA+J)
      IF(ALF.LT.ZERO)GOTO5
      BET=AF(MA+MA+J)
      R=T*BET-ALF
      IF(R.GT.W) THEN
        W=R
        L=J
      ENDIF
    5 CONTINUE
      IF(L.LT.0)RETURN
      BET=AF(MA+MA+L)
      IF(BET.EQ.ZERO)RETURN
C
C     NEW INTERVAL SELECTION
C
      ALF=AF(MA+L)
      IF(BET.LT.ZERO)THEN
        IF(LQ.EQ.2)THEN
          ALFR=ALF
          BETR=BET
        ELSE
          ALFL=ALF
          BETL=BET
          LQ=0
        ENDIF
      ELSE
        IF(LQ.EQ.2)THEN
          ALFL=ALFR
          BETL=BETR
          LQ=0
        ENDIF
        ALFR=ALF
        BETR=BET
      ENDIF
      GOTO 4
      END
