* SUBROUTINE UOKIT2                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
*  PRINT OF EXIT FROM SPARSE INITIAL TRANSFORMATION SUBROUTINE.
*
* PARAMETERS:
*  II  N  TRANSFORMED NUMBER OF VARIABLES.
*  II  M  TRANSFORMED NUMBER OF CONSTRAINTS.
*  II  MROWS DECLARED NUMBER OF ROWS.
*  II  MCOLS DECLARED NUMBER OF COLUMNS.
*  IO  IBEG  BEGINNING OF THE ROW SCHEME OF THE FACTOR U IN CNR.
*  II  PRNR(MCOLS+1) POINTER VECTOR FOR RNR.
*  II  LENR(MCOLS) LENGTHS OF THE ROWS OF THE FACTOR U.
*  II  TYPE(MCOLS) TYPES OF THE VARIABLES:
*                                    =1 FOR NONBASIC LOWER BOUNDED
*                                    =2 FOR NONBASIC UPPER BOUNDED
*                                    =3 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS LOWER BOUND
*                                    =4 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS UPPER BOUND
*                                    =5 FOR NONBASIC FIXED VARIABLE
*                                    =6 FOR NONBASIC FREE VARIABLE
*                                    =-1 FOR BASIC LOWER BOUNDED
*                                    =-2 FOR BASIC UPPER BOUNDED
*                                    =-3 FOR BASIC LOWER AND UPPER
*                     BOUNDED VARIABLE
*                                    =-6 FOR BASIC FREE VARIABLE.
*  II  BCOL(MCOLS) M FIRST ELEMENTS CONTAIN COLUMN INDICES OF BASIC
*              COLUMNS.
*  RI  X(MCOLS) VECTOR OF SOLUTION.
*  RI  C(MCOLS) COST VECTOR.
*  RI  LB(MCOLS) LOWER BOUNDS ON VARIABLES.
*  RI  UB(MCOLS) UPPER BOUNDS ON VARIABLES.
*  RI  LBC(MROWS) RIGHT HAND SIDE OF THE TRANSFORMED CONSTRAINTS.
*  IO  KINP NUMBER OF ARTIFICIALS.
*  IO  MINP NUMBER OF SLACKS.
*
* COMMON DATA :
*  II  JOUT  SYSTEM VARIABLE WHICH SPECIFIES PRINT OF INFORMATION
*         DURING THE OPTIMIZATION PROCESS. JOUT=0-NO PRINT.
*         JOUT=1-PRINT IS REQUIRED.
*  II  KOUT  LEVEL OF PRINT. KOUT=0-NO PRINT. KOUT=1-PRINT OF INITIAL
*         VALUES AND FINAL RESULTS TOGETHER WITH PRINT OF ITERATIONS.
*         KOUT=2-ADDITIONAL PRINT OF STEPSIZE SELECTION.
*         KOUT=3-ADDITIONAL PRINT OF DIRECTION DETERMINATION.
*         KOUT=4-ADDITIONAL PRINT OF MATRIX CORRECTION.
*         KOUT=5-ADDITIONAL PRINT OF NUMERICAL DIFFERENTIATION.
*         KOUT=6-ADDITIONAL PRINT OF DEBUGGING.
*  II  LOUT  PRINT OF ITERATIONS. LOUT=0-STANDARD PAPER SAVING PRINT.
*         LOUT=1-ADDITIONAL PRINT OF FINAL RESULTS OF SUBROUTINES.
*         LOUT=2-ADDITIONAL PRINT OF VARIABLES. LOUT=3-ADDITIONAL
*         PRINT OF THE GRADIENT. LOUT=4-ADDITIONAL PRINT OF THE HESSIAN
*         MATRIX.
*
      SUBROUTINE UOKIT2(N,M,MROWS,MCOLS,IBEG,PRNR,LENR,TYPE,BCOL,X,C,
     & LB,UB,LBC,KINP,MINP)
      $INCLUDE('UMCOMM')
      INTEGER N,M,MROWS,MCOLS,IBEG,PRNR(MCOLS+1),LENR(MROWS),
     & TYPE(MCOLS),BCOL(MCOLS),KINP,MINP
      $FLOAT X(MCOLS),C(MCOLS),LB(MCOLS),UB(MCOLS),LBC(MROWS)
      INTEGER I,IU,NNA
      $FLOAT DENSA,DENSU
      IF(JOUT.EQ.0) RETURN
      IF(ABS(KOUT).LE.3) RETURN
C
C COMPUTE IU,DENSITIES
C
      IU=0
      DO 100 I=1,M
        IU=IU+LENR(I)
  100 CONTINUE
      NNA=PRNR(N+1)-1
      IF(MIN(M,N).GT.0) THEN
        DENSA=NNA
        DENSA=DENSA/(M*N)
      ELSE
        DENSA=0
      ENDIF
      IF(M.GT.0) THEN
        DENSU=2*IU-M
        DENSU=DENSU/(M*M)
      ELSE
        DENSU=0
      ENDIF
      WRITE(IWR,2) N,M,DENSA,NNA
    2 FORMAT(' ','N  =',I4,2X,'M =',I4,2X,'MATRIX DENSITY =',
     *       $P 9.3,2X,'ELEMENTS OF A =',I4)
      WRITE(IWR,7) DENSU,IU,IBEG
    7 FORMAT(' ','DENSITY OF U =',$P 9.3,2X,'ELEMENTS OF U =',
     *       I4,2X,'ELEMENTS OF L =',I4)
      WRITE(IWR,3) NREF,MINP,KINP
    3 FORMAT(' ','REFACTORIZATIONS =',I4,2X,'SLACKS =',I4,2X,
     *       'ARTIFICIALS = ',I4)
      IF(ABS(LOUT).GE.1) WRITE(IWR,5)(X(I),I=1,MIN(N,25))
    5 FORMAT(' ','X   =',5$P 14.5:/(6X,5$P 14.5))
      IF(ABS(LOUT).GE.1) WRITE(IWR,8)(C(I),I=1,MIN(N,25))
    8 FORMAT(' ','GF  =',5$P 14.5:/(6X,5$P 14.5))
      IF(ABS(LOUT).GE.1) WRITE(IWR,4)(BCOL(I),I=1,MIN(M,50))
    4 FORMAT(' ','BCOL=',10I7:/(6X,10I7))
      IF(ABS(LOUT).GE.1.AND.KBF.GT.0)WRITE(IWR,6)(TYPE(I),I=1,MIN(N,50))
    6 FORMAT(' ','IX  =',10I7:/(6X,10I7))
      IF(ABS(LOUT).GE.1.AND.KBF.GT.0) WRITE(IWR, 9)(LB(I),I=1,MIN(N,25))
    9 FORMAT(' ','XL  =',5$P 14.5:/(6X,5$P 14.5))
      IF(ABS(LOUT).GE.1.AND.KBF.GT.0) WRITE(IWR,10)(UB(I),I=1,MIN(N,25))
   10 FORMAT(' ','XU  =',5$P 14.5:/(6X,5$P 14.5))
      IF(ABS(LOUT).GE.1) WRITE(IWR,11)(LBC(I),I=1,MIN(M,25))
   11 FORMAT(' ','CL  =',5$P 14.5:/(6X,5$P 14.5))
      WRITE(IWR,1) ITERL
    1 FORMAT(' ','END OF INITIAL SPARSE TRANSFORMATION : ITERL = ',
     *       I3)
      RETURN
      END
