* SUBROUTINE UOLQD5                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* PRINT OF ITERATIONS IN DUAL RANGE SPACE QUADRATIC PROGRAMMING
* METHOD.
*
* PARAMETERS:
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  RI  S(NF)  DIRECTION VECTOR.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  IA(2*NA)  VECTOR CONTAINING TYPES OF APPROXIMATED FUNCTIONS.
*  II  IAA(NF+1)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  AR((NF+1)*(NF+2)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RI  AZ(NF+1)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RO  PAR  CONSTRAINT VIOLATION.
*  RI  DMAX  MAXIMUM STEPSIZE.
*  RI  SNORM  OPTIMUM LAGRANGE MULTIPLIER.
*  RO  XNORM  MAXIMUM APPROXIMATED FUNCTION VALUE.
*  II  JOLD  INDEX OF THE REMOVED CONSTRAINT.
*  II  JNEW  INDEX OF THE NEW ACTIVE CONSTRAINT.
*  II  MODE  PHASE INDICATOR OF THE ALGORITHM.
*
* COMMON DATA :
*  II  LCS  VARIABLE WHICH SPECIFIES TRUST REGION BOUNDS.
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  NRED  NUMBER OF STEPS.
*  II  IWR  NUMBER OF THE OUTPUT UNIT.
*  II  JOUT  SYSTEM VARIABLE WHICH SPECIFIES PRINT OF INFORMATION
*         DURING THE OPTIMIZATION PROCESS. JOUT=0-NO PRINT.
*         JOUT=1-PRINT IS REQUIRED.
*  II  KOUT  LEVEL OF PRINT. KOUT=0-NO PRINT. KOUT=1-PRINT OF INITIAL
*         VALUES AND FINAL RESULTS TOGETHER WITH PRINT OF ITERATIONS.
*         KOUT=2-ADDITIONAL PRINT OF STEPSIZE SELECTION.
*         KOUT=3-ADDITIONAL PRINT OF DIRECTION DETERMINATION.
*         KOUT=4-ADDITIONAL PRINT OF NUMERICAL DIFFERENTIATION.
*         KOUT=5-ADDITIONAL PRINT OF DEBUGGING.
*  II  LOUT  LEVEL OF PRINT. LOUT=0-PRINT OF SCALAR VALUES.
*         LOUT=1-ADDITIONAL PRINT OF BASIC VECTORS.
*         LOUT=2-ADDITIONAL PRINT OF OTHER VECTORS.
*         LOUT=3-ADDITIONAL PRINT OF MATRICES.
*  II  MOUT  LEVEL OF PRINT ON THE TERMINAL DEVICE.
*         MOUT=0-NO PRINT.
*         MOUT=1-PRINT OF FINAL RESULTS.
*         MOUT=2-ADDITIONAL PRINT OF ITERATIONS.
*
      SUBROUTINE UOLQD5(NF,N,NC,NA,S,IX,IC,IA,IAA,AR,AZ,PAR,DMAX,SNORM,
     & XNORM,JOLD,JNEW,MODE)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N,NC,NA,IX(NF),IC(NC),IA(2*NA),IAA(NF+1),JOLD,JNEW,MODE
      $FLOAT S(NF),AR((NF+1)*(NF+2)/2),AZ(NF+1),PAR,DMAX,SNORM,XNORM
      CHARACTER*8 TEXT
      INTEGER I
      LOGICAL LL
      LL=KOUT.EQ.0.AND.LOUT.GT.0
      TEXT='        '
      IF (JOLD.NE.0) TEXT='DELETION'
      IF (JNEW.NE.0) TEXT='ADDITION'
      IF(ABS(MOUT).GT.3) THEN
      WRITE (ITR,4) MODE,NRED,N,JOLD,JNEW,TEXT
      IF(LL) WRITE (IWR,4) MODE,NRED,N,JOLD,JNEW,TEXT
      ENDIF
    4 FORMAT (' ','MODE =',I4,3X,'NRED =',I4,3X,'N =',I4,3X,'IOLD =',
     & I4,3X,'INEW =',I4,3X,A8)
      IF(JOUT.EQ.0) RETURN
      IF(ABS(KOUT).LE.3) RETURN
      WRITE (IWR,1) MODE,NRED,N,JOLD,JNEW,PAR,SNORM,DMAX,TEXT
    1 FORMAT (' ',I1,2X,I3,2X,'N=',I4,2X,'OLD=',I4,2X,'NEW=',I4,2X,
     & 'V=',$P 9.3,1X,'U=',$P 9.3,1X,'D=',$P 9.3,1X,A8)
      IF (ABS(LOUT).GE.1.AND.JOLD.EQ.0) WRITE(IWR,3) XNORM
    3 FORMAT(' ','Z   =',$P 14.5)
      IF(ABS(LOUT).GE.1) WRITE(IWR,5) (IAA(I),I=1,MIN(NF-N,50))
    5 FORMAT(' ','IAA =',10I7:/(6X,10I7))
      IF(ABS(LOUT).GE.1.AND.(KBF.GT.0.OR.LCS.GT.0)) WRITE(IWR,10)
     & (IX(I),I=1,MIN(NF,50))
   10 FORMAT(' ','IX  =',10I7:/(6X,10I7))
      IF(ABS(LOUT).GE.1.AND.NC.GT.0) WRITE(IWR,6) (IC(I),I=1,
     & MIN(NC,50))
    6 FORMAT(' ','IC  =',10I7:/(6X,10I7))
      IF(ABS(LOUT).GE.1.AND.NA.GT.0) WRITE(IWR,7) (IA(I),I=1,
     & MIN(2*NA,50))
    7 FORMAT(' ','IA  =',10I7:/(6X,10I7))
      IF (ABS(LOUT).GE.2.AND.JOLD.EQ.0) WRITE(IWR,2) (S(I),I=1,
     & MIN(NF,25))
    2 FORMAT(' ','S   =',5$P 14.5:/(6X,5$P 14.5))
      IF(ABS(LOUT).GE.2) WRITE(IWR,8) (AZ(I),I=1,MIN(NF-N,25))
    8 FORMAT(' ','AZ  =',5$P 14.5:/(6X,5$P 14.5))
      IF(ABS(LOUT).GE.3) THEN
      WRITE(IWR,9) (AR(I),I=1,MIN((NF-N)*(NF-N+1)/2,30))
    9 FORMAT(' ','AR  =',5$P 14.5:/(6X,5$P 14.5))
      ENDIF
      RETURN
      END
