* SUBROUTINE UOLQP4                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* PRINT OF ITERATIONS IN PRIMAL RANGE SPACE QUADRATIC PROGRAMMING
* METHOD.
*
* PARAMETERS:
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES
*  II  NC  NUMBER OF LINEAR CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CF(NC)  VALUES OF THE CONSTRAINT FUNCTIONS.
*  RI  CR(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RI  CZ(NF)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RO  DMAX  MAXIMUM STEPSIZE.
*  RO  GMAX  NORM OF THE TRANSFORMED GRADIENT.
*  RI  UMAX  MAXIMUN ABSOLUTE VALUE OF THE NEGATIVE LAGRANGE MULTIPLIER.
*  II  JOLD  INDEX OF THE REMOVED CONSTRAINT.
*  II  JNEW  INDEX OF THE NEW ACTIVE CONSTRAINT.
*  II  MODE  PHASE INDICATOR OF THE ALGORITHM.
*
* COMMON DATA :
*  II  LCS  VARIABLE WHICH SPECIFIES TRUST REGION BOUNDS.
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  NRED  NUMBER OF STEPS.
*  II  IWR  NUMBER OF THE OUTPUT UNIT.
*  II  JOUT  SYSTEM VARIABLE WHICH SPECIFIES PRINT OF INFORMATION
*         DURING THE OPTIMIZATION PROCESS. JOUT=0-NO PRINT.
*         JOUT=1-PRINT IS REQUIRED.
*  II  KOUT  LEVEL OF PRINT. KOUT=0-NO PRINT. KOUT=1-PRINT OF INITIAL
*         VALUES AND FINAL RESULTS TOGETHER WITH PRINT OF ITERATIONS.
*         KOUT=2-ADDITIONAL PRINT OF STEPSIZE SELECTION.
*         KOUT=3-ADDITIONAL PRINT OF DIRECTION DETERMINATION.
*         KOUT=4-ADDITIONAL PRINT OF NUMERICAL DIFFERENTIATION.
*         KOUT=5-ADDITIONAL PRINT OF DEBUGGING.
*  II  LOUT  LEVEL OF PRINT. LOUT=0-PRINT OF SCALAR VALUES.
*         LOUT=1-ADDITIONAL PRINT OF BASIC VECTORS.
*         LOUT=2-ADDITIONAL PRINT OF OTHER VECTORS.
*         LOUT=3-ADDITIONAL PRINT OF MATRICES.
*  II  MOUT  LEVEL OF PRINT ON THE TERMINAL DEVICE.
*         MOUT=0-NO PRINT.
*         MOUT=1-PRINT OF FINAL RESULTS.
*         MOUT=2-ADDITIONAL PRINT OF ITERATIONS.
*
      SUBROUTINE UOLQP4(NF,N,NC,X,IX,IC,ICA,CF,CR,CZ,DMAX,GMAX,UMAX,
     & JOLD,JNEW,MODE)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N,NC,IX(NF),IC(NC),ICA(NF),JOLD,JNEW,MODE
      $FLOAT X(NF),CF(NC),CR(NF*(NF+1)/2),CZ(NF),DMAX,GMAX,UMAX
      INTEGER I
      CHARACTER*8 TEXT
      LOGICAL LL
      LL=KOUT.EQ.0.AND.LOUT.GT.0
      TEXT='        '
      IF (JOLD.NE.0) TEXT='DELETION'
      IF (JNEW.NE.0) TEXT='ADDITION'
      IF(ABS(MOUT).GT.3) THEN
      WRITE (ITR,3) MODE,NRED,N,JOLD,JNEW,TEXT
      IF(LL) WRITE (IWR,3) MODE,NRED,N,JOLD,JNEW,TEXT
      ENDIF
    3 FORMAT (' ','MODE =',I4,3X,'NRED =',I4,3X,'N =',I4,3X,'IOLD =',
     & I4,3X,'INEW =',I4,3X,A8)
      IF(JOUT.EQ.0) RETURN
      IF(ABS(KOUT).LE.3) RETURN
      WRITE (IWR,1) MODE,NRED,N,JOLD,JNEW,GMAX,UMAX,DMAX,TEXT
    1 FORMAT (' ',I1,2X,I3,2X,'N=',I4,2X,'OLD=',I4,2X,'NEW=',I4,2X,
     & 'G=',$P 9.3,1X,'U=',$P 9.3,1X,'D=',$P 9.3,1X,A8)
      IF(ABS(LOUT).GE.1) WRITE(IWR,5) (ICA(I),I=1,MIN(NF-N,50))
    5 FORMAT(' ','ICA =',10I7:/(6X,10I7))
      IF(ABS(LOUT).GE.1.AND.(KBF.GT.0.OR.LCS.GT.0)) WRITE(IWR,6)
     & (IX(I),I=1,MIN(NF,50))
    6 FORMAT(' ','IX  =',10I7:/(6X,10I7))
      IF(ABS(LOUT).GE.1.AND.NC.GT.0) WRITE(IWR,7) (IC(I),I=1,MIN(NC,50))
    7 FORMAT(' ','IC  =',10I7:/(6X,10I7))
      IF(ABS(LOUT).GE.2) WRITE (IWR,2) (X(I),I=1,MIN(NF,25))
    2 FORMAT(' ','X   =',5$P 14.5:/(6X,5$P 14.5))
      IF(ABS(LOUT).GE.2.AND.KBC.GT.0) WRITE (IWR,4) (CF(I),I=1,
     & MIN(NC,25))
    4 FORMAT(' ','CF  =',5$P 14.5:/(6X,5$P 14.5))
      IF(ABS(LOUT).GE.2) WRITE(IWR,8) (CZ(I),I=1,MIN(NF-N,25))
    8 FORMAT(' ','CZ  =',5$P 14.5:/(6X,5$P 14.5))
      IF(ABS(LOUT).GE.3) THEN
      WRITE(IWR,9) (CR(I),I=1,MIN((NF-N)*(NF-N+1)/2,30))
    9 FORMAT(' ','CR  =',5$P 14.5:/(6X,5$P 14.5))
      ENDIF
      RETURN
      END
