$INTERFACE UPLAG7                                             05/12/01

  CALL UPLAG7(NF,NX,NC,$X,$IX,$XL,$XU,$XSL,$XSU,$XZL,$XZU,$XP,$CF,$IC&
,$CL,$CU,$CSL,$CSU,$CZL,$CZU,$CP,RPF3,MEP4)
  $ADD(REAL,'\$X($NF)\$XL($NX)\$XU($NX)\$XSL($NX)\$XSU($NX)\$XZL($NX)&
  \$XZU($NX)\$XP($NX)\$CF($NC)\$CL($NC)\$CU($NC)\$CSL($NC)\$CSU($NC)&
  \$CZL($NC)\$CZU($NC)\$CP($NC)')
  $ADD(INTEGER,'\$IX($NX)\$IC($NC)')
$REM  $ADD(REAL,'\RPF3')
$REM  $ADD(INTEGER,'\NF\NX\NC\MEP4')
  $ADD(INIT)
    NPFX='UPLAG7'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UPLAG7             ALL SYSTEMS                 05/12/01
C PORTABILITY : ALL SYSTEMS
C 05/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF VALUE OF THE AUGMENTED LAGRANGIAN FUNCTION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  XP(NC)  VECTOR CONTAINING PENALTY PARAMETERS FOR VARIABLES.
*  RI  CF(NC)  VECTOR CONTAINING VALUES OF APPROXIMATED FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CP(NC)  VECTOR CONTAINING PENALTY PARAMETERS FOR CONSTRAINTS.
*  RI  RPF3  BARRIER COEFFICIENT.
*  II  MEP4  COMPUTATION OF THE LAGRANGE MULTIPLIERS. MEP4=1-LAGRANGE
*         MULTIPLIERS ARE COMPUTED FROM THE PENALTY TERM.
*         MEP4=2-LAGRANGE MULTIPLIERS ARE COMPUTED FROM VARIOUS TERMS.
*         MEP4=3-LAGRANGE MULTIPLIERS ARE COMPUTED FROM THE BARRIER
*         TERM.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UPLAG7(NF,NX,NC,X,IX,XL,XU,XSL,XSU,XZL,XZU,XP,CF,IC,CL,
     & CU,CSL,CSU,CZL,CZU,CP,RPF3,MEP4)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,IX(NX),IC(NC),MEP4
      $FLOAT X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),XP(NX),
     & CF(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),CP(NC),RPF3
      INTEGER K,KK
      $FLOAT POM,TMP,TEMP
      IF (KBF.GT.0) THEN
      DO 1 K=1,NX
      TMP=RPF3/XP(K)
      KK=ABS(IX(K))
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      POM=XL(K)-X(K)
      TEMP=ABS(TMP+HALF*POM)+SQRT(TMP**2+(HALF*POM)**2)
      IF (TMP+HALF*POM.GE.ZERO) THEN
      XSL(K)=TEMP
      ELSE
      XSL(K)=ABS(TMP*POM)/TEMP
      ENDIF
      IF (MEP4.LE.1) THEN
      XZL(K)=RPF3/(XSL(K)-POM)
      ELSE
      TEMP=ABS(TMP-HALF*POM)+SQRT(TMP**2+(HALF*POM)**2)
      IF (TMP-HALF*POM.GE.ZERO) THEN
      XZL(K)=RPF3/TEMP
      ELSE
      XZL(K)=RPF3*TEMP/ABS(TMP*POM)
      ENDIF
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      POM=X(K)-XU(K)
      TEMP=ABS(TMP+HALF*POM)+SQRT(TMP**2+(HALF*POM)**2)
      IF (TMP+HALF*POM.GE.ZERO) THEN
      XSU(K)=TEMP
      ELSE
      XSU(K)=ABS(TMP*POM)/TEMP
      ENDIF
      IF (MEP4.LE.1) THEN
      XZU(K)=RPF3/(XSU(K)-POM)
      ELSE
      TEMP=ABS(TMP-HALF*POM)+SQRT(TMP**2+(HALF*POM)**2)
      IF (TMP-HALF*POM.GE.ZERO) THEN
      XZU(K)=RPF3/TEMP
      ELSE
      XZU(K)=RPF3*TEMP/ABS(TMP*POM)
      ENDIF
      ENDIF
      ENDIF
    1 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 2 K=1,NC
      TMP=RPF3/CP(K)
      KK=ABS(IC(K))
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      POM=CL(K)-CF(K)
      TEMP=ABS(TMP+HALF*POM)+SQRT(TMP**2+(HALF*POM)**2)
      IF (TMP+HALF*POM.GE.ZERO) THEN
      CSL(K)=TEMP
      ELSE
      CSL(K)=ABS(TMP*POM)/TEMP
      ENDIF
      IF (MEP4.LE.1) THEN
      CZL(K)=RPF3/(CSL(K)-POM)
      ELSE
      TEMP=ABS(TMP-HALF*POM)+SQRT(TMP**2+(HALF*POM)**2)
      IF (TMP-HALF*POM.GE.ZERO) THEN
      CZL(K)=RPF3/TEMP
      ELSE
      CZL(K)=RPF3*TEMP/ABS(TMP*POM)
      ENDIF
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      POM=CF(K)-CU(K)
      TEMP=ABS(TMP+HALF*POM)+SQRT(TMP**2+(HALF*POM)**2)
      IF (TMP+HALF*POM.GE.ZERO) THEN
      CSU(K)=TEMP
      ELSE
      CSU(K)=ABS(TMP*POM)/TEMP
      ENDIF
      IF (MEP4.LE.1) THEN
      CZU(K)=RPF3/(CSU(K)-POM)
      ELSE
      TEMP=ABS(TMP-HALF*POM)+SQRT(TMP**2+(HALF*POM)**2)
      IF (TMP-HALF*POM.GE.ZERO) THEN
      CZU(K)=RPF3/TEMP
      ELSE
      CZU(K)=RPF3*TEMP/ABS(TMP*POM)
      ENDIF
      ENDIF
      ENDIF
    2 CONTINUE
      ENDIF
      RETURN
      END

