$INTERFACE US0L08                                             97/12/01

  $HELP
    UPPER BOUND FOR FUNCTION VALUE
  $CHECK(FMAX,'1.0$P 20',REAL,4)
  $HELP
    TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST
    ON THE CHANGE OF THE FUNCTION VALUE)
  $CHECK(EPS1,'1.0$P-4',REAL,4)
  $HELP
    TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST
    ON THE CHANGE OF THE CONSTRAINT VIOLATION)
  $CHECK(EPS2,'1.0$P 0-1.0$P-4',REAL,4)
  $HELP
    MAXIMUM NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS
  $CHECK(MRED,20,INT,4)
  CALL US0L08($FS,EPS1,EPS2,RO,R,F,FC,FMIN,RMIN,RMAX,SNORM&
,MODE,KTERS,INITS,MRED)
$REM  $ADD(REAL,'\EPS1\EPS2\RO\R\FF\FC\FMIN\RMIN\RMAX\SNORM')
$REM  $ADD(INTEGER,'\MODE\KTERS\INITS\MRED')
  $ADD(INIT)
    NSXX='US0L08'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE US0L08                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  LINE SEARCH WITHOUT DIRECTIONAL DERIVATIVES BASED ON THE MARKOV
*  FILTER.
*
* PARAMETERS :
*  RU  FS(2)  MARKOV FILTER SAMPLES.
*  RI  EPS1  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE FUNCTION VALUE).
*  RI  EPS2  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE CONSTRAINT VIOLATION).
*  RO  RO  INITIAL VALUE OF THE STEPSIZE PARAMETER.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RO  FF  VALUE OF THE OBJECTIVE FUNCTION.
*  RO  FC  L-INFINITY NORM OF THE CONSTRAINT VIOLATION.
*  RI  FMIN  LOWER BOUND FOR VALUE OF THE OBJECTIVE FUNCTION.
*  RI  RMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  SNORM  NORM OF THE DIRECTION VECTOR.
*  IO  MODE  MODE OF LINE SEARCH. MODE=0-INITIAL STEPSIZE.
*         MODE=1-EXTRAPOLATION. MODE=2-INTERPOLATION.
*  II  KTERS  TERMINATION SELECTION. KTERS=1-PERFECT LINE SEARCH.
*         KTERS=2-GOLDSTEIN STEPSIZE. KTERS=3-WEAK WOLFE STEPSIZE.
*         KTERS=4-STRONG WOLFE STEPSIZE. KTERS=5-ARMIJO STEPSIZE.
*         KTERS=6-FIRST STEPSIZE.
*  II  INITS  CHOICE OF THE INITIAL STEPSIZE. INITS=0-INITIAL STEPSIZE
*         IS SPECIFIED IN THE CALLING PROGRAM. INITS=1-UNIT INITIAL
*         STEPSIZE. INITS=2-COMBINED UNIT AND QUADRATICALLY ESTIMATED
*         INITIAL STEPSIZE. INITS=3-QUADRATICALLY ESTIMATED INITIAL
*         STEPSIZE.
*  II  MRED  MAXIMUM NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*
* COMMON DATA :
*  IO  MAXST  MAXIMUM STEPSIZE INDICATOR. MAXST=0 OR MAXST=1 IF MAXIMUM
*         STEPSIZE WAS NOT OR WAS REACHED.
*  IO  NRED  ACTUAL NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*  IO  ITERS  TERMINATION INDICATOR. ITERS=0-ZERO STEP. ITERS=1-PERFECT
*         LINE SEARCH. ITERS=2 GOLDSTEIN STEPSIZE. ITERS=3-WEAK WOLFE
*         STEPSIZE. ITERS=4-STRONG WOLFE STEPSIZE.
*         ITERS=5-ARMIJO STEPSIZE. ITERS=6-FIRST STEPSIZE.
*         ITERS=7-MAXIMUM STEPSIZE. ITERS=8-UNBOUNDED FUNCTION.
*         ITERS=-1-MRED REACHED. ITERS=-2-POSITIVE DIRECTIONAL
*         DERIVATIVE. ITERS=-3-ERROR IN INTERPOLATION.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAM USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UOS0F1  PRINT OF ENTRY TO STEPSIZE SELECTION.
*  S   UOS0F2  PRINT OF EXIT FROM STEPSIZE SELECTION.
*  S   UOS0F3  PRINT OF INFORMATION DURING STEPSIZE SELECTION.
*  S   UOERR1  ERROR MESSAGES.
*
* METHOD :
* SAFEGUARDED EXTRAPOLATION AND INTERPOLATION WITH EXTENDED TERMINATION
* CRITERIA.
*
      SUBROUTINE US0L08(FS,EPS1,EPS2,RO,R,FF,FC,FMIN,RMIN,RMAX,SNORM,
     & MODE,KTERS,INITS,MRED)
      $INCLUDE ('UMCOMM')
      INTEGER MODE,KTERS,INITS,MRED
      $FLOAT FS(2),EPS1,EPS2,RO,R,FF,FC,FMIN,RMIN,RMAX,SNORM
      $FLOAT CON
      PARAMETER(CON=1.0$P 4)
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',0)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      CALL UOS0F1(RO,FF,FC,SNORM,RMAX)
      IF(RMAX.LE.ZERO) THEN
      TSXX='RMAX=0'
      ITERS=0
      GO TO 10
      ENDIF
C
C     INITIAL STEPSIZE SELECTION
C
      TSXX='INITIAL '
      IF (INITS.EQ.0) THEN
      ELSE
      R=ONE
      TSXX='INIT-1  '
      ENDIF
      MAXST=0
      R=MAX(R,RMIN)
      R=MIN(R,RMAX)
      IF (R.GE.RMAX) MAXST=1
      MODE=0
C
C     NEW STEPSIZE SELECTION (EXTRAPOLATION OR INTERPOLATION)
C
    2 IF (MODE.NE.0) THEN
      R=HALF*R
      NRED=NRED+1
      ENDIF
C
C     COMPUTATION OF THE NEW FUNCTION VALUE
C
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
      IF (KTERS.LT.0.OR.KTERS.EQ.6) GO TO 10
      CALL UOS0F3(R,FF,FC,SNORM,MODE)
      IF (FF.GT.FS(1)-EPS1*FS(2).AND.FC.GT.EPS2*FS(2)) GO TO 9
C      IF (FF.GT.CON*ABS(FS(1))) GO TO 9
C      IF (FC.GT.CON*ABS(FS(2))) GO TO 9
      FS(1)=FF
      FS(2)=FC
      ITERS=4
      TSXX='SUCCESS '
      GO TO 10
    9 NRED=0
      MODE=2
      IF(ABS(NRED).GE.MRED) THEN
      ITERS=-1
      TSXX='MAX  INT'
      CALL UOERR1('US0L02',5-ITERS)
      GO TO 10
      ELSE IF (FF.LE.FMIN) THEN
      ITERS=7
      TSXX='F BOUND '
      GO TO 10
      ELSE IF (R.LE.RMIN) THEN
      ITERS=0
      TSXX='ZERO  ST'
      GO TO 10
      ELSE
      GO TO 2
      ENDIF
   10 IF (MAXST.EQ.1) NMAX=NMAX+1
      CALL UOS0F2(R,FF,FC,SNORM)
      CALL UYEPI1(0)
      RETURN
      END
