$INTERFACE US0R02                                             11/12/01

  $HELP
    TYPE OF THE REGULARIZATION METHOD.
      0 - BASIC REGULARIZATION METHOD
      1 - STRICT REGULARIZATION METHOD
  $CHECK(MES,0,'0\1')
  $HELP
    STRATEGY FOR RESTART
      0 - RESTART AFTER UNSUCCESSFUL STEP
      1 - RESTART AFTER LARGE STEP
      2 - RESTART AFTER LARGE BOUNDARY STEP
  $CHECK(MES3,1,'0\1\2')
  $DEFAULT='0.10$P 0'
  $IF(MES>0) $DEFAULT='0.01$P 0'
  $HELP
    FIRST TOLERANCE FOR RATIO DF/DFPR. STEP BOUND IS
    DECREASED IF DF/DFPR<EPS3
  $CHECK(EPS3,DEFAULT,REAL,4)
  $HELP
    SECOND TOLERANCE FOR RATIO DF/DFPR. STEP BOUND IS
    INCREASED IF IT IS ACTIVE AND DF/DFPR>EPS4
  $CHECK(EPS4,'0.95$P 0',REAL,4)
  $HELP
    LOWER BOUND FOR PARAMETER REDUCTION
  $CHECK(BET1,'0.1$P 0',REAL,4)
  $HELP
    UPPER BOUND FOR PARAMETER REDUCTION
  $CHECK(BET2,'1.0$P 0',REAL,4)
  $HELP
    LOWER BOUND FOR PARAMETER EXPANSION
  $CHECK(GAM1,'2.0$P 0',REAL,4)
  $HELP
    UPPER BOUND FOR PARAMETER EXPANSION
  $CHECK(GAM2,'1.0$P 2',REAL,4)
  $HELP
    MAXIMUM STEPSIZE
  $CHECK(XMAX,'1.0$P 3',REAL,2)
  $HELP
    MAXIMUM NUMBER OF ZERO STEPS IN ONE ITERATION
  $CHECK(MRED,10,INT,4)
  CALL US0R02(EPS3,EPS4,BET1,BET2,GAM1,GAM2,RO,R,FO,F,PO,PP,PPP,XDEL&
,XDELO,XMAX,GNORM,SNORM,PAR1,RMAX,KTERS,MES,MES3,MRED,$KDS)
$REM  $ADD(REAL,'\EPS3\EPS4\BET1\BET2\GAM1\GAM2\RO\R\FO\F\PO\PP\PPP&
$REM  \XDEL\XDELO\XMAX\GNORM\PAR1\RMAX')
$REM  $ADD(INTEGER,'\KTERS\MES\MES1\MES2\MES3\MRED')
  $ADD(INIT)
    NSXX='US0R01'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE US0R02                ALL SYSTEMS                11/12/01
C PORTABILITY : ALL SYSTEMS
C 11/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SIMPLE SEARCH WITH TRUST REGION UPDATE.
*
* PARAMETERS :
*  RI  EPS3  FIRST TOLERANCE FOR RATIO DF/DFPR. STEP BOUND IS
*         DECREASED IF DF/DFPR<EPS3.
*  RI  EPS4  SECOND TOLERANCE FOR RATIO DF/DFPR. STEP BOUND IS
*         INCREASED IF IT IS ACTIVE AND DF/DFPR>EPS4.
*  RI  BET1  LOWER BOUND FOR STEPSIZE REDUCTION.
*  RI  BET2  UPPER BOUND FOR STEPSIZE REDUCTION.
*  RI  GAM1  LOWER BOUND FOR STEPSIZE EXPANSION.
*  RI  GAM2  UPPER BOUND FOR STEPSIZE EXPANSION.
*  RO  RO  INITIAL VALUE OF THE STEPSIZE PARAMETER.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RI  FO  INITIAL VALUE OF THE OBJECTIVE FUNCTION.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  PO  INITIAL VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  PP  QUADRATIC PART OF THE PREDICTED FUNCTION VALUE.
*  RI  PPP  CUBIC PART OF THE PREDICTED FUNCTION VALUE.
*  RU  XDEL  TRUST REGION BOUND.
*  RO  XDELO  PREVIOUS TRUST REGION BOUND.
*  RI  XMAX MAXIMUM STEPSIZE.
*  RI  GNORM  EUCLIDEAN NORM OF THE GRADIENT.
*  RA  PAR1  AUXILIARY VARIABLE FOR CONTROLLED SCALING OF VARIABLE
*            METRIC UPDATES.
*  RI  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  II  KTERS  TERMINATION SELECTION. KTERS=1-NORMAL TERMINATION.
*         KTERS=6-FIRST STEPSIZE.
*  II  MES  TYPE OF THE TRUST REGION METHOD. MES=0-BASIC TRUST REGION
*         METHOD. MES=1-STRICT TRUST REGION METHOD.
*  II  MES3  STRATEGY FOR RESTART. MES3=0-RESTART AFTER UNSUCCESSFUL
*         STEP. MES3=1-RESTART AFTER LARGE BOUNDARY STEP. MES3=2-RESTART
*         AFTER LARGE BOUNDARY STEP.
*  II  MRED  MAXIMUM NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*  II  KDS  DEGREE OF COMPUTED DERIVATIVES.
*
* COMMON DATA :
*  IO  MAXST  MAXIMUM STEPSIZE INDICATOR. MAXST=0 OR MAXST=1 IF MAXIMUM
*         STEPSIZE WAS NOT OR WAS REACHED.
*  IU  IDIR INDICATOR FOR DIRECTION DETERMINATION.
*         IDIR=0-BASIC DETERMINATION. IDIR=1-DETERMINATION
*         AFTER STEPSIZE REDUCTION. IDIR=2-DETERMINATION AFTER
*         STEPSIZE EXPANSION.
*  IO  NRED  ACTUAL NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*  IO  ITERS  TERMINATION INDICATOR. ITERS=0-ZERO STEP. ITERS=1-STEP
*         BOUND WAS DECREASED. ITERS=2-STEP BOUND WAS UNCHANGED.
*         ITERS=3-STEP BOUND WAS INCREASED. ITERS=6-FIRST STEPSIZE.
*  II  ITERD TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  TO  TXFU  TEXT INFORMATION ON THE CAUSE OF TERMINATION.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAM USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UOS0G1  PRINT OF ENTRY TO STEPSIZE SELECTION.
*  S   UOS0G2  PRINT OF EXIT FROM STEPSIZE SELECTION.
*  S   UOS0G3  PRINT OF INFORMATION DURING STEPSIZE SELECTION.
*  S   UOERR1  ERROR MESSAGES.
*
* METHOD :
* G.A.SCHULTZ, R.B.SCHNABEL, R.H.BYRD: A FAMILY OF TRUST-REGION-BASED
* ALGORITHMS FOR UNCONSTRAINED MINIMIZATION WITH STRONG GLOBAL
* CONVERGENCE PROPERTIES, SIAM J. NUMER.ANAL. 22 (1985) PP. 47-67.
*
      SUBROUTINE US0R02(EPS3,EPS4,BET1,BET2,GAM2,GAM1,RO,R,FO,F,PO,PP,
     & PPP,XDEL,XDELO,XMAX,GNORM,SNORM,PAR1,RMAX,KTERS,MES,MES3,MRED,
     & KDS)
      $INCLUDE('UMCOMM')
      $FLOAT EPS3,EPS4,BET1,BET2,GAM1,GAM2,RO,R,FO,F,PO,PP,PPP,XDEL,
     & XDELO,XMAX,GNORM,SNORM,PAR1,RMAX
      $FLOAT FQ,FC,FX,DF,DFPR,DP,ALF,BET,XPOM
      INTEGER KTERS,MES,MES3,MRED,KDS,INF
      REAL*8 CON1,CON2,CON3,CON4
      PARAMETER(CON1=1.0$P-8,CON2=1.0$-2,CON3=0.01$P 0,CON4=2.0$P 0) 
*
*     CON1=EPSX, CON2=BETA, CON3=ETA, CON4=ALFM
*     
      INTEGER NRED1,NRED2
      SAVE NRED1,NRED2
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',KDS)
      GO TO (1,2) ISP(NS)
    1 CONTINUE
*      CON3=EPS3
      CALL UOS0G1(RO,FO,XDEL,RMAX)
      IF (IDIR.EQ.0) THEN
      NRED1=0
      NRED2=0
      ENDIF
      IDIR=0
      PAR1=ONE
      XDELO=XDEL
C
C     COMPUTATION OF THE NEW FUNCTION VALUE
C
      R=MIN(ONE,RMAX)
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      IF(KTERS.LT.0.OR.KTERS.GT.5) THEN
      ITERS=6
      TSXX='FIRST ST'
      ELSE
      DF=FO-F
C
C     CUBIC MODEL
C
      DFPR=-R*(PO+R*(PP+R*PPP))
      FQ=FO+R*(PO+R*PP)
      FC=FO-DFPR
      FX=FC-MAX(F,FQ)
      DP=F-FQ
      IF (DF.LT.EPS3*DFPR) THEN
C
C     STEP IS TOO LARGE, IT HAS TO BE REDUCED
C
      IF (DF.LT.ZERO) THEN
      CALL UNCUR2((THREE-TWO*CON3)*PO,(THREE-CON3)*PP,THREE*DP,
     & ALF,INF)
      IF (INF.GE.0) THEN
      XPOM=-(PO+TWO*PP*ALF)/(SNORM**3*ALF**2) 
      XDEL=MIN(MAX(XPOM,GAM1*XDEL),GAM2*XDEL)
      ELSE
      XDEL=GAM1*XDEL
      ENDIF
      ELSE
      XDEL=GAM1*XDEL
      ENDIF
      ITERS=1
      TSXX='LARGE   '
      IF (MES3.LE.1) THEN
      NRED2=NRED2+1
      ELSE
      IF (ITERD.GT.2) NRED2=NRED2+1
      ENDIF
      ELSE IF (DF.LT.EPS4*DFPR) THEN
C
C     STEP IS SUITABLE
C
      ITERS=2
      TSXX='GOOD    '
      ELSE
C
C     STEP IS TOO SMALL, IT HAS TO BE ENLARGED
C
      IF (DF.LT.DFPR) THEN
      XDEL=MAX(BET2*XDEL,1.0D-14)
      ELSE IF (FX.LT.CON1) THEN
      XDEL=MAX(BET2*XDEL,1.0D-14)
      ELSE
      BET=CON2**(ONE/THREE)
      CALL UNCUR1(THREE*CON2*FX,PO,TWO*PP,THREE*DP,BET,ALF,INF)
      IF (INF.LE.0) THEN
      XDEL=MAX(BET1*XDEL,1.0D-14)
      ELSE IF (INF.LE.2) THEN
      ALF=ALF-BET
      IF (ALF.LE.CON4) THEN
      XPOM=CON2*XDEL/ALF**3
      XDEL=MAX(XPOM,1.0D-14)
      ELSE 
      XDEL=MAX(BET1*XDEL,1.0D-14)
      ENDIF
      ELSE
      ALF=ALF-BET
      IF (ALF.LE.CON4) THEN
      XPOM=XDEL+THREE*(FX/SNORM**3)*(CON2-ALF**3)/ALF**3
      XDEL=MAX(XPOM,1.0D-14)
      ELSE 
      XDEL=MAX(BET1*XDEL,1.0D-14)
      ENDIF
      ENDIF
      ENDIF
      ITERS=3
      TSXX='SMALL   '
      ENDIF
      IF (MES.EQ.0.AND.DF.LE.ZERO.OR.MES.EQ.1.AND.DF.LT.EPS3*DFPR)
     & THEN
      IF (NRED1.GE.MRED) THEN
      ITERS=-1
      TSXX='MAX  RED'
      CALL UOERR1('US0R01',5-ITERS)
      ELSE
      IDIR=1
      ITERS=0
      TSXX='ZERO  ST'
      NRED1=NRED1+1
      ENDIF
      ENDIF
      ENDIF
      MAXST=0
      IF (R*SNORM.GE.XMAX) MAXST=1
      CALL UOS0G3(R,F,FO,PO,PP,XDEL,GNORM)
      IF (MES3.EQ.0) THEN
      NRED=NRED1
      ELSE
      NRED=NRED2
      ENDIF
      CALL UOS0G2(R,F,XDEL)
      CALL UYEPI1(0)
      RETURN
      END

