$INTERFACE US1L09                                             04/12/01

  $HELP
    TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST
    ON THE CHANGE OF THE RESIDUAL FUNCTION VALUE)
  $CHECK(EPS4,'1.0$P-8',REAL,4)
  CALL US1L09(KT,$MT,$FT,KS,EPS4,R,F,FQ,RMAX,INITS)
$REM  $ADD(REAL,'\EPS4\PO\RO\R\FF\FC\FMIN\RMIN\RMAX\PAR')
$REM  $ADD(INTEGER,'\MODE\KTERS\INITS\MRED')
  $ADD(INIT)
    NSXX='US1L09'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE US1L09                ALL SYSTEMS                04/12/01
C PORTABILITY : ALL SYSTEMS
C 04/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  LINE SEARCH WITHOUT DIRECTIONAL DERIVATIVES BASED ON A SQP FILTER.
*
* PARAMETERS :
*  IU  KT  NUMBER OF LOCAL FILTER SAMPLES.
*  II  MT  MAXIMUM NUMBER OF LOCAL FILTER SAMPLES.
*  RU  FT(MT,2)  ARRAY CONTAINING LOCAL FILTER SAMPLES.
*  IU  KS  NUMBER OF FILTER SAMPLES.
*  RI  EPS4  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE RIGHT-HAND SIDE VECTOR).
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RO  FF  VALUE OF THE OBJECTIVE FUNCTION.
*  RO  FQ  L-INFINITY NORM OF THE RIGHT-HAND SIDE VECTOR.
*  RI  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  II  INITS  CHOICE OF THE INITIAL STEPSIZE. INITS=0-INITIAL STEPSIZE
*         IS SPECIFIED IN THE CALLING PROGRAM. INITS=1-UNIT INITIAL
*         STEPSIZE. INITS=2-COMBINED UNIT AND QUADRATICALLY ESTIMATED
*         INITIAL STEPSIZE. INITS=3-QUADRATICALLY ESTIMATED INITIAL
*         STEPSIZE.
*
* COMMON DATA :
*  IO  MAXST  MAXIMUM STEPSIZE INDICATOR. MAXST=0 OR MAXST=1 IF MAXIMUM
*         STEPSIZE WAS NOT OR WAS REACHED.
*  IO  ITERS  TERMINATION INDICATOR. ITERS=0-ZERO STEP. ITERS=1-PERFECT
*         LINE SEARCH. ITERS=2 GOLDSTEIN STEPSIZE. ITERS=3-WEAK WOLFE
*         STEPSIZE. ITERS=4-STRONG WOLFE STEPSIZE.
*         ITERS=5-ARMIJO STEPSIZE. ITERS=6-FIRST STEPSIZE.
*         ITERS=7-MAXIMUM STEPSIZE. ITERS=8-UNBOUNDED FUNCTION.
*         ITERS=-1-MRED REACHED. ITERS=-2-POSITIVE DIRECTIONAL
*         DERIVATIVE. ITERS=-3-ERROR IN INTERPOLATION.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAM USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UOS0F1  PRINT OF ENTRY TO STEPSIZE SELECTION.
*  S   UOS0F2  PRINT OF EXIT FROM STEPSIZE SELECTION.
*  S   UOS0F3  PRINT OF INFORMATION DURING STEPSIZE SELECTION.
*  S   UOERR1  ERROR MESSAGES.
*
* METHOD :
* SAFEGUARDED EXTRAPOLATION AND INTERPOLATION WITH EXTENDED TERMINATION
* CRITERIA.
*
      SUBROUTINE US1L09(KT,MT,FT,KS,EPS4,R,FF,FQ,RMAX,INITS)
      $INCLUDE ('UMCOMM')
      INTEGER KT,MT,KS,INITS
      $FLOAT FT(MT,2),EPS4,R,FF,FQ,RMAX
      INTEGER I,J
      $FLOAT FFO,FFP,FQO,FQP
      SAVE FFO,FFP,FQO,FQP
      $FLOAT CON
      PARAMETER(CON=1.2$P 0)
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',1)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      ITERS=2
C      CALL UOS0F1(RO,FF,FQ,SNORM,RMAX)
      IF(RMAX.LE.ZERO) THEN
      TSXX='RMAX=0'
      ITERS=0
      GO TO 10
      ENDIF
      FFO=FF
      FQO=FQ
C
C     INITIAL STEPSIZE SELECTION
C
      TSXX='INIT-1  '
      INITS=1
      MAXST=0
      R=ONE
      R=MIN(R,RMAX)
      IF (R.GE.RMAX) MAXST=1
C
C     COMPUTATION OF THE NEW FUNCTION VALUE
C
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
C      CALL UOS0F3(R,FF,FQ,SNORM,MODE)
      DO 4 I=1,KT
      IF (FF.GE.FT(I,1).AND.FQ.GE.FT(I,2)) GO TO 10
    4 CONTINUE
      IF (FQO-FQ.GE.EPS4*FQO) THEN
      KS=0
      FFP=FF-EPS4*FQO
      FQP=FQ-EPS4*FQO
      I=1
    5 IF (I.GT.KT) GO TO 7
      FFP=MIN(FFP,FT(I,1))
      IF (FT(I,2).GE.FQP) THEN
      DO 6 J=I,KT-1
      FT(J,1)=FT(J+1,1)
      FT(J,2)=FT(J+1,2)
    6 CONTINUE
      KT=KT-1
      ENDIF
      I=I+1
      GO TO 5
    7 IF (KT.GE.MT) THEN
      DO 8 J=1,KT-1
      FT(J,1)=FT(J+1,1)
      FT(J,2)=FT(J+1,2)
    8 CONTINUE
      KT=KT-1
      ENDIF
      KT=KT+1
      FT(KT,1)=FFP
      FT(KT,2)=FQP
      ITERS=8
      ENDIF
   10 INITS=0
C      CALL UOS0F2(R,FF,FC,SNORM)
      CALL UYEPI1(0)
      RETURN
      END
