$INTERFACE UULBI3                                             98/12/01

  $U4='U4'
  $IF(MET<5) $U4=U3
  $IF(MET=7) $U4=U3
  $HELP
    SELECTION OF SELF SCALING:
      1 - SELF SCALING SUPPRESSED
      2 - INITIAL SELF SCALING
  $CHECK(MET1,2,'1\2',4)
  $HELP
    SELECTION OF THE LINE SEARCH MODEL:
      MET2=1 - QUADRATIC MODEL
      MET2=2 - BACKWARD TAYLOR EXPANSION (3 TERMS)
      MET2=3 - BACKWARD TAYLOR EXPANSION (4 TERMS)
      MET2=4 - HOMOGENEOUS MODEL
  $CHECK(MET2,1,'1\2\3\4',4)
  $HELP
    REALIZATION FORM:
      1 - STRATEGY THAT USES 6*MF*NF OPERATIONS
      2 - STRATEGY THAT USES 4*MF*NF OPERATIONS
  $CHECK(MET5,1,'1\2',4)
    $ADD(REAL,'\$XM(2*$NF*$MF)\$XR($MF*($MF+1))\$HR($MF*(2*$MF+1))&
    \$XO($NF)\$G($NF)\$GO($NF)\$U1(2*$MF)\$U2(2*$MF)\$U3(2*$MF)&
    \$U4($MF)')
$REM  $ADD(REAL,'\PAR3')
$REM  $ADD(INTEGER,'\NF\MF')
  CALL UULBI3(NF,MF,MX,$XM,$XR,$HR,$XO,$G,$GO,$U1,$U2,$U3,$U4,R,F,FO&
,P,PO,ETA3,PAR3,MET,MET1,MET2,MET5)
  $ADD(INIT)
    NUXX='UULBI3'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UULBI3                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE VARIABLE METRIC UPDATE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  MF  MAXIMUM NUMBER OF VARIABLE METRIC STEPS.
*  IU  MM  ACTUAL NUMBER OF VARIABLE METRIC STEPS.
*  RU  XM(2*NF*MF)  SET OF DIFFERENCES.
*  RU  XR(MF*(MF+1))  MATRIX TRANS(XM)*XM STORED IN PACKED FORM.
*  RO  HR(MF*(2*MF+1))  MATRIX IN UPDATE REPRESENTATION.
*  RI  XO(NF)  VECTOR OF VARIABLES DIFFERENCE.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  GO(NF)  GRADIENT DIFFERENCE.
*  RU  U(2*MF)  VECTOR TRANS(XM)*G.
*  RU  UO(2*MF)  VECTOR TRANS(XM)*GO.
*  RA  V(2*MF)  AUXILIARY VECTOR.
*  RA  W(MF)  VECTOR CONTAINING NUMBERS TRANS(XO)*B*XO.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  INITIAL VALUE OF THE OBJECTIVE FUNCTION.
*  RO  P  VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  PO  INITIAL VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  PAR  SCALING PARAMETER.
*  RU  ETA PARAMETER ETA FOR VARIABLE METRIC UPDATES.
*  II  MET  METHOD SELECTION. MET=1-BFGS UPDATE. MET=2-DFP UPDATE.
*         MET=3-HOSHINO UPDATE. MET=4-RANK ONE UPDATE. MET=5-CONSTANT
*         PARAMETER UPDATE.
*  II  MET1  SCALING OPTION. MET1=1-NO SCALING. MET1=2-PRELIMINARY
*         SCALING.
*  II  MET5  REALIZATION FORM. MET5=1-THE USE OF 6*MF*NF OPERATIONS.
*         MET5=2-THE USE OF 4*MF*NF OPERATIONS.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXDPGT  TRANSFORMATION OF THE GILL-MURRAY DECOMPOSITION.
*  S   UXDSDP  GENERALIZED DOT PRODUCT OF TWO VECTORS.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UOERR1  ERROR MESAGES.
*
* METHOD :
* LIMITED STORAGE VARIABLE METRIC METHODS.
*
      SUBROUTINE UULBI3(NF,MF,MM,XM,XR,HR,XO,G,GO,U,UO,V,W,R,F,FO,P,PO,
     & ETA,PAR,MET,MET1,MET2,MET5)
      $INCLUDE('UMCOMM')
      INTEGER NF,MF,MM,MET,MET1,MET2,MET5
      $FLOAT XM(2*NF*MF),XR(MF*(MF+1)),HR(MF*(2*MF+1)),XO(NF),G(NF),
     & GO(NF),U(2*MF),UO(2*MF),V(2*MF),W(MF),R,F,FO,P,PO,ETA,PAR,DEN,
     & DIS,POM1,POM2
      INTEGER I,II,J,JJ,KK,LL
      $FLOAT UXVDOT
      $FLOAT A,B,C,RHO,AI,BI,CI,RHOI,HR1,HR2,HR3
      $FLOAT CON4
      PARAMETER (CON4=1.0$P-2)
      ITERH=0
      A=UXVDOT(NF,GO,GO)
      B=UXVDOT(NF,XO,GO)
      C=-R*PO
      IF (B.LE.ZERO) THEN
      ITERH=2
      TUXX='B - NEG.'
      CALL UOERR1('UULBI3',1)
      GO TO 6
      ENDIF
      IF (MET1.EQ.1) THEN
      PAR=ONE
      ELSE
      PAR=B/A
      ENDIF
      IF (MET2.EQ.1) THEN
      RHO=ONE
      ELSE
      POM1=(F-FO)/PO
      POM2=P/PO
      IF (MET2.EQ.2) THEN
      IF (POM1.EQ.POM2.OR.POM2.EQ.ONE) THEN
      RHO=ONE
      ELSE
      RHO=HALF*(POM2-ONE)/(POM2-POM1)
      ENDIF
      ELSE IF (MET2.EQ.3) THEN
      IF (TWO*(POM2-ONE)-THREE*(POM1-ONE).EQ.ZERO) THEN
      RHO=ONE
      ELSE
      RHO=HALF*(POM2-ONE)/(TWO*(POM2-ONE)-THREE*(POM1-ONE))
      ENDIF
      ELSE IF (MET2.EQ.4) THEN
      IF (POM1.EQ.ONE.OR.POM2.EQ.POM1) THEN
      RHO=ONE
      ELSE
      RHO=(POM1-ONE)/(POM2-POM1)
      ENDIF
      ENDIF
      IF(RHO.LE.CON4) RHO=ONE
      IF(RHO*CON4.GE.ONE) RHO=ONE
      ENDIF
      IF (MM.GE.MF) THEN
      II=2*NF
      DO 1 I=1,2*NF*(MM-1)
      XM(I)=XM(I+II)
    1 CONTINUE
      II=2
      JJ=1
      DO 2 I=1,MM-1
      J=JJ+II
      CALL UXVCOP(II,XR(J+2),XR(JJ))
      JJ=J
      IF (MET5.NE.1) THEN
      UO(II-1)=UO(II+1)
      UO(II)=UO(II+2)
      ENDIF
      W(I)=W(I+1)
      II=II+2
    2 CONTINUE
      II=2*(MM-1)*NF+1
      JJ=JJ-1
      ELSE
      II=2*MM*NF+1
      JJ=MM*(MM+1)
      MM=MM+1
      ENDIF
      CALL UXVCOP(NF,XO,XM(II))
      CALL UXVCOP(NF,GO,XM(II+NF))
      IF (MET5.EQ.1) THEN
      CALL UXDRMM(NF,2*MM-2,XM,GO,XR(JJ+1))
      ELSE
      CALL UXDRMM(NF,2*MM,XM,G,U)
      DO 3 I=1,2*MM-2
      XR(JJ+I)=U(I)-UO(I)
    3 CONTINUE
      ENDIF
      JJ=MM*(MM+1)
      XR(JJ)=A
      XR(JJ-1)=B
      W(MM)=C
      II=1
      JJ=0
      KK=0
      LL=0
      DO 5 I=1,MM
      IF (KK.NE.0) THEN
      CALL UXVCOP(LL,XR(II),UO)
      IF (MET1.GT.1) THEN
      DO 4 J=1,I-1
      UO(2*J)=PAR*UO(2*J)
    4 CONTINUE
      ENDIF
      CALL UXDSMM(LL,HR,UO,V)
      ENDIF
      AI=PAR*XR(JJ+2)+UXVDOT(LL,UO,V)
      BI=XR(JJ+1)
      CI=W(I)
      RHOI=ONE
      IF (I.GE.MM) RHOI=RHO
      IF (MET.EQ.1) THEN
      ETA=ONE
      TUXX='BFGS    '
      ELSE IF (MET.EQ.2) THEN
      ETA=ZERO
      TUXX='DFP     '
      ELSE IF (MET.EQ.3) THEN
      ETA=RHOI*BI/(RHOI*BI+AI)
      TUXX='HOSHINO '
      ELSE IF (MET.EQ.4) THEN
      ETA=ONE
      TUXX='BFGS    '
      IF (RHOI*BI.GT.AI) THEN
      ETA=RHOI*BI/(RHOI*BI-AI)
      TUXX='RANK ONE'
      ENDIF
      ELSE IF (MET.EQ.5) THEN
      ETA=ONE
      TUXX='BFGS    '
      IF (CI.GT.ZERO) THEN
      DEN=RHOI*BI-AI
      DIS=RHOI*CI-BI
      IF (RHOI*CI*DEN.LE.AI*DIS) THEN
      DEN=(AI/BI)*(CI/BI)-ONE
      IF (DEN.NE.ZERO) THEN
      ETA=(DIS/BI)/DEN
      IF (ETA.NE.ZERO) THEN
      TUXX='DAVIDON '
      ENDIF
      ENDIF
      ELSE
      IF (DEN.NE.ZERO) THEN
      ETA=RHOI*BI/DEN
      TUXX='RANK ONE'
      ENDIF
      ENDIF
      ENDIF
      ELSE IF (MET.EQ.6) THEN
      ETA=ONE
      TUXX='BFGS    '
      IF (CI.NE.ZERO) THEN
      ETA=ONE+ONE/SQRT(MAX(1.0D-120,ONE-BI*BI/(AI*CI)))
      TUXX='PRECON 1'
      ENDIF
      ELSE IF (MET.EQ.7) THEN
      IF (ETA.LE.ZERO) THEN
      ETA=ONE
      TUXX='BFGS    '
      ELSE IF (ETA.EQ.TWO) THEN
      TUXX='PRECON 2'
      ELSE
      TUXX='GENERAL '
      ENDIF
      ELSE IF (MET.EQ.8) THEN
      ETA=ONE
      TUXX='BFGS    '
      IF (AI*CI.GT.ZERO) THEN
      DEN=BI*BI/(AI*CI)
      IF (DEN.LT.ONE) THEN
      ETA=MAX(1.0D-15,(SQRT(CI/AI)-DEN)/(ONE-DEN))
      TUXX='NEW     '
      ENDIF
      ENDIF
      ENDIF
      HR1=(ETA*AI/BI+RHOI)/BI
      HR2=-ETA/BI
      HR3=(ETA-ONE)/AI
      HR(KK+2*I-1)=HR1
      HR(KK+4*I-2)=HR2
      HR(KK+4*I-1)=HR3
      IF (KK.NE.0) THEN
      CALL UXDSMU(LL,HR,HR3,V)
      CALL UXVSCL(LL,HR2,V,HR(KK+1))
      CALL UXVSCL(LL,HR3,V,HR(KK+2*I))
      ENDIF
      II=II+2*I
      JJ=JJ+2*I+2
      KK=KK+4*I-1
      LL=LL+2
    5 CONTINUE
      IF (MET5.NE.1) CALL UXVCOP(2*MM,U,UO)
      NUPD=NUPD+1
    6 CONTINUE
      RETURN
      END
