$INTERFACE UULBM1                                             98/12/01

  $HELP
    SELECTION OF SELF SCALING:
      1 - SELF SCALING SUPPRESSED
      2 - INITIAL SELF SCALING WITH THE NORM OF D
      3 - INITIAL SELF SCALING WITH THE NORM OF Y
  $CHECK(MET1,2,'1\2\3',4)
  $ADD(REAL,'\$XM($NF*$MF)\$GM($NF*$MF)\$XR($MF*($MF+1)/2)&
  \$GR($MF*($MF+1)/2)\$HR($MF*($MF+1)/2)\$DR($MF)\$XO($NF)&
  \$GO($NF)')
$REM  $ADD(REAL,'\PAR3')
$REM  $ADD(INTEGER,'\NF\MF\INF')
  CALL UULBM1(NF,MF,MX,$XM,$GM,$XR,$GR,$HR,$DR,$XO,$GO,PAR3,ETA0,INF&
,MET,MET1)
  $ADD(INIT)
    NUXX='UULBM1'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UULBM1                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE VARIABLE METRIC UPDATE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  MF  MAXIMUM NUMBER OF VARIABLE METRIC STEPS.
*  IU  MM  ACTUAL NUMBER OF VARIABLE METRIC STEPS.
*  RU  XM(NF*MF)  SET OF VECTOR OF VARIABLES DIFFERENCES.
*  RU  GM(NF*MF)  SET OF GRADIENT DIFFERENCES.
*  RU  XR(MF*(MF+1)/2)  MATRIX TRANS(XM)*XM STORED IN PACKED FORM.
*  RU  GR(MF*(MF+1)/2)  MATRIX TRANS(GM)*XM STORED IN PACKED FORM.
*  RO  HR(MF*(MF+1)/2)  MATRIX IN UPDATE REPRESENTATION.
*  RO  DR(MF)  MODIFIED DIAGONAL OF THE MATRIX GR.
*  RU  XO(NF)  VECTOR OF VARIABLES DIFFERENCE.
*  RI  GO(NF)  GRADIENT DIFFERENCE.
*  RO  PAR3  SCALING PARAMETER.
*  RI  ETA0  MACHINE PRECISION.
*  IU  INF  DECOMPOSITION INDICATOR. INF=0 IF THE MATRIX IS POSITIVE
*         DEFINITE.
*  II  MET  METHOD SELECTION. MET=1-BFGS UPDATE. MET=4-RANK ONE UPDATE.
*  II  MET1  SCALING OPTION. MET1=1-NO SCALING. MET1=2-PRELIMINARY
*         SCALING.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXDPGF  GILL-MURRAY DECOMPOSITION OF A DENSE SYMMETRIC MATRIX
*         WITH CONTROLLING OF POSITIVE DEFINITENESS.
*  S   UXDPGT  TRANSFORMATION OF THE GILL-MURRAY DECOMPOSITION.
*  S   UXDSDP  GENERALIZED DOT PRODUCT OF TWO VECTORS.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UOERR1  ERROR MESAGES.
*
* METHOD :
* LIMITED STORAGE VARIABLE METRIC METHODS.
*
      SUBROUTINE UULBM1(NF,MF,MM,XM,GM,XR,GR,HR,DR,XO,GO,PAR3,ETA0,INF,
     & MET,MET1)
      $INCLUDE('UMCOMM')
      INTEGER NF,MF,MM,INF,MET,MET1
      $FLOAT XM(NF*MF),GM(NF*MF),XR(MF*(MF+1)/2),GR(MF*(MF+1)/2),
     & HR(MF*(MF+1)/2),DR(MF),XO(NF),GO(NF),PAR3,ETA0
      INTEGER I,II,I1,I2,J,JJ,J1,J2,KK
      $FLOAT UXVDOT,UXDSDP
      $FLOAT ALF,BET
C      MM=MIN(NIT-KIT,MF)
      IF (MM.GE.MF) THEN
      I1=1
      J1=1
      DO 1 II=1,MM-1
      I2=I1+NF
      J2=J1+II
      CALL UXVCOP(NF,XM(I2),XM(I1))
      CALL UXVCOP(NF,GM(I2),GM(I1))
      CALL UXVCOP(II,XR(J2+1),XR(J1))
      CALL UXVCOP(II,GR(J2+1),GR(J1))
      I1=I2
      J1=J2
    1 CONTINUE
      J1=J1-1
      ELSE
      I1=MM*NF+1
      J1=MM*(MM+1)/2
      MM=MM+1
      ENDIF
      CALL UXVCOP(NF,XO,XM(I1))
      CALL UXVCOP(NF,GO,GM(I1))
      I1=1
      DO 2 II=1,MM
      XR(J1+II)=UXVDOT(NF,XM(I1),XO)
      GR(J1+II)=UXVDOT(NF,GM(I1),XO)
      I1=I1+NF
    2 CONTINUE
      IF (GR(J1+MM).LE.ZERO) THEN
      ITERH=2
      TUXX='B - NEG.'
      CALL UOERR1('UULBM1',1)
      GOTO 6
      ENDIF
      PAR3=ONE
      IF (MET1.EQ.2) PAR3=GR(J1+MM)/XR(J1+MM)
      IF (MET1.EQ.3) PAR3=UXVDOT(NF,GO,GO)/GR(J1+MM)
      IF (MET.EQ.4) THEN
      CALL UXVDIR(J1+MM,-PAR3,XR,GR,HR)
      ELSE
      CALL UXVSCL(J1+MM,PAR3,XR,HR)
      II=2
      KK=2
      DO 4 I=2,MM
      KK=KK+1
      JJ=2
      DO 3 J=2,I
      HR(KK)=HR(KK)+UXDSDP(J-1,GR,ZERO,GR(II),GR(JJ))
      KK=KK+1
      JJ=JJ+J
    3 CONTINUE
      II=II+I
    4 CONTINUE
      ENDIF
      ALF=ETA0
      CALL UXDPGF(MM,HR,INF,ALF,BET)
      CALL UXDPGT(MM,HR)
      IF (MET.NE.4) THEN
      II=0
      DO 5 I=1,MM
      II=II+I
      DR(I)=ONE/GR(II)
    5 CONTINUE
      ENDIF
      TUXX='BFGS    '
      NUPD=NUPD+1
      ITERH=0
    6 CONTINUE
      RETURN
      END
