* SUBROUTINE UULBV1                ALL SYSTEMS                31/12/13
C PORTABILITY : ALL SYSTEMS
C 31/12/13 VL :                                       ORIGINAL VERSION
*
* PURPOSE :
* CORRECTION TRANSFORMATION OF USED VECTORS AND MATRICES.
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF  MATRICES XM, GM.
*  IU  M  NUMBER OF COLUMNS OF MATRICES XM, GM.
*  II  MM  MAXIMUM NUMBER OF COLUMNS OF MATRICES XM, GM.
*  RU  XR(MM,MM) MATRIX  TRANS(XM)*GM.
*  RU  GR(M*(M+1)/2)  SYMMETRIC MATRIX TRANS(GM)*GM STORED COLUMNWISE.
*  RU  XM(N*MM)  TRANSFORMED RECTANGULAR MATRIX OF VARIABLES DIFFERENCES.
*  RU  GM(N*MM)  TRANSFORMED RECTANGULAR MATRIX OF GRADIENTS DIFFERENCES.
*  RU  RV(M*(M+1)/2)  UPPER TRIANGULAR PART OF XR STORED COLUMNWISE.
*  RU  DV(M)  DIAGONAL OF MATRIX XR.
*  RA  WW(6*MM)  AUXILIARY ARRAY.
*  IA  IND(M)  AUXILIARY ARRAY. 
*  IA  IC(M)  AUXILIARY ARRAY. 
*  IA  IN(M)  AUXILIARY ARRAY. 
*  IA  INC(MM+M)  AUXILIARY ARRAY. 
*  RI  B  INPUT PARAMETER B = TRANS(XO)*GO.
*  RI  AH  INPUT PARAMETER  AH = TRANS(GO)*GO.
*  RI  C  INPUT PARAMETER  C = -R*TRANS(XO)*(G-GO)  (FROM LINE SEARCH).
*  II  MC  MAXIMUM NUMBER OF CORRECTION VECTORS.
*
* SUBPROGRAMS USED :
*  S   UULBV2  COMPUTATION OF TRANS(GO)*HH*GO, WHERE HH IS THE LAST 
*         BUT ONE VARIABLE METRIC AUXILIARY MATRIX.
*  S   UXDCMW  TWO VECTORS MULTIPLIED BY COLUMNWISE STORED RECTANGULAR
*         MATRICES AND ADDED TO ANOTHER TWO VECTORS.
*  S   UXDSMM  MULTIPLICATION OF A DENSE SYMMETRIC MATRIX BY A VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVSUM  ADDITION OF TWO VECTORS.
*  S   UXVSET  A SCALAR IS SET TO ALL ELEMENTS OF A VECTOR.
*
* METHOD :
* 
      SUBROUTINE UULBV1(N,M,MM,XR,GR,XM,GM,RV,DV,WW,IND,IC,IN,INC,B,
     & AH,C,MC)
      INTEGER N,M,MM,MC,IND(M),IC(M),IN(M),INC(MM+M)
      REAL*8 XR(MM,MM),GR(M*(M+1)/2),XM(N*MM),GM(N*MM),RV(M*(M+1)/2),
     & DV(M),WW(MM*6),B,C,AH
      REAL*8 BV,BVP,DELQ,Q,W,A,AV,CV,UXVDOT
      INTEGER J,K,L
      L=M*(M-1)/2
      DV(M)=B
      DO J=1,M
        RV(L+J)=XR(J,M)
        IC(J)=J
        IN(J)=J
        INC(J)=1
      ENDDO
C      
C     COMPUTATION OF A = TRANS(GO)*HH*GO
C
      CALL UULBV2(M,MM,XR,GR,RV,DV,WW(3*MM+1),B/AH,AH,A)
      AV=A
      BVP=B
      CV=C
      DO J=M-1,1,-1
        INC(J)=IND(J)
        IF(J.LT.M-MC)INC(J)=0
        Q=XR(J,M)*XR(M,J)/XR(J,J)
        DELQ=(XR(J,M)-XR(M,J))**2/(B*XR(J,J))
        IF (DELQ.GT.1.0$P-2) INC(J)=0
        IF (J.LE.M-1.AND.AV-XR(J,M)**2/XR(J,J)*INC(J).LE.B*1.0$P-5)
     &   INC(J)=0
        IF (J.LE.M-1.AND.CV-XR(M,J)**2/XR(J,J)*INC(J).LE.B*1.0$P-3)
     &   INC(J)=0
        BV=BVP
        BVP=BVP-Q*INC(J)
        IF (BVP.LT.B*1.0$P-4) INC(J)=0
        W=ABS(BV-AV)/BV*(B/BV-1.0$P 0)
        IF (J.LT.M-1.AND.DELQ+2*(Q/B).LT.1.0$P-10) INC(J)=0
        IF (J.LT.M-1.AND.W.LT.1.0$P 0.AND.DELQ.GT.1.0$P-5) INC(J)=0
        IF (J.LT.M-1.AND.DELQ.GT.MIN(1.0$P-2,(1-BVP/B)**4/2+1.0$P-5)) 
     &   INC(J)=0
        IF(INC(J).EQ.0)BVP=BV
        AV=AV-XR(J,M)**2/XR(J,J)*INC(J)
        CV=CV-XR(M,J)**2/XR(J,J)*INC(J)
      ENDDO
      DO J=1,M
        IF (INC(J).EQ.1) IN(IC(J))=J
        IF (J.LT.M) IC(J+1)=IC(J)+INC(J)
      ENDDO
      CALL UXVSET(M,0.0$P 0,WW)
      CALL UXVSET(M,0.0$P 0,WW(M+1))
      IF (IC(M).LE.1) RETURN
      BV=B
      DO J=1,IC(M)-1
        K=IN(J)
        WW(K)=-XR(K,M)/DV(K)
        WW(M+K)=-XR(M,K)/DV(K)
      ENDDO
      DELQ=GR(L+M)
      DO J=1,M-1
        Q=XR(J,M)
        DO K=1,M-1
          Q=Q+XR(J,K)*WW(K)
          XR(M,J)=XR(M,J)+XR(K,J)*WW(M+K)
        ENDDO
        XR(J,M)=Q
        RV(L+J)=Q
        DELQ=DELQ+WW(J)*GR(L+J)*2.0$P 0
      ENDDO
      IF(M.GT.1)THEN
        CALL UXDSMM(M-1,GR,WW,WW(2*M+1))
        W=UXVDOT(M-1,WW,WW(2*M+1))
        CALL UXVSUM(M-1,WW(2*M+1),GR(L+1),GR(L+1))
        GR(L+M)=DELQ+W
C        
C     COMPUTATION OF XO, GO, BV
C
        CALL UXDCMW(N,M-1,XM,GM,WW(M+1),WW,XM(M*N-N+1),GM(M*N-N+1))
      ENDIF
      BV=0.0$P 0
      L=M*N
      DO J=L-N+1,L
        BV=BV+XM(J)*GM(J)
      ENDDO
      IF (BV.LT.BVP*0.5$P 0)BV=BVP
      RV(M*(M+1)/2)=BV
      DV(M)=BV
      XR(M,M)=BV
      RETURN
      END
