* SUBROUTINE UULBV4                ALL SYSTEMS                31/12/16
C PORTABILITY : ALL SYSTEMS
C 31/12/16 VL :                                       ORIGINAL VERSION
*
* PURPOSE :
* UPDATE OF USED VECTORS AND MATRICES.
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF  MATRICES XM, GM.
*  IU  M  NUMBER OF COLUMNS OF MATRICES XM, GM.
*  II  MM  MAXIMUM NUMBER OF COLUMNS OF MATRICES XM, GM.
*  RU  XR(MM,MM)  MATRIX  XM^T . GM .
*  RU  GR(MM*(MM+1)/2)  SYMMETRIC MATRIX  GM^T . GM  STORED COLUMNWISE.
*  RU  XM(N*MM)  RECTANGULAR MATRIX OF VARIABLES DIFFERENCES.
*  RU  GM(N*MM)  RECTANGULAR MATRIX OF GRADIENTS DIFFERENCES.
*  RI  G(N)  NEW GRADIENT.
*  RU  S(N)  VECTOR OF VARIABLES DIFFERENCE.
*  RI  Y(N)  VECTOR OF GRADIENTS DIFFERENCE.
*  RU  SG(MM)  VECTOR   XM^T . G .
*  RU  YG(MM)  VECTOR   GM^T . G .
*  RU  WW(3*MM)  AUXILIARY VECTOR.
*  RI  A  INPUT PARAMETER  A = Y^T . Y .
*  RI  B  INPUT PARAMETER  B = S^T . Y .
*  RI  R  INPUT STEPSIZE PARAMETER.
*
* SUBPROGRAMS USED :
*  S  UXDRMM  MULTIPLICATION OF TRANSPOSED RECTANGULAR MATRICES  XM, GM
*             BY VECTOR.
*
      SUBROUTINE UULBV4(N,M,MM,XR,GR,XM,GM,G,S,Y,SG,YG,WW,A,B,R)
      INTEGER M,MM,N,I,J,K,L
      $FLOAT G(N),S(N),SG(MM),XM(N*MM),XR(MM*MM),Y(N),YG(MM)
      $FLOAT GR(MM*(MM+1)/2),GM(N*MM),WW(MM*3),A,B,R
c  ***  Shift   M, XM, GM, XR, GR
      IF(M.GE.MM)THEN
        L=M*N-N
        DO I=1,L
          XM(I)=XM(I+N)
          GM(I)=GM(I+N)
        END DO
        DO I=1,M-1
          L=I*(I-1)/2+1
          DO J=1,I
            GR(L)=GR(L+I+1)
            L=L+1
          END DO
        END DO
        DO I=1,M-1
          DO J=1,M-1
            XR((J-1)*MM+I)=XR(J*MM+I+1)
          END DO
        END DO
        K=1
      ELSE
        M=M+1
        K=0
      END IF
c  ***  Update  XM, GM
      DO I=1,M+K-1
c  nasled.radek:  Y^T.S(I)=-T*WW(I+K)   ("zrychleny vypocet")
        WW(2*M+I)=-R*WW(I+K)
        WW(I)=SG(I)
        WW(I+M)=YG(I)
      END DO
      L=M*N-N
      DO I=1,N
        XM(L+I)=S(I)
        GM(L+I)=Y(I)
      END DO
c  ***  Comp of SG=XM^T*G, YG=GM^T*G, GM^T*S (viz vyse), update  XR, GR
      IF(M.GT.1)CALL UXDRMM(N,M-1,XM,G,SG)
      IF(M.GT.1)CALL UXDRMM(N,M-1,GM,G,YG)
      L=M*(M-1)/2
      DO I=1,M-1
        XR(M+I*MM-MM)=WW(I+M+M)
        XR(I+M*MM-MM)=SG(I)-WW(I+K)
        GR(L+I)=YG(I)-WW(I+M+K)
      END DO
      XR(M+M*MM-MM)=B
      GR(L+M)=A
      CALL UXDRMM(N,1,XM(M*N-N+1),G,SG(M))
      CALL UXDRMM(N,1,GM(M*N-N+1),G,YG(M))
      RETURN
      END
