$INTERFACE UULSP1                                             02/12/01

  $GR='GR'
  $HELP
    METHOD OF UPDATE OF HESSIAN MATRIX:
      1 - BFGS UPDATE
      2 - DFP UPDATE
      3 - HOSHINO UPDATE
      4 - SAFEGUARDED RANK ONE UPDATE
      5 - VARIATIONALLY DERIVED PRECONVEX UPDATE
  $CHECK(MET,1,'1\2\3\4\5\6',3)
  $HELP
    SELECTION OF SELF SCALING:
      ABS(MET1)=1 - SELF SCALING SUPPRESSED
      ABS(MET1)=2 - INITIAL SELF SCALING
      ABS(MET1)=3 - CONTROLLED SELF SCALING
      ABS(MET1)=4 - SIMPLIFIED CONTROLLED SELF SCALING
      ABS(MET1)=5 - SELF SCALING IN EACH ITERATION
           MET1>0 - BASIC INITIAL SELF SCALING
           MET1<0 - MODIFIED INITIAL SELF SCALING
  $CHECK(MET1,1,'-5\-4\-3\-2\-1\1\2\3\4\5',4)
  $NET1=MET1
  $IF(NET1=-1) $NET1=1
  $IF(NET1=-2) $NET1=2
  $IF(NET1=-3) $NET1=3
  $IF(NET1=-4) $NET1=4
  $IF(NET1=-5) $NET1=5
  $IF($DEF(MET3DEF))
  $ELSE
    $MET3DEF='MET3'
    $HELP
      COMPUTATION OF THE WEIGHTING PARAMETER:
        0 - PREVIOUS CHOICE
        1 - SIMPLE CHOICE WITH A CONSTANT RELATIVE PARAMETER
        2 - USING QUADRATIC EQUATION
        3 - HEURISTIC CHOICE WITH A RELATIVE PARAMETER LESS
            THEN HALF
        4 - HEURISTIC CHOICE WITH THE OPTIMUM CONDITION NUMBER
            IN THE FIRST ITERATION (FIRST FORMULA)
        5 - HEURISTIC CHOICE WITH THE OPTIMUM CONDITION NUMBER
            IN THE FIRST ITERATION (SECOND FORMULA)
        6 - HEURISTIC CHOICE DERIVED FROM THE LOWEST EIGENVALUE
    $CHECK(MET3,4,'0\1\2\3\4\5\6',4)
  $ENDIF
  $ADD(REAL,'\$XM($NF*$MF)\$GR($MF)\$XO($NF)\$GO($NF)')
$REM  $ADD(INTEGER,'\NF\MF\INF')
  CALL UULSP1(NF,NN,MF,$XM,$GR,$XO,$GO,R,F,FO,P,PO,PAR&
,PAR1,PAR2,$CON1,MET,MET1,1,$MET3DEF,1)
  $ADD(INIT)
    NUXX='UULSP1'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UULSP1                ALL SYSTEMS                02/12/01
C PORTABILITY : ALL SYSTEMS
C 02/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE VARIABLE METRIC UPDATE.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES (NUMBER OF ROWS OF XM).
*  II  M  NUMBER OF COLUMNS OF XM.
*  II  MF  MAXIMUM NUMBER OF COLUMNS OF XM.
*  RI  XM(N*M)  RECTANGULAR MATRIX IN THE PRODUCT FORM SHIFTED BROYDEN
*         METHOD (STORED COLUMNWISE): H-SIGMA*I=XM*TRANS(XM)
*  RO  GR(M)  MATRIX TRANS(XM)*GO.
*  RU  XO(N)  VECTORS OF VARIABLES DIFFERENCE XO AND VECTOR XO-TILDE.
*  RU  GO(N)  GRADIENT DIFFERENCE GO AND VECTOR XM*TRANS(XM)*GO.
*  RI  R  STEPSIZE PARAMETER.
*  RI  F  NEW FUNCTION VALUE.
*  RI  FO  OLD FUNCTION VALUE.
*  RI  P  NEW DIRECTIONAL DERIVATIVE (MULTIPLIED BY R)
*  RI  PO  OLD DIRECTIONAL DERIVATIVE (MULTIPLIED BY R)
*  RU  SIG  SCALING PARAMETER (ZETA AND SIGMA).
*  RI  PAR1  PARAMETER FOR SCALING.
*  RI  PAR2  PARAMETER FOR SCALING.
*  II  MET1  SCALING OPTION. MET1=1-NO SCALING.
*  II  MET2  CORRECTION OPTION. MET1=1-NO CORRECTION.
*  II  MET3  CHOICE OF SIGMA (1-CONSTANT, 2-QUADRATIC EQUATION).
*  II  MET4  SCALING OPTION. MET4=1 - RHO/GAM=A/B.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXDRMM  MULTIPLICATION OF A ROWWISE STORED DENSE RECTANGULAR
*         MATRIX BY A VECTOR.
*  S   UXDCMU  UPDATE OF A COLUMNWISE STORED DENSE RECTANGULAR MATRIX.
*         WITH CONTROLLING OF POSITIVE DEFINITENESS.
*  S   UXVDIR  VECTOR AUGMENTED BY A SCALED VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*
* METHOD :
* SHIFTED BFGS METHOD IN THE PRODUCT FORM.
*
      SUBROUTINE UULSP1(N,M,MF,XM,GR,XO,GO,R,F,FO,P,PO,SIG,
     & PAR1,PAR2,CON,MET,MET1,MET2,MET3,MET4)
      $INCLUDE('UMCOMM')
      INTEGER N,M,MF,MET,MET1,MET2,MET3,MET4
      $FLOAT XM(N*MF),GR(MF),XO(N),GO(N),
     & R,F,FO,P,PO,SIG,PAR1,PAR2,CON
      $FLOAT DEN,DIS,POM,A,B,C,AA,AH,BB,GAM,RHO,PAR,UXVDOT
      INTEGER METS
      IF (MET.LE.0.OR.M.GE.MF) RETURN
      CALL UOU1D1(N,XO,GO)
      METS=MET
      IF (MET.GT.5) METS=1
      B=UXVDOT(N,XO,GO)
      IF (B.LE.ZERO) THEN
      ITERH=2
      TUXX='B - NEG.'
      GO TO 22
      ENDIF
      CALL UXDRMM(N,M,XM,GO,GR)
      AH=UXVDOT(N,GO,GO)
      AA=UXVDOT(M,GR,GR)
      A=AA+AH*SIG
      C=-R*PO
C
C     DETERMINATION OF THE PARAMETER RHO (NONQUADRATIC PROPERTIES)
C
      CALL UURHO1(F,FO,P,PO,B,RHO,MET2)
C
C     DETERMINATION OF THE PARAMETER GAM (SELF SCALING)
C
      CALL UUGAM1(A,B,C,RHO,GAM,PAR,PAR1,PAR2,MET1,MET4)
C
C     DETERMINATION OF THE PARAMETER SIG (SHIFT)
C
      CALL UUSIG1(N,XO,A,B,C,AA,AH,PAR,SIG,SIG,CON,MET3)
C
C     COMPUTATION OF SHIFTED XO AND SHIFTED B
C
      BB=B-AH*SIG
      CALL UXVDIR(N,-SIG,GO,XO,XO)
      IF (RHO.NE.ONE) SIG=RHO*SIG
      IF (M.EQ.0.OR.AA.LE.ZERO) THEN
C
C     BFGS-BASED SHIFTED BFGS UPDATE
C
      POM=ONE
      TUXX='BFGS    '
      NUPD=NUPD+1
      CALL UXDCMU(N,M,XM,-ONE/BB,XO,GR)
      CALL UXVSCL(N,SQRT(PAR/BB),XO,XM(N*M+1))
      GO TO 21
      ENDIF
C
C     BFGS-BASED SHIFTED UPDATES
C
      GO TO (1,2,3,4,5) METS
C
C     BFGS UPDATE
C
    1 POM=ONE
      TUXX='BFGS    '
      NUPD=NUPD+1
      CALL UXDCMU(N,M,XM,-ONE/BB,XO,GR)
      CALL UXVSCL(N,SQRT(PAR/BB),XO,XM(N*M+1))
      GO TO 21
C
C     DFP UPDATE
C
    2 POM=ZERO
      TUXX='DFP     '
      NUPD=NUPD+1
      GO TO 20
C
C     HOSHINO UPDATE
C
    3 DEN=PAR*BB+AA
      DIS=HALF*BB
      POM=PAR*BB/DEN
      TUXX='HOSHINO '
      NUPD=NUPD+1
      GO TO 20
C
C     SAFEGUARDED RANK ONE UPDATE
C
    4 DEN=PAR*BB-AA
      IF (DEN.LE.ZERO) THEN
      POM=ONE
      TUXX='BFGS    '
      CALL UXDCMU(N,M,XM,-ONE/BB,XO,GR)
      CALL UXVSCL(N,SQRT(PAR/BB),XO,XM(N*M+1))
      GO TO 21
      ELSE
      POM=PAR*BB/DEN
      TUXX='RANK ONE'
      NUPD=NUPD+1
      GO TO 20
      ENDIF
C
C     VARIATIONALLY DERIVED PRECONVEX UPDATE
C
    5 POM=TWO
      TUXX='VAR-2   '
      NUPD=NUPD+1
   20 CONTINUE
C
C     GENERAL UPDATE
C
      DEN=SQRT(POM)
      CALL UXDCMD(N,M,XM,GR,DEN*AA/(ONE-DEN)/BB,XO,GO)
      CALL UXDCMU(N,M,XM,-(ONE-DEN)/AA,GO,GR)
      CALL UXVSCL(N,SQRT(PAR/BB),XO,XM(N*M+1))
   21 M=M+1
      IF (GAM.EQ.ONE) GO TO 22
C
C     SCALING
C
      CALL UXVSCL(N*M,SQRT(GAM),XM,XM)
      NSCL=NSCL+1
   22 CONTINUE
      ITERH=0
C      MET=METS
C      CALL UOU1D3(N,H,S,RHO,GAM,PAR,A,B,C,POM,ETA9)
      RETURN
      END

