$INTERFACE UUSSD2                                             97/12/01

  $HELP
    CHOICE OF THE INITIAL APPROXIMATION OF THE HESSIAN
    MATRIX:
      1 - UNIT INITIAL APPROXIMATION
      2 - RANDOM INITIAL APPROXIMATION
  $CHECK(INITH,1,'1\2',4)
  $DEFAULT='5.0D-4'
  $IF(TYPE='L') $DEFAULT='2.0D-2'
  $HELP
    TOLERANCE FOR APPLICATION OF THE BFGS METHOD
  $CHECK(ETA5,DEFAULT,REAL,4)
  $N='NF'
  $ADD(REAL,'\$H($M)\$AH($MAH)')
$REM  $ADD(REAL,'\F\FO\ETA5')
  $ADD(INTEGER,'\$IA($NA)\$IH($NF+1)\$JH($M)\$IAG($NA+1)\$JAG($MA)')
$REM  $ADD(INTEGER,'\$N\NA\M\MA\MAH')
  CALL UUSSD2($N,NA,M,MA,MAH,$IA,$H,$IH,$JH,$AH,$IAG,$JAG,F,FO,ETA5&
,$INITH)
  $ADD(SINIT)
    CALL UXVINS(NA,3,$IA)
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UUSSD2                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* INITIATION OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE HESSIAN MATRIX.
*  II  MB  NUMBER OF NONZERO ELEMENTS OF THE PARTITIONED HESSIAN
*         MATRIX.
*  II  IA(NA)  TYPES OF APPROXIMATED FUNCTIONS.
*  RU  H(M)  POSITIVE DEFINITE APPROXIMATION OF THE SPARSE HESSIAN
*         MATRIX
*  II  IH(NF+1)  POINTERS OF THE DIAGONAL ELEMENTS OF THE SPARSE
*         HESSIAN MATRIX.
*  II  JH(M)  INDICES OF THE NONZERO ELEMENTS OF THE SPARSE HESSIAN
*             MATRIX IN THE PACKED ROW FORM.
*  RU  AH(MB)  POSITIVE DEFINITE APPROXIMATION OF THE PARTITIONED
*         HESSIAN MATRIX.
*  II  IAG(NA+1)  POINTERS OF THE SPARSE JACOBIAN MATRIX.
*  II  JAG(MA)  COLUMN INDICES OF THE SPARSE JACOBIAN MATRIX.
*  RI  F  CURRENT VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RI  ETA5  TOLERANCE FOR A HYBRID METHOD.
*  II  INITH  CHOICE OF THE INITIAL APPROXIMATION OF THE SPARSE
*         HESSIAN MATRIX. INITH=1-UNIT INITIAL APPROXIMATION.
*         INITH=2-RANDOM INITIAL APPROXIMATION.
*
* COMMON DATA :
*  RU  RAN  RANDOM NUMBER.
*  II  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*  II  IREST  RESTART SPECIFICATION. IF IREST=0 DOES NOT HOLD THEN A
*         RESTART IS PERFORMED.
*  IU  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  TO  TUXX   TEXT INFORMATION ON THE RESTART USED.
*
* SUBPROGRAMS USED :
*  S   UXBSMI  GENERATION OF THE PARTITIONED UNIT MATRIX.
*  S   UXBSDO  INITIATION OF A PARTITIONED DIAGONAL MATRIX.
*  S   UXSSMI  GENERATION OF THE SPARSE UNIT MATRIX.
*  S   UXSSDO  INITIATION OF A SPARSE DIAGONAL MATRIX.
*  S   UASSH2  COMPUTATION OF THE SPARSE HESSIAN MATRIX FROM THE
*         PARTITIONED HESSIAN MATRIX.
*  RF  UNRAN1  RANDOM NUMBER GENERATOR.
*  S   UYSET3  DEFINITION OF THE RESTART VARIABLES.
*
      SUBROUTINE UUSSD2(N,NA,M,MA,MB,IA,H,IH,JH,AH,IAG,JAG,F,FO,ETA5,
     & INITH)
      $INCLUDE('UMCOMM')
      INTEGER N,NA,M,MA,MB,IA(NA),IH(N+1),JH(M),IAG(NA+1),JAG(MA),INITH
      $FLOAT H(M),AH(MB),F,FO,ETA5
      INTEGER IUDSD,K,L,KA,NB
      $FLOAT POM,UNRAN1
      $FLOAT CON
      PARAMETER (CON=1.0$P 1)
      SAVE IUDSD
      IF (IREST.LT.0) THEN
      CALL UXBSMI(MB,NA,AH,IAG)
      TUXX='NEWTON  '
      IUDSD=1
      ELSE IF (IREST.EQ.0) THEN
      IF (IDIR.LE.0) THEN
      IF (FO-F.LE.ETA5*FO) THEN
      NB=0
      DO 2 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      IF (IA(KA).GT.0) THEN
      CALL UASSH2(N,NA,MA,L*(L+1)/2,M,H,IH,JH,AH(NB+1),IAG,JAG,KA, ONE)
      ENDIF
      NB=NB+L*(L+1)/2
    2 CONTINUE
      LD=MIN(LD,1)
      IUDSD=0
      ENDIF
      ENDIF
      ELSEIF(IUDSD.EQ.0) THEN
      NB=0
      DO 3 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      CALL UASSH2(N,NA,MA,L*(L+1)/2,M,H,IH,JH,AH(NB+1),IAG,JAG,KA,-ONE)
      NB=NB+L*(L+1)/2
    3 CONTINUE
      CALL UXBSMI(MB,NA,AH,IAG)
      TUXX='NEWTON  '
      IUDSD=1
      ELSEIF(INITH.EQ.1) THEN
      CALL UXSSMI(N,M,H,IH)
      LD=MIN(LD,1)
      TUXX='UNIT    '
      IUDSD=1
      CALL UYSET3
      ELSEIF(INITH.EQ.2) THEN
      POM=(CON-ONE)/(CON+ONE)
      POM=POM/DBLE(N+1)
      DO 1 K=1,MB
      H(K)=POM*(TWO*UNRAN1(RAN)-ONE)
    1 CONTINUE
      CALL UXSSDO(N,M,ONE,H,IH,JH)
      LD=MIN(LD,1)
      TUXX='RANDOM  '
      IUDSD=1
      CALL UYSET3
      ENDIF
      IDECF=0
      RETURN
      END
