$INTERFACE UUSVG1                                             98/12/01

  CALL UUSVG1(NF,MMAX,$H,$IH,$JH,$GN,$S,$RN01,$PSL,$PERM,$XO,$GO,ETA9&
,R,F,FO,P,PO,PAR,PAR1,PAR2,MET,MET1,MET2,MET3)
  $ADD(REAL,'\$H($MMAX)\$GN($NF)\$S($NF)\$XO($NF)\$GO($NF)&
  \$RN01($NF+1)')
  $ADD(INTEGER,'\$IH($NF+1)\$JH($MMAX)\$PSL($NF+1)\$PERM($NF)')
$REM  $ADD(INTEGER,'\NF\MMAX\MET\MET1\MET2\MET3')
$REM  $ADD(REAL,'\R\F\FO\\P\PO\PAR\PAR1\PAR2')
  $HELP
    SELECTION OF SELF SCALING:
      1 - SELF SCALING SUPPRESSED
      2 - INITIAL SELF SCALING
      3 - CONTROLLED SELF SCALING
      4 - SELF SCALING IN EACH ITERATION
  $CHECK(MET1,3,'1\2\3\4',4)
  $HELP
    SELECTION OF THE LINE SEARCH MODEL:
      1 - QUADRATIC MODEL
      2 - USE OF TAYLOR EXPANSION
  $CHECK(MET2,2,'1\2',4)
  $HELP
    METHOD CORRECTION:
      1 - NO CORRECTION
      2 - POWELL'S CORRECTION
  $CHECK(MET3,1,'1\2',4)
  $HELP
    TOLERANCE FOR POSITIVE DEFINITENESS
  $CHECK(ETA2,ETA0,REAL,4)
  $ADD(INIT)
    NUXX='UUSVG1'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UUSVG1                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* VARIABLE METRIC UPDATE OF A SPARSE SYMMETRIC POSITIVE DEFINITE
* MATRIX USING THE FACTORIZATION B=L*D*TRANS(L) FOR DIRECT METHODS.
*
* PARAMETERS :
*  II  NF DECLARED NUMBER OF VARIABLES.
*  II  MMAX  NUMBER OF NONZERO ELEMENTS OF THE MATRIX.
*  RU  H(MMAX)  FACTORIZATION B=L*D*TRANS(L) OF A POSITIVE
*         DEFINITE APPROXIMATION OF THE HESSIAN MATRIX.
*  II  IH(NF)  POINTERS OF THE DIAGONAL ELEMENTS OF H.
*  II  JH(MMAX) INDICES OF THE FACTORS.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RA  RN01(NF) AUXILIARY VECTOR.
*  II  PSL(NF+1) POINTER VECTOR TO THE COMPACT CHOLESKY FACTOR.
*  II  PERM(NF) PERMUTATION VECTOR.
*  RU  XO(NF)  VECTORS OF VARIABLES DIFFERENCE.
*  RI  GO(NF)  GRADIENTS DIFFERENCE.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RI  PO  INITIAL VALUE OF THE DIRECTIONAL DERIVATIVE.
*  II  MET1  SELECTION OF SELF SCALING. MET1=1-SELF SCALING SUPPRESSED.
*         MET1=2-INITIAL SELF SCALING. MET1=3-SELF SCALING IN EACH
*         ITERATION.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  ITERD  TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  II  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXSPCM  MATRIX-VECTOR PRODUCT USING THE FACTORIZATION
*         B=L*D*TRANS(L) STORED IN THE COMPACT FORM.
*  S   UXSPCU  CORRECTION OF A SPARSE SYMMETRIC POSITIVE DEFINITE
*         MATRIX IN THE FACTORED COMPACT FORM B=L*D*TRANS(L).
*  S   UXSPCS  SCALING OF A SPARSE SYMMETRIC POSITIVE DEFINITE MATRIX
*         IN THE FACTORED COMPACT FORM B=L*D*TRANS(L).
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVNEG  COPYING OF A VECTOR WITH THE CHANGE OF THE SIGN.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UOU1D1  PRINT OF ENTRY TO VARIABLE METRIC UPDATE.
*  S   UOU1D2  PRINT OF EXIT FROM VARIABLE METRIC UPDATE.
*  S   UXVSFP  FORWARD PERMUTATION OF A VECTOR.
*  S   UXVSBP  BACKWARD PERMUTATION OF A VECTOR.
*
* METHOD :
* BASIC VARIABLE METRIC METHODS.
*
      SUBROUTINE UUSVG1(NF,MMAX,H,IH,JH,G,S,RN01,PSL,PERM,XO,GO,ETA9,
     & R,F,FO,P,PO,PAR,PAR1,PAR2,MET,MET1,MET2,MET3)
      $INCLUDE('UMCOMM')
      INTEGER NF,MMAX,IH(NF+1),JH(MMAX),PSL(NF+1),PERM(NF),MET,MET1,
     & MET2,MET3
      $FLOAT H(MMAX),G(NF),S(NF),XO(NF),GO(NF),RN01(NF),ETA9,R,F,FO,
     & P,PO,PAR,PAR1,PAR2
      INTEGER MM
      $FLOAT UXVDOT
      $FLOAT DEN,DIS
      $FLOAT A,B,C,GAM,RHO,POM
      LOGICAL L1,L2,L3
      $FLOAT CON,CON3,CON4,CON6,CON7
      PARAMETER(CON=0.1$P 0,
     * CON3=0.2$P 0,CON4=1.0$P-2,CON6=0.5$P 0,CON7=2.5$P 0)
      IF (MET.LE.0) RETURN
      IF(IDECF.NE.1) THEN
      ITERH=-1
      TUXX='BAD DEC1'
      CALL UOERR1('UUSVG1',1)
      GOTO 6
      ENDIF
C
C     FORWARD PERMUTATION
C
      MM=IH(NF+1)-1
      CALL UXVSFP(NF,PERM,G,S)
      CALL UXVSFP(NF,PERM,GO,S)
      CALL UXVSFP(NF,PERM,XO,S)
      CALL UOU1D1(NF,XO,GO)
      IF (MET .GE. 4)  MET = 1
C
C     DETERMINATION OF THE PARAMETERS B, C
C
      B=UXVDOT(NF,XO,GO)
      IF (B .LE. ZERO.AND.MET3.NE.2) THEN
      ITERH = 2
      TUXX = 'B - NEG.'
      GOTO 6
      ENDIF
      A=ZERO
      L1 = MET1 .GE. 3  .OR.  MET1 .EQ. 2  .AND.  NIT .EQ. KIT
      IF (ITERD.NE.1) THEN
      CALL UXVCOP(NF,XO,S)
      CALL UXSPCM(NF,MMAX,H(MM+1),PSL,JH(MM+1),S,RN01,0)
      C = UXVDOT(NF,XO,S)
      ELSE
      IF (ITRAN.EQ.0) THEN
      CALL UXVDIF(NF,GO,G,S)
      ELSE
      CALL UXVNEG(NF,G,S)
      ENDIF
      CALL UXVSCL(NF,R,S,S)
      C = -R * PO
      ENDIF
      IF (C .LE. ZERO) THEN
      ITERH = 3
      TUXX = 'C - NEG.'
      GOTO 6
      ENDIF
      IF(B.LE.ZERO) THEN
C
C     POWELL'S CORRECTION
C
      DIS=(ONE-CON)*C/(C-B)
      CALL UXVDIF(NF,GO,S,GO)
      CALL UXVDIR(NF,DIS,GO,S,GO)
      B=C+DIS*(B-C)
      ENDIF
C
C     DETERMINATION OF THE PARAMETER RHO (NONQUADRATIC PROPERTIES)
C
      IF(MET2.EQ.1) THEN
      RHO=ONE
      ELSEIF(MET2.EQ.2) THEN
      RHO=HALF*B/(FO-F+P)
      IF(RHO/CON4.LE.ONE) RHO=ONE
      IF(RHO*CON4.GE.ONE) RHO=ONE
      ENDIF
      IF (L1)  THEN
C
C     DETERMINATION OF THE PARAMETER GAM (SELF SCALING)
C
      PAR=C/B
      GAM = PAR * RHO
      IF (MET1.GE.3.AND.NIT.NE.KIT) THEN
      L2=PAR2.LE.ZERO
      L3=L2.AND.ABS(PAR1).LE.CON3
      L3=L3.OR.(.NOT.L2.AND.GAM.GT.ONE)
      L3=L3.OR.(L2.AND.PAR1.LT.ZERO.AND.GAM.GT.ONE)
      L3=L3.OR.(L2.AND.PAR1.GT.ZERO.AND.GAM.LT.ONE)
      L3=L3.OR.GAM.LT.CON6
      L3=L3.OR.GAM.GT.CON7
      IF(L3) THEN
      GAM=ONE
      PAR=GAM/RHO
      ENDIF
      ENDIF
      ELSE
      GAM = ONE
      PAR = GAM/RHO
      ENDIF
      GO TO  ( 1, 2, 3)  MET
C
C     BFGS UPDATE
C
    1 POM=ZERO
      CALL UXSPCU(NF,MMAX,H(MM+1),PSL,JH(MM+1),PAR/B,GO,XO)
      CALL UXSPCU(NF,MMAX,H(MM+1),PSL,JH(MM+1),-ONE/C,S,XO)
      TUXX = 'BFGS    '
      NUPD=NUPD+1
      GO TO  4
C
C     DFP UPDATE
C
    2 DEN = PAR * B + C
      DIS = PAR + C/B
      POM=ONE
      CALL UXVDIR(NF,-DIS,GO,S,GO)
      CALL UXSPCU(NF,MMAX,H(MM+1),PSL,JH(MM+1),ONE/DEN,GO,XO)
      CALL UXSPCU(NF,MMAX,H(MM+1),PSL,JH(MM+1),-ONE/DEN,S,XO)
      TUXX = 'DFP     '
      NUPD=NUPD+1
      GO TO  4
C
C     HOSHINO UPDATE
C
    3 DEN = PAR*B + C
      DIS = HALF * B
      POM=PAR*B/DEN
      CALL UXVDIR(NF,PAR,GO,S,S)
      CALL UXSPCU(NF,MMAX,H(MM+1),PSL,JH(MM+1),PAR/DIS,GO,XO)
      CALL UXSPCU(NF,MMAX,H(MM+1),PSL,JH(MM+1),-ONE/DEN,S,XO)
      TUXX = 'HOSHINO '
      NUPD=NUPD+1
    4 ITERH = 0
      IF (GAM .EQ. ONE)  GOTO 5
      CALL UXSPCS(NF,MMAX,H(MM+1),PSL,ONE/GAM)
      NSCL=NSCL+1
    5 CALL UOU1D2(NF,H,S,RHO,GAM,PAR,A,B,C,POM,ETA9)
    6 CONTINUE
C
C BACKWARD PERMUTATION
C
      CALL UXVSBP(NF,PERM,G,S)
      CALL UXVSBP(NF,PERM,GO,S)
      CALL UXVSBP(NF,PERM,XO,S)
C
C SET IDECF
C
      IDECF=1
      RETURN
      END
