* SUBROUTINE UXDCHF               ALL SYSTEMS                16/12/01
C PORTABILITY : ALL SYSTEMS
C 16/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* QR DECOMPOSITION OF A ROWWISE STORED DENSE RECTANGULAR MATRIX A USING
* HOUSEHOLDER TRANSFORMATIONS WITH PIVOTING.
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE MATRIX A.
*  II  M  NUMBER OF ROWS OF THE MATRIX A.
*  RU  A(M*N)  RECTANGULAR MATRIX STORED COLUMNWISE IN THE
*         ONE-DIMENSIONAL ARRAY.
*  IO  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX.
*  RO  D(N)  VECTOR CONTAINING INFORMATION ABOUT HOUSEHOLDER
*         TRANSFORMATIONS.
*  RA  E(N)  AUXILIARY VECTOR.
*  IO  NM  ASSUMED RANK OF THE MATRIX A. 
*
* SUBPROGRAMS USED :
*  S   UXDRMN  EUCLIDEAN NORM OF A PART OF THE ROWWISE STORED
*         RECTANGULAR MATRIX COLUMN.
*
* METHOD :
* P.A.BUSSINGER, G.H.GOLUB : LINEAR LEAST SQUARES SOLUTION BY
* HOUSEHOLDER TRANSFORMATION. NUMER. MATH. 7 (1965) 269-276.
*
      SUBROUTINE UXDCHF(N,M,A,IA,D,E,NM)
      INTEGER N,M,IA(N),NM
      $FLOAT A(M*N),D(N),E(N)
      $FLOAT ALF,BET,POM,UXVDOT,UXVNOR
      INTEGER I,J,K,L,IP,KP
      $FLOAT ZERO,ONE,CON
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0,CON=5.0$P-2)
      NM=MIN(N,M)
C
C     COMPUTATION OF COLUMN NORMS
C
      DO 1 I=1,N
      IA(I)=I
      D(I)=UXVNOR(M,A((I-1)*M+1))
      E(I)=D(I)
    1 CONTINUE
      DO 8 K=1,NM
C
C     COLUMN PIVOTING
C
      L=0
      POM=ZERO
      DO 2 I=K,N
      J=IA(I)
      IF (POM.LT.D(J)) THEN
      POM=D(J)
      L=I
      ENDIF
    2 CONTINUE
      IF (L.EQ.0) THEN
      NM=K-1
      RETURN
      ENDIF
      J=IA(L)
      IA(L)=IA(K)
      IA(K)=J
      L=J
      KP=(J-1)*M
      POM=UXVNOR(M-K+1,A(KP+K))
      IF (A(KP+K).NE.ZERO) POM=SIGN(POM,A(KP+K))
C
C     HOUSEHOLDER TRANSFORMATION
C
      CALL UXVSCL(M-K+1,ONE/POM,A(KP+K),A(KP+K))
      A(KP+K)=A(KP+K)+ONE
      DO 6 I=K+1,N
      J=IA(I)
      IP=(J-1)*M
      ALF=UXVDOT(M-K+1,A(KP+K),A(IP+K))/A(KP+K)
      CALL UXVDIR(M-K+1,-ALF,A(KP+K),A(IP+K),A(IP+K))
C
C     COLUMN NORMS UPDATE
C
      IF (D(J).EQ.ZERO) GO TO 6
      ALF=ONE-(A(IP+K)/D(J))**2
      ALF=MAX(ALF,ZERO)
      BET=ONE+CON*ALF*(D(J)/E(J))**2
      IF (BET.GT.ONE) THEN
      D(J)=D(J)*SQRT(ALF)
      ELSE
      D(J)=UXVNOR(M-K,A(IP+K+1))
      E(J)=D(J)
      ENDIF
    6 CONTINUE
      D(L)=A(KP+K)
      A(KP+K)=-POM
    8 CONTINUE
      RETURN
      END
