* SUBROUTINE UXDCHP               ALL SYSTEMS                16/12/01
C PORTABILITY : ALL SYSTEMS
C 16/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* QR DECOMPOSITION OF A ROWWISE STORED DENSE RECTANGULAR MATRIX A USING
* HOUSEHOLDER TRANSFORMATIONS WITH PIVOTING.
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE MATRIX A.
*  II  M  NUMBER OF ROWS OF THE MATRIX A.
*  RU  A(M*N)  RECTANGULAR MATRIX STORED IN THE TWO-DIMENSIONAL ARRAY.
*  RI  TOL  TOLERANCE FOR PSEUDORANK DETERMINATION.
*  RA  W1(N)  AUXILIARY VECTOR.
*  RA  W2(N)  AUXILIARY VECTOR.
*  IO  NM  ASSUMED RANK OF THE MATRIX A. 
*  IO  NP  PSEUDORANK OF THE MATRIX A.
*
* METHOD :
* P.A.BUSSINGER, G.H.GOLUB : LINEAR LEAST SQUARES SOLUTION BY
* HOUSEHOLDER TRANSFORMATION. NUMER. MATH. 7 (1965) 269-276.
*
      SUBROUTINE UXDCHP(N,M,A,IA,TOL,W1,W2,NM,NP)
      INTEGER M,N,IA(N),NM,NP
      $FLOAT A(M*N),TOL,W1(NM),W2(NM)
      INTEGER I,L
      $FLOAT SMIN,SMAX,SMINPR,SMAXPR,S1,S2,C1,C2
      $FLOAT ZERO,ONE
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
      W1(1)=ONE
      W2(1)=ONE
      SMAX=ABS(A((IA(1)-1)*M+1))
      SMIN=SMAX
      IF (SMAX.EQ.ZERO) THEN
        NP=0
      ELSE
        NP=1
   10   CONTINUE
        IF (NP.LT.NM) THEN
          I=NP+1
          L=(IA(I)-1)*M
          CALL UXVCON(NP,W1,A(L+1),A(L+I),SMIN,SMINPR,S1,C1,2)
          CALL UXVCON(NP,W2,A(L+1),A(L+I),SMAX,SMAXPR,S2,C2,1)
          IF (SMAXPR*TOL.LE.SMINPR) THEN
            DO 20 I = 1,NP
              W1(I)=S1*W1(I)
              W2(I)=S2*W2(I)
   20       CONTINUE
            NP=NP+1
            W1(NP)=C1
            W2(NP)=C2
            SMIN=SMINPR
            SMAX=SMAXPR
            GO TO 10
          END IF
        END IF
      END IF
      RETURN
      END