* SUBROUTINE UXDCQU               ALL SYSTEMS                16/12/01
C PORTABILITY : ALL SYSTEMS
C 16/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* UPDATE OF A QR DECOMPOSITION. THIS QR DECOMPOSITION IS UPDATED
* BY THE RULE Q*R:=Q*R+ALF*X*TRANS(Y).
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE MATRIX Q.
*  II  M  NUMBER OF ROWS OF THE MATRIX Q.
*  RU  A(M,N)  UPPER TRIANGULAR MATRIX STORED COLUMNWISE IN THE
*         TWO-DIMENSIONAL ARRAY.
*  RU  Q(M,N)  RECTANGULAR MATRIX STORED COLUMNWISE IN THE
*         ONE-DIMENSIONAL ARRAY (PART OF THE ORTHOGONAL MATRIX).
*  RI  D(N)  VECTOR CONTAINING INFORMATION ABOUT HOUSEHOLDER
*         TRANSFORMATIONS.
*  RI  ALF  SCALAR PARAMETER.
*  RI  X(M)  INPUT VECTOR.
*  RI  Y(N)  INPUT VECTOR.
*  RA  Z(N)  AUXILIARY VECTOR.
*  IO  INF  INFORMATION. IF INF=0 THEN X LIES IN THE COLUMN SPACE OF Q.
*         IF INF=1 THEN X DOES NOT LIE IN THE COLUMN SPACE OF Q.
*
* SUBPROGRAMS USED :
*  RF  UXVNOR  EUCLIDEAN NORM OF A VECTOR.
*  S   UXVORT  COMPUTATION OF THE PLANE ROTATION MATRIX.
*  S   UXVROT  PLANE ROTATION IS APPLIED TO TWO NUMBERS.
*  S   UXVRTV  PLANE ROTATION IS APPLIED TO TWO VECTORS.
*
* METHOD :
* J.W.DANIEL, W.B.GRAGG, L.KAUFMAN, G.W.STEWARD : REORTHOGONALIZATION
* AND STABLE ALGORITHMS FOR UPDATING THE GRAM-SCHMIDT QR FACTORIZATION.
* MATHEMATICS OF COMPUTATION 30 (1976) 772-795.

      SUBROUTINE UXDCQU(N,M,Q,R,D,ALF,X,Y,Z,INF)
      INTEGER N,M,INF
      $FLOAT Q(N*M),R(N*M),D(N),ALF,X(M),Y(N),Z(N)
      $FLOAT CK,CL,ZK,ZL,UXVNOR
      INTEGER J,JP,K,KP,L,LP,IER
      $FLOAT ONE,CON
      PARAMETER (ONE=1.0$P 0,CON=1.0$P-10)
      INF=0
C
C     COMPUTATION OF THE VECTOR TRANS(Q)*X
C
      CALL UXDCQB(N,M,R,X,D)
      CALL UXDRMM(M,N,Q,X,Z)
      IF (M.GT.N) THEN
C
C     IF X DOES NOT LIE IN THE COLUMN SPACE OF Q WE HAVE TO USE
C     A SUBPROBLEM WHOSE DIMENSION IS BY ONE GREATER (INF=1).
C
      ZK=UXVNOR(M,X)
      CALL UXDCMD(M,N,Q,Z,-ONE,X,X)
      ZL=UXVNOR(M,X)
      IF (ZL.GT.CON*ZK) THEN
      INF=1
      KP=(N-1)*M
      CALL UXVSCL(M,-ONE/ZL,X,X)
      CALL UXVORT(Z(N),ZL,CK,CL,IER)
      IF (IER.EQ.0.OR.IER.EQ.1) THEN
      CALL UXVROT(R(KP+N),ZL,CK,CL,IER)
      CALL UXVRTV(M,Q(KP+1),X,CK,CL,IER)
      ENDIF
      ENDIF
      ENDIF
C
C     APPLICATION OF PLANE ROTATIONS TO THE VECTOR Z SO THAT
C     TRANS(Q1)*Z=E1 WHERE Q1 IS AN ORTHOGONAL MATRIX (ACCUMULATION OF
C     THE PLANE ROTATIONS) AND E1 IS THE FIRST COLUMN OF THE UNIT
C     MATRIX. AT THE SAME TIME BOTH THE UPPER HESSENBERG MATRIX
C     TRANS(Q1)*R AND THE ORTHOGONAL MATRIX Q*Q1 ARE CONSTRUCTED SO THAT
C     Q*Q1*R1=Q*Q1*(TRANS(Q1)*R+ALF*E1*TRANS(Y)) WHERE R1 IS AN UPPER
C     HESSENBERG MATRIX.
C
      LP=N*M
      DO 4 L=N,2,-1
      LP=LP-M
      KP=LP-M
      K=L-1
      CALL UXVORT(Z(K),Z(L),CK,CL,IER)
      IF (IER.EQ.0.OR.IER.EQ.1) THEN
      CALL UXVROT(R(KP+K),Z(L),CK,CL,IER)
      JP=LP
      DO 2 J=L,N
      CALL UXVROT(R(JP+K),R(JP+L),CK,CL,IER)
      JP=JP+M
    2 CONTINUE
      CALL UXVRTV(M,Q(KP+1),Q(LP+1),CK,CL,IER)
      ENDIF
    4 CONTINUE
      Z(1)=ALF*Z(1)
      JP=0
      DO 5 J=1,N
      R(JP+1)=R(JP+1)+Z(1)*Y(J)
      JP=JP+M
    5 CONTINUE
C
C     APPLICATION OF PLANE ROTATIONS TO THE UPPER HESSENBERG MATRIX R1
C     GIVEN ABOVE SO THAT R2=TRANS(Q2)*R1 WHERE Q2 IS AN ORTHOGONAL
C     MATRIX (ACCUMULATION OF THE PLANE ROTATIONS) AND R2 IS AN UPPER
C     TRIANGULAR MATRIX. WE OBTAIN THE NEW QR DECOMPOSITION Q*Q1*Q2*R2.
C
      KP=0
      DO 8 L=2,N
      LP=KP+M
      K=L-1
      CALL UXVORT(R(KP+K),Z(L),CK,CL,IER)
      IF (IER.EQ.0.OR.IER.EQ.1) THEN
      JP=LP
      DO 6 J=L,N
      CALL UXVROT(R(JP+K),R(JP+L),CK,CL,IER)
      JP=JP+M                    
    6 CONTINUE
      CALL UXVRTV(M,Q(KP+1),Q(LP+1),CK,CL,IER)
      ENDIF
      KP=KP+M
    8 CONTINUE
C
C     BACK TRANSFORMATION OF THE GREATER SUBPROBLEM IF INF=1.
C
      KP=(N-1)*M
      IF (INF.EQ.1) THEN
      CALL UXVORT(R(KP+N),ZL,CK,CL,IER)
      IF (IER.EQ.0.OR.IER.EQ.1) THEN
      CALL UXVRTV(M,Q(KP+1),X,CK,CL,IER)
      ENDIF
      ENDIF
      RETURN
      END

