* SUBROUTINE UXDLBU                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* CORRECTION OF A DENSE SYMMETRIC MATRIX A IN THE FACTORED FORM
* A=P*U*D*TRANS(U)*TRANS(P) OBTAINED BY THE SUBROUTINE UXDLBF. THE
* CORRECTION IS DEFINED AS A:=A+ALF*X*TRANS(X) WHERE ALF IS A GIVEN
* SCALING FACTOR AND X IS A GIVEN VECTOR.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RU  A(N*(N+1)/2)  FACTORIZATION A=P*U*D*TRANS(U)*TRANS(P) OBTAINED
*         BY THE SUBROUTINE UXDLBF.
*  II  IA(N)  PERMUTATION FIELD OBTAINED BY THE SUBROUTINE UXDLBF.
*  RI  ALF  SCALING FACTOR IN THE CORRECTION TERM.
*  RI  X(N) VECTOR IN THE CORRECTION TERM.
*  RA  Y(N) AUXILIARY VECTOR.
*  IA  Q(N) AUXILIARY VECTOR.
*
* METHOD :
* D.C.SORENSEN : UPDATING THE SYMMETRIC INDEFINITE FACTORIZATION WITH
* APPLICATION IN A MODIFIED NEWTON'S METHOD. ARGONNE NATIONAL LAB.
* REPORT NO. ANL-77-49, ARGONNE, ILLINOIS, 1977.
*
      SUBROUTINE UXDLBU(N,A,IA,ALF,X,Y,Q)
      $FLOAT ZERO,ONE,TWO,EIGHT,SEVTEN
      PARAMETER ( ZERO=0.0$P+0, ONE=1.0$P+0, TWO=2.0$P+0 )
      PARAMETER ( EIGHT=8.0$P+0, SEVTEN=1.7$P+1 )
      INTEGER N,IA(N),Q(N)
      $FLOAT A(N*(N+1)/2),ALF,X(N),Y(N)
      INTEGER J,K,KD0,KD1,KD2,KD3,KP,KM1,KM2,KM3,Q1,Q2,KT
      $FLOAT SIG,ALPHA,DET,T,S,B1,B2,B3,U1,U0
      $FLOAT D11,D21,D22,D31,D32,D33,L1,L2,T1,T2
C
C     PRODUCING PERMUTATION MATRIX Q FROM PERMUTATION FIELD IA
C
      DO K=1,N
          Q(K) = K
      ENDDO
      DO K=N,1,-1
          KP = ABS(IA(K))
          IF (K.NE.KP) CALL UNSWPI(Q(K),Q(KP))
          IF (IA(K).LT.0) Q(K) = -Q(K)
      ENDDO
C
C     PERMUTATION OF ROWS IN MATRIX A
C
      K = N
      KD1 = N*(N+1)/2
      DO WHILE (K.GT.2)
          KD1 = KD1-K
          IF (IA(K).GT.0) THEN
              DO J=K-1,2,-1
                  KP = ABS(IA(J))
                  IF (KP.NE.J) CALL UNSWAP(A(KD1+KP),A(KD1+J))
              ENDDO
              K = K-1
          ELSE
              KD2 = KD1-K+1
              DO J=K-2,2,-1
                  KP = ABS(IA(J))
                  IF (KP.NE.J) THEN
                      CALL UNSWAP(A(KD1+KP),A(KD1+J))
                      CALL UNSWAP(A(KD2+KP),A(KD2+J))
                  ENDIF
              ENDDO
              K = K-2
              KD1 = KD2
          ENDIF
      ENDDO
C
C     PERMUTATION OF THE VECTOR X, THE RESULT IS A VECTOR Y
C
      CALL UXVCOP(N,X,Y)
      K = N
      DO WHILE (K.GE.1)
          KP = ABS(IA(K))
          IF (KP.NE.K) CALL UNSWAP(Y(K),Y(KP))
          K = K-1
      ENDDO
C
      SIG = ALF
      ALPHA = ( ONE+SQRT( SEVTEN ) ) / EIGHT
      K = N                     ! INDEX OF THE COLUMN
      KD0 = (N+1)*N/2           ! INDEX OF THE ELEMENT A(K,K)
      IF (K.GT.1) KD1 = KD0-K   ! INDEX OF THE ELEMENT A(K-1,K-1)
  101 CONTINUE
      IF (K.LT.1) GOTO 99
C
C     THE PROCESSING BEGINS HERE
C
      KM1 = K-1
      KM2 = K-2
      IF (K.GT.1.AND.IA(K).LT.0) GOTO 115
C
C     THE NEXT BLOCK IS A 1x1 BLOCK
C
      T = Y(K)
      B1 = SIG*T
      D11 = A(KD0)+B1*T
      DO J=1,KM1
          Y(J) = Y(J)-A(KD1+J)*T
      ENDDO
  103 CONTINUE
C
C     ENTER 1x1
C
      IF (K.LE.1) THEN
          A(KD0) = D11
          Q(K) = ABS(Q(K))
          GOTO 99   ! THE DECOMPOSITION IS COMPLETE IF K=1
      ENDIF
C
      U1 = ABS(D11)
      U0 = ABS(B1)
      IF (U1.GE.ALPHA*U0.OR.U1*ABS(SIG).GT.ALPHA*U0*U0) THEN
          SIG = SIG-B1*B1/D11
          B1 = B1/D11
          A(KD0) = D11
C
C         UPDATE THE K-TH COLUMN OF U
C
          DO J=1,KM1
              A(KD1+J) = A(KD1+J)+B1*Y(J)
          ENDDO
          Q(K) = ABS(Q(K))
          KD0 = KD1
          K = KM1
          IF (K.GT.1) KD1 = KD0-K
          GOTO 101
      ENDIF
      IF (K.GT.1) KD2 = KD1-K+1   ! INDEX OF THE ELEMENT A(K-2,K-2)
      IF (KM1.GT.1.AND.IA(KM1).LT.0) GOTO 108
C
C     A 2x2 BLOCK IS FORMED BY COMBINING
C     THE PREVIOUS 1x1 BLOCK WITH BLOCK K
C
      B2 = Y(KM1)
      T = B2
      D21 = B2*B1
      B2 = SIG*B2
      D22 = A(KD1)+B2*T
      L1 = A(KD0-1)
      D22 = D22+L1*D21
      B2 = B2+L1*B1
      D21 = D21+L1*D11
      D22 = D22+L1*D21
C
C     INCLUDE INFORMATION FROM THE (K-1)-ST COLUMN OF U
C
      DO J=1,KM2
          Y(J) = Y(J)-A(KD2+J)*T
          A(KD1+J) = A(KD1+J)-A(KD2+J)*L1
      ENDDO
      GOTO 117
  108 CONTINUE
C
C     IF THIS PORTION OF THE CODE IS REACHED WE ARE IN THE CASE OF A
C     1x1 SINGULAR BLOCK THAT FOLLOWS A 2x2 BLOCK.
C     THIS 2x2 BLOCK IS JOINED TO THE 1x1 BLOCK TO FORM A 3x3 MATRIX D.
C
      T1 = Y(KM1)
      T2 = Y(KM2)
      B2 = SIG*T1
      B3 = SIG*T2
      D22 = A(KD1)+B2*T1
      D32 = A(KD1-1)+B3*T1
      D33 = A(KD2)+B3*T2
      D21 = T1*B1
      D31 = T2*B1
      L1 = A(KD0-1)
      L2 = A(KD0-2)
      T = L2*D11
      D33 = D33+L2*(TWO*D31+T)
      D31 = D31+T
      D32 = D32+L1*D31+L2*D21
      T = L1*D11
      D22 = D22+L1*(TWO*D21+T)
      D21 = D21+T
      B2 = B2+L1*B1
      B3 = B3+L2*B1
      KM3 = K-3
C
C     INCLUDE INFORMATION FROM THE (K-1)-ST AND (K-2)-ND COLUMNS OF U
C
      KD3 = KD2-K+2   ! INDEX OF A(K-3,K-3) INCLUDING ZERO
      DO J=1,KM3
          Y(J) = Y(J)-(A(KD2+J)*T1+A(KD3+J)*T2)
          A(KD1+J) = A(KD1+J)-(A(KD2+J)*L1+A(KD3+J)*L2)
      ENDDO
      U1 = MAX(ABS(D11),ABS(D22),ABS(D33))
      U0 = MAX(ABS(D21),ABS(D31),ABS(D32))
      IF (U1.LT.ALPHA*U0) GOTO 112
C
C     A 1x1 PIVOT WILL BE USED
C
      Q1 = 1
      IF (U1.EQ.ABS(D22).AND.U1.NE.ABS(D11)) THEN
C
C         THE MAX ELEMENT IS D22
C
          CALL UNSWAP(D11,D22)
          CALL UNSWAP(D31,D32)
          CALL UNSWAP(B1,B2)
          CALL UNSWPI(Q(K),Q(KM1))
          Q1 = 2
      ELSEIF (U1.EQ.ABS(D33).AND.U1.NE.ABS(D11)) THEN
C
C         THE MAX ELEMENT IS D33
C
          CALL UNSWAP(D11,D33)
          CALL UNSWAP(D21,D32)
          CALL UNSWAP(B1,B3)
          CALL UNSWPI(Q(K),Q(KM2))
          Q1 = 3
      ENDIF
C
C     THE MAX ELEMENT IS D11
C
      D22 = D22-D21*D21/D11
      D32 = D32-D31*D21/D11
      D33 = D33-D31*D31/D11
      B1 = B1/D11
      B2 = B2-B1*D21
      B3 = B3-B1*D31
      D21 = D21/D11
      D31 = D31/D11
C
      SIG = SIG-D11*B1*B1
C
C     UPDATE THE K-TH COLUMN OF U
C
      DO J=1,KM3
          IF (Q1.EQ.2) THEN
              CALL UNSWAP(A(KD1+J),A(KD2+J))
          ELSEIF (Q1.EQ.3) THEN
              CALL UNSWAP(A(KD1+J),A(KD3+J))
          ENDIF
          A(KD1+J) = A(KD1+J)+D21*A(KD2+J)+D31*A(KD3+J)+B1*Y(J)
      ENDDO
C
C     INTERCHANGE THE CORRESPONDING ROWS OF U
C
      IF (Q1.NE.1) THEN
          KT = KD0
      ENDIF
      IF (Q1.EQ.2) THEN
          DO J=K,N-1
              KT = KT+J
              CALL UNSWAP(A(KT),A(KT-1))
          ENDDO
      ELSEIF (Q1.EQ.3) THEN
          DO J=K,N-1
              KT = KT+J
              CALL UNSWAP(A(KT),A(KT-2))
          ENDDO
      ENDIF
      A(KD0) = D11
      A(KD0-1) = D21
      A(KD0-2) = D31
      D11 = D22
      D22 = D33
      D21 = D32
      B1 = B2
      B2 = B3
      Q(K) = ABS(Q(K))
      K = KM1
      KM1 = KM2
      KM2 = KM1-1
      KD0 = KD1
      KD1 = KD2
      KD2 = KD1-KM1
      GOTO 117
  112 CONTINUE
C
C     A 2x2 PIVOT WILL BE USED
C
      Q1 = 1
      Q2 = 2
      IF (U0.EQ.ABS(D31).AND.U0.NE.ABS(D21)) THEN
C
C         D31 IS THE MAX ELEMENT
C
          CALL UNSWAP(D22,D33)
          CALL UNSWAP(D21,D31)
          CALL UNSWAP(B2,B3)
          CALL UNSWPI(Q(KM1),Q(KM2))
          Q2 = 3
      ELSEIF (U0.EQ.ABS(D32).AND.U0.NE.ABS(D21)) THEN
C
C         D32 IS THE MAX ELEMENT
C
          CALL UNSWAP(D11,D22)
          CALL UNSWAP(D22,D33)
          CALL UNSWAP(D21,D32)
          CALL UNSWAP(D31,D32)
          CALL UNSWAP(B1,B2)
          CALL UNSWAP(B2,B3)
          CALL UNSWPI(Q(K),Q(KM1))
          CALL UNSWPI(Q(KM1),Q(KM2))
          Q1 = 2
          Q2 = 3
      ENDIF
C
C     D21 IS THE MAX ELEMENT, THE 2x2 PIVOT IS DONE HERE
C
      DET = D11*D22-D21*D21
      T = (D22*D31-D21*D32)/DET
      S = (D11*D32-D21*D31)/DET
      B3 = B3-T*B1-S*B2
      D33 = D33-T*D31-S*D32
      D31 = T
      D32 = S
      T = (D22*B1-D21*B2)/DET
      S = (D11*B2-D21*B1)/DET
      SIG = SIG-T*B1-S*B2
      B1 = T
      B2 = S
C
C     UPDATE THE K-TH AND (K-1)-ST COLUMNS OF U
C
      DO J=1,KM3
          IF (Q1.EQ.2) CALL UNSWAP(A(KD1+J),A(KD2+J))
          IF (Q2.EQ.3) CALL UNSWAP(A(KD2+J),A(KD3+J))
          A(KD1+J) = A(KD1+J)+D31*A(KD3+J)+B1*Y(J)
          A(KD2+J) = A(KD2+J)+D32*A(KD3+J)+B2*Y(J)
      ENDDO
C
C     INTERCHANGE THE CORRESPONDING ROWS OF U
C
      IF (Q2.EQ.3) THEN
          KT = KD0
          DO J=K,N-1
              KT = KT+J
              IF (Q1.EQ.2) CALL UNSWAP(A(KT),A(KT-1))
              CALL UNSWAP(A(KT-1),A(KT-2))
          ENDDO
      ENDIF
      A(KD0) = D11
      A(KD0-1) = D21
      A(KD1) = D22
      A(KD0-2) = D31
      A(KD1-1) = D32
      D11 = D33
      B1 = B3
      Q(K) = -ABS(Q(K))
      Q(KM1) = -ABS(Q(KM1))
      K = KM2
      KM1 = K-1
      KM2 = K-2
      KD0 = KD2
      KD1 = KD0-K
      GOTO 103
  115 CONTINUE
C
C     THE DIAGONAL BLOCK BEGINNING AT ENTRY K IS 2x2.
C     THE UPDATED DIAGONAL BLOCK D IS REQUIRED TO SATISFY
C         ABS(D21)*ALPHA.GT.MAX(ABS(D11),ABS(D22)).
C     IF THIS IS NOT SATISFIED, THE BLOCK IS SPLIT INTO TWO 1x1 BLOCKS.
C
      KD2 = KD1-K+1
      T1 = Y(K)
      T2 = Y(KM1)
      B1 = SIG*Y(K)
      B2 = SIG*Y(KM1)
      D11 = A(KD0)+B1*Y(K)
      D21 = A(KD0-1)+B2*Y(K)
      D22 = A(KD1)+B2*Y(KM1)
      DO J=1,KM2
          Y(J) = Y(J)-A(KD1+J)*T1-A(KD2+J)*T2
      ENDDO
  117 CONTINUE
C
C     ENTER 2x2, THE 2x2 BLOCK WILL BE PROCESSED
C
      U1 = MAX(ABS(D11),ABS(D22))
      U0 = ABS(D21)
      IF (U1.GE.ALPHA*U0) GOTO 119
C
C     A 2x2 PIVOT WILL BE USED
C
      DET = D11*D22-D21*D21
      A(KD0) = D11
      A(KD0-1) = D21
      A(KD1) = D22
      Q(K) = -ABS(Q(K))
      Q(KM1) = -ABS(Q(KM1))
      IF (KM1.EQ.1) GOTO 99
      T1 = (D22*B1-D21*B2)/DET
      T2 = (D11*B2-D21*B1)/DET
C
C     UPDATE THE K-TH AND (K-1)-ST COLUMNS OF U
C
      DO J=1,KM2
          A(KD1+J) = A(KD1+J)+T1*Y(J)
          A(KD2+J) = A(KD2+J)+T2*Y(J)
      ENDDO
      SIG = SIG-T1*B1-T2*B2
      K = KM2
      KD0 = KD2
      KD1 = KD0 - K
      GOTO 101
  119 CONTINUE
C
C     A 1x1 PIVOT WILL BE USED
C
      IF (U1.EQ.ABS(D22).AND.U1.NE.ABS(D11)) THEN
          CALL UNSWAP(D11,D22)
          CALL UNSWAP(B1,B2)
          CALL UNSWPI(Q(K),Q(KM1))
          DO J=1,KM2
              CALL UNSWAP(A(KD1+J),A(KD2+J))
          ENDDO
          KT = KD0
          DO J=K,N-1
              KT = KT+J
              CALL UNSWAP(A(KT),A(KT-1))
          ENDDO
      ENDIF
C
C     PROCESS THE TWO 1x1 BLOCKS
C
      D22 = D22-D21*D21/D11
      D21 = D21/D11
      B2 = B2-B1*D21
      B1 = B1/D11
C
C     UPDATE THE K-TH COLUMN OF U
C
      DO J=1,KM2
          A(KD1+J) = A(KD1+J)+D21*A(KD2+J)+B1*Y(J)
      ENDDO
      A(KD0) = D11
      A(KD0-1) = D21
      SIG = SIG-B1*D11*B1
      D11 = D22
      B1 = B2
      Q(K) = ABS(Q(K))
      K = KM1
      KM1 = KM2
      KM2 = K-2
      KD0 = KD1
      KD1 = KD2
      GOTO 103
C
C     PRODUCING UPDATED PERMUTATION FIELD IA FROM UPDATED PERMUTATION MATRIX Q
C
   99 CONTINUE
      DO K=N,2,-1
          KP = K
   91     KT = ABS(Q(KP))
          IF (KT.LE.K) THEN
              IA(K) = KT
              IF (Q(K).LT.0) IA(K) = -IA(K)
          ELSE
              KP = KT
              GOTO 91
          ENDIF
      ENDDO
      IA(1) = 1
      IF (Q(1).LT.0) IA(1) = -1
C
C     PERMUTATION OF ROWS IN UPDATED MATRIX A
C
      K = 1
      KD1 = 0
      DO WHILE (K.LE.N)
          IF (IA(K).GT.0) THEN
              DO J=2,K-1
                  KP = ABS(IA(J))
                  IF (KP.NE.J) CALL UNSWAP(A(KD1+KP),A(KD1+J))
              ENDDO
              KD1 = KD1+K
              K = K+1
          ELSE
              KD0 = KD1+K
              DO J=2,K-1
                  KP = ABS(IA(J))
                  IF (KP.NE.J) THEN
                      CALL UNSWAP(A(KD1+KP),A(KD1+J))
                      CALL UNSWAP(A(KD0+KP),A(KD0+J))
                  ENDIF
              ENDDO
              KD1 = KD0+K+1
              K = K+2
          ENDIF
      ENDDO
C
      RETURN
      END
