* SUBROUTINE UXDPGU                ALL SYSTEMS                89/12/01
C PORTABILITY : ALL SYSTEMS
C 89/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* CORRECTION OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX A+E IN THE
* FACTORED FORM A+E=L*D*TRANS(L) OBTAINED BY THE SUBROUTINE UXDPGF.
* THE CORRECTION IS DEFINED AS A+E:=A+E+ALF*X*TRANS(X) WHERE ALF IS A
* GIVEN SCALING FACTOR AND X IS A GIVEN VECTOR.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RU  A(N*(N+1)/2) FACTORIZATION A+E=L*D*TRANS(L) OBTAINED BY THE
*         SUBROUTINE UXDPGF.
*  RI  ALF  SCALING FACTOR IN THE CORRECTION TERM.
*  RI  X(N)  VECTOR IN THE CORRECTION TERM.
*  RA  Y(N) AUXILIARY VECTOR.
*
* METHOD :
* P.E.GILL, W.MURRAY, M.SAUNDERS: METHODS FOR COMPUTING AND MODIFYING
* THE LDV FACTORS OF A MATRIX, MATH. OF COMP. 29 (1974) PP. 1051-1077.
*
      SUBROUTINE UXDPGU( N, A, ALF, X, Y)
      INTEGER N
      $FLOAT  A(N*(N+1)/2), ALF, X(N), Y(N)
      $FLOAT  B, D, P, R, T, TO, ALFR
      INTEGER  I, J, II, IJ
      $FLOAT ZERO, ONE, FOUR, CON
      PARAMETER  (ZERO = 0.0$P 0, ONE = 1.0$P 0,
     &            FOUR = 4.0$P 0, CON=1.0$P-8)
      IF (ALF .GE. ZERO) THEN
C
C     FORWARD CORRECTION IN CASE WHEN THE SCALING FACTOR IS NONNEGATIVE
C
      ALFR = SQRT(ALF)
      CALL UXVSCL(N, ALFR, X, Y)
      TO = ONE
      II = 0
      DO 3  I = 1, N
      II = II + I
      D = A(II)
      P = Y(I)
      T = TO + P*P/D
      R = TO / T
      A(II) = D / R
      B = P / (D*T)
      IF (A(II) .LE. FOUR * D) THEN
C
C     AN EASY FORMULA FOR LIMITED DIAGONAL ELEMENT
C
      IJ = II
      DO 1  J = I+1, N
      IJ = IJ + J - 1
      D = A(IJ)
      Y(J) = Y(J) - P*D
      A(IJ) = D + B*Y(J)
    1 CONTINUE
      ELSE
C
C     A MORE COMPLICATE BUT NUMERICALLY STABLE FORMULA FOR UNLIMITED
C     DIAGONAL ELEMENT
C
      IJ = II
      DO 2  J = I+1, N
      IJ = IJ + J - 1
      D = A(IJ)
      A(IJ) = R*D + B*Y(J)
      Y(J) = Y(J) - P*D
    2 CONTINUE
      ENDIF
      TO = T
    3 CONTINUE
      ELSE
C
C     BACKWARD CORRECTION IN CASE WHEN THE SCALING FACTOR IS NEGATIVE
C
      ALFR = SQRT( -ALF)
      CALL UXVSCL(N, ALFR, X, Y)
      TO = ONE
      IJ = 0
      DO 5  I = 1, N
      D = Y(I)
      DO 4  J = 1, I-1
      IJ = IJ + 1
      D = D - A(IJ) * Y(J)
    4 CONTINUE
      Y(I) = D
      IJ = IJ + 1
      TO = TO - D*D/A(IJ)
    5 CONTINUE
      IF (TO .LE. ZERO)  TO = CON
      II = N*(N+1)/2
      DO 7  I = N, 1, -1
      D = A(II)
      P = Y(I)
      T = TO + P*P/D
      A(II) = D * TO / T
      B = -P / (D*TO)
      TO = T
      IJ = II
      DO 6  J = I+1, N
      IJ = IJ + J - 1
      D = A(IJ)
      A(IJ) = D + B*Y(J)
      Y(J) = Y(J) + P*D
    6 CONTINUE
      II = II - I
    7 CONTINUE
      ENDIF
      RETURN
      END
