* SUBROUTINE UXDRHC               ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* QR DECOMPOSITION OF TRANSPOSE OF A DENSE RECTANGULAR MATRIX A USING
* HOUSEHOLDER TRANSFORMATIONS WITH INCREMENTAL CONDITION ESTIMATION.
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE MATRIX A.
*  II  M  NUMBER OF ROWS OF THE MATRIX A.
*  RU  A(M*N)  RECTANGULAR MATRIX STORED ROWWISE IN THE
*         ONE-DIMENSIONAL ARRAY.
*  IO  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX.
*  RO  D(N)  VECTOR CONTAINING INFORMATION ABOUT HOUSEHOLDER
*         TRANSFORMATIONS.
*  RA  E(N)  AUXILIARY VECTOR.
*  RA  V(N)  AUXILIARY VECTOR.
*  IO  NM  ASSUMED RANK OF THE MATRIX A. 
*
*  S   UXDRMN  EUCLIDEAN NORM OF A PART OF THE ROWWISE STORED
*         RECTANGULAR MATRIX COLUMN.
*
* METHOD :
* C.H.BISCHOF : INCREMENTAL CONDITION ESTIMATION. SIAM J. MATRIX ANAL.
* APPL. 11 (1990) 312-322.
*
      SUBROUTINE UXDRHC(N,M,A,IA,D,E,V,NM)
      INTEGER N,M,IA(N),NM
      $FLOAT A(N*M),D(N),E(N),V(N)
      $FLOAT POM,DEN,XEST,XPOM,XNEW,ALF,BET,ETA,ETAI,EPS,EPSI,S,C,UXDRMN
      INTEGER I,J,K,L,IP,JP,KP,MODE
      $FLOAT ZERO,ONE,TWO,HALF,CON
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0,TWO=2.0$P 0,HALF=5.0$P-1,
     * CON=5.0$P-2)
      NM=MIN(N,M)
      DO 1 I=1,N
      IA(I)=I
      D(I)=UXDRMN(N,M,A,I,1)
      E(I)=D(I)
      V(I)=ZERO
    1 CONTINUE
      XEST=ZERO
      L=0
      DO 8 K=1,NM
      J=0
      DO 2 I=K,N
      IP=IA(I)
      ALF=V(IP)
      DEN=D(IP)**2
      IF (DEN.GT.ZERO) THEN
      IF (ALF.EQ.ZERO) THEN
      IF (XEST*DEN.GT.ONE) THEN
      XPOM=XEST
      JP=1
      ELSE
      XPOM=ONE/DEN
      JP=2
      ENDIF
      IF (XNEW.GT.XPOM.OR.I.EQ.K) THEN
      XNEW=XPOM
      MODE=JP
      J=I
      KP=IP
      ENDIF
      ELSE
      BET=DEN*XEST+ALF**2-ONE
      ETAI=HALF*BET/ABS(ALF)
      EPSI=SQRT(ETAI**2+ONE)
      XPOM=(ONE+ABS(ALF)*(ETAI+EPSI))/DEN
      IF (XNEW.GT.XPOM.OR.I.EQ.K) THEN
      XNEW=XPOM
      ETA=ETAI
      EPS=EPSI
      MODE=0
      J=I
      KP=IP
      ENDIF
      ENDIF
      ENDIF
    2 CONTINUE
      IF (J.EQ.0) THEN
      NM=K-1
      RETURN
      ENDIF
      IA(J)=IA(K)
      IA(K)=KP
      IF (MODE.EQ.1) THEN
      S=ONE
      C=ZERO
      ELSE IF (MODE.EQ.2) THEN
      S=ZERO
      C=ONE
      ELSE
      S=SQRT((ONE+ETA/EPS)/TWO)
      C=SQRT((ONE-ETA/EPS)/TWO)
      ENDIF
      XEST=XNEW
      POM=UXDRMN(N,M,A,KP,K)
      IF (A(L+KP).NE.ZERO) POM=SIGN(POM,A(L+KP))
      JP=L
      DO 3 J=K,M
      A(JP+KP)=A(JP+KP)/POM
      JP=JP+N
    3 CONTINUE
      A(L+KP)=A(L+KP)+ONE
      DO 6 I=K+1,N
      IP=IA(I)
      ALF=ZERO
      JP=L
      DO 4 J=K,M
      ALF=ALF+A(JP+KP)*A(JP+IP)
      JP=JP+N
    4 CONTINUE
      ALF=ALF/A(L+KP)
      JP=L
      DO 5 J=K,M
      A(JP+IP)=A(JP+IP)-ALF*A(JP+KP)
      JP=JP+N
    5 CONTINUE
      IF (D(IP).NE.ZERO) THEN
      ALF=ONE-(A(L+IP)/D(IP))**2
      ALF=MAX(ALF,ZERO)
      BET=ONE+CON*ALF*(D(IP)/E(IP))**2
      IF (BET.GT.ONE) THEN
      D(IP)=D(IP)*SQRT(ALF)
      ELSE
      D(IP)=UXDRMN(N,M,A,IP,K+1)
      E(IP)=D(IP)
      ENDIF
      ENDIF
      IF (MODE.EQ.1) THEN
      ELSE IF (MODE.EQ.2) THEN
      V(IP)=D(IP)/XPOM
      ELSE
      V(IP)=S*V(IP)+D(IP)*(C-V(KP)*S)/D(KP)
      ENDIF
    6 CONTINUE
      DO 7 I=1,K-1
      IP=IA(I)
      V(IP)=V(IP)*S
    7 CONTINUE
      V(KP)=C
      D(KP)=A(L+KP)
      A(L+KP)=-POM
      L=L+N
    8 CONTINUE
      RETURN
      END
