* SUBROUTINE UXDSBF                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* FACTORIZATION A=P*L*D*TRANS(L)*TRANS(P) OF A DENSE SYMMETRIC MATRIX
* A WHERE P IS A PERMUTATION MATRIX, D IS A BLOCK DIAGONAL MATRIX (WITH
* 1*1 OR 2*2 BLOCKS) AND L IS A LOWER TRIANGULAR MATRIX.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RU  A(N*(N+1)/2)  ON INPUT A GIVEN DENSE SYMMETRIC MATRIX A STORED
*         IN PACKED FORM. ON OUTPUT THE COMPUTED FACTORIZATION
*         A=P*L*D*TRANS(L)*TRANS(P).
*  IO  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  IO  INF  AN INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. IF
*         INF=0 THEN A IS POSITIVE SEMIDEFINITE. IF INF>0 THEN A IS
*         INDEFINITE.
*  RU  ALF  ON INPUT DESIRED TOLERANCE FOR NONSINGULARITY. ON OUTPUT
*         A NEGATIVE NUMBER IF A IS SINGULAR OR NEARLY SINGULAR.
*
* METHOD :
* R.FLETCHER, T.L.FREEMAN: A MODIFIED NEWTON METHOD FOR MINIMIZATION,
* J.OPTIMIZATION THEORY APPL. 23 (1977) PP. 35-372.
*
      SUBROUTINE UXDSBF(N,A,IA,INF,ALF)
      INTEGER N, IA(N), INF
      $FLOAT  A(N*(N+1)/2), ALF
      $FLOAT  AMAXD, AMAXN, DET, AMJ, AKJ, ALJ, TOL
      INTEGER  IAMP, IAKP, IALP, I, J, K, L, M, IP, JP, KP, LP, MP, JJ,
     &         IJ, KK, KL, LL, KJ, LJ, IK, IL, MM, MJ, IM, NP, IND
      $FLOAT ZERO, HALF, CON, DEL
      PARAMETER (ZERO = 0.0$P 0, HALF = 0.5$P 0, CON = 0.64$P 0,
     & DEL=1.0$P-30)
      IND(I,J) = MAX(I,J) * (MAX(I,J) - 1) / 2 + MIN(I,J)
      ALF = MAX(ABS(ALF),DEL)
      INF = 0
      DO 1 I=1,N
      IA(I) = I
    1 CONTINUE
      NP=1
    8 IF (NP.LE.N) THEN
      AMAXD = ZERO
      AMAXN = ZERO
C
C     DETERMINATION OF BOTH MAXIMUM DIAGONAL AND MAXIMUM OFF-DIAGONAL
C     ELEMENTS
C
      DO 3 JP=NP,N
      J = IA(JP)
      JJ = IND(J,J)
      IF ( ABS(A(JJ)) .GE. AMAXD ) THEN
      M = J
      MP = JP
      AMAXD = ABS(A(JJ))
      ENDIF
      DO 2 IP=NP,JP
      I = IA(IP)
      IJ = IND(I,J)
      IF ( ABS(A(IJ))  .GE. AMAXN )  THEN
      K = I
      L = J
      KP = IP
      LP = JP
      AMAXN = ABS(A(IJ))
      ENDIF
    2 CONTINUE
    3 CONTINUE
      IF (NP.EQ.1) TOL=ALF*MAX(ALF,AMAXN)
      IF (AMAXN .LE. TOL) THEN
      ALF=-HALF
      AMAXD=TOL
      AMAXN=TOL
      ENDIF
      IF (AMAXD .GE. CON*AMAXN) THEN
C
C     GAUSSIAN ELIMINATION WITH A 1*1 PIVOT
C
      MM = IND(M,M)
      IF (ALF .LE. ZERO) A(MM) = TOL
      DET = A(MM)
      IAMP = IA(MP)
      IA(MP) = IA(NP)
      IA(NP) = IAMP
      DO 5 JP=NP+1, N
      J = IA(JP)
      MJ = IND(M,J)
      AMJ = A(MJ)
      A(MJ) = AMJ/DET
      DO 4 IP=NP+1, JP
      I = IA(IP)
      IM = IND(I,M)
      IJ = IND(I,J)
      A(IJ) = A(IJ) - A(IM)*AMJ
    4 CONTINUE
    5 CONTINUE
      ELSE
C
C     GAUSSIAN ELIMINATION WITH A 2*2 PIVOT
C
      KK = IND(K,K)
      KL = IND(K,L)
      LL = IND(L,L)
      DET = A(KK)*A(LL) - A(KL)*A(KL)
      IAKP = IA(KP)
      IA(KP) = IA(NP)
      IA(NP)=-IAKP
      NP = NP + 1
      IALP = IA(LP)
      IA(LP) = IA(NP)
      IA(NP) = -IALP
      DO 7 JP=NP+1, N
      J = IA(JP)
      KJ = IND(K,J)
      LJ = IND(L,J)
      AKJ = A(KJ)
      ALJ = A(LJ)
      A(KJ) = (A(LL)*AKJ-A(KL)*ALJ)/DET
      A(LJ) = (A(KK)*ALJ-A(KL)*AKJ)/DET
      DO 6 IP=NP+1, JP
      I = IA(IP)
      IK = IND(I,K)
      IL = IND(I,L)
      IJ = IND(I,J)
      A(IJ) = A(IJ) - A(IK)*AKJ - A(IL)*ALJ
    6 CONTINUE
    7 CONTINUE
      IF (A(KK)+A(LL).LT.ZERO) INF=1
      ENDIF
      IF (DET .LT. ZERO) INF = 1
      NP=NP+1
      GO TO 8
      ENDIF
      RETURN
      END
