* SUBROUTINE UXDSGC                ALL SYSTEMS               91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  DECOMPOSITION A+ALF*I=P*L*D*TRANS(L)*TRANS(P) OF A DENSE SYMMETRIC
*  POSITIVE DEFINITE MATRIX A+ALF*I WHERE I IS UNIT MATRIX, D IS
*  DIAGONAL POSITIVE DEFINITE MATRIX AND L IS LOWER TRIANGULAR MATRIX.
*  IF A IS SUFFICIENTLY POSITIVE DEFINITE THEN ALF=0.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2) GIVEN DENSE SYMMETRIC MATRIX A STORED IN THE PACKED
*         FORM.
*  RO  B(N*(N+1)/2) COMPUTED FACTORIZATION A+ALF*I=L*D*TRANS(L).
*  IO  IB(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RA  C(N)  AUXILIARY VECTOR
*  RI  TOL  TOLERANCE FOR POSITIVE DEFINITENESS.
*  RU  ALF  SCALAR PARAMETER (SHIFT) IN THE FACTORIZATION.
*  IU  M  COUNT OF DECOMPOSITIONS.
*
* SUBPROGRAMS USED :
*  S   UXDSGF  SCHNABELL-ESKOW DECOMPOSITION OF A DENSE SYMMETRIC
*         MATRIX.
*  S   UXDSDA  ADDITION OF A SCALED UNIT MATRIX TO A DENSE SYMMETRIC
*         MATRIX.
*  S   UXDSMC  COPYING OF A DENSE SYMMETRIC MATRIX.
*
* METHOD :
* R.B.SCHNABEL, E.ESKOW: A NEW CHOLESKI FACTORIZATION, SIAM J.
* SCI. STAT. COMPUT. 11 (1990), PP. 1136-1158.
*
      SUBROUTINE UXDSGC(N,A,B,IB,C,TOL,ALF,M)
      INTEGER N,IB(N),M
      $FLOAT A(N*(N+1)/2),B(N*(N+1)/2),C(N),TOL,ALF
      $FLOAT BET,TEMP
      INTEGER INF
      $FLOAT ZERO
      PARAMETER (ZERO=0.0$P 0)
      ALF=MAX(ALF,ZERO)
C
C     DECOMPOSITION OF THE SHIFTED MATRIX
C
      CALL UXDSMC(N,A,B)
      IF (ALF.GT.ZERO) CALL UXDSDA(N,B,ALF)
      M=M+1
      TEMP=TOL
      CALL UXDSGF(N,B,IB,C,INF,TEMP,BET)
      IF (INF.EQ.0) RETURN
C
C     NEW SHIFT
C
      ALF=ALF+BET
C
C     DECOMPOSITION OF THE SHIFTED MATRIX
C
      CALL UXDSMC(N,A,B)
      IF (ALF.GT.ZERO) CALL UXDSDA(N,B,ALF)
      M=M+1
      TEMP=TOL
      CALL UXDSGF(N,B,IB,C,INF,TEMP,BET)
      RETURN
      END
