* SUBROUTINE UXDSGN                ALL SYSTEMS               92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* ESTIMATION OF THE MINIMUM EIGENVALUE AND THE CORRESPONDING EIGENVECTOR
* OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX A+E USING THE
* FACTORIZATION A+E=L*D*TRANS(L) OBTAINED BY THE SUBROUTINE UXDPGF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2) FACTORIZATION A+E=L*D*TRANS(L) OBTAINED BY THE
*         SUBROUTINE UXDSGF.
*  II  IA(N)  PERMUTATION VECTOR WHICH CONTAINS INFORMATION OBTAINED
*         BY THE SUBROUTINE UXDSGF.
*  RO  X(N)  ESTIMATED EIGENVECTOR.
*  RO  ALF  ESTIMATED EIGENVALUE.
*  II  JOB  OPTION. IF JOB=0 THEN ONLY ESTIMATED EIGENVALUE IS
*         COMPUTED. IS JOB>0 THEN BOTH ESTIMATED EIGENVALUE AND
*         ESTIMATED EIGENVECTOR ARE COMPUTED BY JOB ITERATIONS.
*
* SUBPROGRAMS USED :
*  S   UXDSGB  BACK SUBSTITUTION.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*
* METHOD :
* A.K.CLINE, C.B.MOLER, G.W.STEWART, J.H.WILKINSON : AN ESTIMATE
* FOR THE CONDITION NUMBER OF A MATRIX. SIAM J. NUMER. ANAL. 16
* (1979) 368-373.
*
      SUBROUTINE UXDSGN(N,A,IA,X,ALF,JOB)
      INTEGER N,IA(N),JOB
      $FLOAT A(N*(N+1)/2),X(N),ALF
      $FLOAT XP,XM,FP,FM,UXVDOT
      INTEGER I,K,IP,KP,II,IK,IND
      $FLOAT ZERO,ONE
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
      IND(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
C
C     COMPUTATION OF THE VECTOR V WITH POSSIBLE MAXIMUM NORM SUCH
C     THAT  L*D**(1/2)*V=U  WHERE U HAS ELEMENTS +1 OR -1
C
      DO 2 I=1,N
      IP=IA(I)
      X(IP)=ZERO
    2 CONTINUE
      DO 6 K=1,N
      KP=IA(K)
      XP=-X(KP)+ONE
      XM=-X(KP)-ONE
      FP=ABS(XP)
      FM=ABS(XM)
      DO 3 I=K+1,N
      IP=IA(I)
      IK=IND(IP,KP)
      FP=FP+ABS(X(IP)+A(IK)*XP)
      FM=FM+ABS(X(IP)+A(IK)*XM)
    3 CONTINUE
      IF(FP.GE.FM) THEN
      X(KP)=XP
      DO 4 I=K+1,N
      IP=IA(I)
      IK=IND(IP,KP)
      X(IP)=X(IP)+A(IK)*XP
    4 CONTINUE
      ELSE
      X(KP)=XM
      DO 5 I=K+1,N
      IP=IA(I)
      IK=IND(IP,KP)
      X(IP)=X(IP)+A(IK)*XM
    5 CONTINUE
      ENDIF
    6 CONTINUE
C
C     COMPUTATION OF THE VECTOR X SUCH THAT
C     D**(1/2)*TRANS(L)*X=V
C
      FM=ZERO
      DO 7 I=1,N
      IP=IA(I)
      II=IND(I,I)
      IF (JOB.LE.0) THEN
      FP=SQRT(A(II))
      X(IP)=X(IP)/FP
      FM=FM+X(IP)*X(IP)
      ELSE
      X(IP)=X(IP)/A(II)
      ENDIF
    7 CONTINUE
      FP=$DBLE(N)
      IF (JOB.LE.0) THEN
C
C     FIRST ESTIMATION OF THE MINIMUM EIGENVALUE BY THE FORMULA
C     ALF=(TRANS(U)*U)/(TRANS(V)*V)
C
      ALF=FP/FM
      RETURN
      ENDIF
      FM=ZERO
      DO 9 K=N,1,-1
      KP=IA(K)
      DO 8 I=K+1,N
      IP=IA(I)
      IK=IND(IP,KP)
      X(KP)=X(KP)-A(IK)*X(IP)
    8 CONTINUE
      FM=FM+X(KP)*X(KP)
    9 CONTINUE
      FM=SQRT(FM)
      IF (JOB.LE.1) THEN
C
C     SECOND ESTIMATION OF THE MINIMUM EIGENVALUE BY THE FORMULA
C     ALF=SQRT(TRANS(U)*U)/SQRT(TRANS(X)*X)
C
      ALF=SQRT(FP)/FM
      ELSE
C
C     INVERSE ITERATIONS
C
      DO 11 K=2,JOB
C
C     SCALING THE VECTOR X BY ITS NORM
C
      DO 10 I=1,N
      IP=IA(I)
      X(IP)=X(IP)/FM
   10 CONTINUE
      CALL UXDSGB(N,A,IA,X,0)
      FM=SQRT(UXVDOT(N,X,X))
   11 CONTINUE
      ALF=ONE/FM
      ENDIF
C
C     SCALING THE VECTOR X BY ITS NORM
C
      DO 12 I=1,N
      IP=IA(I)
      X(IP)=X(IP)/FM
   12 CONTINUE
      RETURN
      END
