* SUBROUTINE UXDSGU                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* CORRECTION OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX A+E IN THE
* FACTORED FORM A+E=P*L*D*TRANS(L)*TRANS(P) OBTAINED BY THE SUBROUTINE
* UXDSGF. THE CORRECTION IS DEFINED AS A+E:=A+E+ALF*X*TRANS(X) WHERE
* ALF IS A GIVEN SCALING FACTOR AND X IS A GIVEN VECTOR.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RU  A(N*(N+1)/2) FACTORIZATION A+E=L*D*TRANS(L) OBTAINED BY THE
*         SUBROUTINE UXDPGF.
*  II  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RI  ALF  SCALING FACTOR IN THE CORRECTION TERM.
*  RI  X(N)  VECTOR IN THE CORRECTION TERM.
*  RA  Y(N) AUXILIARY VECTOR.
*
* METHOD :
* P.E.GILL, W.MURRAY, M.SAUNDERS: METHODS FOR COMPUTING AND MODIFYING
* THE LDV FACTORS OF A MATRIX, MATH. OF COMP. 29 (1974) PP. 1051-1077.
*
      SUBROUTINE UXDSGU( N, A, IA, ALF, X, Y)
      INTEGER N, IA(N)
      $FLOAT  A(N*(N+1)/2), ALF, X(N), Y(N)
      $FLOAT  B, D, P, R, T, TO, ALFR
      INTEGER  I, J, IP, JP, II, IJ, IND
      $FLOAT ZERO, ONE, FOUR, CON
      PARAMETER  (ZERO = 0.0$P 0, ONE = 1.0$P 0,
     &            FOUR = 4.0$P 0, CON=1.0$P-8)
      IND(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
      IF (ALF .GE. ZERO) THEN
C
C     FORWARD CORRECTION IN CASE WHEN THE SCALING FACTOR IS NONNEGATIVE
C
      ALFR = SQRT(ALF)
      CALL UXVSCL(N, ALFR, X, Y)
      TO = ONE
      DO 3  I = 1, N
      IP = IA(I)
      II = IND(IP,IP)
      D = A(II)
      P = Y(IP)
      T = TO + P*P/D
      R = TO / T
      A(II) = D / R
      B = P / (D*T)
      IF (A(II) .LE. FOUR * D) THEN
C
C     AN EASY FORMULA FOR LIMITED DIAGONAL ELEMENT
C
      DO 1  J = I+1, N
      JP = IA(J)
      IJ = IND(IP,JP)
      D = A(IJ)
      Y(JP) = Y(JP) - P*D
      A(IJ) = D + B*Y(JP)
    1 CONTINUE
      ELSE
C
C     A MORE COMPLICATE BUT NUMERICALLY STABLE FORMULA FOR UNLIMITED
C     DIAGONAL ELEMENT
C
      DO 2  J = I+1, N
      JP = IA(J)
      IJ = IND(IP,JP)
      D = A(IJ)
      A(IJ) = R*D + B*Y(JP)
      Y(JP) = Y(JP) - P*D
    2 CONTINUE
      ENDIF
      TO = T
    3 CONTINUE
      ELSE
C
C     BACKWARD CORRECTION IN CASE WHEN THE SCALING FACTOR IS NEGATIVE
C
      ALFR = SQRT( -ALF)
      CALL UXVSCL(N, ALFR, X, Y)
      TO = ONE
      DO 5  I = 1, N
      IP = IA(I)
      II = IND(IP,IP)
      D = Y(IP)
      DO 4  J = 1, I-1
      JP = IA(J)
      IJ = IND(IP,JP)
      D = D - A(IJ) * Y(JP)
    4 CONTINUE
      Y(IP) = D
      TO = TO - D*D/A(II)
    5 CONTINUE
      IF (TO .LE. ZERO)  TO = CON
      DO 7  I = N, 1, -1
      IP = IA(I)
      II = IND(IP,IP)
      D = A(II)
      P = Y(IP)
      T = TO + P*P/D
      A(II) = D * TO / T
      B = -P / (D*TO)
      TO = T
      DO 6  J = I+1, N
      JP = IA(J)
      IJ = IND(IP,JP)
      D = A(IJ)
      A(IJ) = D + B*Y(JP)
      Y(JP) = Y(JP) + P*D
    6 CONTINUE
    7 CONTINUE
      ENDIF
      RETURN
      END
