* SUBROUTINE UXDTGV                ALL SYSTEMS               91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* ROW AND COLUMN PERMUTATION OF THE FACTORIZATION
* A=TRANS(P)*TRANS(R)*R*P.
*
* PARAMETERS :
*  II  N  ORDER OF THE MATRIX A.
*  RU  A(N*(N+1)/2) UPPER TRIANGULAR MATRIX.
*  IU  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RU  B(N)  RIGHT HAND SIDE VECTOR OBTAINED USING ORTHOGONAL
*         TRANSFORMATION OF A SET OF LINEAR EQUATIONS.
*  II  KI  FIRST INDEX FOR ROW AND COLUMN PERMUTATION.
*  II  MI  SECOND INDEX FOR ROW AND COLUMN PERMUTATION.
*
* SUBPROGRAMS USED :
*  S   UXVORT  COMPUTATION OF THE PLANE ROTATION MATRIX.
*  S   UXVROT  PLANE ROTATION IS APPLIED TO TWO NUMBERS.
*
      SUBROUTINE UXDTGV(N,A,IA,B,KI,MI)
      INTEGER N,IA(N),KI,MI
      $FLOAT A(N*(N+1)/2),B(N)
      $FLOAT S,C
      INTEGER I,J,K,M,IP,JP,KP,LP,MP,II,IJ,IK,IM,LJ,LL,LM,MM,IER,IND
      IND(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
      IF (KI.NE.MI) THEN
      K=MIN(KI,MI)
      M=MAX(KI,MI)
C
C     ROTATIONS APPLIED TO THE MI-TH COLUMN OF THE MATRIX A
C
      KP=IA(K)
      MP=IA(M)
      MM=IND(MP,MP)
      IP=MP
      IM=MM
      DO 3 I=M-1,K,-1
      LP=IP
      LM=IM
      IP=IA(I)
      II=IND(IP,IP)
      IM=IND(IP,MP)
      CALL UXVORT(A(IM),A(LM),C,S,IER)
      CALL UXVROT(A(II),A(LM),C,S,IER)
      DO 2 J=I+1,N
      IF (J.NE.M) THEN
      JP=IA(J)
      IJ=IND(IP,JP)
      LJ=IND(LP,JP)
      CALL UXVROT(A(IJ),A(LJ),C,S,IER)
      ENDIF
    2 CONTINUE
      CALL UXVROT(B(I),B(I+1),C,S,IER)
    3 CONTINUE
C
C     ROTATIONS APPLIED TO THE NEW SUBDIAGONAL ELEMENTS OF THE MATRIX A
C
      DO 5 I=K+2,M
      LL=IND(LP,LP)
      IP=IA(I)
      IM=IND(IP,MP)
      CALL UXVORT(A(LL),A(IM),C,S,IER)
      CALL UXVROT(A(LM),A(IM),C,S,IER)
      DO 4 J=I,N
      IF (J.NE.M) THEN
      JP=IA(J)
      LJ=IND(LP,JP)
      IJ=IND(IP,JP)
      CALL UXVROT(A(LJ),A(IJ),C,S,IER)
      ENDIF
    4 CONTINUE
      CALL UXVROT(B(I-1),B(I),C,S,IER)
      LP=IP
      LM=IM
    5 CONTINUE
C
C     FINAL PERMUTATIONS
C
      DO 6 I=N,K,-1
      IP=IA(I)
      IK=IND(IP,KP)
      IM=IND(IP,MP)
      C=A(IK)
      A(IK)=A(IM)
      A(IM)=C
    6 CONTINUE
      IA(M)=KP
      IA(K)=MP
      ENDIF
      RETURN
      END
