* SUBROUTINE UXEGGV                ALL SYSTEMS                16/12/01
C PORTABILITY : ALL SYSTEMS
C 16/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* RANK ONE UPDATE OF FACTORIZATION P*A=L*U OBTAINED BY THE SUBROUTINE
* UXEGGF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  ND  DECLARED ORDER OF THE MATRIX A.
*  RU  A(N,N) THE FACTORIZATION P*A=L*U COMPUTED BY THE SUBROUTINE 
*         UXEGGF.
*  IU  IA(N) A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  IO  IB(N) NEW PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RU  X(N) INPUT VECTOR.
*  RU  Y(N) INPUT VECTOR.
*  RU  D(N) AUXILIARY VECTOR
*  RU  Z(N) AUXILIARY VECTOR
*  IO  INF INDICATOR. INF=0 - CORRECT COMPUTATION. INF=1 - DIVISION BY 
*         ZERO.       
*
* METHOD :
* BACK SUBSTITUTION.
*
      SUBROUTINE UXEGGV(N,ND,A,IA,IB,X,Y,D,Z,INF)
      INTEGER N,ND,IA(N),IB(N),INF
      $FLOAT A(ND,N),X(N),Y(N),D(N),Z(N)
      INTEGER I,J,K,KP1,KP2
      $FLOAT Q,TAU,ZERO,ONE
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
      INF=0
      CALL UXVIPR(N,IA,IB,1)
C      
C     TRANSFORMATION OF THE INPUT VECTOR
C
      CALL UXVPER(N,IA,X,1)
      DO 20 K=1,N
      IF (X(K).NE.ZERO) THEN
      DO 10 I=K+1,N
      X(I)=X(I)-A(I,K)*X(K)
   10 CONTINUE
      ENDIF
   20 CONTINUE      
C      
C     THE BACKWARD CYCLE - ZEROING ELEMENTS OF THE INPUT VECTOR
C
      D(N)=ONE
      DO 30 K=N-1,1,-1
      D(K)=ONE
      KP1=K+1
      KP2=K+2
      IF (X(KP1).NE.ZERO) THEN
C      IF (ABS(X(K)).LT.1.0$P-1*ABS(X(KP1))) THEN
      IF (ABS(X(K)).LT.1.0$P-1*ABS(A(KP1,K)*X(K)+X(KP1))) THEN 
C      
C     POSSIBLE PERMUTATION
C
      CALL UXVSWP(K-1,A(K,1),A(KP1,1),ND)
      IF (KP2.LE.N) CALL UXVSWP(N-KP1,A(KP2,K),A(KP2,KP1),1)
      CALL UXVSWP(N-K,A(K,KP1),A(KP1,KP1),ND)
      CALL UXVSWP(1,D(K),D(KP1),1)
      CALL UXVSWP(1,X(K),X(KP1),1)
      Q=A(KP1,K)
      A(KP1,K)=ZERO
      Z(K)=A(K,K)
      A(K,K)=ZERO
      J=IB(K)
      IB(K)=IB(KP1)
      IB(KP1)=J
      ELSE
      Q=ZERO
      Z(K)=ZERO
      ENDIF 
C      
C     ZEROING THE ELEMENT X(K+1)
C
      TAU=-X(KP1)/X(K)
      CALL UXVELT(N-K,A(K,KP1),A(KP1,KP1),ND,TAU)
      Z(K)=Z(K)+TAU*A(K,K)
      IF (KP2.LE.N) CALL UXVELT(N-KP1,A(KP2,KP1),A(KP2,K),1,-TAU)
      D(K)=D(K)-TAU*Q
      A(KP1,K)=A(KP1,K)-TAU*D(KP1)  
C      
C     ZEROING THE ELEMENT L(K,K+1)
C
      IF (Q.NE.ZERO) THEN
      TAU=-Q/D(K)
      D(K+1)=D(K+1)+TAU*A(KP1,K)
      IF (KP2.LE.N) CALL UXVELT(N-KP1,A(KP2,K),A(KP2,KP1),1,TAU)
      CALL UXVELT(N-K,A(KP1,KP1),A(K,KP1),ND,-TAU)
      A(K,K)=A(K,K)-TAU*Z(K)
      ENDIF
      ELSE
      Z(K)=ZERO
      ENDIF
   30 CONTINUE
C      
C     UPDATE OF THE UPPER HESSENBERG MATRIX
C
      DO 40 K=1,N
      A(1,K)=A(1,K)+X(1)*Y(K)
   40 CONTINUE     
C      
C     THE FORWARD CYCLE - ZEROING OFF-DIAGONAL ELEMENTS OF THE UPPER 
C     HESSENBERG MATRIX
C
      DO 50 K=1,N-1
      KP1=K+1
      KP2=K+2
      IF (Z(K).NE.ZERO) THEN 
C      IF (ABS(A(K,K)).LT.1.0$P-1*ABS(Z(K))) THEN
      IF (ABS(A(K,K)).LT.1.0$P-1*ABS(A(KP1,K)*A(K,K)+Z(K))) THEN 
C      
C     POSSIBLE PERMUTATION
C
      CALL UXVSWP(K-1,A(K,1),A(KP1,1),ND)
      IF (KP2.LE.N) CALL UXVSWP(N-KP1,A(KP2,K),A(KP2,KP1),1)
      CALL UXVSWP(N-K,A(K,KP1),A(KP1,KP1),ND)
      CALL UXVSWP(1,D(K),D(KP1),1)
      CALL UXVSWP(1,A(K,K),Z(K),1)
      Q=A(KP1,K)
      A(KP1,K)=ZERO
      J=IB(K)
      IB(K)=IB(KP1)
      IB(KP1)=J
      ELSE
      Q=ZERO
      ENDIF 
C      
C     ZEROING THE ELEMENT U(K+1,K)
C
      TAU=-Z(K)/A(K,K)
      CALL UXVELT(N-K,A(K,KP1),A(KP1,KP1),ND,TAU)
      IF (KP2.LE.N) CALL UXVELT(N-KP1,A(KP2,KP1),A(KP2,K),1,-TAU)
      D(K)=D(K)-TAU*Q
      A(KP1,K)=A(KP1,K)-TAU*D(KP1)  
C      
C     ZEROING THE ELEMENT L(K,K+1)
C
      IF (Q.NE.ZERO) THEN
      TAU=-Q/D(K)
      D(K+1)=D(K+1)+TAU*A(KP1,K)
      IF (KP2.LE.N) CALL UXVELT(N-KP1,A(KP2,K),A(KP2,KP1),1,TAU)
      CALL UXVELT(N-K,A(KP1,KP1),A(K,KP1),ND,-TAU)
      ENDIF
      ENDIF
  50  CONTINUE       
C      
C     FINAL SCALING
C
      DO 60 K=1,N
      IF (D(K).NE.ONE) THEN
      IF (K.LT.N) CALL UXVSCG(N-K,ONE/D(K),A(K+1,K),A(K+1,K),1)
      CALL UXVSCG(N-K+1,D(K),A(K,K),A(K,K),ND)
      ENDIF
   60 CONTINUE
      CALL UXVIPR(N,IB,IA,-1)   
      RETURN
      END       
        
      
