* SUBROUTINE UXSGFB                 ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A SPARSE NONSYMMETRIC
* MATRIX A USING THE FACTORIZATION P A Q = L*U.
*
* PARAMETERS:
*  II  N  SIZE OF THE SYSTEM SOLVED.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX - OF THE ARRAY AU.
*  II  NEL  NUMBER OF TRIPLES IN ARRAYS (LI,LJ,CM).
*  II  NUP  DECLARED LENGTH OF THE ARRAYS LI,LJ,CM.
*  II  PTL(N) POINTER VECTOR OF THE COLUMNS OF THE L-FACTOR.
*  II  LENL(N) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  II  CNR(NZ) ARRAY THAT CONTAINS THE INDICES OF THE FACTORS.
*  II  PTR(N) POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  II  LENR(N) LENGTHS OF THE ROWS OF THE FACTOR U.
*  II  ORDC(N) COLUMN PERMUTATION OF THE FACTOR U.
*  II  ORDR(N) ROW PERMUTATION OF THE FACTOR U.
*  II  ORDL(N) ROW PERMUTATION OF THE ORIGINAL FACTOR L.
*  II  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  II  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RI  CM(NUP) UPDATE MULTIPLIERS.
*  RI  AU(NZ) NUMERICAL VALUES OF THE FACTORS.
*  RU  RHS(N) ON INPUT : RIGHT HAND SIDE OF THE SYSTEM. ON OUTPUT
*             SYSTEM SOLUTION.
*  RA  WRO1(N)  AUXILIARY VECTOR.
*  II  JOB  OPTION. IF JOB=0 THEN X:=A**(-1)*X. IF JOB>0 THEN
*         X:=L**(-1)*X. IF JOB<0 THEN X:=U**(-1)*X.
*
* METHOD :
* BACK SUBSTITUTION
*
      SUBROUTINE UXSGFB(N,NZ,NAU,NEL,NUP,PTL,LENL,CNR,PTR,
     *                  LENR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,RHS,WR01,JOB)
      INTEGER N,NZ,NAU,NEL,NUP,JOB
      INTEGER PTL(N),PTR(N),LENL(N),LENR(N),ORDR(N),ORDC(N),ORDL(N)
      INTEGER CNR(NZ)
      INTEGER LI(NUP),LJ(NUP)
      $FLOAT CM(NUP)
      $FLOAT AU(NAU)
      $FLOAT RHS(N),WR01(N)
      INTEGER I,J,K,L,I1
C
C     FTRANL
C
      IF(JOB.LT.0) GO TO 400
      DO 300 I=1,N
      L=ORDL(I)
            DO 200 J=PTL(I),PTL(I)+LENL(I)-1
            K=CNR(J)
            RHS(K)=RHS(K)-RHS(L)*AU(J)
200         CONTINUE
300   CONTINUE
C
C     L-UPDATES ARE USED
C
      DO 500 I=1,NEL
      RHS(LI(I))=RHS(LI(I))-CM(I)*RHS(LJ(I))
500   CONTINUE
      IF(JOB.GT.0) GO TO 900
C
C     JUMP LABEL IF JOB.LT.0
C
400   CONTINUE
C
C     BTRANU
C
      DO 800 I=N,1,-1
            I1=ORDR(I)
            DO 700 J=PTR(I1)+1,PTR(I1)+LENR(I1)-1
            K=CNR(J)
            RHS(I1)=RHS(I1)-AU(J)*WR01(K)
700         CONTINUE
            WR01(I)=RHS(I1)/AU(PTR(I1))
800   CONTINUE
      DO 850 I=1,N
      RHS(ORDC(I))=WR01(I)
850   CONTINUE
C
C     JUMP LABEL IF JOB.GT.0
C
900   CONTINUE
      RETURN
      END
