* SUBROUTINE UXSIB2                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DU : ORIGINAL VERSION
*
* PURPOSE :
* AUXILIARY SUBROUTINE FOR UXSIBB.
* THIS SUBROUTINE PERFORMS BACKWARD ELIMINATION OPERATIONS
*
      SUBROUTINE UXSIB2(N, A, LA, IW, MW, W, MAXFNT, RHS, IW2, NBLK,
     & LATOP)
C
C     USING THE FACTORS DINVERSE AND U
C     STORED IN A/IW AFTER UXSIBF.
C
      INTEGER N, LA, MW, MAXFNT, NBLK, LATOP
      INTEGER IW2(NBLK), APOS, APOS2
      INTEGER IW(MW)
      DOUBLE PRECISION A(LA), RHS(N), W(MAXFNT), W1, W2
C     N - MUST BE SET TO THE ORDER OF THE MATRIX. NOT ALTERED
C         BY UXSIB2.
C     A - MUST BE SET TO HOLD THE REAL VALUES CORRESPONDING
C         TO THE FACTORS OF DINVERSE AND U.  THIS MUST BE
C         UNCHANGED SINCE THE PRECEDING CALL TO UXSIBF.  NOT ALTERED
C         BY UXSIB2.
C     LA- LENGTH OF ARRAY A. NOT ALTERED BY UXSIB2.
C     IW- HOLDS THE INTEGER INDEXING
C         INFORMATION FOR THE MATRIX FACTORS IN A.  THIS MUST BE
C         UNCHANGED SINCE THE PRECEDING CALL TO UXSIBF.  NOT ALTERED
C         BY UXSIB2.
C     MW- LENGTH OF ARRAY IW.  NOT ALTERED BY UXSIB2.
C     W - USED
C         AS WORKSPACE BY UXSIB2 TO HOLD THE COMPONENTS OF THE RIGHT
C         HAND SIDES CORRESPONDING TO CURRENT BLOCK PIVOTAL ROWS.
C     MAXFNT- INTEGER VARIABLE.  MUST BE SET TO THE LARGEST NUMBER OF
C         VARIABLES IN ANY BLOCK PIVOT ROW.  THIS VALUE WAS GIVEN
C         ON OUTPUT FROM UXSIBF.  NOT ALTERED BY UXSIB2.
C     RHS - ON INPUT,
C         MUST BE SET TO HOLD THE RIGHT HAND SIDE MODIFIED BY THE
C         FORWARD SUBSTITUTION OPERATIONS.  ON OUTPUT, RHS WILL HOLD
C         THE SOLUTION VECTOR.
C     IW2 - ON ENTRY IW2(I) (I = 1,NBLK)
C         MUST HOLD POINTERS TO THE
C         BEGINNING OF EACH BLOCK PIVOT IN ARRAY IW, AS SET BY
C         UXSIB1.
C     NBLK - NUMBER OF BLOCK PIVOT ROWS. NOT ALTERED BY UXSIB2.
C     LATOP- IT IS THE POSITION IN
C         A OF THE LAST ENTRY IN THE  FACTORS. IT MUST BE UNCHANGED
C         SINCE THE CALL TO UXSIB1.  IT IS NOT ALTERED BY UXSIB2.
      INTEGER IBLK, IFR, IFRLVL, ILVL, IOVFLO, IPIV,  IPOS, IRHS,
     &        I1RHS, I2RHS, IIPIV, IIRHS,  IST, JPIV,
     &        J, J1, J2, JJ,  JJ1, JJ2, JPOS, K,  LIELL,
     &        LOOP, NEMIN, NPIV
      INTRINSIC MIN0, IABS
C
C     SEE BLOCK DATA FOR COMMENTS ON VARIABLES IN COMMON
C     IFRLVL IS USED TO CONTROL WHETHER DIRECT OR INDIRECT ACCESS IS
C     USED BY UXSIBB.  INDIRECT ACCESS IS EMPLOYED
C     IN FORWARD AND BACK SUBSTITUTION RESPECTIVELY IF THE SIZE OF
C     A BLOCK IS LESS THAN IFRLVL(MIN0(10,NPIV)) AND
C     IFRLVL(10+MIN0(10,NPIV)) RESPECTIVELY, WHERE NPIV IS THE NUMBER
C     OF PIVOTS IN THE BLOCK.
C
      COMMON /UMMAT3/ IOVFLO, NEMIN, IFRLVL(20)
C
C     APOS. RUNNING POINTER TO CURRENT PIVOT POSITION IN ARRAY A.
C     IPOS. RUNNING POINTER TO BEGINNING OF CURRENT BLOCK PIVOT ROW.
C
      APOS = LATOP + 1
      NPIV = 0
      IBLK = NBLK + 1
C
C     RUN THROUGH BLOCK PIVOT ROWS IN THE REVERSE ORDER.
C
      DO 180 LOOP=1,N
        IF (NPIV.GT.0) GO TO 110
        IBLK = IBLK - 1
        IF (IBLK.LT.1) GO TO 190
        IPOS = IW2(IBLK)
C
C     IABS(LIELL) IS NUMBER OF VARIABLES (COLUMNS) IN BLOCK PIVOT ROW.
C
        LIELL = -IW(IPOS)
C
C     NPIV IS NUMBER OF PIVOTS (ROWS) IN BLOCK PIVOT.
C
        NPIV = 1
        IF (LIELL.GT.0) GO TO 10
        LIELL = -LIELL
        IPOS = IPOS + 1
        NPIV = IW(IPOS)
   10   JPOS = IPOS + NPIV
        J2 = IPOS + LIELL
        ILVL = MIN0(10,NPIV) + 10
        IF (LIELL.LT.IFRLVL(ILVL)) GO TO 110
C
C     PERFORM OPERATIONS USING DIRECT ADDRESSING.
C
        J1 = IPOS + 1
C
C     LOAD APPROPRIATE COMPONENTS OF RHS INTO W.
C
        IFR = 0
        DO 20 JJ=J1,J2
          J = IABS(IW(JJ)+0)
          IFR = IFR + 1
          W(IFR) = RHS(J)
   20   CONTINUE
C
C     JPIV IS USED AS A FLAG SO THAT IPIV IS INCREMENTED CORRECTLY
C     AFTER THE USE OF A 2 BY 2 PIVOT.
C
        JPIV = 1
C
C     PERFORM ELIMINATIONS.
C
        DO 90 IIPIV=1,NPIV
          JPIV = JPIV - 1
          IF (JPIV.EQ.1) GO TO 90
          IPIV = NPIV - IIPIV + 1
          IF (IPIV.EQ.1) GO TO 30
C
C     JUMP IF WE HAVE A 2 BY 2 PIVOT.
C
          IF (IW(JPOS-1).LT.0) GO TO 60
C
C     PERFORM BACK-SUBSTITUTION USING 1 BY 1 PIVOT.
C
   30     JPIV = 1
          APOS = APOS - (LIELL+1-IPIV)
          IST = IPIV + 1
          W1 = W(IPIV)*A(APOS)
          IF (LIELL.LT.IST) GO TO 50
          JJ1 = APOS + 1
          DO 40 J=IST,LIELL
            W1 = W1 + A(JJ1)*W(J)
            JJ1 = JJ1 + 1
   40     CONTINUE
   50     W(IPIV) = W1
          JPOS = JPOS - 1
          GO TO 90
C
C     PERFORM BACK-SUBSTITUTION OPERATIONS WITH 2 BY 2 PIVOT
C
   60     JPIV = 2
          APOS2 = APOS - (LIELL+1-IPIV)
          APOS = APOS2 - (LIELL+2-IPIV)
          IST = IPIV + 1
          W1 = W(IPIV-1)*A(APOS) + W(IPIV)*A(APOS+1)
          W2 = W(IPIV-1)*A(APOS+1) + W(IPIV)*A(APOS2)
          IF (LIELL.LT.IST) GO TO 80
          JJ1 = APOS + 2
          JJ2 = APOS2 + 1
          DO 70 J=IST,LIELL
            W1 = W1 + W(J)*A(JJ1)
            W2 = W2 + W(J)*A(JJ2)
            JJ1 = JJ1 + 1
            JJ2 = JJ2 + 1
   70     CONTINUE
   80     W(IPIV-1) = W1
          W(IPIV) = W2
          JPOS = JPOS - 2
   90   CONTINUE
C
C     RELOAD WORKING VECTOR INTO SOLUTION VECTOR.
C
        IFR = 0
        DO 100 JJ=J1,J2
          J = IABS(IW(JJ)+0)
          IFR = IFR + 1
          RHS(J) = W(IFR)
  100   CONTINUE
        NPIV = 0
        GO TO 180
C
C     PERFORM OPERATIONS USING INDIRECT ADDRESSING.
C
  110   IF (NPIV.EQ.1) GO TO 120
C
C     JUMP IF WE HAVE A 2 BY 2 PIVOT.
C
        IF (IW(JPOS-1).LT.0) GO TO 150
C
C     PERFORM BACK-SUBSTITUTION USING 1 BY 1 PIVOT.
C
  120   NPIV = NPIV - 1
        APOS = APOS - (J2-JPOS+1)
        IIRHS = IW(JPOS)
        W1 = RHS(IIRHS)*A(APOS)
        J1 = JPOS + 1
        IF (J1.GT.J2) GO TO 140
        K = APOS + 1
        DO 130 J=J1,J2
          IRHS = IABS(IW(J)+0)
          W1 = W1 + A(K)*RHS(IRHS)
          K = K + 1
  130   CONTINUE
  140   RHS(IIRHS) = W1
        JPOS = JPOS - 1
        GO TO 180
C
C     PERFORM OPERATIONS WITH 2 BY 2 PIVOT
C
  150   NPIV = NPIV - 2
        APOS2 = APOS - (J2-JPOS+1)
        APOS = APOS2 - (J2-JPOS+2)
        I1RHS = -IW(JPOS-1)
        I2RHS = IW(JPOS)
        W1 = RHS(I1RHS)*A(APOS) + RHS(I2RHS)*A(APOS+1)
        W2 = RHS(I1RHS)*A(APOS+1) + RHS(I2RHS)*A(APOS2)
        J1 = JPOS + 1
        IF (J1.GT.J2) GO TO 170
        JJ1 = APOS + 2
        JJ2 = APOS2 + 1
        DO 160 J=J1,J2
          IRHS = IABS(IW(J)+0)
          W1 = W1 + RHS(IRHS)*A(JJ1)
          W2 = W2 + RHS(IRHS)*A(JJ2)
          JJ1 = JJ1 + 1
          JJ2 = JJ2 + 1
  160   CONTINUE
  170   RHS(I1RHS) = W1
        RHS(I2RHS) = W2
        JPOS = JPOS - 2
  180 CONTINUE
  190 RETURN
      END
