* SUBROUTINE UXSIBS                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DU : ORIGINAL VERSION
*
* PURPOSE :
* THIS SUBROUTINE COMPUTES A MINIMUM DEGREE ORDERING OR ACCEPTS A GIVEN
* ORDERING. IT COMPUTES ASSOCIATED ASSEMBLY AND ELIMINATION INFORMATION
* FOR UXSIBF.
*
* PARAMETERS :
*  II  N  MUST BE SET TO THE ORDER OF THE MATRIX. IT IS NOT ALTERED.
*  II  M  MUST BE SET TO THE NUMBER OF NON-ZEROS. IT IS NOT ALTERED.
*  II  KA(M)  ENTRY K (K=1,M) OF KA MUST BE SET TO THE ROW INDEX OF
*         THE NON-ZERO IN A. KA IS UNALTERED BY UXSIBS.
*  II  JA(M)  ENTRY K (K=1,M) OF JA MUST BE SET TO THE COLUMN INDEX
*         OF THE NON-ZERO IN A. JA IS UNALTERED BY UXSIBS.
*  IA  IW(MW)  USED AS WORKSPACE.
*  II  MW  LENGTH OF ARRAY IW. IT MUST BE AT LEAST 2*M+3*N FOR THE
*         INF=0 ENTRY AND AT LEAST M+3*N FOR THE INF=1 ENTRY. IT IS
*         ALTERED.
*  IA  IW1(N,2)  USED AS WORKSPACE.
*  IO  IW2(N,3) CONTAINS INFORMATION NEEDED BY UXSIBF. IW2(I,1) HOLDS
*         THE POSITION OF VARIABLE I IN IN THE PIVOT ORDER. IW2(I,2),
*         IW2(I,3) HOLD THE NUMBER OF ELIMINATIONS, ASSEMBLIES AT
*         MAJOR STEP I, I=1,2,...,NS. NOTE THAT WHEN AN ORDER IS GIVEN
*         IT MAY BE REPLACED BY ANOTHER ORDER THAT GIVES IDENTICAL
*         NUMERICAL RESULTS.
*  IO  NS  IT CONTAINS THE NUMBER OF MAJOR STEPS NEEDED FOR A DEFINITE
*         MATRIX AND MUST BE PASSED UNCHANGED TO UXSIBF.
*  IO  INF  A ZERO VALUE INDICATES NO ERROR HAS BEEN DETECTED.
*
      SUBROUTINE UXSIBS(N, M, KA, JA, IW, MW, IW1, IW2, NS,
     & ITERM)
      INTEGER N, M, MW, NS, ITERM
      INTEGER KA(*),JA(*),IW(MW),IW1(N,2),IW2(N,3)
      INTEGER IERROR,  IFRLVL, IOVFLO, IWFR,
     &        L1, L2, LDIAG, LMW, LP, MP,
     &        NCMPA, NCMPBI, NCMPBR, NEIG, NEMIN,
     &        NIRADU,  NIRBDU, NIRNEC, NIRTOT, NRLADU, NRLBDU,
     &        NRLNEC, NRLTOT,  NTWO, INF
      DOUBLE PRECISION U, OPS
      INTRINSIC MIN0
      COMMON /UMMAT1/ U, LP, MP, LDIAG
      COMMON /UMMAT2/ OPS, IERROR, NRLTOT, NIRTOT, NRLNEC, NIRNEC,
     & NRLADU, NIRADU, NRLBDU, NIRBDU, NCMPA, NCMPBR, NCMPBI, NTWO,NEIG
      COMMON /UMMAT3/ IOVFLO, NEMIN, IFRLVL(20)
      IF (N.LT.1 .OR. N.GT.IOVFLO) GO TO 70
      IF (M.LT.0) GO TO 100
      LMW = MW - 2*N
      L1 = LMW + 1
      L2 = L1 + N
      IF (MW.LT.2*M+3*N+1) GO TO 130
C
C     SORT
C
      CALL UXSIS1(N, M, KA, JA, IW, LMW, IW1, IW1(1,2), IW(L1),
     & IWFR, INF)
      IF (INF.NE.0) THEN
      CALL UOERR1('UXSIS1',73)
      ITERM=-73
      RETURN
      ENDIF
C
C     ANALYZE USING MINIMUM DEGREE ORDERING
C
      CALL UXSIS2(N, IW1, IW, LMW, IWFR, IW(L1), IW(L2), IW2(1,2),
     & IW2(1,3), IW2)
      GO TO 60
C
C     SORT USING GIVEN ORDER
C
      IF (MW.LT.M+3*N+1) GO TO 120
      CALL UXSIS4(N, M, KA, JA, IW2, IW, LMW, IW1, IW1(1,2),
     & IW(L1), IWFR, INF)
      IF (INF.NE.0) THEN
      CALL UOERR1('UXSIS4',73)
      ITERM=-73
      RETURN
      ENDIF
C
C     ANALYZE USING GIVEN ORDER
C
      CALL UXSIS5(N, IW1, IW, LMW, IWFR, IW2, IW2(1,2), IW(L1),
     & IW(L2))
C
C     PERFORM DEPTH-FIRST SEARCH OF ASSEMBLY TREE
C
   60 CALL UXSIS6(N, IW1, IW(L1), IW2, IW2(1,2), IW2(1,3),
     & IW(L2), NS)
C
C     EVALUATE STORAGE AND OPERATION COUNT REQUIRED BY UXSIBF IN THE
C     DEFINITE CASE.
C     SET IW(1) SO THAT ARRAYS IW AND KA CAN BE TESTED FOR EQUIVALENCE
C     IN UXSIS7.
C
      IW(1)=KA(1)+1
      CALL UXSIS7(N, M, KA, JA, IW2, IW2(1,3), IW2(1,2),
     & IW(L2), NS, IW1, IW1(1,2), IW)
      RETURN
   70 INF = -1
      ITERM = -61
      CALL UOERR1('UXSIBS',61)
      RETURN
  100 INF = -2
      ITERM = -62
      CALL UOERR1('UXSIBS',62)
      RETURN
  120 IERROR = M + 3*N + 1
      GO TO 140
  130 IERROR = 2*M + 3*N + 1
  140 INF = -3
      ITERM = -65
      CALL UOERR1('UXSIBS',65)
      RETURN
      END
      BLOCK DATA UXSIB3
      INTEGER IFRLVL, IOVFLO,
     &        LDIAG,  LP,
     &        MP,
     &        NEMIN
      DOUBLE PRECISION U
C
C     U IS USED TO CONTROL THRESHOLD PIVOTING. IT IS NOT ALTERED.
C     LP MUST BE SET TO THE STREAM NUMBER FOR ERROR MESSAGES.
C     ERROR MESSAGES ARE SUPPRESSED IF LP IS NOT POSITIVE.
C     IT IS NOT ALTERED.
C     MP MUST BE SET TO THE STREAM NUMBER FOR DIAGNOSTIC MESSAGES.
C     DIAGNOSTIC MESSAGES ARE SUPPRESSED IF MP IS NOT POSITIVE.
C     IT IS NOT ALTERED.
C     LDIAG CONTROLS THE LEVEL OF DIAGNOSTIC PRINTING.
C     0 NO PRINTING
C     1 PRINTING OF SCALAR PARAMETERS AND FIRST PARTS OF ARRAYS.
C     2 PRINTING OF SCALAR PARAMETERS AND WHOLE OF ARRAYS.
C
      COMMON /UMMAT1/ U, LP, MP, LDIAG
C
C     OPS ACCUMULATES THE NO. OF MULTIPLY/ADD PAIRS NEEDED TO CREATE THE
C     TRIANGULAR FACTORIZATION, IN THE DEFINITE CASE.
C     IERROR HOLDS ADDITIONAL INFORMATION IN THE EVENT OF ERRORS.
C     IF INF=-3 IERROR HOLDS A LENGTH THAT MAY SUFFICE FOR IW.
C     IF INF=-4 IERROR HOLDS A LENGTH THAT MAY SUFFICE FOR A.
C     IF INF=-5 IERROR IS SET TO THE PIVOT STEP AT WHICH SINGULARITY
C               WAS DETECTED.
C     IF INF=-6 IERROR IS SET TO THE PIVOT STEP AT WHICH A CHANGE OF
C               PIVOT SIGN WAS FOUND.
C     IF INF= 1 IERROR HOLDS THE NUMBER OF FAULTY ENTRIES.
C     IF INF= 2 IERROR IS SET TO THE NUMBER OF SIGNS CHANGES IN
C               THE PIVOTS.
C     IF INF= 3 IERROR IS SET TO THE RANK OF THE MATRIX.
C     NRLTOT,NIRTOT REAL AND INTEGER STRORAGE RESPECTIVELY REQUIRED
C     FOR THE FACTORIZATION IF NO COMPRESSES ARE ALLOWED.
C     NRLNEC,NIRNEC REAL AND INTEGER STORAGE RESPECTIVELY REQUIRED FOR
C     THE FACTORIZATION IF COMPRESSES ARE ALLOWED AND THE MATRIX IS
C     DEFINITE.
C     NRLADU,NIRADU REAL AND INTEGER STORAGE RESPECTIVELY FOR THE
C     MATRIX FACTORS AS CALCULATED BY UXSIBS FOR THE DEFINITE CASE.
C     NRLBDU,NIRBDU REAL AND INTEGER STORAGE RESPECTIVELY FOR THE
C     MATRIX FACTORS AS FOUND  BY UXSIBF.
C     NCMPA ACCUMULATES THE NUMBER OF TIMES THE ARRAY IW IS COMPRESSED
C     BY UXSIBS.
C     NCMPBR AND NCMPBI ACCUMULATE THE NUMBER OF COMPRESSES OF THE
C     REALS AND INTEGERS PERFORMED BY UXSIBF.
C     NTWO IS USED BY UXSIBF TO RECORD THE NUMBER OF 2*2 PIVOTS USED.
C     NEIG IS USED BY ME27B/BD TO RECORD THE NUMBER OF NEGATIVE EIGEN-
C     VALUES OF A.
C     IOVFLO IS THE LARGEST INTEGER SUCH THAT ALL INTEGERS I IN THE
C     RANGE-IOVFLO.LE.I.LE.IOVFLO CAN BE HANDLED BY THE SHORTEST INTEGER
C     TYPE IN USE.
C     NEMIN MUST BE SET TO THE MINIMUM NUMBER OF ELIMINATIONS IN A STEP
C     THAT IS AUTOMATICALLY ACCEPTED. IF TWO ADJACENT STEPS CAN BE
C     COMBINED AND EACH HAS LESS ELIMINATIONS THEN THEY ARE COMBINED.
C     IFRLVL MUST BE SET TO CONTROL WHETHER DIRECT OR INDIRECT ACCESS IS
C     USED BY UXSIBB.  INDIRECT ACCESS IS EMPLOYED
C     IN FORWARD AND BACK SUBSTITUTION RESPECTIVELY IF THE SIZE OF
C     A BLOCK IS LESS THAN IFRLVL(MIN0(10,NPIV)) AND
C     IFRLVL(10+MIN0(10,NPIV)) RESPECTIVELY, WHERE NPIV IS THE NUMBER
C     OF PIVOTS IN THE BLOCK.
C
      COMMON /UMMAT3/ IOVFLO, NEMIN, IFRLVL(20)
      DATA LP /6/, LDIAG /0/, U /0.1D0/, NEMIN /1/, MP /6/,
     & IOVFLO /2139062143/, IFRLVL(1), IFRLVL(2), IFRLVL(3),IFRLVL(4),
     & IFRLVL(5), IFRLVL(6), IFRLVL(7), IFRLVL(8), IFRLVL(9),
     & IFRLVL(10), IFRLVL(11), IFRLVL(12), IFRLVL(13), IFRLVL(14),
     & IFRLVL(15), IFRLVL(16), IFRLVL(17), IFRLVL(18), IFRLVL(19),
     & IFRLVL(20) /32639,32639,32639,32639,14,9,8,8,9,10,
     &             32639,32639,32639,32689,24,11,9,8,9,10/
      END
