* SUBROUTINE UXSPCD                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF A DIRECTION OF NEGATIVE CURVATURE WITH RESPECT TO A
* SPARSE SYMMETRIC MATRIX A USING THE FACTORIZATION A+E=L*D*TRANS(L)
* STORED IN THE COMPACT SPARSE FORM.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  MMAX  LENGTH OF THE PRINCIPAL MATRIX VECTORS (SL,A).
*  RI  A(MMAX)      FACTORIZATION A+E=L*D*TRANS(L) OBTAINED BY THE
*         SUBROUTINE UXSPGF.IT CONTAINS THE NUMERICAL VALUES OF THE
*         FACTORS STORED IN THE COMPACT FORM ACCORDING TO THE
*         INFORMATION IN THE VECTORS PSL,SL.
*  II  PSL(N+1)  POINTER VECTOR OF THE FACTORIZED MATRIX A.
*  II  SL(MMAX)  COMPACT SHEME OF THE FACTORIZED MATRIX A.
*  RO  X(N)  COMPUTED DIRECTION OF NEGATIVE CURVATURE (I.E.
*         TRANS(X)*A*X<0) IF IT EXISTS.
*  II  INF  INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. THE
*         DIRECTION OF NEGATIVE CURVATURE EXISTS ONLY IF INF>0.
*
* METHOD :
* P.E.GILL, W.MURRAY : NEWTON TYPE METHODS FOR UNCONSTRAINED AND
* LINEARLY CONSTRAINED OPTIMIZATION, MATH. PROGRAMMING 28 (1974)
* PP. 311-350.
*
      SUBROUTINE UXSPCD(N,MMAX,A,PSL,SL,X,INF)
      INTEGER  N, MMAX, INF,PSL(N+1),SL(MMAX)
      $FLOAT  A(MMAX),X(N)
      INTEGER  I, J, IS
      $FLOAT ZERO, ONE
      PARAMETER (ZERO = 0.0D0, ONE = 1.0D0)
C
C     RIGHT HAND SIDE FORMATION
C
      DO 100 I=1,N
      X(I) = ZERO
100   CONTINUE
      IF (INF .LE. 0) RETURN
      X(INF) = ONE
C
C     BACK SUBSTITUTION
C
      DO 300 I=INF-1,1,-1
      IS=SL(I)+N+1
      DO 200 J=PSL(I)+I,PSL(I+1)+I-1
      X(I)=X(I)-A(J)*X(SL(IS))
      IS=IS+1
200   CONTINUE
300   CONTINUE
      RETURN
      END
