* SUBROUTINE UXSPQF                ALL SYSTEMS                90/12/01
C PORTABILITY : ALL SYSTEMS
C 90/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL  FACTORIZATION A+E=L*D*TRANS(L) OF A SPARSE
* SYMMETRIC POSITIVE DEFINITE MATRIX A+E WHERE D AND E ARE DIAGONAL
* POSITIVE DEFINITE MATRICES AND L IS A LOWER TRIANGULAR MATRIX. IF
* A IS SUFFICIENTLY POSITIVE DEFINITE THEN E=0. THE STRUCTURE ON
* INPUT WAS OBTAINED BY THE SYMBOLIC FACTORIZATION AND IT MAKES
* USE OF THE COMPACT SCHEME OF STORING THE SPARSE MATRIX IN THE
* POINTER ARRAY PSL ,INDEX ARRAY SL. NUMERICAL VALUES OF THE FACTOR
* CAN BE FOUND IN THE ARRAY A.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  MMAX LENGTH OF THE ARRAYS A,SL.
*  RU  A(MMAX) ON INPUT NUMERICAL VALUES OF THE LOWER HALF OF THE
*      MATRIX THAT IS BEEING FACTORIZED(INCLUDING THE DIAGONAL
*      ELEMENTS. ON OUTPUT IT CONTAINS FACTORS L AND D AS IF THEY
*      FORM THE LOWER HALF OF THE MATRIX.STRUCTURE INFORMATION
*      IS SAVED IN THE COMPACT SCHEME IN THE PAIR OF VECTORS PSL
*      AND SL.
*  II  PSL(NF+1) POINTER VECTOR OF THE FACTOR
*  II  SL(MMAX) STRUCTURE OF THE FACTOR IN THE COMPACT FORM
*  IA  WN11(NF+1) AUXILIARY VECTOR.
*  IA  WN12(NF+1) AUXILIARY VECTOR.
*  RA  RN01(NF+1) AUXILIARY VECTOR.
*  IO  INF  AN INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. IF
*         INF=0 THEN A IS SUFFICIENTLY POSITIVE DEFINITE AND E=0. IF
*         INF<0 THEN A IS NOT SUFFICIENTLY POSITIVE DEFINITE AND E>0. IF
*         INF>0 THEN THEN A IS INDEFINITE AND INF IS AN INDEX OF THE
*         MOST NEGATIVE DIAGONAL ELEMENT USED IN THE FACTORIZATION
*         PROCESS.
*  RU  ALF  ON INPUT A DESIRED TOLERANCE FOR POSITIVE DEFINITENESS. ON
*         OUTPUT THE MOST NEGATIVE DIAGONAL ELEMENT USED IN THE
*         FACTORIZATION PROCESS (IF INF>0).
*  RO  TAU  MAXIMUM DIAGONAL ELEMENT OF THE MATRIX E.
*
* METHOD :
* S.C.EISENSTAT,M.C.GURSKY,M.H.SCHULTZ,A.H.SHERMAN:YALE SPARSE MATRIX
* PACKAGE I. THE SYMMETRIC CODES,YALE UNIV. RES. REPT.
* NO.112,1977.
*
      SUBROUTINE UXSPQF(N,MMAX,A,PSL,SL,WN11,WN12,RN01,INF)
      INTEGER N,MMAX,PSL(N+1),SL(MMAX),WN11(N+1),WN12(N+1),INF
      $FLOAT A(MMAX),RN01(N)
      $FLOAT ZERO,RHO,SIG,TADD,TBDD
      INTEGER  I, J, K, II
      INTEGER ISTRT,ISTOP,NEWK,KPB,ISUB
      PARAMETER (ZERO=0.0D0)
      INF=0
      DO 100 I=1,N
      WN11(I)=0
      RN01(I)=ZERO
100   CONTINUE
      DO 600 J=1,N
C
C     DETERMINATION OF A DIAGONAL CORRECTION
C
      SIG=A(PSL(J)+J-1)
      RHO=ZERO
      NEWK=WN11(J)
200   K=NEWK
      IF(K.EQ.0) GO TO 400
      NEWK=WN11(K)
      KPB=WN12(K)
      TADD=A(KPB+K)
      TBDD=TADD*A(PSL(K)+K-1)
      RHO=RHO+TADD*TBDD
      ISTRT=KPB+1
      ISTOP=PSL(K+1)-1
      IF(ISTOP.LT.ISTRT) GO TO 200
C
      WN12(K)=ISTRT
      I=SL(K)+(KPB-PSL(K))+1
      ISUB=SL(N+1+I)
      WN11(K)=WN11(ISUB)
      WN11(ISUB)=K
      DO 300 II=ISTRT,ISTOP
      ISUB=SL(N+1+I)
      RN01(ISUB)=RN01(ISUB)+A(II+K)*TBDD
      I=I+1
300   CONTINUE
      GO TO 200
400   SIG=A(PSL(J)+J-1)-RHO
      ISTRT=PSL(J)
      ISTOP=PSL(J+1)-1
      IF(ISTOP.LT.ISTRT) GO TO 370
      WN12(J)=ISTRT
      I=SL(J)
      ISUB=SL(N+1+I)
      WN11(J)=WN11(ISUB)
      WN11(ISUB)=J
      DO 500 II=ISTRT,ISTOP
      ISUB=SL(N+1+I)
      A(II+J)=(A(II+J)-RN01(ISUB))
      RN01(ISUB)=ZERO
      I=I+1
500   CONTINUE
370   RHO=SIG
C
C     GAUSSIAN ELIMINATION
C
      A(PSL(J)+J-1)=RHO
      IF(RHO.EQ.ZERO) GO TO 600
      IF(RHO.LT.ZERO) THEN
      INF=1
      RETURN
      ENDIF 
      DO 550 II=ISTRT,ISTOP
      A(II+J)=A(II+J)/RHO
550   CONTINUE
600   CONTINUE
      RETURN
      END
