* SUBROUTINE UXSPT4                ALL SYSTEMS                03/12/01
C PORTABILITY : ALL SYSTEMS
C 03/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* INCOMPLETE CHOLESKY FACTORIZATION A+E=L*D*TRANS(L) OF A SPARSE
* SYMMETRIC POSITIVE DEFINITE MATRIX A+E WHERE D AND E ARE DIAGONAL
* POSITIVE DEFINITE MATRICES AND L IS A LOWER TRIANGULAR MATRIX.
* METHOD IS BASED ON AJIZ-JENNINGS REJECTION WITHOUT PERMUTATION
* (DIAG_ONE=0 VARIANT ONLY). SPARSE LEFT-LOOKING IMPLEMENTATION.

* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  M NUMBER OF NONZERO ELEMENTS OF THE MATRIX A.
*  II  MMAX DIMENSION OF THE ARRAY AP.
*  RI  A(M)  SPARSE SYMMETRIC (USUALLY POSITIVE DEFINITE) MATRIX.
*  II  IA(N+1)  POINTERS OF THE DIAGONAL ELEMENTS OF A.
*  II  JA(M)  INDICES OF THE NONZERO ELEMENTS OF A.
*  RO  AP(MMAX)  INCOMPLETE FACTORIZATION A+E=L*D*TRANS(L).
*  IO  IP(N+1)  POINTERS OF THE DIAGONAL ELEMENTS OF A.
*  IO  JP(MMAX)  INDICES OF THE NONZERO ELEMENTS OF A.
*  IA  LIST(N+1)  AUXILIARY ARRAY.
*  IA  STRT(N+1)  AUXILIARY ARRAY.
*  IA  WN01(N+1)  AUXILIARY ARRAY.
*  IA  WN02(N+1)  AUXILIARY ARRAY.
*  RA  WR01(N+1)  AUXILIARY ARRAY.
*  RA  DIAG(N)  AUXILIARY ARRAY.
*  RI  PSI  DROPPING PARAMETER.
*  IO  INF  AN INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. IF
*         INF=0 THEN A IS SUFFICIENTLY POSITIVE DEFINITE AND E=0. IF
*         INF<0 THEN A IS NOT SUFFICIENTLY POSITIVE DEFINITE AND E>0. IF
*         INF>0 THEN A IS INDEFINITE AND INF IS AN INDEX OF THE
*         MOST NEGATIVE DIAGONAL ELEMENT USED IN THE FACTORIZATION
*         PROCESS.
*  RU  ALF  ON INPUT A DESIRED TOLERANCE FOR POSITIVE DEFINITENESS.
*         ON OUTPUT THE MOST NEGATIVE DIAGONAL ELEMENT USED IN THE
*         FACTORIZATION PROCESS (IF INF>0).
*  RO  TAU  MAXIMUM DIAGONAL ELEMENT OF THE MATRIX E.
*
* METHOD :
* P.E.GILL, W.MURRAY : NEWTON TYPE METHODS FOR UNCONSTRAINED AND
* LINEARLY CONSTRAINED OPTIMIZATION, MATH. PROGRAMMING 28 (1974)
* PP. 311-350.
*
      SUBROUTINE UXSPT4(N,M,MMAX,A,IA,JA,AP,IP,JP,LIST,START,WN01,
     & WN02,WR01,DIAG,PSI,INF,ALF,TAU)
      INTEGER N,M,MMAX,IA(N+1),JA(M),IP(N+1),JP(MMAX),LIST(N+1),
     & START(N+1),WN01(N+1),WN02(N+1),INF
      $FLOAT A(M),AP(MMAX),WR01(N+1),DIAG(N),PSI,ALF,TAU
      INTEGER JNDIAG,INDOLD,COLNEX,COLCUR,NXTROW
      $FLOAT NDIAG,DEL,TEMP,PSI2,DIAGJ,TESTMP
      INTEGER I,J,K,II,LL,IND,IND2,ISTRT,ISTOP,NJA
      DOUBLE PRECISION TEMP2,MULT,MULT1,BET,GAM,PTOL
      PTOL=ALF
      NJA=IA(N+1)-1
      IF(MMAX.LE.NJA) THEN
        INF=-11
        RETURN
      END IF
C
C     INITIALIZE AN AUXILIARY VECTOR
C
      INF=0
      CALL UXVINS(N,0,WN01)
C
C     MOVE PRECONDITIONER INTO THE FINAL DATA STRUCTURES
C
      CALL UXVICP(N+1,IA,IP)
      CALL UXVICP(NJA,JA,JP)
      CALL UXVCOP(NJA,A,AP)
C
C     GILL-MURRAY MODIFICATION
C
      ALF=0.0$P 0
      BET=0.0$P 0
      GAM=0.0$P 0
      TAU=0.0$P 0
      DO 60 I=1,N
        J=IA(I)
        TEMP=A(J)
        BET=MAX(BET,ABS(TEMP))
        DO 50 J=IA(I)+1,IA(I+1)-1
          TEMP=A(J)
          GAM=MAX(GAM,ABS(TEMP))
 50     CONTINUE
 60   CONTINUE
      BET=MAX(PTOL,BET,GAM/$DBLE(N))
      DEL=PTOL*BET
C
C     INTIALIZE WORKING VECTORS
C
      CALL UXVSET(N,0.0$P 0,WR01)
      CALL UXVINS(N,0,LIST)
      DO I=1,N
        J=IA(I)
        DIAG(I)=A(J)
      END DO
C
C     LOOP FOR COLUMNS
C
      DO 400 J=1,N
        IND2=0
        COLNEX=LIST(J)
 200    COLCUR=COLNEX
        IF(COLCUR.NE.0) THEN
          COLNEX=LIST(COLCUR)
          JNDIAG=START(COLCUR)
          NDIAG=AP(JNDIAG)
          TEMP=NDIAG*AP(IP(COLCUR))
          DIAG(J)=DIAG(J)-NDIAG*TEMP
          ISTRT=JNDIAG+1
          ISTOP=IP(COLCUR+1)-1
C
C     UPDATE LIST FOR THE ROW NXTROW
C
          IF(ISTRT.LE.ISTOP) THEN
            START(COLCUR)=ISTRT
            NXTROW=JP(ISTRT)
            LIST(COLCUR)=LIST(NXTROW)
            LIST(NXTROW)=COLCUR
          END IF
C
C     GATHER UPDATES FROM PREVIOUS COLUMNS
C     MUST BE TESTED WHETHER USING A VECTOR IN NODAL VERSION
C     IS NECESSARY FOR MORE EFFICIENT IMPLEMENTATION SINCE
C     CACHE CAN ADOPT IN THE SAME WAY A PART OF VECTOR A.
C
          DO 300 II=ISTRT,ISTOP
            K=JP(II)
            IF(WN01(K).EQ.0) THEN
              IND2=IND2+1
              WN02(IND2)=K
              WN01(K)=IND2
            END IF
            WR01(K)=WR01(K)-AP(II)*TEMP
 300      CONTINUE
C
C     BACK TO NEXT COLUMN HAVING A NONZERO IN A ROW I
C
          GO TO 200
        END IF
C
C     ADD CONTRIBUTIONS FROM THE ORIGINAL J-TH COLUMN
C     COMPUTE AVERAGE SIZE OF AN ORIGINAL NONZERO
C
        DO 500 II=IA(J)+1,IA(J+1)-1
          K=JA(II)
          TEMP=A(II)
          WR01(K)=WR01(K)+TEMP
          IF(WN01(K).EQ.0) THEN
            IND2=IND2+1
            WN02(IND2)=K
            WN01(K)=IND2
          END IF
 500    CONTINUE
C
C     SORT CONTRIBUTIONS IN WN02
C
        CALL UXVSRT(IND2,WN02)
C
C     DEFINE NEXT UNUSED POSITION
C
        IND=IP(J)+1
        IF(IND+IND2.GE.MMAX) THEN
          INF=-12
          RETURN
        END IF
C
C     AJIZ-JENNINGS DROPPING
C
        PSI2=PSI*PSI
        DIAGJ=DIAG(J)
        TESTMP=PSI2*DIAGJ
        INDOLD=IND
        DO 510 II=1,IND2
          TEMP=WR01(WN02(II))
          TEMP2=TEMP*TEMP
          WN01(WN02(II))=0
          WR01(WN02(II))=0.0$P 0
          K=WN02(II)
          IF(TEMP2.GE.TESTMP*DIAG(K)) THEN
            AP(IND)=TEMP
            JP(IND)=WN02(II)
            IND=IND+1
          ELSE
            MULT1=SQRT(DIAGJ/DIAG(K))
            MULT=MULT1*ABS(TEMP)
            DIAG(J)=DIAG(J)+MULT
            MULT=ABS(TEMP)/MULT1
            DIAG(K)=DIAG(K)+MULT
          END IF
 510    CONTINUE
        IP(J+1)=IND
C
C     TEST DIAGONAL ELEMENT SIZE
C
        IF (ALF.GT.DIAG(J)) THEN
          ALF=DIAG(J)
          LL=J
        ENDIF
C        TEMP=MAX(ABS(DIAG(J)),GAM/BET,DEL)
        TEMP=MAX(ABS(DIAG(J)),DEL)
        IF (TAU.LT.TEMP-DIAG(J)) THEN
          TAU=TEMP-DIAG(J)
          INF=-1
        ENDIF
        DIAG(J)=1.0$P 0/TEMP
C        IF(ABS(DIAG(J)).LT.DEL) THEN
C          DIAG(J)=DEL
C        END IF
C        DIAG(J)=1.0D0/ABS(DIAG(J))
C
C     INVERSE DIAGONAL ELEMENT INTO DECOMPOSED AP
C
        AP(IP(J))=DIAG(J)
        JP(IP(J))=J
C
C     DEFINE START,TOP,IP(J+1)
C
        IF(IND.NE.IP(J)+1) THEN
          START(J)=IP(J)+1
          COLNEX=JP(IP(J)+1)
          LIST(J)=LIST(COLNEX)
          LIST(COLNEX)=J
        END IF
C
  400 CONTINUE
      IF (LL.GT.0.AND.ABS(ALF).GT.DEL) INF=LL
      RETURN
      END

