* SUBROUTINE UXSPTB                ALL SYSTEMS                94/12/01
C PORTABILITY : ALL SYSTEMS
C 94/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A SPARSE SYMMETRIC
* POSITIVE DEFINITE MATRIX A+E USING INCOMPLETE ILUT-TYPE FACTORIZATION
* A+E=L*D*TRANS(L) OBTAINED BY THE SUBROUTINE UXSPTF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  MMAX NUMBER OF NONZERO ELEMENTS OF THE MATRIX A.
*  RI  A(MMAX) INCOMPLETE FACTORIZATION A+E=L*D*TRANS(L) OBTAINED BY THE
*         SUBROUTINE UXSPTF.
*  II  IA(N+1)  POINTERS OF THE DIAGONAL ELEMENTS OF A.
*  II  JA(MMAX)  INDICES OF THE NONZERO ELEMENTS OF A.
*  RU  X(N)  ON INPUT THE RIGHT HAND SIDE OF A SYSTEM OF LINEAR
*         EQUATIONS. ON OUTPUT THE SOLUTION OF A SYSTEM OF LINEAR
*         EQUATIONS.
*  II  JOB  OPTION. IF JOB=0 THEN X:=(A+E)**(-1)*X. IF JOB>0 THEN
*         X:=L**(-1)*X. IF JOB<0 THEN X:=TRANS(L)**(-1)*X.
*
* METHOD :
* BACK SUBSTITUTION
*
      SUBROUTINE UXSPTB(N,MMAX,A,IA,JA,X,JOB)
      INTEGER N,MMAX,IA(N+1),JA(MMAX),JOB
      $FLOAT A(MMAX),X(N)
      INTEGER I,J,K
      $FLOAT TEMP,SUM
C
C     FIRST PHASE
C
      IF(JOB.GE.0) THEN
        DO 300 I=1,N
          K=IA(I)
          IF (K.LE.0) GO TO 300
          TEMP=X(I)*A(K)
          DO 200 J=IA(I)+1,IA(I+1)-1
            K=JA(J)
            IF (K.GT.0) X(K)=X(K)-A(J)*TEMP
200       CONTINUE
300     CONTINUE
      END IF
C
C     SECOND PHASE
C
      IF (JOB.EQ.0) THEN
        DO 400 I=1,N
          K=IA(I)
          IF (K.LE.0) GO TO 400
          X(I)=X(I)*A(K)
400     CONTINUE
      ENDIF
C
C     THIRD PHASE
C
      IF(JOB.LE.0) THEN
        DO 600 I=N,1,-1
          K=IA(I)
          IF (K.LE.0) GO TO 600
          TEMP=A(K)
          SUM=0.0$P 0
          DO 500 J=IA(I)+1,IA(I+1)-1
            K=JA(J)
            IF (K.GT.0) SUM=SUM+A(J)*X(K)
500       CONTINUE
          SUM=SUM*TEMP
          X(I)=X(I)-SUM
600     CONTINUE
      END IF
      RETURN
      END
