* SUBROUTINE UXSRB1                ALL SYSTEMS               91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
*  GENERATES LEVEL STRUCTURE ROOTED AT THE INPUT NODE.
*
*  PARAMETERS :
*  II  ROOT THE NODE AT WHICH THE LEVEL STRUCTURE IS TO
*        BE ROOTED.
*  II  XADJ(N+1) POINTER VECTOR FOR THE ADJACENCY STRUCTURE.
*  II  ADJNCY(*) ADJACENCY STRUCTURE OF THE INPUT GRAPH.
*  II  MASK(N)  USED TO SPECIFY A SECTION SUBGRAPH.
*        NODES WITH MASK(I)=0 ARE IGNORED.
*  IO  NLVL(N)  NUMBER OF LEVELS IN THE LEVEL STRUCTURE.
*  IO  XLS(N+1) POINTER VECTOR FOR THE ROOTED LEVEL STRUCTURE.
*  IO  LS(N) INDICES OF THE ROOTED LEVEL STRUCTURE.
*
* SEE GEORGE-LIU: SPARSPAK
*
      SUBROUTINE  UXSRB1 (ROOT,XADJ,ADJNCY,MASK,NLVL,
     *                     XLS,LS)
C
         INTEGER    ADJNCY(*), LS(*), MASK(*), XLS(*), XADJ(*)
         INTEGER    I     , J     , JSTOP , JSTRT , LBEGIN,
     *              CCSIZE, LVLEND, LVSIZE, NBR   , NLVL  ,
     *              NODE  , ROOT
C
C INITIALIZATION
C
         MASK(ROOT) = 0
         LS(1) = ROOT
         NLVL = 0
         LVLEND = 0
         CCSIZE = 1
C
C        LBEGIN IS THE POINTER TO THE BEGINNING OF THE CURRENT
C        LEVEL, AND LVLEND POINTS TO THE END OF THIS LEVEL.
C
  100    CONTINUE
             LBEGIN = LVLEND + 1
             LVLEND = CCSIZE
             NLVL = NLVL + 1
             XLS(NLVL) = LBEGIN
C
C            GENERATE THE NEXT LEVEL BY FINDING ALL THE MASKED
C            NEIGHBORS OF NODES IN THE CURRENT LEVEL.
C
             DO  300  I = LBEGIN, LVLEND
                 NODE = LS(I)
                 JSTRT = XADJ(NODE)
                 JSTOP = XADJ(NODE+1) - 1
                 IF  ( JSTOP .LT. JSTRT )  GO TO 300
                     DO  200  J = JSTRT, JSTOP
                         NBR = ADJNCY(J)
                         IF  ( MASK(NBR) .EQ. 0 )  GO TO 200
                             CCSIZE = CCSIZE + 1
                             LS(CCSIZE) = NBR
                             MASK(NBR) = 0
  200                CONTINUE
  300        CONTINUE
C
C            COMPUTE THE CURRENT LEVEL WIDTH.
C            IF IT IS NONZERO, GENERATE THE NEXT LEVEL.
C
             LVSIZE = CCSIZE - LVLEND
             IF  ( LVSIZE .GT. 0 )  GO TO 100
C
C
C        RESET MASK TO ONE FOR THE NODES IN THE LEVEL STRUCTURE.
C
         XLS(NLVL+1) = LVLEND + 1
         DO  400  I = 1, CCSIZE
             NODE = LS(I)
             MASK(NODE) = 1
  400    CONTINUE
         RETURN
C
      END
