* SUBROUTINE UXTQEP                ALL SYSTEMS                06/12/01
C PORTABILITY : ALL SYSTEMS
C 06/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE EIGENVALUES AND EIGENVECTORS OF A SYMMETRIC
* TRIDIAGONAL MATRIX BY THE QL METHOD.
*
* PARAMETERS :
*  II  N  ORDER OF THE TRIDIAGONAL MATRIX T.
*  RU  D(N)  ON INPUT DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T.
*         ON OUTPUT EIGENVALUES OF THE TRIDIAGONAL MATRIX T.
*  RU  E(N)  ON INPUT SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T
*         ON POSITIONS FROM 2 TO N (E(1) IS ARBITRARY).
*  RO  Z(N*N)  EIGENVECTORS OF THE TRIDIAGONAL MATRIX T.
*  RO  IER  ERROR INDICATOR.
*
      SUBROUTINE UXTQEP(N,D,E,Z,IER)
      INTEGER N,IER
      $FLOAT D(N),E(N),Z(N*N)
      INTEGER I,II,IP0,IP1,J,JP0,K,KP0,L,L1,M,MML
      $FLOAT B,C,F,G,H,P,R,S,PARM
      IER=0
      PARM=1.0$P-16
      IF (N.LE.1) RETURN
      DO 102 I=2,N
      E(I-1)=E(I)
  102 CONTINUE
      F=0.0$P 0
      B=0.0$P 0
      E(N)=0.0$P 0
      DO 240 L=1,N
      J=0
      H=PARM*(ABS(D(L))+ABS(E(L)))
      IF (B.LT.H) B=H
      DO 110 M=L,N
      IF (ABS(E(M)).LE.B) GO TO 120
  110 CONTINUE
  120 IF (M.LE.L) GO TO 210
  130 IF (J.GE.30) THEN
      IER=L
      RETURN
      ENDIF
      J=J+1
      L1=L+1
      G=D(L)
      P=(D(L1)-G)/(2.0$P 0*E(L))
      R=SQRT(1.0$P 0+P*P)
      D(L)=E(L)/(P+SIGN(R,P))
      H=G-D(L)
      DO 140 I=L1,N
      D(I)=D(I)-H
  140 CONTINUE
      F=F+H
      P=D(M)
      C=1.0$P 0
      S=0.0$P 0
      MML=M-L
      IP1=(M-1)*N
      DO 200 II=1,MML
      IP0=IP1-N
      I=M-II
      G=C*E(I)
      H=C*P
      IF (ABS(P).GE.ABS(E(I))) THEN
      C=E(I)/P
      R=SQRT(1.0$P 0+C*C)
      E(I+1)=S*P*R
      S=C/R
      C=1.0$P 0/R
      ELSE
      C=P/E(I)
      R=SQRT(1.0$P 0+C*C)
      E(I+1)=S*E(I)*R
      S=1.0$P 0/R
      C=C*S
      ENDIF
      P=C*D(I)-S*G
      D(I+1)=H+S*(C*G+S*D(I))
      DO 180 K=1,N
      H=Z(K+IP1)
      Z(K+IP1)=S*Z(K+IP0)+C*H
      Z(K+IP0)=C*Z(K+IP0)-S*H
  180 CONTINUE
      IP1=IP0
  200 CONTINUE
      E(L)=S*P
      D(L)=C*P
      IF (ABS(E(L)).GT.B) GO TO 130
  210 D(L)=D(L)+F
  240 CONTINUE
      IP0=0
      DO 300 II=2,N
      I=II-1
      K=I
      P=D(I)
      KP0=IP0
      JP0=IP0
      DO 260 J=II,N
      JP0=JP0+N
      IF (D(J).LT.P) THEN
      K=J
      KP0=JP0
      P=D(J)
      ENDIF
  260 CONTINUE
      IF (K.LE.I) GO TO 290
      D(K)=D(I)
      D(I)=P
      DO 280 J=1,N
      P=Z(J+IP0)
      Z(J+IP0)=Z(J+KP0)
      Z(J+KP0)=P
  280 CONTINUE
  290 IP0=IP0+N
  300 CONTINUE
      RETURN
      END
