* SUBROUTINE UXTSMC                ALL SYSTEMS                10/12/01
C PORTABILITY : ALL SYSTEMS
C 10/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* CHOLESKI DECOMPOSITION OF A TRIDIAGONAL MATRIX.
*
* PARAMETERS :
*  II  N  ORDER OF THE TRIDIAGONAL MATRIX T.
*  RU  D(N)  ON INPUT DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T.
*         ON OUTPUT ELEMENTS OF THE DIAGONAL MATRIX D IN THE
*         DECOMPOSITION T=L*D*TRANS(L).
*  RU  E(N)  ON INPUT SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T.
*         ON OUTPUT SUBDIAGONAL ELEMENTS OF THE LOWER TRIANGULAR MATRIX L
*         IN THE DECOMPOSITION T=L*D*TRANS(L).
*  II  JOB  OPTION. JOB=0-NO CORRECTION. JOB=1-SUPERDIAGONAL ELEMENTS
*         ARE DIVIDED BY TWO. JOB=2-SUPERDIAGONAL ELEMENT IS DIVIDED
*         BY TWO ONLY IF THE CORRESPONDING SUBDETERMINANT IS NEGATIVE.
*         JOB=3-SUPERDIAGONAL ELEMENTS ARE MODIFIED IN SUCH A WAY
*         THAT THE NEGAGIVE SUBDETERMINANTS ARE CHANGED TO BE ZEROS.
*
      SUBROUTINE UXTSMC(N,D,E,JOB)
      INTEGER N,JOB
      $FLOAT  D(N),E(N)
      INTEGER I
      $FLOAT BET,BETA
      IF (JOB.LE.0) THEN
      ELSE IF (JOB.EQ.1) THEN
      DO 1 I=1,N-1
      E(I)=0.5$P 0*E(I)
    1 CONTINUE
      ELSE IF (JOB.EQ.2) THEN
      DO 2 I=1,N-1
      IF (D(I)*D(I+1).LT.4.0$P 0*E(I)*E(I)) E(I)=0.5$P 0*E(I)
    2 CONTINUE
      ELSE IF (JOB.EQ.3) THEN
      DO 3 I=1,N-1
      IF (D(I)*D(I+1).LT.4.0$P 0*E(I)*E(I))
     & E(I)=0.5$P 0*SIGN(SQRT(ABS(D(I)*D(I+1))),E(I))
    3 CONTINUE
      ENDIF
      RETURN
      END
