* SUBROUTINE UXVORG               ALL SYSTEMS                16/12/01
C PORTABILITY : ALL SYSTEMS
C 16/12/01 LU : ORIGINAL VERSION
*
* PURPOSE : COMPUTATION OF THE HOUSEHOLDER MATRIX THAT ZEROES LAST 
* N-1 ELEMENTS OF A GIVEN VECTOR.
* 
*
* PARAMETERS :
*  II  N  THE ORDER OF THE ELEMENTARY REFLECTOR.
*  RU  ALF  ON INPUT THE FIRST ELEMENT OF THE TRANSFORMED VECTOR.
*         ON OUTPUT NORM OF THE TRANSFORMED VECTOR. 
*  RU  X(*)  ON INPUT THE TRANSFORMED VECTOR. ON OUTPUT THE VECTOR
*         DEFINING THE ELEMENTARY REFLECTOR.
*  II  K  INCREMENT BETWEEN ELEMENTS OF X.
*  RO  D  THE SCALAR PARAMETER OF THE ELEMENTARY REFLECTOR. 
*
* SUBPROGRAMS USED :
*  S   UXVNOG  EUCLIDEAN NORM OF A VECTOR.
*  S   UXVSCG  SCALING OF A VECTOR. 
*
      SUBROUTINE UXVORG(N,ALF,X,K,D)
      INTEGER N,K
      $FLOAT ALF,X(*),D
      INTEGER J,L
      $FLOAT BET,RSMIN,SMIN,XNORM,UXVNOG
      $FLOAT ONE,ZERO
      PARAMETER (ONE=1.0$P 0,ZERO=0.0$P 0)
      IF (N.LE.1) THEN
        D=ZERO
        RETURN
      END IF
      XNORM=UXVNOG(N-1,X,K)
      IF (XNORM.EQ.ZERO) THEN
        D=ZERO
      ELSE
*
*     GENERAL CASE
*
        BET=-SIGN(SQRT(ALF**2+XNORM**2),ALF)
        SMIN=1.0$P-16
        IF (ABS(BET).LT.SMIN) THEN
*
*     XNORM AND BETA MAY BE INNACCURATE - SCALE X AND RECOMPUTE THEM
*
          RSMIN=ONE/SMIN
          L=0
   10     CONTINUE
          L=L + 1
          CALL UXVSCG(N-1,RSMIN,X,X,K)
          BET=BET*RSMIN
          ALF=ALF*RSMIN
          IF (ABS(BET).LT.SMIN) GO TO 10
*
*     NEW ETA IS AT MOST 1 AND AT LEAST SMIN
*
          XNORM=UXVNOG(N-1,X,K)
          BET=-SIGN(SQRT(ALF**2+XNORM**2),ALF)
          D=(BET-ALF)/BET
          CALL UXVSCG(N-1,ONE/(ALF-BET),X,X,K)
*
*     ALF IS SUBNORMAL - IT MAY LOSE RELATIVE ACCURACY
*
          ALF=BET
          DO 20 J=1, L
            ALF=ALF*SMIN
   20     CONTINUE
        ELSE
          D=(BET-ALF)/BET
          CALL UXVSCG(N-1,ONE/(ALF-BET),X,X,K)
          ALF=BET
        END IF
      END IF
      RETURN
      END
