$INTERFACE UYAGS1                                             99/12/01

  CALL UYAGS1(NF,N,$IX,$H,$G,$GO,$GP,$GN,$S,$GS,ALFN,ALFV)
  $ADD(INTEGER,'\$IX($NF)')
  $ADD(REAL,'\$H($NF*($NF+1)/2)\$G($NF)\$GO($NF)\$GP($NF)\$GN($NF)&
  \$S($NF)\$GS($NF)')
  $ADD(REAL,'\ALFN\ALFV')
$REM  $ADD(INTEGER,'\NF\N')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYAGS1                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SUBGRADIENT AGGREGATION FOR NONSMOOTH VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  H(N*(N+1)/2)  POSITIVE DEFINITE APPROXIMATION OF THE INVERSE
*         HESSIAN MATRIX.
*  RI  G(NF)  SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  GO(NF)  PREVIOUS SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GV(NF)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GN(NF)  REDUCED AGGREGATED SUBGRADIENT.
*  RI  S(NF)  DIRECTION VECTOR.
*  RA  U(N)  AUXILIARY VECTOR.
*  RO  ALF  LINEARIZATION TERM.
*  RU  ALFV  AGGREGATED LINEARIZATION TERM.
*
      SUBROUTINE UYAGS1(NF,N,IX,H,G,GO,GV,GN,S,U,ALF,ALFV)
      INTEGER NF,N,IX(NF)
      $FLOAT H(N*(N+1)/2),G(NF),GO(NF),GV(NF),GN(N),S(NF),U(N),ALF,
     & ALFV
      $FLOAT A,B,ALFM,LAM1,LAM2,PQ,PR,PRQR,QQP,QR,RR,RRP,RRQ,W,W1,W2
      INTEGER I,J,K,L
      $FLOAT ZERO,ONE
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
      ALFM=ZERO
C
C     General routine - here always input parameter ALFM=0
C
      RR=ALFV+ALFV
      RRP=ALFV-ALFM
      RRQ=ALFV-ALF
      K=0
      DO 1 I=1,NF
        IF (IX(I).GE.0) THEN
          K=K+1
          RR=RR-S(I)*GV(I)
          U(K)=GO(I)-GV(I)
          PR=G(I)-GV(I)
          RRP=RRP+S(I)*U(K)
          RRQ=RRQ+S(I)*PR
          S(K)=PR
        ENDIF
    1 CONTINUE
      PQ=ZERO
      PR=ZERO
      QR=ZERO
      QQP=ALF-ALFM
      PRQR=ZERO
      K=0
      DO 4 I=1,NF
        IF (IX(I).GE.0) THEN
          K=K+1
          L=K*(K-1)/2+1
          W1=ZERO
          W2=ZERO
          DO 2 J=1,K-1
            W=H(L)
            W1=W1+W*S(J)
            W2=W2+W*U(J)
    2       L=L+1
          DO 3 J=K,N
            W=H(L)
            W1=W1+W*S(J)
            W2=W2+W*U(J)
    3       L=L+J
          PR=PR+U(K)*W2
          QR=QR+S(K)*W1
          PQ=PQ+(S(K)-U(K))*(W1-W2)
          QQP=QQP+G(I)*(W1-W2)
          PRQR=PRQR+U(K)*W1
        ENDIF
    4 CONTINUE
      IF(PR.LE.ZERO.OR.QR.LE.ZERO)GOTO10
      A=RRQ/QR
      B=PRQR/QR
      W=PRQR*B-PR
      IF(W.EQ.ZERO)GOTO10
      LAM1=(A*PRQR-RRP)/W
      LAM2=A-LAM1*B
      IF(LAM1*(LAM1-ONE).LT.ZERO.AND.LAM2*(LAM1+LAM2-ONE).LT.ZERO)
     & GOTO40
C
C     MINIMUM ON THE BOUNDARY
C
   10 LAM1=ZERO
      LAM2=ZERO
      IF(ALF.LE.ALFV)LAM2=ONE
      IF(QR.GT.ZERO)LAM2=MIN(ONE,MAX(ZERO,RRQ/QR))
      W=(LAM2*QR-RRQ-RRQ)*LAM2
      A=ZERO
      IF(ALFM.LE.ALFV)A=ONE
      IF(PR.GT.ZERO)A=MIN(ONE,MAX(ZERO,RRP/PR))
      B=(A*PR-RRP-RRP)*A
      IF(B.LT.W)THEN
        W=B
        LAM1=A
        LAM2=ZERO
      ENDIF
      IF(QQP*(QQP-PQ).GE.ZERO)GOTO40
      IF(QR-RRQ-RRQ-QQP*QQP/PQ.GE.W)GOTO40
      LAM1=QQP/PQ
      LAM2=ONE-LAM1
   40 IF(LAM1.EQ.ZERO.AND.LAM2*(LAM2-ONE).LT.ZERO.AND.RRP-LAM2*PRQR
     &.GT.ZERO.AND.PR.GT.ZERO)LAM1=MIN(ONE-LAM2,(RRP-LAM2*PRQR)/PR)
      A=ONE-LAM1-LAM2
      K=0
      DO 5 I=1,NF
        GV(I)=LAM1*GO(I)+LAM2*G(I)+A*GV(I)
        IF (IX(I).GE.0) THEN
          K=K+1
          GN(K)=GV(I)
        ENDIF
    5 CONTINUE
      ALFV=LAM1*ALFM+LAM2*ALF+A*ALFV
      RETURN
      END
