$INTERFACE UYALU1                                             04/12/01

  CALL UYALU1(NF,NN,$XM,$XR,$GR,$G,$GO,$GP,$S,$GS,PAR,ALFN,ALFV)
  $ADD(REAL,'\$XM($NF*$MF)\$XR($MF)\$GR($MF)\$G($NF)\$GO($NF)\$GP($NF)&
  \$S($NF)\$GS($NF)')
  $ADD(REAL,'\ALFN\ALFV')
$REM  $ADD(REAL,'\PAR')
$REM  $ADD(INTEGER,'\NF\NN')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYALU1                ALL SYSTEMS                04/12/01
C PORTABILITY : ALL SYSTEMS
C 04/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SUBGRADIENT AGGREGATION FOR NONSMOOTH VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  M  NUMBER OF COLUMNS OF XM.
*  RI  XM(N*M)  RECTANGULAR MATRIX IN THE PRODUCT FORM SHIFTED BROYDEN
*         METHOD (STORED COLUMNWISE): H-SIGMA*I=XM*TRANS(XM)
*  RA  XR(M)  AUXILIARY VECTOR.
*  RA  GR(M)  AUXILIARY VECTOR.
*  RI  G(N)  SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  GO(N)  PREVIOUS SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GV(N)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  S(N)  DIRECTION VECTOR.
*  RA  U(N)  AUXILIARY VECTOR.
*  RI  PAR  SHIFT PARAMETER.
*  RO  ALF  LINEARIZATION TERM.
*  RU  ALFV  AGGREGATED LINEARIZATION TERM.
*
      SUBROUTINE UYALU1(N,M,XM,XR,GR,G,GO,GV,S,U,PAR,ALF,ALFV)
      INTEGER N,M
      $FLOAT XM(N*M),XR(M),GR(M),G(N),GO(N),GV(N),S(N),U(N),PAR,ALF,ALFV
      $FLOAT A,B,ALFM,LAM1,LAM2,PQ,PR,PRQR,QQP,QR,RR,RRP,RRQ,W
      INTEGER I
      $FLOAT ZERO,ONE,UXVDOT
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
      ALFM=ZERO
C
C     General routine - here always input parameter ALFM=0
C
      RR=ALFV+ALFV
      RRP=ALFV-ALFM
      RRQ=ALFV-ALF
      DO 1 I=1,N
        A=S(I)
        U(I)=GO(I)-GV(I)
        S(I)=G(I)-GV(I)
        RR=RR-A*GV(I)
        RRP=RRP+A*U(I)
        RRQ=RRQ+A*S(I)
    1 CONTINUE
      PQ=ZERO
      PR=ZERO
      QR=ZERO
      QQP=ALF-ALFM
      PRQR=ZERO
      CALL UXDRMM(N,M,XM,U,XR)
      CALL UXDRMM(N,M,XM,S,GR)
      DO 4 I=1,N
        PR=PR+PAR*U(I)**2
        QR=QR+PAR*S(I)**2
        PRQR=PRQR+PAR*U(I)*S(I)
        PQ=PQ+PAR*(S(I)-U(I))**2
        QQP=QQP+PAR*G(I)*(S(I)-U(I))
    4 CONTINUE
      DO 2 I=1,M
        PR=PR+XR(I)**2
        QR=QR+GR(I)**2
        PRQR=PRQR+XR(I)*GR(I)
        PQ=PQ+(GR(I)-XR(I))**2
        GR(I)=GR(I)-XR(I)
C        QQP=QQP+G(I)*(GR(I)-XR(I))
    2 CONTINUE
      CALL UXDRMM(N,M,XM,G,XR)
      QQP=QQP+UXVDOT(M,XR,GR)
      IF (PR.LE.ZERO.OR.QR.LE.ZERO) GO TO 10
      A=RRQ/QR
      B=PRQR/QR
      W=PRQR*B-PR
      IF (W.EQ.ZERO) GO TO 10
      LAM1=(A*PRQR-RRP)/W
      LAM2=A-LAM1*B
      IF (LAM1*(LAM1-ONE).LT.ZERO.AND.LAM2*(LAM1+LAM2-ONE).LT.ZERO)
     & GO TO 40
C
C     MINIMUM ON THE BOUNDARY
C
   10 LAM1=ZERO
      LAM2=ZERO
      IF (ALF.LE.ALFV) LAM2=ONE
      IF (QR.GT.ZERO) LAM2=MIN(ONE,MAX(ZERO,RRQ/QR))
      W=(LAM2*QR-RRQ-RRQ)*LAM2
      A=ZERO
      IF (ALFM.LE.ALFV) A=ONE
      IF (PR.GT.ZERO) A=MIN(ONE,MAX(ZERO,RRP/PR))
      B=(A*PR-RRP-RRP)*A
      IF(B.LT.W)THEN
        W=B
        LAM1=A
        LAM2=ZERO
      ENDIF
      IF (QQP*(QQP-PQ).GE.ZERO) GO TO 40
      IF (QR-RRQ-RRQ-QQP*QQP/PQ.GE.W) GO TO 40
      LAM1=QQP/PQ
      LAM2=ONE-LAM1
   40 IF (LAM1.EQ.ZERO.AND.LAM2*(LAM2-ONE).LT.ZERO.AND.RRP-LAM2*PRQR
     & .GT.ZERO.AND.PR.GT.ZERO) LAM1=MIN(ONE-LAM2,(RRP-LAM2*PRQR)/PR)
      A=ONE-LAM1-LAM2
      ALFV=LAM1*ALFM+LAM2*ALF+A*ALFV
      DO 5 I=1,N
        GV(I)=LAM1*GO(I)+LAM2*G(I)+A*GV(I)
    5 CONTINUE
      RETURN
      END
