$INTERFACE UYFUT3                                             98/12/01

  $IF ($DEF(LOCALSTOP))
    $LOCALSTOP
  $ENDIF
  $HELP
    MAXIMUM NUMBER OF ITERATIONS
  $CHECK(MIT,1000,INT,3)
  $HELP
    MAXIMUM NUMBER OF COMPUTED FUNCTION VALUES
  $CHECK(MFV,1000,INT,3)
  $HELP
    MAXIMUM NUMBER OF COMPUTED GRADIENT VALUES
  $CHECK(MFG,10000,INT,3)
  $HELP
    LOWER BOUND FOR STEPLENGTH
  $CHECK(TOLX,'1.0$P-8',REAL,3)
  $HELP
    LOWER BOUND FOR FUNCTION VALUE
  $CHECK(TOLB,'1.0$P-16',REAL,3)
  CALL UYFUT3($N,F,DMAX,ITES,IRES1,IRES2)
$REM  $ADD(INTEGER,'\$N\ITES')
$REM  $ADD(REAL,'\F\DMAX')
  $ADD(INIT)
    ITES=1
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYFUT3                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* TERMINATION CRITERIA AND TEST ON RESTART.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RI  DMAX  MAXIMUM RELATIVE DIFFERENCE OF VARIABLES.
*  II  ITES  SYSTEM VARIBLE WHICH SPECIFIES TERMINATION. IF ITES=0
*         THEN TERMINATION IS SUPPRESSED.
*  II  IRES1  RESTART SPECIFICATION. RESTART IS PERFORMED AFTER
*         IRES1*N+IRES2 ITERATIONS.
*  II  IRES2  RESTART SPECIFICATION. RESTART IS PERFORMED AFTER
*         IRES1*N+IRES2 ITERATIONS.
*
* COMMON DATA :
*  II  ITERS  TERMINATION INDICATOR FOR STEPLENGTH DETERMINATION.
*         ITERS=0 FOR ZERO STEP.
*  IO  ITERM  TERMINATION INDICATOR. ITERM=1-TERMINATION AFTER MTESX
*         UNSUFFICIENT STEPLENGTHS. ITERM=2-TERMINATION AFTER MTESF
*         UNSUFFICIENT FUNCTION DECREASES. ITERM=3-TERMINATION ON LOWER
*         BOUND FOR FUNCTION VALUE. ITERM=4-TERMINATION ON LOWER BOUND
*         FOR GRADIENT. ITERM=11-TERMINATION AFTER MAXIMUM NUMBER OF
*         ITERATIONS. ITERM=12-TERMINATION AFTER MAXIMUM NUMBER OF
*         COMPUTED FUNCTION VALUES.
*  IU  IREST  RESTART INDICATOR. RESTART IS PERFORMED IF IREST>0.
*  II  MTESX  MAXIMUM NUMBER OF TESTS ON STEPLENGTH.
*  IU  NTESX  ACTUAL NUMBER OF TESTS ON STEPLENGTH.
*  II  MTESF  MAXIMUM NUMBER OF TESTS ON FUNCTION DECREASE.
*  IU  NTESF  ACTUAL NUMBER OF TESTS ON FUNCTION DECREASE.
*  II  LIT  NUMBER OF THE FIRST ITERATION.
*  II  MIT  MAXIMUM NUMBER OF ITERATIONS.
*  IU  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  LFV  NUMBER OF THE FIRST COMPUTED FUNCTION VALUE.
*  II  MFV  MAXIMUM NUMBER OF COMPUTED FUNCTION VALUES.
*  IU  NFV  ACTUAL NUMBER OF COMPUTED FUNCTION VALUES.
*  II  LFG  NUMBER OF THE FIRST COMPUTED GRADIENT VALUE.
*  II  MFG  MAXIMUM NUMBER OF COMPUTED GRADIENT VALUES.
*  IU  NFG  ACTUAL NUMBER OF COMPUTED GRADIENT VALUES.
*  II  KD  DEGREE OF REQUIRED DERIVATIVES.
*  II  JOUT  SYSTEM VARIABLE WHICH SPECIFIES PRINT OF INFORMATION
*         DURING THE OPTIMIZATION PROCESS. JOUT=0-NO PRINT.
*         JOUT=1-PRINT IS REQUIRED.
*  II  KOUT  LEVEL OF PRINT. KOUT=0-NO PRINT. KOUT=1-PRINT OF INITIAL
*         VALUES AND FINAL RESULTS TOGETHER WITH PRINT OF ITERATIONS.
*         KOUT=2-ADDITIONAL PRINT OF STEPSIZE SELECTION.
*         KOUT=3-ADDITIONAL PRINT OF DIRECTION DETERMINATION.
*         KOUT=4-ADDITIONAL PRINT OF MATRIX CORRECTION.
*         KOUT=5-ADDITIONAL PRINT OF NUMERICAL DIFFERENTIATION.
*         KOUT=6-ADDITIONAL PRINT OF DEBUGGING.
*  II  KOUT1  LOWER ITERATION IN WHICH INFORMATION IS PRINTED.
*  II  KOUT2  UPPER ITERATION IN WHICH INFORMATION IS PRINTED.
*  II  KOUT3  NUMBER OF ITERATIONS AFTER THAT INFORMATION IS PRINTED.
*  RI  TOLX  LOWER BOUND FOR STEPLENGTH.
*  RI  TOLB  LOWER BOUND FOR FUNCTION VALUE.
*  RI  TOLG  LOWER BOUND FOR GRADIENT.
*  TU  TPXX  TEXT INFORMATION ON THE CAUSE OF RESTART.
*  TO  TXFU  TEXT INFORMATION ON THE CAUSE OF TERMINATION.
*
* SUBPROGRAMS USED :
*  S   UYRES1  DEFINITION OF THE RESTART VARIABLES.
*
      SUBROUTINE UYFUT3(N,F,DMAX,ITES,IRES1,IRES2)
      $INCLUDE ('UMCOMM')
      INTEGER N,ITES,IRES1,IRES2
      $FLOAT  F,DMAX
      IF (ITERM.LT.0) RETURN
      IF (ITES .LE.0) GOTO 2
      IF ( F .LE. TOLB)  THEN
      ITERM = 3
      TXFU = 'FV BOUND'
      RETURN
      ENDIF
      IF (NIT .GT. LIT .AND. DMAX .LE. TOLX)  THEN
      ITERM = 4
      TXFU = 'STEP TOL'
      RETURN
      ENDIF
      IF (NIT-LIT .GE. MIT)  THEN
      ITERM = 11
      TXFU = 'IT LIMIT'
      CALL UOERR1('UYFUT3',-1)
      RETURN
      ENDIF
      IF (NFV-LFV .GE. MFV)  THEN
      ITERM = 12
      TXFU = 'FV LIMIT'
      CALL UOERR1('UYFUT3',-2)
      RETURN
      ENDIF
      IF ( NFG-LFG .GE. MFG)  THEN
      ITERM = 13
      TXFU = 'FG LIMIT'
      CALL UOERR1('UYFUT1',-3)
      RETURN
      ENDIF
    2 ITERM = 0
      TPXX='        '
      IF ( N .GT. 0 .AND. NIT-KIT .GE. IRES1*N + IRES2)   THEN
      CALL UYRES1('NORMAL  ')
      ENDIF
      NIT = NIT + 1
      JOUT=0
      IF(KOUT.EQ.0.OR.KOUT3.LE.0) RETURN
      IF(KOUT1.LE.KOUT2) THEN
        IF(NIT-LIT.GE.KOUT1.AND.NIT-LIT.LE.KOUT2) JOUT=1
      ELSE
        IF(NIT-LIT.GE.KOUT1.OR. NIT-LIT.LE.KOUT2) JOUT=1
      ENDIF
      IF(NIT-LIT-(INT((NIT-LIT)/KOUT3)*KOUT3).NE.0) JOUT=0
      RETURN
      END
