* SUBROUTINE UYSSLO                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF A SMALLEST-LAST ORDERING OF THE COLUMNS OF A.
*
* PARAMETERS :
*  II  NF DECLARED NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  IA(NA+1) ROW POINTERS OF THE SPARSE JACOBIAN MATRIX.
*  II  JA(MA) COLUMN INDICES OF THE SPARSE JACOBIAN MATRIX.
*  II  IB(NF+1) COLUMN POINTERS OF THE SPARSE JACOBIAN MATRIX.
*  II  JB(MA) ROW INDICES OF THE SPARSE JACOBIAN MATRIX.
*  II  IDEG(NF) VECTOR SPECIFYING DEGREE SEQUENCE.
*  IO  LIST(NF) VECTOR SPECIFYING INCIDENCE-DEGREE ORDERING.
*  IA  IW1(NF)  AUXILIARY VECTOR.
*  IA  IW2(NF)  AUXILIARY VECTOR.
*  IA  IW3(NF)  AUXILIARY VECTOR.
*  IA  IW4(NF)  AUXILIARY VECTOR.
*  IO  MAXC  SIZE OF THE LARGEST CLIQUE FOUND DURING THE ORDERING.
*
      SUBROUTINE UYSSLO(NF,NA,MA,IA,JA,IB,JB,IDEG,LIST,IW1,IW2,IW3,IW4,
     & MAXC)
      INTEGER MAXC
      INTEGER NF,NA,MA,IA(NA+1),JA(MA),IB(NF+1),JB(MA),IDEG(NF),
     & LIST(NF),IW1(0:NF-1),IW2(NF),IW3(NF),IW4(NF)
      INTEGER IC,IP,IR,JC,JP,MDEG,NUMD,NUMO
C
C     INITIALIZATION BLOCK
C
      MDEG = NF
      DO 10 JP = 1, NF
      IW1(JP-1) = 0
      IW4(JP) = NF
      LIST(JP) = IDEG(JP)
      MDEG = MIN(MDEG,IDEG(JP))
   10 CONTINUE
      DO 20 JP = 1, NF
      NUMD = IDEG(JP)
      IW2(JP) = 0
      IW3(JP) = IW1(NUMD)
      IF (IW1(NUMD) .GT. 0) IW2(IW1(NUMD)) = JP
      IW1(NUMD) = JP
   20 CONTINUE
      MAXC = 0
      NUMO = NF
C
C     BEGINNING OF ITERATION LOOP
C
   30 CONTINUE
C
C     MARK THE SIZE OF THE LARGEST CLIQUE
C     FOUND DURING THE ORDERING
C
      IF (MDEG + 1 .EQ. NUMO .AND. MAXC .EQ. 0) MAXC = NUMO
C
C     CHOOSE A COLUMN JC OF MINIMAL DEGREE MDEG
C
   40 CONTINUE
      JC = IW1(MDEG)
      IF (JC .GT. 0) GO TO 50
      MDEG = MDEG + 1
      GO TO 40
   50 CONTINUE
      LIST(JC) = NUMO
      NUMO = NUMO - 1
C
C     TERMINATION TEST
C
      IF (NUMO .EQ. 0) GO TO 80
C
C     DELETE COLUMN JC FROM THE MDEG LIST
C
      IW1(MDEG) = IW3(JC)
      IF (IW3(JC) .GT. 0) IW2(IW3(JC)) = 0
C
C     FIND ALL COLUMNS ADJACENT TO COLUMN JC
C
      IW4(JC) = 0
C
C     DETERMINE ALL POSITIONS (IR,JC) WHICH CORRESPOND
C     TO NON-ZEROES IN THE MATRIX
C
      DO 70 JP = IB(JC), IB(JC+1)-1
      IR = JB(JP)
C
C     FOR EACH ROW IR, DETERMINE ALL POSITIONS (IR,IC)
C     WHICH CORRESPOND TO NON-ZEROES IN THE MATRIX
C
      DO 60 IP = IA(IR), IA(IR+1)-1
      IC = JA(IP)
C
C     ARRAY IW4 MARKS COLUMNS WHICH ARE ADJACENT TO
C     COLUMN JC
C
      IF (IW4(IC) .GT. NUMO) THEN
      IW4(IC) = NUMO
C
C     UPDATE THE POINTERS TO THE CURRENT DEGREE LISTS
C
      NUMD = LIST(IC)
      LIST(IC) = LIST(IC) - 1
      MDEG = MIN(MDEG,LIST(IC))
C
C     DELETE COLUMN IC FROM THE NUMD LIST
C
      IF (IW2(IC) .EQ. 0) THEN
      IW1(NUMD) = IW3(IC)
      ELSE
      IW3(IW2(IC)) = IW3(IC)
      END IF
      IF (IW3(IC) .GT. 0) IW2(IW3(IC)) = IW2(IC)
C
C     ADD COLUMN IC TO THE NUMD-1 LIST
C
      IW2(IC) = 0
      IW3(IC) = IW1(NUMD-1)
      IF (IW1(NUMD-1) .GT. 0) IW2(IW1(NUMD-1)) = IC
      IW1(NUMD-1) = IC
      END IF
   60 CONTINUE
   70 CONTINUE
C
C     END OF ITERATION LOOP
C
      GO TO 30
   80 CONTINUE
C
C     INVERT THE ARRAY LIST
C
      DO 90 JC = 1, NF
      IW2(LIST(JC)) = JC
   90 CONTINUE
      DO 100 JP = 1, NF
      LIST(JP) = IW2(JP)
  100 CONTINUE
      RETURN
      END
