$INTERFACE UYTHUT                                             02/12/01

  $HELP
    TOLERANCE FOR LINEAR INDEPENDENCE.
  $CHECK(EPS7,'1.0$P-5',REAL,4)
  $ADD(REAL,'\$XM($NF*$MF)\$XR($MF*($MF+1)/2)\$HR($MF*($MF+1)/2)&
  \$G($NF)\$GR($MF)')
$REM  $ADD(INTEGER,'\NF\MF\INF')
  CALL UYTHUT(NF,NN,MF,$XM,$XR,$HR,$G,$GR,GNORM,$EPS7,MOS,INF,'$DECOMP')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYTHUT                ALL SYSTEMS                02/12/01
C PORTABILITY : ALL SYSTEMS
C 02/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE REDUCED HESSIAN VARIABLE METRIC TRANSFORMATIONS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  IU  N  CURRENT NUMBER OF VARIABLE METRIC STEPS.
*  II  MF  MAXIMUM NUMBER OF VARIABLE METRIC STEPS.
*  RU  XM(NF*MF)  SET OF VECTOR OF VARIABLES DIFFERENCES.
*  RU  XR(MF*(MF+1)/2)  CHOLESKI DECOMPOSITION OF THE REDUCED
*         HESSIAN MATRIX.
*  RU  HR(MF*(MF+1)/2)  TRIANGULAR TRANFORMATION MATRIX.
*  RI  G(NF) GRADIENT VECTOR.
*  RO  GR(MF) REDUCED GRADIENT VECTOR.
*  RI  GNORM  NORM OF THE GRADIENT VECTOR.
*  RI  EPS7  TOLERANCE FOR LINEAR DEPENDENCE.
*  II  MOS  OPTION. MOS=1-INVERSION OF REDUCED HESSIAN MATRIX.
*         MOS=2-REDUCED INVERSION OF HESSIAN MATRIX.
*  IO  INF  LINEAR DEPENDENCE INDICATOR. INF=0 IF THE CURRENT GRADIENT
*         IS A LINEAR COMBINATION OF BASIC VECTORS. INF=1 OTHERWISE.
*  CO  TEXT  TYPE OF DECOMPOSITION. TEXT='R'-RECURSIVE QR DECOMPOSITION
*         WITHOUT PERMUTATIONS. TEXT='I'-INVERSION.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXDRMM  VECTOR MULTIPLIED BY A ROWWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*
      SUBROUTINE UYTHUT(NF,N,MF,XM,XR,HR,G,GR,GNORM,EPS7,MOS,INF,TEXT)
      $INCLUDE('UMCOMM')
      INTEGER NF,N,MF,MOS,INF
      $FLOAT XM(NF*MF),XR(MF*(MF+1)/2),HR(MF*(MF+1)/2),G(NF),GR(MF),
     & GNORM,EPS7
      CHARACTER*1 TEXT
      $FLOAT RHO,UXVDOT
      INTEGER I,IER
      IF (N.LT.MF) RETURN
      RHO=ABS(GR(N))
      IF (RHO.LE.EPS7*GNORM) THEN
      INF=0
      ELSE
      INF=1
      IF (TEXT.EQ.'R') THEN
      CALL UXDPRH(N,XR,HR,GR,GR,GR,IER,1)
      ELSE
      CALL UXDSMH(N,XR,HR,GR,GR,GR,IER,MOS,1)
      ENDIF
      DO 2 I=1,NF*(N-1)
      XM(I)=XM(I+NF)
    2 CONTINUE
      RHO=SQRT(ABS(UXVDOT(NF,G,G)-UXVDOT(N-1,GR,GR)))
      GR(N)=RHO
      CALL UXVCOP(NF,G,XM((N-1)*NF+1))
      CALL UXVCOP(N,GR,HR(N*(N-1)/2+1))
      ENDIF
      RETURN
      END
