$INTERFACE UYTRFS                                             98/12/01

  CALL UYTRFS(NF,MCL,$X,$IX,$XN,$XO,$XL,$XU,$G,$GO,$CF,$CFD,$IC,$CL&
,$CU,$CG,$S,RO,FP,FO,F,PO,P,CMAX,CMAXO,RMAX,KREM,INEW)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$XO($NF)\$XL($NF)\$XU($NF)\$G($NF)&
  \$GO($NF)\$CF($NC)\$CFD($NC)\$CL($NC)\$CU($NC)\$CG($NF*$MCL)\$S($NF)')
  $ADD(INTEGER,'\$IX($NX)\$IC($NC)')
$REM  $ADD(REAL,'\RO\FP\FO\F\PO\P\CMAX\CMAXO\RMAX')
$REM  $ADD(INTEGER,'\NF\MCL\KREM\INEW')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYTRFS               ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SCALED AND REDUCED DIRECTION VECTOR IS BACK TRANSFORMED.
* VECTORS X,G AND VALUES F,P ARE SAVED.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  NC  NUMBER OF LINEARIZED CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  XO(NF)  SAVED VECTOR OF VARIABLES.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  GO(NF)  SAVED GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  CF(NF)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCYIONS.
*  RO  CFD(NF)  VECTOR CONTAINING INCREMENTS OF THE CONSTRAINT
*         FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RO  S(NF)  DIRECTION VECTOR.
*  RO  RO  SAVED VALUE OF THE STEPSIZE PARAMETER.
*  RO  FP  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RU  FO  SAVED VALUE OF THE OBJECTIVE FUNCTION.
*  RI  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RO  PO  SAVED VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  P  VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  CMAXO  SAVED VALUE OF THE CONSTRAINT VIOLATION.
*  RI  CMAX  VALUE OF THE CONSTRAINT VIOLATION.
*  RO  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  II  KREM  INDICATION OF LINEARLY DEPENDENT GRADIENTS.
*  IO  INEW  INDEX OF THE NEW ACTIVE FUNCTION.
*
* COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* SUBPROGRAMS USED :
*  S   ULMAXS  DETERMINATION OF THE MAXIMUM STEPSIZE USING SIMPLE
*         BOUNDS.
*  S   ULMAXL  DETERMINATION OF THE MAXIMUM STEPSIZE USING LINEAR
*         CONSTRAINTS.
*  S   UXVCOP  COPYING OF A VECTOR.
*
      SUBROUTINE UYTRFS(NF,NC,X,IX,XN,XO,XL,XU,G,GO,CF,CFD,IC,CL,CU,CG,
     & S,RO,FP,FO,F,PO,P,CMAX,CMAXO,RMAX,KREM,INEW)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NC,IX(NF),IC(NC),KREM,INEW
      $FLOAT X(NF),XN(NF),XO(NF),XL(NF),XU(NF),G(NF),GO(NF),CF(NC),
     & CFD(NC),CL(NC),CU(NC),CG(NF*NC),S(NF),RO,FP,FO,F,PO,P,CMAX,
     & CMAXO,RMAX
      FP = FO
      RO = 0.0$P 0
      FO = F
      PO = P
      CMAXO = CMAX
      CALL UXVCOP(NF,X,XO)
      CALL UXVCOP(NF,G,GO)
      INEW=0
      CALL ULMAXL(NF,NC,CF,CFD,IC,CL,CU,CG,S,RMAX,KBC,KREM,INEW)
      IF (NORMF.GT.0) CALL UXVMUL(NF,XN,S,S,1)
      CALL ULMAXS(NF,X,IX,XL,XU,S,RMAX,KBF,KREM,INEW)
      RETURN
      END
