$INTERFACE UYTRPG                                            98/12/01

  $HELP
    PRECISION FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
  $CHECK(EPS7,'1.0$P-8',REAL,4)
  CALL UYTRPG(NF,N,MCL,MC,$MMAX,$IX,$XN,$IC,$ICA,$CG,$ICG,$JCG,$CR,$ICR&
,$JCR,$CZ,$G,$GN,$EPS7,UMAX,GMAX,IOLD)
  $ADD(REAL,'\$XN($NF)\$CG($MC)\$CR($MMAX)&
  \$CZ($NF)\$G($NF)\$GN($NF)')
  $ADD(INTEGER,'\$IX($NX)\$IC($NC)\$ICA($NF)&
  \$ICG($NC+1)\$JCG($MC)\$ICR($NF+1)\$JCR($MMAX)')
$REM  $ADD(INTEGER,'\NF\N\MCL\MC\IOLD')
$REM  $ADD(REAL,'\UMAX\GMAX')
  $ADD(INIT)
    ITRAN=2
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYTRPG               ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* GRADIENT OF THE OBJECTIVE FUNCTION IS SCALED AND REDUCED.
* TEST VALUES GMAX AND UMAX ARE COMPUTED.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS IN THE FIELD CG.
*  II  MMAX  MAXIMUM NUMBER OF NONZERO ELEMENTS IN THE FIELD CR.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CG(MC)  SPARSE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(MCL+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC) COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  RI  CR(MMAX)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  II  ICR(NF+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CR.
*  II  JCR(MMAX) COLUMN INDICES OF ELEMENTS IN THE FIELD CR.
*  RO  CZ(NF)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  GN(NF)  TRANSFORMED GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  UMAX  MAXIMUN ABSOLUTE VALUE OF THE NEGATIVE LAGRANGE MULTIPLIER.
*  RI  GMAX  NORM OF THE TRANSFORMED GRADIENT.
*  II  IOLD  INDEX OF THE REMOVED CONSTRAINT.
*
* COMMON DATA :
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* SUBPROGRAMS USED :
*  S   UMSLAG  NEGATIVE PROJECTED GRADIENT IS DETERMINED.
*  S   ULTLAG  THE OPTIMUM LAGRANGE MULTIPLIER IS DETERMINED.
*  S   UMVLAG  AN AUXILIARY VECTOR IS DETERMINED.
*  S   UXSPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXVSET  INITIATION OF A VECTOR.
*
      SUBROUTINE UYTRPG(NF,N,NC,MC,MMAX,IX,XN,IC,ICA,CG,ICG,JCG,CR,ICR,
     & JCR,CZ,G,GN,EPS7,UMAX,GMAX,IOLD)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N,NC,MC,MMAX,IX(NF),IC(NC),ICA(NF),ICG(NC+1),JCG(MC),
     & ICR(NF+1),JCR(MMAX),IOLD
      $FLOAT XN(NF),CG(MC),CR(MMAX),CZ(NF),G(NF),GN(NF),EPS7,UMAX,GMAX
      IF (NORMF.GT.0) THEN
      CALL UXVMUL(NF,XN,G,GN,1)
      ELSE
      CALL UXVCOP(NF,G,GN)
      ENDIF
      IF (NF.GT.N) THEN
      CALL UMVLAG(NF,N,NC,NC,NC,MC,MC,ICA,CG,ICG,JCG,CG,ICG,JCG,GN,CZ)
      CALL UXSPRB(NF-N,MMAX,CR,ICR,JCR,CZ,0)
      CALL ULTLAG(NF,N,NC,NC,NC,IX,IC,ICA,CZ,IC,EPS7,UMAX,IOLD)
      ELSE
      IOLD=0
      UMAX=0.0$P 0
      ENDIF
      IF (N.GT.0) THEN
      CALL UXVNEG(NF,GN,GN)
      CALL UMSLAG(NF,N,NC,NC,NC,MC,MC,ICA,CG,ICG,JCG,CZ,CG,ICG,JCG,GN,
     & EPS7,GMAX)
      ELSE
      CALL UXVSET(NF,0.0$P 0,GN)
      GMAX=0.0$P 0
      ENDIF
      RETURN
      END
